/****************************************************************************
** Meta object code from reading C++ file 'GRPCClient.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../internal/frontend/bridge-gui/bridgepp/bridgepp/GRPC/GRPCClient.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'GRPCClient.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8bridgepp23KnowledgeBaseSuggestionE_t {};
} // unnamed namespace

template <> constexpr inline auto bridgepp::KnowledgeBaseSuggestion::qt_create_metaobjectdata<qt_meta_tag_ZN8bridgepp23KnowledgeBaseSuggestionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "bridgepp::KnowledgeBaseSuggestion",
        "url",
        "title"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'url'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KnowledgeBaseSuggestion, qt_meta_tag_ZN8bridgepp23KnowledgeBaseSuggestionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject bridgepp::KnowledgeBaseSuggestion::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8bridgepp23KnowledgeBaseSuggestionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8bridgepp23KnowledgeBaseSuggestionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8bridgepp23KnowledgeBaseSuggestionE_t>.metaTypes,
    nullptr
} };

void bridgepp::KnowledgeBaseSuggestion::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<KnowledgeBaseSuggestion *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->url; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->title; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->url, *reinterpret_cast<QString*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->title, *reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN8bridgepp16UserNotificationE_t {};
} // unnamed namespace

template <> constexpr inline auto bridgepp::UserNotification::qt_create_metaobjectdata<qt_meta_tag_ZN8bridgepp16UserNotificationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "bridgepp::UserNotification",
        "title",
        "subtitle",
        "body",
        "userID"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'title'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'subtitle'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'body'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'userID'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UserNotification, qt_meta_tag_ZN8bridgepp16UserNotificationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject bridgepp::UserNotification::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8bridgepp16UserNotificationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8bridgepp16UserNotificationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8bridgepp16UserNotificationE_t>.metaTypes,
    nullptr
} };

void bridgepp::UserNotification::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<UserNotification *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->title; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->subtitle; break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->body; break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->userID; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->title, *reinterpret_cast<QString*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->subtitle, *reinterpret_cast<QString*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->body, *reinterpret_cast<QString*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->userID, *reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN8bridgepp10GRPCClientE_t {};
} // unnamed namespace

template <> constexpr inline auto bridgepp::GRPCClient::qt_create_metaobjectdata<qt_meta_tag_ZN8bridgepp10GRPCClientE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "bridgepp::GRPCClient",
        "internetStatus",
        "",
        "isOn",
        "toggleAutostartFinished",
        "resetFinished",
        "reportBugFinished",
        "reportBugSuccess",
        "reportBugError",
        "reportBugFallback",
        "certificateInstallSuccess",
        "certificateInstallCanceled",
        "certificateInstallFailed",
        "showMainWindow",
        "knowledgeBasSuggestionsReceived",
        "QList<KnowledgeBaseSuggestion>",
        "suggestions",
        "repairStarted",
        "allUsersLoaded",
        "userNotificationReceived",
        "UserNotification",
        "notification",
        "cantMoveDiskCache",
        "diskCachePathChanged",
        "QUrl",
        "path",
        "diskCachePathChangeFinished",
        "imapPortStartupError",
        "smtpPortStartupError",
        "imapPortChangeError",
        "smtpPortChangeError",
        "imapConnectionModeChangeError",
        "smtpConnectionModeChangeError",
        "mailServerSettingsChanged",
        "imapPort",
        "smtpPort",
        "useSSLForIMAP",
        "useSSLForSMTP",
        "changeMailServerSettingsFinished",
        "loginUsernamePasswordError",
        "errMsg",
        "loginFreeUserError",
        "loginConnectionError",
        "login2FARequested",
        "username",
        "login2FAError",
        "login2FAErrorAbort",
        "loginFidoRequested",
        "login2FAOrFidoRequested",
        "login2PasswordRequested",
        "login2PasswordError",
        "login2PasswordErrorAbort",
        "loginFinished",
        "userID",
        "wasSignedOut",
        "loginAlreadyLoggedIn",
        "loginHvRequested",
        "hvUrl",
        "loginHvError",
        "loginFidoTouchRequested",
        "loginFidoTouchCompleted",
        "loginFidoPinRequired",
        "loginFidoPinInvalid",
        "loginFidoPinBlocked",
        "loginFidoError",
        "updateManualError",
        "updateForceError",
        "updateSilentError",
        "updateManualReady",
        "version",
        "updateManualRestartNeeded",
        "updateForce",
        "updateSilentRestartNeeded",
        "updateIsLatestVersion",
        "checkUpdatesFinished",
        "updateVersionChanged",
        "toggleSplitModeFinished",
        "userDisconnected",
        "userChanged",
        "userBadEvent",
        "errorMessage",
        "usedBytesChanged",
        "usedBytes",
        "imapLoginFailed",
        "syncStarted",
        "syncFinished",
        "syncProgress",
        "progress",
        "elapsedMs",
        "remainingMs",
        "changeKeychainFinished",
        "hasNoKeychain",
        "rebuildKeychain",
        "certIsReady",
        "addressChanged",
        "address",
        "addressChangedLogout",
        "apiCertIssue",
        "genericError",
        "ErrorInfo",
        "info"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'internetStatus'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'toggleAutostartFinished'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resetFinished'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'reportBugFinished'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'reportBugSuccess'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'reportBugError'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'reportBugFallback'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'certificateInstallSuccess'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'certificateInstallCanceled'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'certificateInstallFailed'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showMainWindow'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'knowledgeBasSuggestionsReceived'
        QtMocHelpers::SignalData<void(QList<KnowledgeBaseSuggestion> const &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'repairStarted'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'allUsersLoaded'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'userNotificationReceived'
        QtMocHelpers::SignalData<void(UserNotification const &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'cantMoveDiskCache'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'diskCachePathChanged'
        QtMocHelpers::SignalData<void(QUrl const &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Signal 'diskCachePathChangeFinished'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'imapPortStartupError'
        QtMocHelpers::SignalData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'smtpPortStartupError'
        QtMocHelpers::SignalData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'imapPortChangeError'
        QtMocHelpers::SignalData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'smtpPortChangeError'
        QtMocHelpers::SignalData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'imapConnectionModeChangeError'
        QtMocHelpers::SignalData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'smtpConnectionModeChangeError'
        QtMocHelpers::SignalData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'mailServerSettingsChanged'
        QtMocHelpers::SignalData<void(qint32, qint32, bool, bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 34 }, { QMetaType::Int, 35 }, { QMetaType::Bool, 36 }, { QMetaType::Bool, 37 },
        }}),
        // Signal 'changeMailServerSettingsFinished'
        QtMocHelpers::SignalData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loginUsernamePasswordError'
        QtMocHelpers::SignalData<void(QString const &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 40 },
        }}),
        // Signal 'loginFreeUserError'
        QtMocHelpers::SignalData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loginConnectionError'
        QtMocHelpers::SignalData<void(QString const &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 40 },
        }}),
        // Signal 'login2FARequested'
        QtMocHelpers::SignalData<void(QString const &)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 44 },
        }}),
        // Signal 'login2FAError'
        QtMocHelpers::SignalData<void(QString const &)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 40 },
        }}),
        // Signal 'login2FAErrorAbort'
        QtMocHelpers::SignalData<void(QString const &)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 40 },
        }}),
        // Signal 'loginFidoRequested'
        QtMocHelpers::SignalData<void(QString const &)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 44 },
        }}),
        // Signal 'login2FAOrFidoRequested'
        QtMocHelpers::SignalData<void(QString const &)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 44 },
        }}),
        // Signal 'login2PasswordRequested'
        QtMocHelpers::SignalData<void(QString const &)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 44 },
        }}),
        // Signal 'login2PasswordError'
        QtMocHelpers::SignalData<void(QString const &)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 40 },
        }}),
        // Signal 'login2PasswordErrorAbort'
        QtMocHelpers::SignalData<void(QString const &)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 40 },
        }}),
        // Signal 'loginFinished'
        QtMocHelpers::SignalData<void(QString const &, bool)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 53 }, { QMetaType::Bool, 54 },
        }}),
        // Signal 'loginAlreadyLoggedIn'
        QtMocHelpers::SignalData<void(QString const &)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 53 },
        }}),
        // Signal 'loginHvRequested'
        QtMocHelpers::SignalData<void(QString const &)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 57 },
        }}),
        // Signal 'loginHvError'
        QtMocHelpers::SignalData<void(QString const &)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 40 },
        }}),
        // Signal 'loginFidoTouchRequested'
        QtMocHelpers::SignalData<void(QString const &)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 44 },
        }}),
        // Signal 'loginFidoTouchCompleted'
        QtMocHelpers::SignalData<void(QString const &)>(60, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 44 },
        }}),
        // Signal 'loginFidoPinRequired'
        QtMocHelpers::SignalData<void(QString const &)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 44 },
        }}),
        // Signal 'loginFidoPinInvalid'
        QtMocHelpers::SignalData<void(QString const &)>(62, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 40 },
        }}),
        // Signal 'loginFidoPinBlocked'
        QtMocHelpers::SignalData<void(QString const &)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 40 },
        }}),
        // Signal 'loginFidoError'
        QtMocHelpers::SignalData<void(QString const &)>(64, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 40 },
        }}),
        // Signal 'updateManualError'
        QtMocHelpers::SignalData<void()>(65, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateForceError'
        QtMocHelpers::SignalData<void()>(66, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateSilentError'
        QtMocHelpers::SignalData<void()>(67, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateManualReady'
        QtMocHelpers::SignalData<void(QString const &)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 69 },
        }}),
        // Signal 'updateManualRestartNeeded'
        QtMocHelpers::SignalData<void()>(70, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateForce'
        QtMocHelpers::SignalData<void(QString const &)>(71, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 69 },
        }}),
        // Signal 'updateSilentRestartNeeded'
        QtMocHelpers::SignalData<void()>(72, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateIsLatestVersion'
        QtMocHelpers::SignalData<void()>(73, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'checkUpdatesFinished'
        QtMocHelpers::SignalData<void()>(74, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateVersionChanged'
        QtMocHelpers::SignalData<void()>(75, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'toggleSplitModeFinished'
        QtMocHelpers::SignalData<void(QString const &)>(76, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 53 },
        }}),
        // Signal 'userDisconnected'
        QtMocHelpers::SignalData<void(QString const &)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 44 },
        }}),
        // Signal 'userChanged'
        QtMocHelpers::SignalData<void(QString const &)>(78, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 53 },
        }}),
        // Signal 'userBadEvent'
        QtMocHelpers::SignalData<void(QString const &, QString const &)>(79, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 53 }, { QMetaType::QString, 80 },
        }}),
        // Signal 'usedBytesChanged'
        QtMocHelpers::SignalData<void(QString const &, qint64)>(81, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 53 }, { QMetaType::LongLong, 82 },
        }}),
        // Signal 'imapLoginFailed'
        QtMocHelpers::SignalData<void(QString const &)>(83, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 44 },
        }}),
        // Signal 'syncStarted'
        QtMocHelpers::SignalData<void(QString const &)>(84, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 53 },
        }}),
        // Signal 'syncFinished'
        QtMocHelpers::SignalData<void(QString const &)>(85, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 53 },
        }}),
        // Signal 'syncProgress'
        QtMocHelpers::SignalData<void(QString const &, double, qint64, qint64)>(86, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 53 }, { QMetaType::Double, 87 }, { QMetaType::LongLong, 88 }, { QMetaType::LongLong, 89 },
        }}),
        // Signal 'changeKeychainFinished'
        QtMocHelpers::SignalData<void()>(90, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasNoKeychain'
        QtMocHelpers::SignalData<void()>(91, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rebuildKeychain'
        QtMocHelpers::SignalData<void()>(92, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'certIsReady'
        QtMocHelpers::SignalData<void()>(93, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'addressChanged'
        QtMocHelpers::SignalData<void(QString const &)>(94, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 95 },
        }}),
        // Signal 'addressChangedLogout'
        QtMocHelpers::SignalData<void(QString const &)>(96, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 95 },
        }}),
        // Signal 'apiCertIssue'
        QtMocHelpers::SignalData<void()>(97, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'genericError'
        QtMocHelpers::SignalData<void(ErrorInfo)>(98, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 99, 100 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GRPCClient, qt_meta_tag_ZN8bridgepp10GRPCClientE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject bridgepp::GRPCClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8bridgepp10GRPCClientE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8bridgepp10GRPCClientE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8bridgepp10GRPCClientE_t>.metaTypes,
    nullptr
} };

void bridgepp::GRPCClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GRPCClient *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->internetStatus((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->toggleAutostartFinished(); break;
        case 2: _t->resetFinished(); break;
        case 3: _t->reportBugFinished(); break;
        case 4: _t->reportBugSuccess(); break;
        case 5: _t->reportBugError(); break;
        case 6: _t->reportBugFallback(); break;
        case 7: _t->certificateInstallSuccess(); break;
        case 8: _t->certificateInstallCanceled(); break;
        case 9: _t->certificateInstallFailed(); break;
        case 10: _t->showMainWindow(); break;
        case 11: _t->knowledgeBasSuggestionsReceived((*reinterpret_cast<std::add_pointer_t<QList<KnowledgeBaseSuggestion>>>(_a[1]))); break;
        case 12: _t->repairStarted(); break;
        case 13: _t->allUsersLoaded(); break;
        case 14: _t->userNotificationReceived((*reinterpret_cast<std::add_pointer_t<UserNotification>>(_a[1]))); break;
        case 15: _t->cantMoveDiskCache(); break;
        case 16: _t->diskCachePathChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 17: _t->diskCachePathChangeFinished(); break;
        case 18: _t->imapPortStartupError(); break;
        case 19: _t->smtpPortStartupError(); break;
        case 20: _t->imapPortChangeError(); break;
        case 21: _t->smtpPortChangeError(); break;
        case 22: _t->imapConnectionModeChangeError(); break;
        case 23: _t->smtpConnectionModeChangeError(); break;
        case 24: _t->mailServerSettingsChanged((*reinterpret_cast<std::add_pointer_t<qint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint32>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 25: _t->changeMailServerSettingsFinished(); break;
        case 26: _t->loginUsernamePasswordError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 27: _t->loginFreeUserError(); break;
        case 28: _t->loginConnectionError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 29: _t->login2FARequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 30: _t->login2FAError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 31: _t->login2FAErrorAbort((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 32: _t->loginFidoRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 33: _t->login2FAOrFidoRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 34: _t->login2PasswordRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 35: _t->login2PasswordError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 36: _t->login2PasswordErrorAbort((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 37: _t->loginFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 38: _t->loginAlreadyLoggedIn((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 39: _t->loginHvRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 40: _t->loginHvError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 41: _t->loginFidoTouchRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 42: _t->loginFidoTouchCompleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 43: _t->loginFidoPinRequired((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 44: _t->loginFidoPinInvalid((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 45: _t->loginFidoPinBlocked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 46: _t->loginFidoError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 47: _t->updateManualError(); break;
        case 48: _t->updateForceError(); break;
        case 49: _t->updateSilentError(); break;
        case 50: _t->updateManualReady((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 51: _t->updateManualRestartNeeded(); break;
        case 52: _t->updateForce((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 53: _t->updateSilentRestartNeeded(); break;
        case 54: _t->updateIsLatestVersion(); break;
        case 55: _t->checkUpdatesFinished(); break;
        case 56: _t->updateVersionChanged(); break;
        case 57: _t->toggleSplitModeFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 58: _t->userDisconnected((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 59: _t->userChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 60: _t->userBadEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 61: _t->usedBytesChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 62: _t->imapLoginFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 63: _t->syncStarted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 64: _t->syncFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 65: _t->syncProgress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[4]))); break;
        case 66: _t->changeKeychainFinished(); break;
        case 67: _t->hasNoKeychain(); break;
        case 68: _t->rebuildKeychain(); break;
        case 69: _t->certIsReady(); break;
        case 70: _t->addressChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 71: _t->addressChangedLogout((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 72: _t->apiCertIssue(); break;
        case 73: _t->genericError((*reinterpret_cast<std::add_pointer_t<ErrorInfo>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(bool )>(_a, &GRPCClient::internetStatus, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::toggleAutostartFinished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::resetFinished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::reportBugFinished, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::reportBugSuccess, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::reportBugError, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::reportBugFallback, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::certificateInstallSuccess, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::certificateInstallCanceled, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::certificateInstallFailed, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::showMainWindow, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QList<KnowledgeBaseSuggestion> const & )>(_a, &GRPCClient::knowledgeBasSuggestionsReceived, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::repairStarted, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::allUsersLoaded, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(UserNotification const & )>(_a, &GRPCClient::userNotificationReceived, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::cantMoveDiskCache, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QUrl const & )>(_a, &GRPCClient::diskCachePathChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::diskCachePathChangeFinished, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::imapPortStartupError, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::smtpPortStartupError, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::imapPortChangeError, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::smtpPortChangeError, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::imapConnectionModeChangeError, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::smtpConnectionModeChangeError, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(qint32 , qint32 , bool , bool )>(_a, &GRPCClient::mailServerSettingsChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::changeMailServerSettingsFinished, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::loginUsernamePasswordError, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::loginFreeUserError, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::loginConnectionError, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::login2FARequested, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::login2FAError, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::login2FAErrorAbort, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::loginFidoRequested, 32))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::login2FAOrFidoRequested, 33))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::login2PasswordRequested, 34))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::login2PasswordError, 35))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::login2PasswordErrorAbort, 36))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & , bool )>(_a, &GRPCClient::loginFinished, 37))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::loginAlreadyLoggedIn, 38))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::loginHvRequested, 39))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::loginHvError, 40))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::loginFidoTouchRequested, 41))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::loginFidoTouchCompleted, 42))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::loginFidoPinRequired, 43))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::loginFidoPinInvalid, 44))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::loginFidoPinBlocked, 45))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::loginFidoError, 46))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::updateManualError, 47))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::updateForceError, 48))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::updateSilentError, 49))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::updateManualReady, 50))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::updateManualRestartNeeded, 51))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::updateForce, 52))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::updateSilentRestartNeeded, 53))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::updateIsLatestVersion, 54))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::checkUpdatesFinished, 55))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::updateVersionChanged, 56))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::toggleSplitModeFinished, 57))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::userDisconnected, 58))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::userChanged, 59))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & , QString const & )>(_a, &GRPCClient::userBadEvent, 60))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & , qint64 )>(_a, &GRPCClient::usedBytesChanged, 61))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::imapLoginFailed, 62))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::syncStarted, 63))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::syncFinished, 64))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & , double , qint64 , qint64 )>(_a, &GRPCClient::syncProgress, 65))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::changeKeychainFinished, 66))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::hasNoKeychain, 67))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::rebuildKeychain, 68))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::certIsReady, 69))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::addressChanged, 70))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(QString const & )>(_a, &GRPCClient::addressChangedLogout, 71))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)()>(_a, &GRPCClient::apiCertIssue, 72))
            return;
        if (QtMocHelpers::indexOfMethod<void (GRPCClient::*)(ErrorInfo )>(_a, &GRPCClient::genericError, 73))
            return;
    }
}

const QMetaObject *bridgepp::GRPCClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *bridgepp::GRPCClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8bridgepp10GRPCClientE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int bridgepp::GRPCClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 74)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 74;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 74)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 74;
    }
    return _id;
}

// SIGNAL 0
void bridgepp::GRPCClient::internetStatus(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void bridgepp::GRPCClient::toggleAutostartFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void bridgepp::GRPCClient::resetFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void bridgepp::GRPCClient::reportBugFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void bridgepp::GRPCClient::reportBugSuccess()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void bridgepp::GRPCClient::reportBugError()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void bridgepp::GRPCClient::reportBugFallback()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void bridgepp::GRPCClient::certificateInstallSuccess()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void bridgepp::GRPCClient::certificateInstallCanceled()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void bridgepp::GRPCClient::certificateInstallFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void bridgepp::GRPCClient::showMainWindow()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void bridgepp::GRPCClient::knowledgeBasSuggestionsReceived(QList<KnowledgeBaseSuggestion> const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void bridgepp::GRPCClient::repairStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void bridgepp::GRPCClient::allUsersLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void bridgepp::GRPCClient::userNotificationReceived(UserNotification const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void bridgepp::GRPCClient::cantMoveDiskCache()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void bridgepp::GRPCClient::diskCachePathChanged(QUrl const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void bridgepp::GRPCClient::diskCachePathChangeFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void bridgepp::GRPCClient::imapPortStartupError()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void bridgepp::GRPCClient::smtpPortStartupError()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void bridgepp::GRPCClient::imapPortChangeError()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void bridgepp::GRPCClient::smtpPortChangeError()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void bridgepp::GRPCClient::imapConnectionModeChangeError()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void bridgepp::GRPCClient::smtpConnectionModeChangeError()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void bridgepp::GRPCClient::mailServerSettingsChanged(qint32 _t1, qint32 _t2, bool _t3, bool _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 25
void bridgepp::GRPCClient::changeMailServerSettingsFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void bridgepp::GRPCClient::loginUsernamePasswordError(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 26, nullptr, _t1);
}

// SIGNAL 27
void bridgepp::GRPCClient::loginFreeUserError()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void bridgepp::GRPCClient::loginConnectionError(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 28, nullptr, _t1);
}

// SIGNAL 29
void bridgepp::GRPCClient::login2FARequested(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 29, nullptr, _t1);
}

// SIGNAL 30
void bridgepp::GRPCClient::login2FAError(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 30, nullptr, _t1);
}

// SIGNAL 31
void bridgepp::GRPCClient::login2FAErrorAbort(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 31, nullptr, _t1);
}

// SIGNAL 32
void bridgepp::GRPCClient::loginFidoRequested(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 32, nullptr, _t1);
}

// SIGNAL 33
void bridgepp::GRPCClient::login2FAOrFidoRequested(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 33, nullptr, _t1);
}

// SIGNAL 34
void bridgepp::GRPCClient::login2PasswordRequested(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 34, nullptr, _t1);
}

// SIGNAL 35
void bridgepp::GRPCClient::login2PasswordError(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 35, nullptr, _t1);
}

// SIGNAL 36
void bridgepp::GRPCClient::login2PasswordErrorAbort(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 36, nullptr, _t1);
}

// SIGNAL 37
void bridgepp::GRPCClient::loginFinished(QString const & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 37, nullptr, _t1, _t2);
}

// SIGNAL 38
void bridgepp::GRPCClient::loginAlreadyLoggedIn(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 38, nullptr, _t1);
}

// SIGNAL 39
void bridgepp::GRPCClient::loginHvRequested(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 39, nullptr, _t1);
}

// SIGNAL 40
void bridgepp::GRPCClient::loginHvError(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 40, nullptr, _t1);
}

// SIGNAL 41
void bridgepp::GRPCClient::loginFidoTouchRequested(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 41, nullptr, _t1);
}

// SIGNAL 42
void bridgepp::GRPCClient::loginFidoTouchCompleted(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 42, nullptr, _t1);
}

// SIGNAL 43
void bridgepp::GRPCClient::loginFidoPinRequired(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 43, nullptr, _t1);
}

// SIGNAL 44
void bridgepp::GRPCClient::loginFidoPinInvalid(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 44, nullptr, _t1);
}

// SIGNAL 45
void bridgepp::GRPCClient::loginFidoPinBlocked(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 45, nullptr, _t1);
}

// SIGNAL 46
void bridgepp::GRPCClient::loginFidoError(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 46, nullptr, _t1);
}

// SIGNAL 47
void bridgepp::GRPCClient::updateManualError()
{
    QMetaObject::activate(this, &staticMetaObject, 47, nullptr);
}

// SIGNAL 48
void bridgepp::GRPCClient::updateForceError()
{
    QMetaObject::activate(this, &staticMetaObject, 48, nullptr);
}

// SIGNAL 49
void bridgepp::GRPCClient::updateSilentError()
{
    QMetaObject::activate(this, &staticMetaObject, 49, nullptr);
}

// SIGNAL 50
void bridgepp::GRPCClient::updateManualReady(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 50, nullptr, _t1);
}

// SIGNAL 51
void bridgepp::GRPCClient::updateManualRestartNeeded()
{
    QMetaObject::activate(this, &staticMetaObject, 51, nullptr);
}

// SIGNAL 52
void bridgepp::GRPCClient::updateForce(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 52, nullptr, _t1);
}

// SIGNAL 53
void bridgepp::GRPCClient::updateSilentRestartNeeded()
{
    QMetaObject::activate(this, &staticMetaObject, 53, nullptr);
}

// SIGNAL 54
void bridgepp::GRPCClient::updateIsLatestVersion()
{
    QMetaObject::activate(this, &staticMetaObject, 54, nullptr);
}

// SIGNAL 55
void bridgepp::GRPCClient::checkUpdatesFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 55, nullptr);
}

// SIGNAL 56
void bridgepp::GRPCClient::updateVersionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 56, nullptr);
}

// SIGNAL 57
void bridgepp::GRPCClient::toggleSplitModeFinished(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 57, nullptr, _t1);
}

// SIGNAL 58
void bridgepp::GRPCClient::userDisconnected(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 58, nullptr, _t1);
}

// SIGNAL 59
void bridgepp::GRPCClient::userChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 59, nullptr, _t1);
}

// SIGNAL 60
void bridgepp::GRPCClient::userBadEvent(QString const & _t1, QString const & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 60, nullptr, _t1, _t2);
}

// SIGNAL 61
void bridgepp::GRPCClient::usedBytesChanged(QString const & _t1, qint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 61, nullptr, _t1, _t2);
}

// SIGNAL 62
void bridgepp::GRPCClient::imapLoginFailed(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 62, nullptr, _t1);
}

// SIGNAL 63
void bridgepp::GRPCClient::syncStarted(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 63, nullptr, _t1);
}

// SIGNAL 64
void bridgepp::GRPCClient::syncFinished(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 64, nullptr, _t1);
}

// SIGNAL 65
void bridgepp::GRPCClient::syncProgress(QString const & _t1, double _t2, qint64 _t3, qint64 _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 65, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 66
void bridgepp::GRPCClient::changeKeychainFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 66, nullptr);
}

// SIGNAL 67
void bridgepp::GRPCClient::hasNoKeychain()
{
    QMetaObject::activate(this, &staticMetaObject, 67, nullptr);
}

// SIGNAL 68
void bridgepp::GRPCClient::rebuildKeychain()
{
    QMetaObject::activate(this, &staticMetaObject, 68, nullptr);
}

// SIGNAL 69
void bridgepp::GRPCClient::certIsReady()
{
    QMetaObject::activate(this, &staticMetaObject, 69, nullptr);
}

// SIGNAL 70
void bridgepp::GRPCClient::addressChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 70, nullptr, _t1);
}

// SIGNAL 71
void bridgepp::GRPCClient::addressChangedLogout(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 71, nullptr, _t1);
}

// SIGNAL 72
void bridgepp::GRPCClient::apiCertIssue()
{
    QMetaObject::activate(this, &staticMetaObject, 72, nullptr);
}

// SIGNAL 73
void bridgepp::GRPCClient::genericError(ErrorInfo _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 73, nullptr, _t1);
}
QT_WARNING_POP
