// Copyright (c) 2026 Proton AG
//
// This file is part of Proton Mail Bridge.
//
// Proton Mail Bridge is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Proton Mail Bridge is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Proton Mail Bridge.  If not, see <https://www.gnu.org/licenses/>.

// Code generated by ./credits.sh at Thu Jun 4 15:59:03 UTC 2026. DO NOT EDIT.

package bridge

const Credits = "cloud.google.com/go/auth;cloud.google.com/go/auth/oauth2adapt;cloud.google.com/go/compute/metadata;github.com/0xAX/notificator;github.com/Masterminds/semver/v3;github.com/ProtonMail/bcrypt;github.com/ProtonMail/gluon;github.com/ProtonMail/go-autostart;github.com/ProtonMail/go-crypto;github.com/ProtonMail/go-keychain;github.com/ProtonMail/go-libfido2;github.com/ProtonMail/go-message;github.com/ProtonMail/go-mime;github.com/ProtonMail/go-proton-api;github.com/ProtonMail/go-smtp;github.com/ProtonMail/go-srp;github.com/ProtonMail/gopenpgp/v2;github.com/ProtonMail/resty/v2;github.com/ProtonMail/winhello;github.com/PuerkitoBio/goquery;github.com/abiosoft/ishell;github.com/abiosoft/readline;github.com/allan-simon/go-singleinstance;github.com/andybalholm/cascadia;github.com/bradenaw/juniper;github.com/bytedance/sonic;github.com/cespare/xxhash/v2;github.com/chenzhuoyu/base64x;github.com/clipperhouse/displaywidth;github.com/clipperhouse/uax29/v2;github.com/cloudflare/circl;github.com/cpuguy83/go-md2man/v2;github.com/cronokirby/saferith;github.com/cucumber/gherkin-go/v19;github.com/cucumber/godog;github.com/cucumber/messages-go/v16;github.com/danieljoos/wincred;github.com/davecgh/go-spew;github.com/docker/docker-credential-helpers;github.com/elastic/go-sysinfo;github.com/elastic/go-windows;github.com/emersion/go-imap;github.com/emersion/go-imap-id;github.com/emersion/go-message;github.com/emersion/go-sasl;github.com/emersion/go-smtp;github.com/emersion/go-textwrapper;github.com/emersion/go-vcard;github.com/fatih/color;github.com/felixge/fgprof;github.com/felixge/httpsnoop;github.com/flynn-archive/go-shlex;github.com/fxamacker/cbor/v2;github.com/gabriel-vasile/mimetype;github.com/getsentry/sentry-go;github.com/gin-contrib/sse;github.com/gin-gonic/gin;github.com/go-ctap/ctaphid;github.com/go-ctap/winhello;github.com/go-logr/logr;github.com/go-logr/stdr;github.com/go-playground/locales;github.com/go-playground/universal-translator;github.com/go-playground/validator/v10;github.com/go-resty/resty/v2;github.com/goccy/go-json;github.com/godbus/dbus;github.com/gofrs/uuid;github.com/golang/mock;github.com/google/go-cmp;github.com/google/pprof;github.com/google/s2a-go;github.com/google/uuid;github.com/googleapis/enterprise-certificate-proxy;github.com/googleapis/gax-go/v2;github.com/hashicorp/errwrap;github.com/hashicorp/go-immutable-radix;github.com/hashicorp/go-memdb;github.com/hashicorp/go-multierror;github.com/hashicorp/golang-lru;github.com/jaytaylor/html2text;github.com/jeandeaual/go-locale;github.com/joeshaw/multierror;github.com/json-iterator/go;github.com/keybase/go-keychain;github.com/keys-pub/go-libfido2;github.com/klauspost/cpuid/v2;github.com/ldclabs/cose;github.com/leodido/go-urn;github.com/mattn/go-colorable;github.com/mattn/go-isatty;github.com/mattn/go-runewidth;github.com/mattn/go-sqlite3;github.com/miekg/dns;github.com/modern-go/concurrent;github.com/modern-go/reflect2;github.com/olekukonko/cat;github.com/olekukonko/errors;github.com/olekukonko/ll;github.com/olekukonko/tablewriter;github.com/pbnjay/memory;github.com/pelletier/go-toml/v2;github.com/pierrec/lz4/v4;github.com/pkg/errors;github.com/pkg/profile;github.com/pmezard/go-difflib;github.com/prometheus/procfs;github.com/russross/blackfriday/v2;github.com/sirupsen/logrus;github.com/spf13/pflag;github.com/ssor/bom;github.com/stretchr/objx;github.com/stretchr/testify;github.com/twitchyliquid64/golang-asm;github.com/ugorji/go/codec;github.com/urfave/cli/v2;github.com/vmihailenco/msgpack/v5;github.com/vmihailenco/tagparser/v2;github.com/x448/float16;github.com/xrash/smetrics;gitlab.com/c0b/go-ordered-json;go.opentelemetry.io/auto/sdk;go.opentelemetry.io/contrib/instrumentation/net/http/otelhttp;go.opentelemetry.io/otel;go.opentelemetry.io/otel/metric;go.opentelemetry.io/otel/trace;go.uber.org/goleak;golang.org/x/arch;golang.org/x/crypto;golang.org/x/exp;golang.org/x/mod;golang.org/x/net;golang.org/x/oauth2;golang.org/x/sync;golang.org/x/sys;golang.org/x/text;golang.org/x/tools;google.golang.org/api;google.golang.org/genproto/googleapis/rpc;google.golang.org/grpc;google.golang.org/protobuf;gopkg.in/yaml.v3;howett.net/plist;;Qt 6.8.2 by Qt group;"
