/****************************************************************************
** Meta object code from reading C++ file 'QMLBackend.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../internal/frontend/bridge-gui/bridge-gui/QMLBackend.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QMLBackend.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10QMLBackendE_t {};
} // unnamed namespace

template <> constexpr inline auto QMLBackend::qt_create_metaobjectdata<qt_meta_tag_ZN10QMLBackendE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QMLBackend",
        "showSplashScreenChanged",
        "",
        "value",
        "showOnStartupChanged",
        "goosChanged",
        "diskCachePathChanged",
        "QUrl",
        "url",
        "imapPortChanged",
        "port",
        "smtpPortChanged",
        "useSSLForSMTPChanged",
        "useSSLForIMAPChanged",
        "isAutomaticUpdateOnChanged",
        "isBetaEnabledChanged",
        "isAllMailVisibleChanged",
        "isTelemetryDisabledChanged",
        "isDisabled",
        "colorSchemeNameChanged",
        "scheme",
        "isDoHEnabledChanged",
        "logsPathChanged",
        "path",
        "licensePathChanged",
        "releaseNotesLinkChanged",
        "link",
        "dependencyLicensesLinkChanged",
        "landingPageLinkChanged",
        "appnameChanged",
        "appname",
        "vendorChanged",
        "vendor",
        "versionChanged",
        "version",
        "tagChanged",
        "tag",
        "currentEmailClientChanged",
        "email",
        "currentKeychainChanged",
        "keychain",
        "bugCategoriesChanged",
        "QVariantList",
        "bugCategories",
        "bugQuestionsChanged",
        "bugQuestions",
        "availableKeychainChanged",
        "keychains",
        "hostnameChanged",
        "hostname",
        "isAutostartOnChanged",
        "usersChanged",
        "UserList*",
        "users",
        "dockIconVisibleChanged",
        "trayIconVisibleChanged",
        "receivedUserNotification",
        "bridgepp::UserNotification",
        "notification",
        "loginFlowStarted",
        "toggleAutostartFinished",
        "cantMoveDiskCache",
        "diskCachePathChangeFinished",
        "loginUsernamePasswordError",
        "errorMsg",
        "loginFreeUserError",
        "loginConnectionError",
        "login2FARequested",
        "username",
        "login2FAError",
        "login2FAErrorAbort",
        "loginFidoRequested",
        "login2FAOrFidoRequested",
        "login2PasswordRequested",
        "login2PasswordError",
        "login2PasswordErrorAbort",
        "loginFinished",
        "index",
        "wasSignedOut",
        "loginAlreadyLoggedIn",
        "loginFidoTouchRequested",
        "loginFidoTouchCompleted",
        "loginFidoPinRequired",
        "loginFidoPinInvalid",
        "loginFidoPinBlocked",
        "loginFidoError",
        "loginHvRequested",
        "hvUrl",
        "loginHvError",
        "updateManualReady",
        "updateManualRestartNeeded",
        "updateManualError",
        "updateForce",
        "updateForceError",
        "updateSilentRestartNeeded",
        "updateSilentError",
        "updateIsLatestVersion",
        "checkUpdatesFinished",
        "imapPortStartupError",
        "smtpPortStartupError",
        "imapPortChangeError",
        "smtpPortChangeError",
        "imapConnectionModeChangeError",
        "smtpConnectionModeChangeError",
        "changeMailServerSettingsFinished",
        "changeKeychainFinished",
        "notifyHasNoKeychain",
        "notifyRebuildKeychain",
        "addressChanged",
        "address",
        "addressChangedLogout",
        "apiCertIssue",
        "userDisconnected",
        "userBadEvent",
        "userID",
        "description",
        "internetOff",
        "internetOn",
        "resetFinished",
        "reportBugFinished",
        "bugReportSendSuccess",
        "bugReportSendFallback",
        "bugReportSendError",
        "certificateInstallSuccess",
        "certificateInstallCanceled",
        "certificateInstallFailed",
        "showMainWindow",
        "hideMainWindow",
        "showHelp",
        "showSettings",
        "selectUser",
        "forceShowWindow",
        "genericError",
        "title",
        "imapLoginWhileSignedOut",
        "receivedKnowledgeBaseSuggestions",
        "QList<bridgepp::KnowledgeBaseSuggestion>",
        "suggestions",
        "repairStarted",
        "allUsersLoaded",
        "fatalError",
        "bridgepp::Exception",
        "e",
        "toggleAutostart",
        "active",
        "toggleBeta",
        "changeIsAllMailVisible",
        "isVisible",
        "toggleIsTelemetryDisabled",
        "changeColorScheme",
        "setDiskCachePath",
        "login",
        "password",
        "loginHv",
        "login2FA",
        "code",
        "loginFido",
        "pin",
        "login2Password",
        "loginAbort",
        "abortFidoAssertion",
        "toggleDoH",
        "toggleAutomaticUpdate",
        "makeItActive",
        "updateCurrentMailClient",
        "changeKeychain",
        "guiReady",
        "quit",
        "restart",
        "forceLauncher",
        "launcher",
        "checkUpdates",
        "installUpdate",
        "triggerReset",
        "reportBug",
        "category",
        "emailClient",
        "includeLogs",
        "installTLSCertificate",
        "exportTLSCertificates",
        "onResetFinished",
        "onVersionChanged",
        "setMailServerSettings",
        "imapPort",
        "smtpPort",
        "useSSLForIMAP",
        "useSSLForSMTP",
        "sendBadEventUserFeedback",
        "doResync",
        "triggerRepair",
        "userNotificationDismissed",
        "setNormalTrayIcon",
        "setErrorTrayIcon",
        "stateString",
        "statusIcon",
        "setWarnTrayIcon",
        "setUpdateTrayIcon",
        "internetStatusChanged",
        "isOn",
        "onMailServerSettingsChanged",
        "onGenericError",
        "bridgepp::ErrorInfo",
        "info",
        "onLoginFinished",
        "onLoginAlreadyLoggedIn",
        "onUserBadEvent",
        "errorMessage",
        "onIMAPLoginFailed",
        "processUserNotification",
        "buildYear",
        "getCursorPos",
        "QPoint",
        "isPortFree",
        "nativePath",
        "areSameFileOrFolder",
        "lhs",
        "rhs",
        "getBugCategory",
        "categoryId",
        "getQuestionSet",
        "setQuestionAnswer",
        "questionId",
        "answer",
        "getQuestionAnswer",
        "collectAnswers",
        "clearAnswers",
        "isTLSCertificateInstalled",
        "openExternalLink",
        "requestKnowledgeBaseSuggestions",
        "categoryID",
        "showOnStartup",
        "showSplashScreen",
        "goos",
        "logsPath",
        "licensePath",
        "releaseNotesLink",
        "dependencyLicensesLink",
        "landingPageLink",
        "isAutostartOn",
        "isBetaEnabled",
        "isAllMailVisible",
        "isTelemetryDisabled",
        "colorSchemeName",
        "diskCachePath",
        "isDoHEnabled",
        "isAutomaticUpdateOn",
        "currentEmailClient",
        "availableKeychain",
        "currentKeychain",
        "dockIconVisible",
        "trayIconVisible"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'showSplashScreenChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showOnStartupChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'goosChanged'
        QtMocHelpers::SignalData<void(QString const &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'diskCachePathChanged'
        QtMocHelpers::SignalData<void(QUrl const &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'imapPortChanged'
        QtMocHelpers::SignalData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Signal 'smtpPortChanged'
        QtMocHelpers::SignalData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Signal 'useSSLForSMTPChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'useSSLForIMAPChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'isAutomaticUpdateOnChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'isBetaEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'isAllMailVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'isTelemetryDisabledChanged'
        QtMocHelpers::SignalData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Signal 'colorSchemeNameChanged'
        QtMocHelpers::SignalData<void(QString const &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Signal 'isDoHEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'logsPathChanged'
        QtMocHelpers::SignalData<void(QUrl const &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 23 },
        }}),
        // Signal 'licensePathChanged'
        QtMocHelpers::SignalData<void(QUrl const &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 23 },
        }}),
        // Signal 'releaseNotesLinkChanged'
        QtMocHelpers::SignalData<void(QUrl const &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 26 },
        }}),
        // Signal 'dependencyLicensesLinkChanged'
        QtMocHelpers::SignalData<void(QUrl const &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 26 },
        }}),
        // Signal 'landingPageLinkChanged'
        QtMocHelpers::SignalData<void(QUrl const &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 26 },
        }}),
        // Signal 'appnameChanged'
        QtMocHelpers::SignalData<void(QString const &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 },
        }}),
        // Signal 'vendorChanged'
        QtMocHelpers::SignalData<void(QString const &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 32 },
        }}),
        // Signal 'versionChanged'
        QtMocHelpers::SignalData<void(QString const &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 34 },
        }}),
        // Signal 'tagChanged'
        QtMocHelpers::SignalData<void(QString const &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 36 },
        }}),
        // Signal 'currentEmailClientChanged'
        QtMocHelpers::SignalData<void(QString const &)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 38 },
        }}),
        // Signal 'currentKeychainChanged'
        QtMocHelpers::SignalData<void(QString const &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 40 },
        }}),
        // Signal 'bugCategoriesChanged'
        QtMocHelpers::SignalData<void(QVariantList const &)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 42, 43 },
        }}),
        // Signal 'bugQuestionsChanged'
        QtMocHelpers::SignalData<void(QVariantList const &)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 42, 45 },
        }}),
        // Signal 'availableKeychainChanged'
        QtMocHelpers::SignalData<void(QStringList const &)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 47 },
        }}),
        // Signal 'hostnameChanged'
        QtMocHelpers::SignalData<void(QString const &)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 49 },
        }}),
        // Signal 'isAutostartOnChanged'
        QtMocHelpers::SignalData<void(bool)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'usersChanged'
        QtMocHelpers::SignalData<void(UserList *)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 52, 53 },
        }}),
        // Signal 'dockIconVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'trayIconVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'receivedUserNotification'
        QtMocHelpers::SignalData<void(bridgepp::UserNotification const &)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 57, 58 },
        }}),
        // Signal 'loginFlowStarted'
        QtMocHelpers::SignalData<void() const>(59, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'toggleAutostartFinished'
        QtMocHelpers::SignalData<void()>(60, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cantMoveDiskCache'
        QtMocHelpers::SignalData<void()>(61, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'diskCachePathChangeFinished'
        QtMocHelpers::SignalData<void()>(62, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loginUsernamePasswordError'
        QtMocHelpers::SignalData<void(QString const &)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 64 },
        }}),
        // Signal 'loginFreeUserError'
        QtMocHelpers::SignalData<void()>(65, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loginConnectionError'
        QtMocHelpers::SignalData<void(QString const &)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 64 },
        }}),
        // Signal 'login2FARequested'
        QtMocHelpers::SignalData<void(QString const &)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 },
        }}),
        // Signal 'login2FAError'
        QtMocHelpers::SignalData<void(QString const &)>(69, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 64 },
        }}),
        // Signal 'login2FAErrorAbort'
        QtMocHelpers::SignalData<void(QString const &)>(70, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 64 },
        }}),
        // Signal 'loginFidoRequested'
        QtMocHelpers::SignalData<void(QString const &)>(71, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 },
        }}),
        // Signal 'login2FAOrFidoRequested'
        QtMocHelpers::SignalData<void(QString const &)>(72, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 },
        }}),
        // Signal 'login2PasswordRequested'
        QtMocHelpers::SignalData<void(QString const &)>(73, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 },
        }}),
        // Signal 'login2PasswordError'
        QtMocHelpers::SignalData<void(QString const &)>(74, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 64 },
        }}),
        // Signal 'login2PasswordErrorAbort'
        QtMocHelpers::SignalData<void(QString const &)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 64 },
        }}),
        // Signal 'loginFinished'
        QtMocHelpers::SignalData<void(int, bool)>(76, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 77 }, { QMetaType::Bool, 78 },
        }}),
        // Signal 'loginAlreadyLoggedIn'
        QtMocHelpers::SignalData<void(int)>(79, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 77 },
        }}),
        // Signal 'loginFidoTouchRequested'
        QtMocHelpers::SignalData<void(QString const &)>(80, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 },
        }}),
        // Signal 'loginFidoTouchCompleted'
        QtMocHelpers::SignalData<void(QString const &)>(81, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 },
        }}),
        // Signal 'loginFidoPinRequired'
        QtMocHelpers::SignalData<void(QString const &)>(82, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 },
        }}),
        // Signal 'loginFidoPinInvalid'
        QtMocHelpers::SignalData<void(QString const &)>(83, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 64 },
        }}),
        // Signal 'loginFidoPinBlocked'
        QtMocHelpers::SignalData<void(QString const &)>(84, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 64 },
        }}),
        // Signal 'loginFidoError'
        QtMocHelpers::SignalData<void(QString const &)>(85, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 64 },
        }}),
        // Signal 'loginHvRequested'
        QtMocHelpers::SignalData<void(QString const &)>(86, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 87 },
        }}),
        // Signal 'loginHvError'
        QtMocHelpers::SignalData<void(QString const &)>(88, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 64 },
        }}),
        // Signal 'updateManualReady'
        QtMocHelpers::SignalData<void(QString const &)>(89, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 34 },
        }}),
        // Signal 'updateManualRestartNeeded'
        QtMocHelpers::SignalData<void()>(90, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateManualError'
        QtMocHelpers::SignalData<void()>(91, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateForce'
        QtMocHelpers::SignalData<void(QString const &)>(92, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 34 },
        }}),
        // Signal 'updateForceError'
        QtMocHelpers::SignalData<void()>(93, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateSilentRestartNeeded'
        QtMocHelpers::SignalData<void()>(94, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateSilentError'
        QtMocHelpers::SignalData<void()>(95, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateIsLatestVersion'
        QtMocHelpers::SignalData<void()>(96, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'checkUpdatesFinished'
        QtMocHelpers::SignalData<void()>(97, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'imapPortStartupError'
        QtMocHelpers::SignalData<void()>(98, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'smtpPortStartupError'
        QtMocHelpers::SignalData<void()>(99, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'imapPortChangeError'
        QtMocHelpers::SignalData<void()>(100, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'smtpPortChangeError'
        QtMocHelpers::SignalData<void()>(101, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'imapConnectionModeChangeError'
        QtMocHelpers::SignalData<void()>(102, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'smtpConnectionModeChangeError'
        QtMocHelpers::SignalData<void()>(103, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'changeMailServerSettingsFinished'
        QtMocHelpers::SignalData<void()>(104, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'changeKeychainFinished'
        QtMocHelpers::SignalData<void()>(105, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notifyHasNoKeychain'
        QtMocHelpers::SignalData<void()>(106, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notifyRebuildKeychain'
        QtMocHelpers::SignalData<void()>(107, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'addressChanged'
        QtMocHelpers::SignalData<void(QString const &)>(108, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 109 },
        }}),
        // Signal 'addressChangedLogout'
        QtMocHelpers::SignalData<void(QString const &)>(110, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 109 },
        }}),
        // Signal 'apiCertIssue'
        QtMocHelpers::SignalData<void()>(111, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'userDisconnected'
        QtMocHelpers::SignalData<void(QString const &)>(112, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 },
        }}),
        // Signal 'userBadEvent'
        QtMocHelpers::SignalData<void(QString const &, QString const &)>(113, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 114 }, { QMetaType::QString, 115 },
        }}),
        // Signal 'internetOff'
        QtMocHelpers::SignalData<void()>(116, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'internetOn'
        QtMocHelpers::SignalData<void()>(117, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resetFinished'
        QtMocHelpers::SignalData<void()>(118, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'reportBugFinished'
        QtMocHelpers::SignalData<void()>(119, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bugReportSendSuccess'
        QtMocHelpers::SignalData<void()>(120, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bugReportSendFallback'
        QtMocHelpers::SignalData<void()>(121, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bugReportSendError'
        QtMocHelpers::SignalData<void()>(122, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'certificateInstallSuccess'
        QtMocHelpers::SignalData<void()>(123, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'certificateInstallCanceled'
        QtMocHelpers::SignalData<void()>(124, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'certificateInstallFailed'
        QtMocHelpers::SignalData<void()>(125, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showMainWindow'
        QtMocHelpers::SignalData<void()>(126, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hideMainWindow'
        QtMocHelpers::SignalData<void()>(127, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showHelp'
        QtMocHelpers::SignalData<void()>(128, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showSettings'
        QtMocHelpers::SignalData<void()>(129, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectUser'
        QtMocHelpers::SignalData<void(QString const &, bool)>(130, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 114 }, { QMetaType::Bool, 131 },
        }}),
        // Signal 'genericError'
        QtMocHelpers::SignalData<void(QString const &, QString const &)>(132, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 133 }, { QMetaType::QString, 115 },
        }}),
        // Signal 'imapLoginWhileSignedOut'
        QtMocHelpers::SignalData<void(QString const &)>(134, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 },
        }}),
        // Signal 'receivedKnowledgeBaseSuggestions'
        QtMocHelpers::SignalData<void(QList<bridgepp::KnowledgeBaseSuggestion> const &)>(135, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 136, 137 },
        }}),
        // Signal 'repairStarted'
        QtMocHelpers::SignalData<void()>(138, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'allUsersLoaded'
        QtMocHelpers::SignalData<void()>(139, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fatalError'
        QtMocHelpers::SignalData<void(bridgepp::Exception const &) const>(140, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 141, 142 },
        }}),
        // Slot 'toggleAutostart'
        QtMocHelpers::SlotData<void(bool)>(143, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 144 },
        }}),
        // Slot 'toggleBeta'
        QtMocHelpers::SlotData<void(bool)>(145, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 144 },
        }}),
        // Slot 'changeIsAllMailVisible'
        QtMocHelpers::SlotData<void(bool)>(146, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 147 },
        }}),
        // Slot 'toggleIsTelemetryDisabled'
        QtMocHelpers::SlotData<void(bool)>(148, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Slot 'changeColorScheme'
        QtMocHelpers::SlotData<void(QString const &)>(149, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'setDiskCachePath'
        QtMocHelpers::SlotData<void(QUrl const &) const>(150, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 23 },
        }}),
        // Slot 'login'
        QtMocHelpers::SlotData<void(QString const &, QString const &) const>(151, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 }, { QMetaType::QString, 152 },
        }}),
        // Slot 'loginHv'
        QtMocHelpers::SlotData<void(QString const &, QString const &) const>(153, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 }, { QMetaType::QString, 152 },
        }}),
        // Slot 'login2FA'
        QtMocHelpers::SlotData<void(QString const &, QString const &) const>(154, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 }, { QMetaType::QString, 155 },
        }}),
        // Slot 'loginFido'
        QtMocHelpers::SlotData<void(QString const &, QString const &) const>(156, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 }, { QMetaType::QString, 157 },
        }}),
        // Slot 'login2Password'
        QtMocHelpers::SlotData<void(QString const &, QString const &) const>(158, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 }, { QMetaType::QString, 152 },
        }}),
        // Slot 'loginAbort'
        QtMocHelpers::SlotData<void(QString const &) const>(159, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 },
        }}),
        // Slot 'abortFidoAssertion'
        QtMocHelpers::SlotData<void(QString const &) const>(160, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 },
        }}),
        // Slot 'toggleDoH'
        QtMocHelpers::SlotData<void(bool)>(161, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 144 },
        }}),
        // Slot 'toggleAutomaticUpdate'
        QtMocHelpers::SlotData<void(bool)>(162, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 163 },
        }}),
        // Slot 'updateCurrentMailClient'
        QtMocHelpers::SlotData<void()>(164, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'changeKeychain'
        QtMocHelpers::SlotData<void(QString const &)>(165, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 40 },
        }}),
        // Slot 'guiReady'
        QtMocHelpers::SlotData<void()>(166, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'quit'
        QtMocHelpers::SlotData<void() const>(167, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'restart'
        QtMocHelpers::SlotData<void() const>(168, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'forceLauncher'
        QtMocHelpers::SlotData<void(QString) const>(169, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 170 },
        }}),
        // Slot 'checkUpdates'
        QtMocHelpers::SlotData<void() const>(171, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'installUpdate'
        QtMocHelpers::SlotData<void() const>(172, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'triggerReset'
        QtMocHelpers::SlotData<void() const>(173, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reportBug'
        QtMocHelpers::SlotData<void(QString const &, QString const &, QString const &, QString const &, bool) const>(174, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 175 }, { QMetaType::QString, 115 }, { QMetaType::QString, 109 }, { QMetaType::QString, 176 },
            { QMetaType::Bool, 177 },
        }}),
        // Slot 'installTLSCertificate'
        QtMocHelpers::SlotData<void()>(178, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'exportTLSCertificates'
        QtMocHelpers::SlotData<void() const>(179, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onResetFinished'
        QtMocHelpers::SlotData<void()>(180, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onVersionChanged'
        QtMocHelpers::SlotData<void()>(181, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setMailServerSettings'
        QtMocHelpers::SlotData<void(int, int, bool, bool) const>(182, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 183 }, { QMetaType::Int, 184 }, { QMetaType::Bool, 185 }, { QMetaType::Bool, 186 },
        }}),
        // Slot 'sendBadEventUserFeedback'
        QtMocHelpers::SlotData<void(QString const &, bool)>(187, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 114 }, { QMetaType::Bool, 188 },
        }}),
        // Slot 'triggerRepair'
        QtMocHelpers::SlotData<void() const>(189, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'userNotificationDismissed'
        QtMocHelpers::SlotData<void()>(190, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setNormalTrayIcon'
        QtMocHelpers::SlotData<void()>(191, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setErrorTrayIcon'
        QtMocHelpers::SlotData<void(QString const &, QString const &)>(192, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 193 }, { QMetaType::QString, 194 },
        }}),
        // Slot 'setWarnTrayIcon'
        QtMocHelpers::SlotData<void(QString const &, QString const &)>(195, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 193 }, { QMetaType::QString, 194 },
        }}),
        // Slot 'setUpdateTrayIcon'
        QtMocHelpers::SlotData<void(QString const &, QString const &)>(196, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 193 }, { QMetaType::QString, 194 },
        }}),
        // Slot 'internetStatusChanged'
        QtMocHelpers::SlotData<void(bool)>(197, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 198 },
        }}),
        // Slot 'onMailServerSettingsChanged'
        QtMocHelpers::SlotData<void(int, int, bool, bool)>(199, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 183 }, { QMetaType::Int, 184 }, { QMetaType::Bool, 185 }, { QMetaType::Bool, 186 },
        }}),
        // Slot 'onGenericError'
        QtMocHelpers::SlotData<void(bridgepp::ErrorInfo const &)>(200, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 201, 202 },
        }}),
        // Slot 'onLoginFinished'
        QtMocHelpers::SlotData<void(QString const &, bool)>(203, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 114 }, { QMetaType::Bool, 78 },
        }}),
        // Slot 'onLoginAlreadyLoggedIn'
        QtMocHelpers::SlotData<void(QString const &)>(204, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 114 },
        }}),
        // Slot 'onUserBadEvent'
        QtMocHelpers::SlotData<void(QString const &, QString const &)>(205, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 114 }, { QMetaType::QString, 206 },
        }}),
        // Slot 'onIMAPLoginFailed'
        QtMocHelpers::SlotData<void(QString const &)>(207, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 },
        }}),
        // Slot 'processUserNotification'
        QtMocHelpers::SlotData<void(bridgepp::UserNotification const &)>(208, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 57, 58 },
        }}),
        // Method 'buildYear'
        QtMocHelpers::MethodData<QString()>(209, 2, QMC::AccessPublic, QMetaType::QString),
        // Method 'getCursorPos'
        QtMocHelpers::MethodData<QPoint() const>(210, 2, QMC::AccessPublic, 0x80000000 | 211),
        // Method 'isPortFree'
        QtMocHelpers::MethodData<bool(int) const>(212, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 10 },
        }}),
        // Method 'nativePath'
        QtMocHelpers::MethodData<QString(QUrl const &) const>(213, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Method 'areSameFileOrFolder'
        QtMocHelpers::MethodData<bool(QUrl const &, QUrl const &) const>(214, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 7, 215 }, { 0x80000000 | 7, 216 },
        }}),
        // Method 'getBugCategory'
        QtMocHelpers::MethodData<QString(quint8) const>(217, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::UChar, 218 },
        }}),
        // Method 'getQuestionSet'
        QtMocHelpers::MethodData<QVariantList(quint8) const>(219, 2, QMC::AccessPublic, 0x80000000 | 42, {{
            { QMetaType::UChar, 218 },
        }}),
        // Method 'setQuestionAnswer'
        QtMocHelpers::MethodData<void(quint8, QString const &)>(220, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 221 }, { QMetaType::QString, 222 },
        }}),
        // Method 'getQuestionAnswer'
        QtMocHelpers::MethodData<QString(quint8) const>(223, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::UChar, 221 },
        }}),
        // Method 'collectAnswers'
        QtMocHelpers::MethodData<QString(quint8) const>(224, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::UChar, 218 },
        }}),
        // Method 'clearAnswers'
        QtMocHelpers::MethodData<void()>(225, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'isTLSCertificateInstalled'
        QtMocHelpers::MethodData<bool()>(226, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'openExternalLink'
        QtMocHelpers::MethodData<void(QString const &)>(227, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Method 'openExternalLink'
        QtMocHelpers::MethodData<void()>(227, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Method 'requestKnowledgeBaseSuggestions'
        QtMocHelpers::MethodData<void(qint8) const>(228, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::SChar, 229 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'showOnStartup'
        QtMocHelpers::PropertyData<bool>(230, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'showSplashScreen'
        QtMocHelpers::PropertyData<bool>(231, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'goos'
        QtMocHelpers::PropertyData<QString>(232, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'logsPath'
        QtMocHelpers::PropertyData<QUrl>(233, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 14),
        // property 'licensePath'
        QtMocHelpers::PropertyData<QUrl>(234, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 15),
        // property 'releaseNotesLink'
        QtMocHelpers::PropertyData<QUrl>(235, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 16),
        // property 'dependencyLicensesLink'
        QtMocHelpers::PropertyData<QUrl>(236, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 17),
        // property 'landingPageLink'
        QtMocHelpers::PropertyData<QUrl>(237, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 18),
        // property 'appname'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags, 19),
        // property 'vendor'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags, 20),
        // property 'version'
        QtMocHelpers::PropertyData<QString>(34, QMetaType::QString, QMC::DefaultPropertyFlags, 21),
        // property 'tag'
        QtMocHelpers::PropertyData<QString>(36, QMetaType::QString, QMC::DefaultPropertyFlags, 22),
        // property 'hostname'
        QtMocHelpers::PropertyData<QString>(49, QMetaType::QString, QMC::DefaultPropertyFlags, 28),
        // property 'isAutostartOn'
        QtMocHelpers::PropertyData<bool>(238, QMetaType::Bool, QMC::DefaultPropertyFlags, 29),
        // property 'isBetaEnabled'
        QtMocHelpers::PropertyData<bool>(239, QMetaType::Bool, QMC::DefaultPropertyFlags, 9),
        // property 'isAllMailVisible'
        QtMocHelpers::PropertyData<bool>(240, QMetaType::Bool, QMC::DefaultPropertyFlags, 10),
        // property 'isTelemetryDisabled'
        QtMocHelpers::PropertyData<bool>(241, QMetaType::Bool, QMC::DefaultPropertyFlags, 11),
        // property 'colorSchemeName'
        QtMocHelpers::PropertyData<QString>(242, QMetaType::QString, QMC::DefaultPropertyFlags, 12),
        // property 'diskCachePath'
        QtMocHelpers::PropertyData<QUrl>(243, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
        // property 'useSSLForIMAP'
        QtMocHelpers::PropertyData<bool>(185, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'useSSLForSMTP'
        QtMocHelpers::PropertyData<bool>(186, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'imapPort'
        QtMocHelpers::PropertyData<int>(183, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 4),
        // property 'smtpPort'
        QtMocHelpers::PropertyData<int>(184, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 5),
        // property 'isDoHEnabled'
        QtMocHelpers::PropertyData<bool>(244, QMetaType::Bool, QMC::DefaultPropertyFlags, 13),
        // property 'isAutomaticUpdateOn'
        QtMocHelpers::PropertyData<bool>(245, QMetaType::Bool, QMC::DefaultPropertyFlags, 8),
        // property 'currentEmailClient'
        QtMocHelpers::PropertyData<QString>(246, QMetaType::QString, QMC::DefaultPropertyFlags, 23),
        // property 'availableKeychain'
        QtMocHelpers::PropertyData<QStringList>(247, QMetaType::QStringList, QMC::DefaultPropertyFlags, 27),
        // property 'currentKeychain'
        QtMocHelpers::PropertyData<QString>(248, QMetaType::QString, QMC::DefaultPropertyFlags, 24),
        // property 'bugCategories'
        QtMocHelpers::PropertyData<QVariantList>(43, 0x80000000 | 42, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 25),
        // property 'bugQuestions'
        QtMocHelpers::PropertyData<QVariantList>(45, 0x80000000 | 42, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 26),
        // property 'users'
        QtMocHelpers::PropertyData<UserList*>(53, 0x80000000 | 52, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 30),
        // property 'dockIconVisible'
        QtMocHelpers::PropertyData<bool>(249, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 31),
        // property 'trayIconVisible'
        QtMocHelpers::PropertyData<bool>(250, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 32),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QMLBackend, qt_meta_tag_ZN10QMLBackendE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QMLBackend::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QMLBackendE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QMLBackendE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10QMLBackendE_t>.metaTypes,
    nullptr
} };

void QMLBackend::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QMLBackend *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->showSplashScreenChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->showOnStartupChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->goosChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->diskCachePathChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 4: _t->imapPortChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->smtpPortChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->useSSLForSMTPChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->useSSLForIMAPChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->isAutomaticUpdateOnChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->isBetaEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->isAllMailVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->isTelemetryDisabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->colorSchemeNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->isDoHEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->logsPathChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 15: _t->licensePathChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 16: _t->releaseNotesLinkChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 17: _t->dependencyLicensesLinkChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 18: _t->landingPageLinkChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 19: _t->appnameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->vendorChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->versionChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->tagChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->currentEmailClientChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->currentKeychainChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 25: _t->bugCategoriesChanged((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1]))); break;
        case 26: _t->bugQuestionsChanged((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1]))); break;
        case 27: _t->availableKeychainChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 28: _t->hostnameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 29: _t->isAutostartOnChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 30: _t->usersChanged((*reinterpret_cast<std::add_pointer_t<UserList*>>(_a[1]))); break;
        case 31: _t->dockIconVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: _t->trayIconVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->receivedUserNotification((*reinterpret_cast<std::add_pointer_t<bridgepp::UserNotification>>(_a[1]))); break;
        case 34: _t->loginFlowStarted(); break;
        case 35: _t->toggleAutostartFinished(); break;
        case 36: _t->cantMoveDiskCache(); break;
        case 37: _t->diskCachePathChangeFinished(); break;
        case 38: _t->loginUsernamePasswordError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 39: _t->loginFreeUserError(); break;
        case 40: _t->loginConnectionError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 41: _t->login2FARequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 42: _t->login2FAError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 43: _t->login2FAErrorAbort((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 44: _t->loginFidoRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 45: _t->login2FAOrFidoRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 46: _t->login2PasswordRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 47: _t->login2PasswordError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 48: _t->login2PasswordErrorAbort((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 49: _t->loginFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 50: _t->loginAlreadyLoggedIn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 51: _t->loginFidoTouchRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 52: _t->loginFidoTouchCompleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 53: _t->loginFidoPinRequired((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 54: _t->loginFidoPinInvalid((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 55: _t->loginFidoPinBlocked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 56: _t->loginFidoError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 57: _t->loginHvRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 58: _t->loginHvError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 59: _t->updateManualReady((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 60: _t->updateManualRestartNeeded(); break;
        case 61: _t->updateManualError(); break;
        case 62: _t->updateForce((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 63: _t->updateForceError(); break;
        case 64: _t->updateSilentRestartNeeded(); break;
        case 65: _t->updateSilentError(); break;
        case 66: _t->updateIsLatestVersion(); break;
        case 67: _t->checkUpdatesFinished(); break;
        case 68: _t->imapPortStartupError(); break;
        case 69: _t->smtpPortStartupError(); break;
        case 70: _t->imapPortChangeError(); break;
        case 71: _t->smtpPortChangeError(); break;
        case 72: _t->imapConnectionModeChangeError(); break;
        case 73: _t->smtpConnectionModeChangeError(); break;
        case 74: _t->changeMailServerSettingsFinished(); break;
        case 75: _t->changeKeychainFinished(); break;
        case 76: _t->notifyHasNoKeychain(); break;
        case 77: _t->notifyRebuildKeychain(); break;
        case 78: _t->addressChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 79: _t->addressChangedLogout((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 80: _t->apiCertIssue(); break;
        case 81: _t->userDisconnected((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 82: _t->userBadEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 83: _t->internetOff(); break;
        case 84: _t->internetOn(); break;
        case 85: _t->resetFinished(); break;
        case 86: _t->reportBugFinished(); break;
        case 87: _t->bugReportSendSuccess(); break;
        case 88: _t->bugReportSendFallback(); break;
        case 89: _t->bugReportSendError(); break;
        case 90: _t->certificateInstallSuccess(); break;
        case 91: _t->certificateInstallCanceled(); break;
        case 92: _t->certificateInstallFailed(); break;
        case 93: _t->showMainWindow(); break;
        case 94: _t->hideMainWindow(); break;
        case 95: _t->showHelp(); break;
        case 96: _t->showSettings(); break;
        case 97: _t->selectUser((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 98: _t->genericError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 99: _t->imapLoginWhileSignedOut((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 100: _t->receivedKnowledgeBaseSuggestions((*reinterpret_cast<std::add_pointer_t<QList<bridgepp::KnowledgeBaseSuggestion>>>(_a[1]))); break;
        case 101: _t->repairStarted(); break;
        case 102: _t->allUsersLoaded(); break;
        case 103: _t->fatalError((*reinterpret_cast<std::add_pointer_t<bridgepp::Exception>>(_a[1]))); break;
        case 104: _t->toggleAutostart((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 105: _t->toggleBeta((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 106: _t->changeIsAllMailVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 107: _t->toggleIsTelemetryDisabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 108: _t->changeColorScheme((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 109: _t->setDiskCachePath((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 110: _t->login((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 111: _t->loginHv((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 112: _t->login2FA((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 113: _t->loginFido((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 114: _t->login2Password((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 115: _t->loginAbort((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 116: _t->abortFidoAssertion((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 117: _t->toggleDoH((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 118: _t->toggleAutomaticUpdate((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 119: _t->updateCurrentMailClient(); break;
        case 120: _t->changeKeychain((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 121: _t->guiReady(); break;
        case 122: _t->quit(); break;
        case 123: _t->restart(); break;
        case 124: _t->forceLauncher((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 125: _t->checkUpdates(); break;
        case 126: _t->installUpdate(); break;
        case 127: _t->triggerReset(); break;
        case 128: _t->reportBug((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 129: _t->installTLSCertificate(); break;
        case 130: _t->exportTLSCertificates(); break;
        case 131: _t->onResetFinished(); break;
        case 132: _t->onVersionChanged(); break;
        case 133: _t->setMailServerSettings((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 134: _t->sendBadEventUserFeedback((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 135: _t->triggerRepair(); break;
        case 136: _t->userNotificationDismissed(); break;
        case 137: _t->setNormalTrayIcon(); break;
        case 138: _t->setErrorTrayIcon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 139: _t->setWarnTrayIcon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 140: _t->setUpdateTrayIcon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 141: _t->internetStatusChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 142: _t->onMailServerSettingsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 143: _t->onGenericError((*reinterpret_cast<std::add_pointer_t<bridgepp::ErrorInfo>>(_a[1]))); break;
        case 144: _t->onLoginFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 145: _t->onLoginAlreadyLoggedIn((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 146: _t->onUserBadEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 147: _t->onIMAPLoginFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 148: _t->processUserNotification((*reinterpret_cast<std::add_pointer_t<bridgepp::UserNotification>>(_a[1]))); break;
        case 149: { QString _r = _t->buildYear();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 150: { QPoint _r = _t->getCursorPos();
            if (_a[0]) *reinterpret_cast<QPoint*>(_a[0]) = std::move(_r); }  break;
        case 151: { bool _r = _t->isPortFree((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 152: { QString _r = _t->nativePath((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 153: { bool _r = _t->areSameFileOrFolder((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 154: { QString _r = _t->getBugCategory((*reinterpret_cast<std::add_pointer_t<quint8>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 155: { QVariantList _r = _t->getQuestionSet((*reinterpret_cast<std::add_pointer_t<quint8>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 156: _t->setQuestionAnswer((*reinterpret_cast<std::add_pointer_t<quint8>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 157: { QString _r = _t->getQuestionAnswer((*reinterpret_cast<std::add_pointer_t<quint8>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 158: { QString _r = _t->collectAnswers((*reinterpret_cast<std::add_pointer_t<quint8>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 159: _t->clearAnswers(); break;
        case 160: { bool _r = _t->isTLSCertificateInstalled();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 161: _t->openExternalLink((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 162: _t->openExternalLink(); break;
        case 163: _t->requestKnowledgeBaseSuggestions((*reinterpret_cast<std::add_pointer_t<qint8>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< UserList* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(bool )>(_a, &QMLBackend::showSplashScreenChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(bool )>(_a, &QMLBackend::showOnStartupChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::goosChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QUrl const & )>(_a, &QMLBackend::diskCachePathChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(int )>(_a, &QMLBackend::imapPortChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(int )>(_a, &QMLBackend::smtpPortChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(bool )>(_a, &QMLBackend::useSSLForSMTPChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(bool )>(_a, &QMLBackend::useSSLForIMAPChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(bool )>(_a, &QMLBackend::isAutomaticUpdateOnChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(bool )>(_a, &QMLBackend::isBetaEnabledChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(bool )>(_a, &QMLBackend::isAllMailVisibleChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(bool )>(_a, &QMLBackend::isTelemetryDisabledChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::colorSchemeNameChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(bool )>(_a, &QMLBackend::isDoHEnabledChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QUrl const & )>(_a, &QMLBackend::logsPathChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QUrl const & )>(_a, &QMLBackend::licensePathChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QUrl const & )>(_a, &QMLBackend::releaseNotesLinkChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QUrl const & )>(_a, &QMLBackend::dependencyLicensesLinkChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QUrl const & )>(_a, &QMLBackend::landingPageLinkChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::appnameChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::vendorChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::versionChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::tagChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::currentEmailClientChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::currentKeychainChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QVariantList const & )>(_a, &QMLBackend::bugCategoriesChanged, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QVariantList const & )>(_a, &QMLBackend::bugQuestionsChanged, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QStringList const & )>(_a, &QMLBackend::availableKeychainChanged, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::hostnameChanged, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(bool )>(_a, &QMLBackend::isAutostartOnChanged, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(UserList * )>(_a, &QMLBackend::usersChanged, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(bool )>(_a, &QMLBackend::dockIconVisibleChanged, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(bool )>(_a, &QMLBackend::trayIconVisibleChanged, 32))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(bridgepp::UserNotification const & )>(_a, &QMLBackend::receivedUserNotification, 33))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)() const>(_a, &QMLBackend::loginFlowStarted, 34))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::toggleAutostartFinished, 35))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::cantMoveDiskCache, 36))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::diskCachePathChangeFinished, 37))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::loginUsernamePasswordError, 38))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::loginFreeUserError, 39))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::loginConnectionError, 40))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::login2FARequested, 41))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::login2FAError, 42))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::login2FAErrorAbort, 43))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::loginFidoRequested, 44))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::login2FAOrFidoRequested, 45))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::login2PasswordRequested, 46))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::login2PasswordError, 47))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::login2PasswordErrorAbort, 48))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(int , bool )>(_a, &QMLBackend::loginFinished, 49))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(int )>(_a, &QMLBackend::loginAlreadyLoggedIn, 50))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::loginFidoTouchRequested, 51))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::loginFidoTouchCompleted, 52))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::loginFidoPinRequired, 53))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::loginFidoPinInvalid, 54))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::loginFidoPinBlocked, 55))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::loginFidoError, 56))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::loginHvRequested, 57))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::loginHvError, 58))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::updateManualReady, 59))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::updateManualRestartNeeded, 60))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::updateManualError, 61))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::updateForce, 62))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::updateForceError, 63))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::updateSilentRestartNeeded, 64))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::updateSilentError, 65))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::updateIsLatestVersion, 66))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::checkUpdatesFinished, 67))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::imapPortStartupError, 68))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::smtpPortStartupError, 69))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::imapPortChangeError, 70))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::smtpPortChangeError, 71))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::imapConnectionModeChangeError, 72))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::smtpConnectionModeChangeError, 73))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::changeMailServerSettingsFinished, 74))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::changeKeychainFinished, 75))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::notifyHasNoKeychain, 76))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::notifyRebuildKeychain, 77))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::addressChanged, 78))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::addressChangedLogout, 79))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::apiCertIssue, 80))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::userDisconnected, 81))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & , QString const & )>(_a, &QMLBackend::userBadEvent, 82))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::internetOff, 83))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::internetOn, 84))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::resetFinished, 85))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::reportBugFinished, 86))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::bugReportSendSuccess, 87))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::bugReportSendFallback, 88))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::bugReportSendError, 89))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::certificateInstallSuccess, 90))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::certificateInstallCanceled, 91))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::certificateInstallFailed, 92))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::showMainWindow, 93))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::hideMainWindow, 94))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::showHelp, 95))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::showSettings, 96))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & , bool )>(_a, &QMLBackend::selectUser, 97))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & , QString const & )>(_a, &QMLBackend::genericError, 98))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QString const & )>(_a, &QMLBackend::imapLoginWhileSignedOut, 99))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(QList<bridgepp::KnowledgeBaseSuggestion> const & )>(_a, &QMLBackend::receivedKnowledgeBaseSuggestions, 100))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::repairStarted, 101))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)()>(_a, &QMLBackend::allUsersLoaded, 102))
            return;
        if (QtMocHelpers::indexOfMethod<void (QMLBackend::*)(bridgepp::Exception const & ) const>(_a, &QMLBackend::fatalError, 103))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 30:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< UserList* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->showOnStartup(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->showSplashScreen(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->goos(); break;
        case 3: *reinterpret_cast<QUrl*>(_v) = _t->logsPath(); break;
        case 4: *reinterpret_cast<QUrl*>(_v) = _t->licensePath(); break;
        case 5: *reinterpret_cast<QUrl*>(_v) = _t->releaseNotesLink(); break;
        case 6: *reinterpret_cast<QUrl*>(_v) = _t->dependencyLicensesLink(); break;
        case 7: *reinterpret_cast<QUrl*>(_v) = _t->landingPageLink(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->appname(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->vendor(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->version(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->tag(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->hostname(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->isAutostartOn(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->isBetaEnabled(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->isAllMailVisible(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->isTelemetryDisabled(); break;
        case 17: *reinterpret_cast<QString*>(_v) = _t->colorSchemeName(); break;
        case 18: *reinterpret_cast<QUrl*>(_v) = _t->diskCachePath(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->useSSLForIMAP(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->useSSLForSMTP(); break;
        case 21: *reinterpret_cast<int*>(_v) = _t->imapPort(); break;
        case 22: *reinterpret_cast<int*>(_v) = _t->smtpPort(); break;
        case 23: *reinterpret_cast<bool*>(_v) = _t->isDoHEnabled(); break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->isAutomaticUpdateOn(); break;
        case 25: *reinterpret_cast<QString*>(_v) = _t->currentEmailClient(); break;
        case 26: *reinterpret_cast<QStringList*>(_v) = _t->availableKeychain(); break;
        case 27: *reinterpret_cast<QString*>(_v) = _t->currentKeychain(); break;
        case 28: *reinterpret_cast<QVariantList*>(_v) = _t->bugCategories(); break;
        case 29: *reinterpret_cast<QVariantList*>(_v) = _t->bugQuestions(); break;
        case 30: *reinterpret_cast<UserList**>(_v) = _t->users_; break;
        case 31: *reinterpret_cast<bool*>(_v) = _t->dockIconVisible(); break;
        case 32: *reinterpret_cast<bool*>(_v) = _t->trayIconVisible(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setShowSplashScreen(*reinterpret_cast<bool*>(_v)); break;
        case 19: _t->setUseSSLForIMAP(*reinterpret_cast<bool*>(_v)); break;
        case 20: _t->setUseSSLForSMTP(*reinterpret_cast<bool*>(_v)); break;
        case 21: _t->setIMAPPort(*reinterpret_cast<int*>(_v)); break;
        case 22: _t->setSMTPPort(*reinterpret_cast<int*>(_v)); break;
        case 30:
            if (QtMocHelpers::setProperty(_t->users_, *reinterpret_cast<UserList**>(_v)))
                Q_EMIT _t->usersChanged(_t->users_);
            break;
        case 31: _t->setDockIconVisible(*reinterpret_cast<bool*>(_v)); break;
        case 32: _t->setTrayIconVisible(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QMLBackend::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QMLBackend::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QMLBackendE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QMLBackend::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 164)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 164;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 164)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 164;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 33;
    }
    return _id;
}

// SIGNAL 0
void QMLBackend::showSplashScreenChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QMLBackend::showOnStartupChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QMLBackend::goosChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QMLBackend::diskCachePathChanged(QUrl const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QMLBackend::imapPortChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QMLBackend::smtpPortChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QMLBackend::useSSLForSMTPChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QMLBackend::useSSLForIMAPChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QMLBackend::isAutomaticUpdateOnChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QMLBackend::isBetaEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QMLBackend::isAllMailVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QMLBackend::isTelemetryDisabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QMLBackend::colorSchemeNameChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QMLBackend::isDoHEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QMLBackend::logsPathChanged(QUrl const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QMLBackend::licensePathChanged(QUrl const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void QMLBackend::releaseNotesLinkChanged(QUrl const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void QMLBackend::dependencyLicensesLinkChanged(QUrl const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void QMLBackend::landingPageLinkChanged(QUrl const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void QMLBackend::appnameChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void QMLBackend::vendorChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void QMLBackend::versionChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void QMLBackend::tagChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void QMLBackend::currentEmailClientChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void QMLBackend::currentKeychainChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1);
}

// SIGNAL 25
void QMLBackend::bugCategoriesChanged(QVariantList const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 25, nullptr, _t1);
}

// SIGNAL 26
void QMLBackend::bugQuestionsChanged(QVariantList const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 26, nullptr, _t1);
}

// SIGNAL 27
void QMLBackend::availableKeychainChanged(QStringList const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 27, nullptr, _t1);
}

// SIGNAL 28
void QMLBackend::hostnameChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 28, nullptr, _t1);
}

// SIGNAL 29
void QMLBackend::isAutostartOnChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 29, nullptr, _t1);
}

// SIGNAL 30
void QMLBackend::usersChanged(UserList * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 30, nullptr, _t1);
}

// SIGNAL 31
void QMLBackend::dockIconVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 31, nullptr, _t1);
}

// SIGNAL 32
void QMLBackend::trayIconVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 32, nullptr, _t1);
}

// SIGNAL 33
void QMLBackend::receivedUserNotification(bridgepp::UserNotification const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 33, nullptr, _t1);
}

// SIGNAL 34
void QMLBackend::loginFlowStarted()const
{
    QMetaObject::activate(const_cast< QMLBackend *>(this), &staticMetaObject, 34, nullptr);
}

// SIGNAL 35
void QMLBackend::toggleAutostartFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 35, nullptr);
}

// SIGNAL 36
void QMLBackend::cantMoveDiskCache()
{
    QMetaObject::activate(this, &staticMetaObject, 36, nullptr);
}

// SIGNAL 37
void QMLBackend::diskCachePathChangeFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 37, nullptr);
}

// SIGNAL 38
void QMLBackend::loginUsernamePasswordError(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 38, nullptr, _t1);
}

// SIGNAL 39
void QMLBackend::loginFreeUserError()
{
    QMetaObject::activate(this, &staticMetaObject, 39, nullptr);
}

// SIGNAL 40
void QMLBackend::loginConnectionError(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 40, nullptr, _t1);
}

// SIGNAL 41
void QMLBackend::login2FARequested(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 41, nullptr, _t1);
}

// SIGNAL 42
void QMLBackend::login2FAError(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 42, nullptr, _t1);
}

// SIGNAL 43
void QMLBackend::login2FAErrorAbort(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 43, nullptr, _t1);
}

// SIGNAL 44
void QMLBackend::loginFidoRequested(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 44, nullptr, _t1);
}

// SIGNAL 45
void QMLBackend::login2FAOrFidoRequested(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 45, nullptr, _t1);
}

// SIGNAL 46
void QMLBackend::login2PasswordRequested(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 46, nullptr, _t1);
}

// SIGNAL 47
void QMLBackend::login2PasswordError(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 47, nullptr, _t1);
}

// SIGNAL 48
void QMLBackend::login2PasswordErrorAbort(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 48, nullptr, _t1);
}

// SIGNAL 49
void QMLBackend::loginFinished(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 49, nullptr, _t1, _t2);
}

// SIGNAL 50
void QMLBackend::loginAlreadyLoggedIn(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 50, nullptr, _t1);
}

// SIGNAL 51
void QMLBackend::loginFidoTouchRequested(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 51, nullptr, _t1);
}

// SIGNAL 52
void QMLBackend::loginFidoTouchCompleted(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 52, nullptr, _t1);
}

// SIGNAL 53
void QMLBackend::loginFidoPinRequired(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 53, nullptr, _t1);
}

// SIGNAL 54
void QMLBackend::loginFidoPinInvalid(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 54, nullptr, _t1);
}

// SIGNAL 55
void QMLBackend::loginFidoPinBlocked(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 55, nullptr, _t1);
}

// SIGNAL 56
void QMLBackend::loginFidoError(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 56, nullptr, _t1);
}

// SIGNAL 57
void QMLBackend::loginHvRequested(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 57, nullptr, _t1);
}

// SIGNAL 58
void QMLBackend::loginHvError(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 58, nullptr, _t1);
}

// SIGNAL 59
void QMLBackend::updateManualReady(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 59, nullptr, _t1);
}

// SIGNAL 60
void QMLBackend::updateManualRestartNeeded()
{
    QMetaObject::activate(this, &staticMetaObject, 60, nullptr);
}

// SIGNAL 61
void QMLBackend::updateManualError()
{
    QMetaObject::activate(this, &staticMetaObject, 61, nullptr);
}

// SIGNAL 62
void QMLBackend::updateForce(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 62, nullptr, _t1);
}

// SIGNAL 63
void QMLBackend::updateForceError()
{
    QMetaObject::activate(this, &staticMetaObject, 63, nullptr);
}

// SIGNAL 64
void QMLBackend::updateSilentRestartNeeded()
{
    QMetaObject::activate(this, &staticMetaObject, 64, nullptr);
}

// SIGNAL 65
void QMLBackend::updateSilentError()
{
    QMetaObject::activate(this, &staticMetaObject, 65, nullptr);
}

// SIGNAL 66
void QMLBackend::updateIsLatestVersion()
{
    QMetaObject::activate(this, &staticMetaObject, 66, nullptr);
}

// SIGNAL 67
void QMLBackend::checkUpdatesFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 67, nullptr);
}

// SIGNAL 68
void QMLBackend::imapPortStartupError()
{
    QMetaObject::activate(this, &staticMetaObject, 68, nullptr);
}

// SIGNAL 69
void QMLBackend::smtpPortStartupError()
{
    QMetaObject::activate(this, &staticMetaObject, 69, nullptr);
}

// SIGNAL 70
void QMLBackend::imapPortChangeError()
{
    QMetaObject::activate(this, &staticMetaObject, 70, nullptr);
}

// SIGNAL 71
void QMLBackend::smtpPortChangeError()
{
    QMetaObject::activate(this, &staticMetaObject, 71, nullptr);
}

// SIGNAL 72
void QMLBackend::imapConnectionModeChangeError()
{
    QMetaObject::activate(this, &staticMetaObject, 72, nullptr);
}

// SIGNAL 73
void QMLBackend::smtpConnectionModeChangeError()
{
    QMetaObject::activate(this, &staticMetaObject, 73, nullptr);
}

// SIGNAL 74
void QMLBackend::changeMailServerSettingsFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 74, nullptr);
}

// SIGNAL 75
void QMLBackend::changeKeychainFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 75, nullptr);
}

// SIGNAL 76
void QMLBackend::notifyHasNoKeychain()
{
    QMetaObject::activate(this, &staticMetaObject, 76, nullptr);
}

// SIGNAL 77
void QMLBackend::notifyRebuildKeychain()
{
    QMetaObject::activate(this, &staticMetaObject, 77, nullptr);
}

// SIGNAL 78
void QMLBackend::addressChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 78, nullptr, _t1);
}

// SIGNAL 79
void QMLBackend::addressChangedLogout(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 79, nullptr, _t1);
}

// SIGNAL 80
void QMLBackend::apiCertIssue()
{
    QMetaObject::activate(this, &staticMetaObject, 80, nullptr);
}

// SIGNAL 81
void QMLBackend::userDisconnected(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 81, nullptr, _t1);
}

// SIGNAL 82
void QMLBackend::userBadEvent(QString const & _t1, QString const & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 82, nullptr, _t1, _t2);
}

// SIGNAL 83
void QMLBackend::internetOff()
{
    QMetaObject::activate(this, &staticMetaObject, 83, nullptr);
}

// SIGNAL 84
void QMLBackend::internetOn()
{
    QMetaObject::activate(this, &staticMetaObject, 84, nullptr);
}

// SIGNAL 85
void QMLBackend::resetFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 85, nullptr);
}

// SIGNAL 86
void QMLBackend::reportBugFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 86, nullptr);
}

// SIGNAL 87
void QMLBackend::bugReportSendSuccess()
{
    QMetaObject::activate(this, &staticMetaObject, 87, nullptr);
}

// SIGNAL 88
void QMLBackend::bugReportSendFallback()
{
    QMetaObject::activate(this, &staticMetaObject, 88, nullptr);
}

// SIGNAL 89
void QMLBackend::bugReportSendError()
{
    QMetaObject::activate(this, &staticMetaObject, 89, nullptr);
}

// SIGNAL 90
void QMLBackend::certificateInstallSuccess()
{
    QMetaObject::activate(this, &staticMetaObject, 90, nullptr);
}

// SIGNAL 91
void QMLBackend::certificateInstallCanceled()
{
    QMetaObject::activate(this, &staticMetaObject, 91, nullptr);
}

// SIGNAL 92
void QMLBackend::certificateInstallFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 92, nullptr);
}

// SIGNAL 93
void QMLBackend::showMainWindow()
{
    QMetaObject::activate(this, &staticMetaObject, 93, nullptr);
}

// SIGNAL 94
void QMLBackend::hideMainWindow()
{
    QMetaObject::activate(this, &staticMetaObject, 94, nullptr);
}

// SIGNAL 95
void QMLBackend::showHelp()
{
    QMetaObject::activate(this, &staticMetaObject, 95, nullptr);
}

// SIGNAL 96
void QMLBackend::showSettings()
{
    QMetaObject::activate(this, &staticMetaObject, 96, nullptr);
}

// SIGNAL 97
void QMLBackend::selectUser(QString const & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 97, nullptr, _t1, _t2);
}

// SIGNAL 98
void QMLBackend::genericError(QString const & _t1, QString const & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 98, nullptr, _t1, _t2);
}

// SIGNAL 99
void QMLBackend::imapLoginWhileSignedOut(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 99, nullptr, _t1);
}

// SIGNAL 100
void QMLBackend::receivedKnowledgeBaseSuggestions(QList<bridgepp::KnowledgeBaseSuggestion> const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 100, nullptr, _t1);
}

// SIGNAL 101
void QMLBackend::repairStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 101, nullptr);
}

// SIGNAL 102
void QMLBackend::allUsersLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 102, nullptr);
}

// SIGNAL 103
void QMLBackend::fatalError(bridgepp::Exception const & _t1)const
{
    QMetaObject::activate<void>(const_cast< QMLBackend *>(this), &staticMetaObject, 103, nullptr, _t1);
}
QT_WARNING_POP
