// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: bridge.proto
// Protobuf C++ Version: 7.35.0

#include "bridge.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::grpc::AddLogEntryRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::GuiReadyResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::ReportBugRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::LoginRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::LoginAbortRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::ImapSmtpSettings
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::AvailableKeychainsResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::User
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UserSplitModeRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UserBadEventFeedbackRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UserListResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::ConfigureAppleMailRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::EventStreamRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::StreamEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::AppEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::InternetStatusEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::ToggleAutostartFinishedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::ResetFinishedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::ReportBugFinishedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::ReportBugSuccessEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::ReportBugErrorEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::ShowMainWindowEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::ReportBugFallbackEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::CertificateInstallSuccessEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::CertificateInstallCanceledEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::CertificateInstallFailedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::RepairStartedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::AllUsersLoadedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::KnowledgeBaseSuggestion
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::KnowledgeBaseSuggestionsEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::LoginEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::LoginErrorEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::LoginTfaRequestedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::LoginFidoRequestedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::LoginTfaOrFidoRequestedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::LoginFidoTouchEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::LoginFidoPinRequired
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::LoginTwoPasswordsRequestedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::LoginFinishedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::LoginHvRequestedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UpdateEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UpdateErrorEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UpdateManualReadyEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UpdateManualRestartNeededEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UpdateForceEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UpdateSilentRestartNeeded
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UpdateIsLatestVersion
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UpdateCheckFinished
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UpdateVersionChanged
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::DiskCacheEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::DiskCacheErrorEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::DiskCachePathChangedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::DiskCachePathChangeFinishedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::MailServerSettingsEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::MailServerSettingsErrorEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::MailServerSettingsChangedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::ChangeMailServerSettingsFinishedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::KeychainEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::ChangeKeychainFinishedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::HasNoKeychainEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::RebuildKeychainEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::MailEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::AddressChangedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::AddressChangedLogoutEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::ApiCertIssueEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UserEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::ToggleSplitModeFinishedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UserDisconnectedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UserChangedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UserBadEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UsedBytesChangedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::ImapLoginFailedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::SyncStartedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::SyncFinishedEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::SyncProgressEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::UserNotificationEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
        // ::grpc::GenericErrorEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_bridge_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace grpc {
class UserSplitModeRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<UserSplitModeRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(UserSplitModeRequest, _impl_._has_bits_);
};

constexpr UserSplitModeRequest::ParseTableT_ UserSplitModeRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UserSplitModeRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UserSplitModeRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // bool active = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(UserSplitModeRequest, _impl_.active_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(UserSplitModeRequest, _impl_.active_)}},
      // string userID = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(UserSplitModeRequest, _impl_.userid_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string userID = 1;
      {PROTOBUF_FIELD_OFFSET(UserSplitModeRequest, _impl_.userid_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // bool active = 2;
      {PROTOBUF_FIELD_OFFSET(UserSplitModeRequest, _impl_.active_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    // no aux_entries
    {{
      "\31\6\0\0\0\0\0\0"
      "grpc.UserSplitModeRequest"
      "userID"
    }},
  };
}


inline constexpr UserSplitModeRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        userid_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        active_{false} {}

template <typename>
constexpr UserSplitModeRequest::UserSplitModeRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UserSplitModeRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UserSplitModeRequest(arena);
}
constexpr auto UserSplitModeRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(UserSplitModeRequest), alignof(UserSplitModeRequest));
}
constexpr auto UserSplitModeRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UserSplitModeRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UserSplitModeRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UserSplitModeRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UserSplitModeRequest>(), &UserSplitModeRequest::ByteSizeLong,
              &UserSplitModeRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UserSplitModeRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[8],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UserSplitModeRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UserSplitModeRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UserSplitModeRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UserSplitModeRequest::InternalGenerateClassData_(
            _default, &UserSplitModeRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UserSplitModeRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UserSplitModeRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UserSplitModeRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UserSplitModeRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UserSplitModeRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UserSplitModeRequestGlobalsTypeInternal UserSplitModeRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UserSplitModeRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UserSplitModeRequest_globals_.GetClassData();
#else
  return UserSplitModeRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UserNotificationEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<UserNotificationEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(UserNotificationEvent, _impl_._has_bits_);
};

constexpr UserNotificationEvent::ParseTableT_ UserNotificationEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UserNotificationEvent, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UserNotificationEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string userID = 4;
      {::_pbi::TcParser::FastUS1,
       {34, 3, 0,
        PROTOBUF_FIELD_OFFSET(UserNotificationEvent, _impl_.userid_)}},
      // string title = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(UserNotificationEvent, _impl_.title_)}},
      // string subtitle = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(UserNotificationEvent, _impl_.subtitle_)}},
      // string body = 3;
      {::_pbi::TcParser::FastUS1,
       {26, 2, 0,
        PROTOBUF_FIELD_OFFSET(UserNotificationEvent, _impl_.body_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string title = 1;
      {PROTOBUF_FIELD_OFFSET(UserNotificationEvent, _impl_.title_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string subtitle = 2;
      {PROTOBUF_FIELD_OFFSET(UserNotificationEvent, _impl_.subtitle_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string body = 3;
      {PROTOBUF_FIELD_OFFSET(UserNotificationEvent, _impl_.body_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string userID = 4;
      {PROTOBUF_FIELD_OFFSET(UserNotificationEvent, _impl_.userid_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\32\5\10\4\6\0\0\0"
      "grpc.UserNotificationEvent"
      "title"
      "subtitle"
      "body"
      "userID"
    }},
  };
}


inline constexpr UserNotificationEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        title_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        subtitle_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        body_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        userid_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr UserNotificationEvent::UserNotificationEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UserNotificationEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UserNotificationEvent(arena);
}
constexpr auto UserNotificationEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(UserNotificationEvent), alignof(UserNotificationEvent));
}
constexpr auto UserNotificationEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UserNotificationEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UserNotificationEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UserNotificationEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UserNotificationEvent>(), &UserNotificationEvent::ByteSizeLong,
              &UserNotificationEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UserNotificationEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[75],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UserNotificationEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UserNotificationEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UserNotificationEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UserNotificationEvent::InternalGenerateClassData_(
            _default, &UserNotificationEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UserNotificationEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UserNotificationEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UserNotificationEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UserNotificationEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UserNotificationEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UserNotificationEventGlobalsTypeInternal UserNotificationEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UserNotificationEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UserNotificationEvent_globals_.GetClassData();
#else
  return UserNotificationEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UserDisconnectedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<UserDisconnectedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(UserDisconnectedEvent, _impl_._has_bits_);
};

constexpr UserDisconnectedEvent::ParseTableT_ UserDisconnectedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UserDisconnectedEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UserDisconnectedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string username = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(UserDisconnectedEvent, _impl_.username_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string username = 1;
      {PROTOBUF_FIELD_OFFSET(UserDisconnectedEvent, _impl_.username_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\32\10\0\0\0\0\0\0"
      "grpc.UserDisconnectedEvent"
      "username"
    }},
  };
}


inline constexpr UserDisconnectedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        username_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr UserDisconnectedEvent::UserDisconnectedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UserDisconnectedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UserDisconnectedEvent(arena);
}
constexpr auto UserDisconnectedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(UserDisconnectedEvent), alignof(UserDisconnectedEvent));
}
constexpr auto UserDisconnectedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UserDisconnectedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UserDisconnectedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UserDisconnectedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UserDisconnectedEvent>(), &UserDisconnectedEvent::ByteSizeLong,
              &UserDisconnectedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UserDisconnectedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[67],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UserDisconnectedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UserDisconnectedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UserDisconnectedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UserDisconnectedEvent::InternalGenerateClassData_(
            _default, &UserDisconnectedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UserDisconnectedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UserDisconnectedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UserDisconnectedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UserDisconnectedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UserDisconnectedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UserDisconnectedEventGlobalsTypeInternal UserDisconnectedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UserDisconnectedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UserDisconnectedEvent_globals_.GetClassData();
#else
  return UserDisconnectedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UserChangedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<UserChangedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(UserChangedEvent, _impl_._has_bits_);
};

constexpr UserChangedEvent::ParseTableT_ UserChangedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UserChangedEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UserChangedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string userID = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(UserChangedEvent, _impl_.userid_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string userID = 1;
      {PROTOBUF_FIELD_OFFSET(UserChangedEvent, _impl_.userid_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\25\6\0\0\0\0\0\0"
      "grpc.UserChangedEvent"
      "userID"
    }},
  };
}


inline constexpr UserChangedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        userid_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr UserChangedEvent::UserChangedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UserChangedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UserChangedEvent(arena);
}
constexpr auto UserChangedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(UserChangedEvent), alignof(UserChangedEvent));
}
constexpr auto UserChangedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UserChangedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UserChangedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UserChangedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UserChangedEvent>(), &UserChangedEvent::ByteSizeLong,
              &UserChangedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UserChangedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[68],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UserChangedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UserChangedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UserChangedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UserChangedEvent::InternalGenerateClassData_(
            _default, &UserChangedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UserChangedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UserChangedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UserChangedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UserChangedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UserChangedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UserChangedEventGlobalsTypeInternal UserChangedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UserChangedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UserChangedEvent_globals_.GetClassData();
#else
  return UserChangedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UserBadEventFeedbackRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<UserBadEventFeedbackRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(UserBadEventFeedbackRequest, _impl_._has_bits_);
};

constexpr UserBadEventFeedbackRequest::ParseTableT_ UserBadEventFeedbackRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UserBadEventFeedbackRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UserBadEventFeedbackRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // bool doResync = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(UserBadEventFeedbackRequest, _impl_.doresync_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(UserBadEventFeedbackRequest, _impl_.doresync_)}},
      // string userID = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(UserBadEventFeedbackRequest, _impl_.userid_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string userID = 1;
      {PROTOBUF_FIELD_OFFSET(UserBadEventFeedbackRequest, _impl_.userid_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // bool doResync = 2;
      {PROTOBUF_FIELD_OFFSET(UserBadEventFeedbackRequest, _impl_.doresync_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    // no aux_entries
    {{
      "\40\6\0\0\0\0\0\0"
      "grpc.UserBadEventFeedbackRequest"
      "userID"
    }},
  };
}


inline constexpr UserBadEventFeedbackRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        userid_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        doresync_{false} {}

template <typename>
constexpr UserBadEventFeedbackRequest::UserBadEventFeedbackRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UserBadEventFeedbackRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UserBadEventFeedbackRequest(arena);
}
constexpr auto UserBadEventFeedbackRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(UserBadEventFeedbackRequest), alignof(UserBadEventFeedbackRequest));
}
constexpr auto UserBadEventFeedbackRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UserBadEventFeedbackRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UserBadEventFeedbackRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UserBadEventFeedbackRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UserBadEventFeedbackRequest>(), &UserBadEventFeedbackRequest::ByteSizeLong,
              &UserBadEventFeedbackRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UserBadEventFeedbackRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[9],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UserBadEventFeedbackRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UserBadEventFeedbackRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UserBadEventFeedbackRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UserBadEventFeedbackRequest::InternalGenerateClassData_(
            _default, &UserBadEventFeedbackRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UserBadEventFeedbackRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UserBadEventFeedbackRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UserBadEventFeedbackRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UserBadEventFeedbackRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UserBadEventFeedbackRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UserBadEventFeedbackRequestGlobalsTypeInternal UserBadEventFeedbackRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UserBadEventFeedbackRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UserBadEventFeedbackRequest_globals_.GetClassData();
#else
  return UserBadEventFeedbackRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UserBadEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<UserBadEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(UserBadEvent, _impl_._has_bits_);
};

constexpr UserBadEvent::ParseTableT_ UserBadEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UserBadEvent, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UserBadEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string errorMessage = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(UserBadEvent, _impl_.errormessage_)}},
      // string userID = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(UserBadEvent, _impl_.userid_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string userID = 1;
      {PROTOBUF_FIELD_OFFSET(UserBadEvent, _impl_.userid_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string errorMessage = 2;
      {PROTOBUF_FIELD_OFFSET(UserBadEvent, _impl_.errormessage_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\21\6\14\0\0\0\0\0"
      "grpc.UserBadEvent"
      "userID"
      "errorMessage"
    }},
  };
}


inline constexpr UserBadEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        userid_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        errormessage_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr UserBadEvent::UserBadEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UserBadEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UserBadEvent(arena);
}
constexpr auto UserBadEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(UserBadEvent), alignof(UserBadEvent));
}
constexpr auto UserBadEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UserBadEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UserBadEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UserBadEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UserBadEvent>(), &UserBadEvent::ByteSizeLong,
              &UserBadEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UserBadEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[69],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UserBadEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UserBadEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UserBadEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UserBadEvent::InternalGenerateClassData_(
            _default, &UserBadEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UserBadEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UserBadEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UserBadEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UserBadEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UserBadEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UserBadEventGlobalsTypeInternal UserBadEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UserBadEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UserBadEvent_globals_.GetClassData();
#else
  return UserBadEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class User::_Internal {
 public:
  using HasBits = decltype(::std::declval<User>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(User, _impl_._has_bits_);
};

constexpr User::ParseTableT_ User::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(User, _impl_._has_bits_),
      0, // no _extensions_
      9, 120,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294966784,  // skipmap
      offsetof(ParseTableT_, field_entries),
      9,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::User>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string id = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(User, _impl_.id_)}},
      // string username = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 2, 0,
        PROTOBUF_FIELD_OFFSET(User, _impl_.username_)}},
      // string avatarText = 3;
      {::_pbi::TcParser::FastUS1,
       {26, 3, 0,
        PROTOBUF_FIELD_OFFSET(User, _impl_.avatartext_)}},
      // .grpc.UserState state = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(User, _impl_.state_), 5>(),
       {32, 5, 0,
        PROTOBUF_FIELD_OFFSET(User, _impl_.state_)}},
      // bool splitMode = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(User, _impl_.splitmode_), 6>(),
       {40, 6, 0,
        PROTOBUF_FIELD_OFFSET(User, _impl_.splitmode_)}},
      // int64 usedBytes = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(User, _impl_.usedbytes_), 7>(),
       {48, 7, 0,
        PROTOBUF_FIELD_OFFSET(User, _impl_.usedbytes_)}},
      // int64 totalBytes = 7;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(User, _impl_.totalbytes_), 8>(),
       {56, 8, 0,
        PROTOBUF_FIELD_OFFSET(User, _impl_.totalbytes_)}},
      // bytes password = 8;
      {::_pbi::TcParser::FastBS1,
       {66, 4, 0,
        PROTOBUF_FIELD_OFFSET(User, _impl_.password_)}},
      // repeated string addresses = 9;
      {::_pbi::TcParser::FastUR1,
       {74, 0, 0,
        PROTOBUF_FIELD_OFFSET(User, _impl_.addresses_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // string id = 1;
      {PROTOBUF_FIELD_OFFSET(User, _impl_.id_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string username = 2;
      {PROTOBUF_FIELD_OFFSET(User, _impl_.username_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string avatarText = 3;
      {PROTOBUF_FIELD_OFFSET(User, _impl_.avatartext_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .grpc.UserState state = 4;
      {PROTOBUF_FIELD_OFFSET(User, _impl_.state_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // bool splitMode = 5;
      {PROTOBUF_FIELD_OFFSET(User, _impl_.splitmode_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // int64 usedBytes = 6;
      {PROTOBUF_FIELD_OFFSET(User, _impl_.usedbytes_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // int64 totalBytes = 7;
      {PROTOBUF_FIELD_OFFSET(User, _impl_.totalbytes_), _Internal::kHasBitsOffset + 8, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // bytes password = 8;
      {PROTOBUF_FIELD_OFFSET(User, _impl_.password_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // repeated string addresses = 9;
      {PROTOBUF_FIELD_OFFSET(User, _impl_.addresses_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
    }},
    // no aux_entries
    {{
      "\11\2\10\12\0\0\0\0\0\11\0\0\0\0\0\0"
      "grpc.User"
      "id"
      "username"
      "avatarText"
      "addresses"
    }},
  };
}


inline constexpr User::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        addresses_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::grpc::User,
            PROTOBUF_FIELD_OFFSET(::grpc::User, _impl_.addresses_)>()
         }
        ,
        id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        username_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        avatartext_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        password_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        state_{static_cast< ::grpc::UserState >(0)},
        splitmode_{false},
        usedbytes_{::int64_t{0}},
        totalbytes_{::int64_t{0}} {}

template <typename>
constexpr User::User(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL User::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) User(arena);
}
constexpr auto User::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(User), alignof(User));
}
constexpr auto User::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &User::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<User>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &User::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<User>(), &User::ByteSizeLong,
              &User::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(User, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[7],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UserGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UserGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 User_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(User::InternalGenerateClassData_(
            _default, &User_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<User>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UserGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) User _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<User>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UserGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UserGlobalsTypeInternal User_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* User_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return User_globals_.GetClassData();
#else
  return User_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UsedBytesChangedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<UsedBytesChangedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(UsedBytesChangedEvent, _impl_._has_bits_);
};

constexpr UsedBytesChangedEvent::ParseTableT_ UsedBytesChangedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UsedBytesChangedEvent, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UsedBytesChangedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int64 usedBytes = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(UsedBytesChangedEvent, _impl_.usedbytes_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(UsedBytesChangedEvent, _impl_.usedbytes_)}},
      // string userID = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(UsedBytesChangedEvent, _impl_.userid_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string userID = 1;
      {PROTOBUF_FIELD_OFFSET(UsedBytesChangedEvent, _impl_.userid_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int64 usedBytes = 2;
      {PROTOBUF_FIELD_OFFSET(UsedBytesChangedEvent, _impl_.usedbytes_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
    }},
    // no aux_entries
    {{
      "\32\6\0\0\0\0\0\0"
      "grpc.UsedBytesChangedEvent"
      "userID"
    }},
  };
}


inline constexpr UsedBytesChangedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        userid_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        usedbytes_{::int64_t{0}} {}

template <typename>
constexpr UsedBytesChangedEvent::UsedBytesChangedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UsedBytesChangedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UsedBytesChangedEvent(arena);
}
constexpr auto UsedBytesChangedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(UsedBytesChangedEvent), alignof(UsedBytesChangedEvent));
}
constexpr auto UsedBytesChangedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UsedBytesChangedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UsedBytesChangedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UsedBytesChangedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UsedBytesChangedEvent>(), &UsedBytesChangedEvent::ByteSizeLong,
              &UsedBytesChangedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UsedBytesChangedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[70],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UsedBytesChangedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UsedBytesChangedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UsedBytesChangedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UsedBytesChangedEvent::InternalGenerateClassData_(
            _default, &UsedBytesChangedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UsedBytesChangedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UsedBytesChangedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UsedBytesChangedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UsedBytesChangedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UsedBytesChangedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UsedBytesChangedEventGlobalsTypeInternal UsedBytesChangedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UsedBytesChangedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UsedBytesChangedEvent_globals_.GetClassData();
#else
  return UsedBytesChangedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UpdateVersionChanged::_Internal {
 public:
};

constexpr UpdateVersionChanged::ParseTableT_ UpdateVersionChanged::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UpdateVersionChanged,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UpdateVersionChanged>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr UpdateVersionChanged::UpdateVersionChanged(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL UpdateVersionChanged::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UpdateVersionChanged(arena);
}
constexpr auto UpdateVersionChanged::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(UpdateVersionChanged), alignof(UpdateVersionChanged));
}
constexpr auto UpdateVersionChanged::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UpdateVersionChanged::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<UpdateVersionChanged>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UpdateVersionChanged::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<UpdateVersionChanged>(), &UpdateVersionChanged::ByteSizeLong,
              &UpdateVersionChanged::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UpdateVersionChanged, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[48],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UpdateVersionChangedGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UpdateVersionChangedGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UpdateVersionChanged_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UpdateVersionChanged::InternalGenerateClassData_(
            _default, &UpdateVersionChanged_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UpdateVersionChanged>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UpdateVersionChangedGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UpdateVersionChanged _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UpdateVersionChanged>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UpdateVersionChangedGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UpdateVersionChangedGlobalsTypeInternal UpdateVersionChanged_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UpdateVersionChanged_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UpdateVersionChanged_globals_.GetClassData();
#else
  return UpdateVersionChanged_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UpdateSilentRestartNeeded::_Internal {
 public:
};

constexpr UpdateSilentRestartNeeded::ParseTableT_ UpdateSilentRestartNeeded::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UpdateSilentRestartNeeded,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UpdateSilentRestartNeeded>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr UpdateSilentRestartNeeded::UpdateSilentRestartNeeded(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL UpdateSilentRestartNeeded::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UpdateSilentRestartNeeded(arena);
}
constexpr auto UpdateSilentRestartNeeded::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(UpdateSilentRestartNeeded), alignof(UpdateSilentRestartNeeded));
}
constexpr auto UpdateSilentRestartNeeded::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UpdateSilentRestartNeeded::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<UpdateSilentRestartNeeded>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UpdateSilentRestartNeeded::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<UpdateSilentRestartNeeded>(), &UpdateSilentRestartNeeded::ByteSizeLong,
              &UpdateSilentRestartNeeded::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UpdateSilentRestartNeeded, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[45],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UpdateSilentRestartNeededGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UpdateSilentRestartNeededGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UpdateSilentRestartNeeded_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UpdateSilentRestartNeeded::InternalGenerateClassData_(
            _default, &UpdateSilentRestartNeeded_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UpdateSilentRestartNeeded>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UpdateSilentRestartNeededGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UpdateSilentRestartNeeded _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UpdateSilentRestartNeeded>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UpdateSilentRestartNeededGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UpdateSilentRestartNeededGlobalsTypeInternal UpdateSilentRestartNeeded_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UpdateSilentRestartNeeded_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UpdateSilentRestartNeeded_globals_.GetClassData();
#else
  return UpdateSilentRestartNeeded_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UpdateManualRestartNeededEvent::_Internal {
 public:
};

constexpr UpdateManualRestartNeededEvent::ParseTableT_ UpdateManualRestartNeededEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UpdateManualRestartNeededEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UpdateManualRestartNeededEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr UpdateManualRestartNeededEvent::UpdateManualRestartNeededEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL UpdateManualRestartNeededEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UpdateManualRestartNeededEvent(arena);
}
constexpr auto UpdateManualRestartNeededEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(UpdateManualRestartNeededEvent), alignof(UpdateManualRestartNeededEvent));
}
constexpr auto UpdateManualRestartNeededEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UpdateManualRestartNeededEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<UpdateManualRestartNeededEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UpdateManualRestartNeededEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<UpdateManualRestartNeededEvent>(), &UpdateManualRestartNeededEvent::ByteSizeLong,
              &UpdateManualRestartNeededEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UpdateManualRestartNeededEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[43],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UpdateManualRestartNeededEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UpdateManualRestartNeededEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UpdateManualRestartNeededEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UpdateManualRestartNeededEvent::InternalGenerateClassData_(
            _default, &UpdateManualRestartNeededEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UpdateManualRestartNeededEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UpdateManualRestartNeededEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UpdateManualRestartNeededEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UpdateManualRestartNeededEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UpdateManualRestartNeededEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UpdateManualRestartNeededEventGlobalsTypeInternal UpdateManualRestartNeededEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UpdateManualRestartNeededEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UpdateManualRestartNeededEvent_globals_.GetClassData();
#else
  return UpdateManualRestartNeededEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UpdateManualReadyEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<UpdateManualReadyEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(UpdateManualReadyEvent, _impl_._has_bits_);
};

constexpr UpdateManualReadyEvent::ParseTableT_ UpdateManualReadyEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UpdateManualReadyEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UpdateManualReadyEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string version = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(UpdateManualReadyEvent, _impl_.version_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string version = 1;
      {PROTOBUF_FIELD_OFFSET(UpdateManualReadyEvent, _impl_.version_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\33\7\0\0\0\0\0\0"
      "grpc.UpdateManualReadyEvent"
      "version"
    }},
  };
}


inline constexpr UpdateManualReadyEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        version_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr UpdateManualReadyEvent::UpdateManualReadyEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UpdateManualReadyEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UpdateManualReadyEvent(arena);
}
constexpr auto UpdateManualReadyEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(UpdateManualReadyEvent), alignof(UpdateManualReadyEvent));
}
constexpr auto UpdateManualReadyEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UpdateManualReadyEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UpdateManualReadyEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UpdateManualReadyEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UpdateManualReadyEvent>(), &UpdateManualReadyEvent::ByteSizeLong,
              &UpdateManualReadyEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UpdateManualReadyEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[42],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UpdateManualReadyEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UpdateManualReadyEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UpdateManualReadyEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UpdateManualReadyEvent::InternalGenerateClassData_(
            _default, &UpdateManualReadyEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UpdateManualReadyEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UpdateManualReadyEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UpdateManualReadyEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UpdateManualReadyEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UpdateManualReadyEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UpdateManualReadyEventGlobalsTypeInternal UpdateManualReadyEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UpdateManualReadyEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UpdateManualReadyEvent_globals_.GetClassData();
#else
  return UpdateManualReadyEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UpdateIsLatestVersion::_Internal {
 public:
};

constexpr UpdateIsLatestVersion::ParseTableT_ UpdateIsLatestVersion::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UpdateIsLatestVersion,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UpdateIsLatestVersion>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr UpdateIsLatestVersion::UpdateIsLatestVersion(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL UpdateIsLatestVersion::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UpdateIsLatestVersion(arena);
}
constexpr auto UpdateIsLatestVersion::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(UpdateIsLatestVersion), alignof(UpdateIsLatestVersion));
}
constexpr auto UpdateIsLatestVersion::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UpdateIsLatestVersion::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<UpdateIsLatestVersion>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UpdateIsLatestVersion::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<UpdateIsLatestVersion>(), &UpdateIsLatestVersion::ByteSizeLong,
              &UpdateIsLatestVersion::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UpdateIsLatestVersion, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[46],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UpdateIsLatestVersionGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UpdateIsLatestVersionGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UpdateIsLatestVersion_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UpdateIsLatestVersion::InternalGenerateClassData_(
            _default, &UpdateIsLatestVersion_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UpdateIsLatestVersion>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UpdateIsLatestVersionGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UpdateIsLatestVersion _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UpdateIsLatestVersion>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UpdateIsLatestVersionGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UpdateIsLatestVersionGlobalsTypeInternal UpdateIsLatestVersion_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UpdateIsLatestVersion_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UpdateIsLatestVersion_globals_.GetClassData();
#else
  return UpdateIsLatestVersion_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UpdateForceEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<UpdateForceEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(UpdateForceEvent, _impl_._has_bits_);
};

constexpr UpdateForceEvent::ParseTableT_ UpdateForceEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UpdateForceEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UpdateForceEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string version = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(UpdateForceEvent, _impl_.version_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string version = 1;
      {PROTOBUF_FIELD_OFFSET(UpdateForceEvent, _impl_.version_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\25\7\0\0\0\0\0\0"
      "grpc.UpdateForceEvent"
      "version"
    }},
  };
}


inline constexpr UpdateForceEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        version_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr UpdateForceEvent::UpdateForceEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UpdateForceEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UpdateForceEvent(arena);
}
constexpr auto UpdateForceEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(UpdateForceEvent), alignof(UpdateForceEvent));
}
constexpr auto UpdateForceEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UpdateForceEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UpdateForceEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UpdateForceEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UpdateForceEvent>(), &UpdateForceEvent::ByteSizeLong,
              &UpdateForceEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UpdateForceEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[44],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UpdateForceEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UpdateForceEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UpdateForceEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UpdateForceEvent::InternalGenerateClassData_(
            _default, &UpdateForceEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UpdateForceEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UpdateForceEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UpdateForceEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UpdateForceEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UpdateForceEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UpdateForceEventGlobalsTypeInternal UpdateForceEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UpdateForceEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UpdateForceEvent_globals_.GetClassData();
#else
  return UpdateForceEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UpdateErrorEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<UpdateErrorEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(UpdateErrorEvent, _impl_._has_bits_);
};

constexpr UpdateErrorEvent::ParseTableT_ UpdateErrorEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UpdateErrorEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UpdateErrorEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .grpc.UpdateErrorType type = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(UpdateErrorEvent, _impl_.type_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(UpdateErrorEvent, _impl_.type_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.UpdateErrorType type = 1;
      {PROTOBUF_FIELD_OFFSET(UpdateErrorEvent, _impl_.type_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr UpdateErrorEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        type_{static_cast< ::grpc::UpdateErrorType >(0)} {}

template <typename>
constexpr UpdateErrorEvent::UpdateErrorEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UpdateErrorEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UpdateErrorEvent(arena);
}
constexpr auto UpdateErrorEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(UpdateErrorEvent), alignof(UpdateErrorEvent));
}
constexpr auto UpdateErrorEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UpdateErrorEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UpdateErrorEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UpdateErrorEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UpdateErrorEvent>(), &UpdateErrorEvent::ByteSizeLong,
              &UpdateErrorEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UpdateErrorEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[41],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UpdateErrorEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UpdateErrorEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UpdateErrorEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UpdateErrorEvent::InternalGenerateClassData_(
            _default, &UpdateErrorEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UpdateErrorEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UpdateErrorEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UpdateErrorEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UpdateErrorEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UpdateErrorEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UpdateErrorEventGlobalsTypeInternal UpdateErrorEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UpdateErrorEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UpdateErrorEvent_globals_.GetClassData();
#else
  return UpdateErrorEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UpdateCheckFinished::_Internal {
 public:
};

constexpr UpdateCheckFinished::ParseTableT_ UpdateCheckFinished::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UpdateCheckFinished,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UpdateCheckFinished>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr UpdateCheckFinished::UpdateCheckFinished(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL UpdateCheckFinished::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UpdateCheckFinished(arena);
}
constexpr auto UpdateCheckFinished::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(UpdateCheckFinished), alignof(UpdateCheckFinished));
}
constexpr auto UpdateCheckFinished::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UpdateCheckFinished::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<UpdateCheckFinished>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UpdateCheckFinished::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<UpdateCheckFinished>(), &UpdateCheckFinished::ByteSizeLong,
              &UpdateCheckFinished::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UpdateCheckFinished, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[47],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UpdateCheckFinishedGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UpdateCheckFinishedGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UpdateCheckFinished_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UpdateCheckFinished::InternalGenerateClassData_(
            _default, &UpdateCheckFinished_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UpdateCheckFinished>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UpdateCheckFinishedGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UpdateCheckFinished _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UpdateCheckFinished>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UpdateCheckFinishedGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UpdateCheckFinishedGlobalsTypeInternal UpdateCheckFinished_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UpdateCheckFinished_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UpdateCheckFinished_globals_.GetClassData();
#else
  return UpdateCheckFinished_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ToggleSplitModeFinishedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<ToggleSplitModeFinishedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ToggleSplitModeFinishedEvent, _impl_._has_bits_);
};

constexpr ToggleSplitModeFinishedEvent::ParseTableT_ ToggleSplitModeFinishedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ToggleSplitModeFinishedEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::ToggleSplitModeFinishedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string userID = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ToggleSplitModeFinishedEvent, _impl_.userid_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string userID = 1;
      {PROTOBUF_FIELD_OFFSET(ToggleSplitModeFinishedEvent, _impl_.userid_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\41\6\0\0\0\0\0\0"
      "grpc.ToggleSplitModeFinishedEvent"
      "userID"
    }},
  };
}


inline constexpr ToggleSplitModeFinishedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        userid_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr ToggleSplitModeFinishedEvent::ToggleSplitModeFinishedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ToggleSplitModeFinishedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ToggleSplitModeFinishedEvent(arena);
}
constexpr auto ToggleSplitModeFinishedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ToggleSplitModeFinishedEvent), alignof(ToggleSplitModeFinishedEvent));
}
constexpr auto ToggleSplitModeFinishedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ToggleSplitModeFinishedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ToggleSplitModeFinishedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ToggleSplitModeFinishedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ToggleSplitModeFinishedEvent>(), &ToggleSplitModeFinishedEvent::ByteSizeLong,
              &ToggleSplitModeFinishedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ToggleSplitModeFinishedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[66],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ToggleSplitModeFinishedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ToggleSplitModeFinishedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ToggleSplitModeFinishedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ToggleSplitModeFinishedEvent::InternalGenerateClassData_(
            _default, &ToggleSplitModeFinishedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ToggleSplitModeFinishedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ToggleSplitModeFinishedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ToggleSplitModeFinishedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ToggleSplitModeFinishedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ToggleSplitModeFinishedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ToggleSplitModeFinishedEventGlobalsTypeInternal ToggleSplitModeFinishedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ToggleSplitModeFinishedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ToggleSplitModeFinishedEvent_globals_.GetClassData();
#else
  return ToggleSplitModeFinishedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ToggleAutostartFinishedEvent::_Internal {
 public:
};

constexpr ToggleAutostartFinishedEvent::ParseTableT_ ToggleAutostartFinishedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ToggleAutostartFinishedEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::ToggleAutostartFinishedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr ToggleAutostartFinishedEvent::ToggleAutostartFinishedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL ToggleAutostartFinishedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ToggleAutostartFinishedEvent(arena);
}
constexpr auto ToggleAutostartFinishedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ToggleAutostartFinishedEvent), alignof(ToggleAutostartFinishedEvent));
}
constexpr auto ToggleAutostartFinishedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ToggleAutostartFinishedEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<ToggleAutostartFinishedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ToggleAutostartFinishedEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<ToggleAutostartFinishedEvent>(), &ToggleAutostartFinishedEvent::ByteSizeLong,
              &ToggleAutostartFinishedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ToggleAutostartFinishedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[16],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ToggleAutostartFinishedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ToggleAutostartFinishedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ToggleAutostartFinishedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ToggleAutostartFinishedEvent::InternalGenerateClassData_(
            _default, &ToggleAutostartFinishedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ToggleAutostartFinishedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ToggleAutostartFinishedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ToggleAutostartFinishedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ToggleAutostartFinishedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ToggleAutostartFinishedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ToggleAutostartFinishedEventGlobalsTypeInternal ToggleAutostartFinishedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ToggleAutostartFinishedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ToggleAutostartFinishedEvent_globals_.GetClassData();
#else
  return ToggleAutostartFinishedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class SyncStartedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<SyncStartedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(SyncStartedEvent, _impl_._has_bits_);
};

constexpr SyncStartedEvent::ParseTableT_ SyncStartedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(SyncStartedEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::SyncStartedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string userID = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(SyncStartedEvent, _impl_.userid_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string userID = 1;
      {PROTOBUF_FIELD_OFFSET(SyncStartedEvent, _impl_.userid_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\25\6\0\0\0\0\0\0"
      "grpc.SyncStartedEvent"
      "userID"
    }},
  };
}


inline constexpr SyncStartedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        userid_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr SyncStartedEvent::SyncStartedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL SyncStartedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) SyncStartedEvent(arena);
}
constexpr auto SyncStartedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(SyncStartedEvent), alignof(SyncStartedEvent));
}
constexpr auto SyncStartedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &SyncStartedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<SyncStartedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &SyncStartedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<SyncStartedEvent>(), &SyncStartedEvent::ByteSizeLong,
              &SyncStartedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(SyncStartedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[72],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SyncStartedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SyncStartedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 SyncStartedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(SyncStartedEvent::InternalGenerateClassData_(
            _default, &SyncStartedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<SyncStartedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SyncStartedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) SyncStartedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<SyncStartedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SyncStartedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SyncStartedEventGlobalsTypeInternal SyncStartedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* SyncStartedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return SyncStartedEvent_globals_.GetClassData();
#else
  return SyncStartedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class SyncProgressEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<SyncProgressEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(SyncProgressEvent, _impl_._has_bits_);
};

constexpr SyncProgressEvent::ParseTableT_ SyncProgressEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(SyncProgressEvent, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::SyncProgressEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int64 remainingMs = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(SyncProgressEvent, _impl_.remainingms_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(SyncProgressEvent, _impl_.remainingms_)}},
      // string userID = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(SyncProgressEvent, _impl_.userid_)}},
      // double progress = 2;
      {::_pbi::TcParser::FastF64S1,
       {17, 1, 0,
        PROTOBUF_FIELD_OFFSET(SyncProgressEvent, _impl_.progress_)}},
      // int64 elapsedMs = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(SyncProgressEvent, _impl_.elapsedms_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(SyncProgressEvent, _impl_.elapsedms_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string userID = 1;
      {PROTOBUF_FIELD_OFFSET(SyncProgressEvent, _impl_.userid_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // double progress = 2;
      {PROTOBUF_FIELD_OFFSET(SyncProgressEvent, _impl_.progress_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
      // int64 elapsedMs = 3;
      {PROTOBUF_FIELD_OFFSET(SyncProgressEvent, _impl_.elapsedms_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // int64 remainingMs = 4;
      {PROTOBUF_FIELD_OFFSET(SyncProgressEvent, _impl_.remainingms_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
    }},
    // no aux_entries
    {{
      "\26\6\0\0\0\0\0\0"
      "grpc.SyncProgressEvent"
      "userID"
    }},
  };
}


inline constexpr SyncProgressEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        userid_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        progress_{0},
        elapsedms_{::int64_t{0}},
        remainingms_{::int64_t{0}} {}

template <typename>
constexpr SyncProgressEvent::SyncProgressEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL SyncProgressEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) SyncProgressEvent(arena);
}
constexpr auto SyncProgressEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(SyncProgressEvent), alignof(SyncProgressEvent));
}
constexpr auto SyncProgressEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &SyncProgressEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<SyncProgressEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &SyncProgressEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<SyncProgressEvent>(), &SyncProgressEvent::ByteSizeLong,
              &SyncProgressEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(SyncProgressEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[74],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SyncProgressEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SyncProgressEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 SyncProgressEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(SyncProgressEvent::InternalGenerateClassData_(
            _default, &SyncProgressEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<SyncProgressEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SyncProgressEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) SyncProgressEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<SyncProgressEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SyncProgressEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SyncProgressEventGlobalsTypeInternal SyncProgressEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* SyncProgressEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return SyncProgressEvent_globals_.GetClassData();
#else
  return SyncProgressEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class SyncFinishedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<SyncFinishedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(SyncFinishedEvent, _impl_._has_bits_);
};

constexpr SyncFinishedEvent::ParseTableT_ SyncFinishedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(SyncFinishedEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::SyncFinishedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string userID = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(SyncFinishedEvent, _impl_.userid_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string userID = 1;
      {PROTOBUF_FIELD_OFFSET(SyncFinishedEvent, _impl_.userid_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\26\6\0\0\0\0\0\0"
      "grpc.SyncFinishedEvent"
      "userID"
    }},
  };
}


inline constexpr SyncFinishedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        userid_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr SyncFinishedEvent::SyncFinishedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL SyncFinishedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) SyncFinishedEvent(arena);
}
constexpr auto SyncFinishedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(SyncFinishedEvent), alignof(SyncFinishedEvent));
}
constexpr auto SyncFinishedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &SyncFinishedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<SyncFinishedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &SyncFinishedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<SyncFinishedEvent>(), &SyncFinishedEvent::ByteSizeLong,
              &SyncFinishedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(SyncFinishedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[73],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SyncFinishedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SyncFinishedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 SyncFinishedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(SyncFinishedEvent::InternalGenerateClassData_(
            _default, &SyncFinishedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<SyncFinishedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SyncFinishedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) SyncFinishedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<SyncFinishedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SyncFinishedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SyncFinishedEventGlobalsTypeInternal SyncFinishedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* SyncFinishedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return SyncFinishedEvent_globals_.GetClassData();
#else
  return SyncFinishedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ShowMainWindowEvent::_Internal {
 public:
};

constexpr ShowMainWindowEvent::ParseTableT_ ShowMainWindowEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ShowMainWindowEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::ShowMainWindowEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr ShowMainWindowEvent::ShowMainWindowEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL ShowMainWindowEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ShowMainWindowEvent(arena);
}
constexpr auto ShowMainWindowEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ShowMainWindowEvent), alignof(ShowMainWindowEvent));
}
constexpr auto ShowMainWindowEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ShowMainWindowEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<ShowMainWindowEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ShowMainWindowEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<ShowMainWindowEvent>(), &ShowMainWindowEvent::ByteSizeLong,
              &ShowMainWindowEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ShowMainWindowEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[21],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ShowMainWindowEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ShowMainWindowEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ShowMainWindowEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ShowMainWindowEvent::InternalGenerateClassData_(
            _default, &ShowMainWindowEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ShowMainWindowEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ShowMainWindowEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ShowMainWindowEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ShowMainWindowEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ShowMainWindowEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ShowMainWindowEventGlobalsTypeInternal ShowMainWindowEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ShowMainWindowEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ShowMainWindowEvent_globals_.GetClassData();
#else
  return ShowMainWindowEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ResetFinishedEvent::_Internal {
 public:
};

constexpr ResetFinishedEvent::ParseTableT_ ResetFinishedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ResetFinishedEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::ResetFinishedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr ResetFinishedEvent::ResetFinishedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL ResetFinishedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ResetFinishedEvent(arena);
}
constexpr auto ResetFinishedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ResetFinishedEvent), alignof(ResetFinishedEvent));
}
constexpr auto ResetFinishedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ResetFinishedEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<ResetFinishedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ResetFinishedEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<ResetFinishedEvent>(), &ResetFinishedEvent::ByteSizeLong,
              &ResetFinishedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ResetFinishedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[17],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ResetFinishedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ResetFinishedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ResetFinishedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ResetFinishedEvent::InternalGenerateClassData_(
            _default, &ResetFinishedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ResetFinishedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ResetFinishedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ResetFinishedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ResetFinishedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ResetFinishedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ResetFinishedEventGlobalsTypeInternal ResetFinishedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ResetFinishedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ResetFinishedEvent_globals_.GetClassData();
#else
  return ResetFinishedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ReportBugSuccessEvent::_Internal {
 public:
};

constexpr ReportBugSuccessEvent::ParseTableT_ ReportBugSuccessEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ReportBugSuccessEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::ReportBugSuccessEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr ReportBugSuccessEvent::ReportBugSuccessEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL ReportBugSuccessEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ReportBugSuccessEvent(arena);
}
constexpr auto ReportBugSuccessEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ReportBugSuccessEvent), alignof(ReportBugSuccessEvent));
}
constexpr auto ReportBugSuccessEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ReportBugSuccessEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<ReportBugSuccessEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ReportBugSuccessEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<ReportBugSuccessEvent>(), &ReportBugSuccessEvent::ByteSizeLong,
              &ReportBugSuccessEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ReportBugSuccessEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[19],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ReportBugSuccessEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ReportBugSuccessEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ReportBugSuccessEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ReportBugSuccessEvent::InternalGenerateClassData_(
            _default, &ReportBugSuccessEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ReportBugSuccessEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ReportBugSuccessEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ReportBugSuccessEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ReportBugSuccessEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ReportBugSuccessEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ReportBugSuccessEventGlobalsTypeInternal ReportBugSuccessEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ReportBugSuccessEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ReportBugSuccessEvent_globals_.GetClassData();
#else
  return ReportBugSuccessEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ReportBugRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<ReportBugRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_._has_bits_);
};

constexpr ReportBugRequest::ParseTableT_ ReportBugRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_._has_bits_),
      0, // no _extensions_
      7, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967168,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::ReportBugRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string osType = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_.ostype_)}},
      // string osVersion = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_.osversion_)}},
      // string title = 3;
      {::_pbi::TcParser::FastUS1,
       {26, 2, 0,
        PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_.title_)}},
      // string description = 4;
      {::_pbi::TcParser::FastUS1,
       {34, 3, 0,
        PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_.description_)}},
      // string address = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 4, 0,
        PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_.address_)}},
      // string emailClient = 6;
      {::_pbi::TcParser::FastUS1,
       {50, 5, 0,
        PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_.emailclient_)}},
      // bool includeLogs = 7;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(ReportBugRequest, _impl_.includelogs_), 6>(),
       {56, 6, 0,
        PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_.includelogs_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string osType = 1;
      {PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_.ostype_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string osVersion = 2;
      {PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_.osversion_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string title = 3;
      {PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_.title_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string description = 4;
      {PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_.description_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string address = 5;
      {PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_.address_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string emailClient = 6;
      {PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_.emailclient_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // bool includeLogs = 7;
      {PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_.includelogs_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    // no aux_entries
    {{
      "\25\6\11\5\13\7\13\0"
      "grpc.ReportBugRequest"
      "osType"
      "osVersion"
      "title"
      "description"
      "address"
      "emailClient"
    }},
  };
}


inline constexpr ReportBugRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        ostype_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        osversion_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        title_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        description_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        address_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        emailclient_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        includelogs_{false} {}

template <typename>
constexpr ReportBugRequest::ReportBugRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ReportBugRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ReportBugRequest(arena);
}
constexpr auto ReportBugRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ReportBugRequest), alignof(ReportBugRequest));
}
constexpr auto ReportBugRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ReportBugRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ReportBugRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ReportBugRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ReportBugRequest>(), &ReportBugRequest::ByteSizeLong,
              &ReportBugRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ReportBugRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ReportBugRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ReportBugRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ReportBugRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ReportBugRequest::InternalGenerateClassData_(
            _default, &ReportBugRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ReportBugRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ReportBugRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ReportBugRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ReportBugRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ReportBugRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ReportBugRequestGlobalsTypeInternal ReportBugRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ReportBugRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ReportBugRequest_globals_.GetClassData();
#else
  return ReportBugRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ReportBugFinishedEvent::_Internal {
 public:
};

constexpr ReportBugFinishedEvent::ParseTableT_ ReportBugFinishedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ReportBugFinishedEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::ReportBugFinishedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr ReportBugFinishedEvent::ReportBugFinishedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL ReportBugFinishedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ReportBugFinishedEvent(arena);
}
constexpr auto ReportBugFinishedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ReportBugFinishedEvent), alignof(ReportBugFinishedEvent));
}
constexpr auto ReportBugFinishedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ReportBugFinishedEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<ReportBugFinishedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ReportBugFinishedEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<ReportBugFinishedEvent>(), &ReportBugFinishedEvent::ByteSizeLong,
              &ReportBugFinishedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ReportBugFinishedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[18],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ReportBugFinishedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ReportBugFinishedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ReportBugFinishedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ReportBugFinishedEvent::InternalGenerateClassData_(
            _default, &ReportBugFinishedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ReportBugFinishedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ReportBugFinishedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ReportBugFinishedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ReportBugFinishedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ReportBugFinishedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ReportBugFinishedEventGlobalsTypeInternal ReportBugFinishedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ReportBugFinishedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ReportBugFinishedEvent_globals_.GetClassData();
#else
  return ReportBugFinishedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ReportBugFallbackEvent::_Internal {
 public:
};

constexpr ReportBugFallbackEvent::ParseTableT_ ReportBugFallbackEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ReportBugFallbackEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::ReportBugFallbackEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr ReportBugFallbackEvent::ReportBugFallbackEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL ReportBugFallbackEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ReportBugFallbackEvent(arena);
}
constexpr auto ReportBugFallbackEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ReportBugFallbackEvent), alignof(ReportBugFallbackEvent));
}
constexpr auto ReportBugFallbackEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ReportBugFallbackEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<ReportBugFallbackEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ReportBugFallbackEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<ReportBugFallbackEvent>(), &ReportBugFallbackEvent::ByteSizeLong,
              &ReportBugFallbackEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ReportBugFallbackEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[22],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ReportBugFallbackEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ReportBugFallbackEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ReportBugFallbackEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ReportBugFallbackEvent::InternalGenerateClassData_(
            _default, &ReportBugFallbackEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ReportBugFallbackEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ReportBugFallbackEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ReportBugFallbackEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ReportBugFallbackEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ReportBugFallbackEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ReportBugFallbackEventGlobalsTypeInternal ReportBugFallbackEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ReportBugFallbackEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ReportBugFallbackEvent_globals_.GetClassData();
#else
  return ReportBugFallbackEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ReportBugErrorEvent::_Internal {
 public:
};

constexpr ReportBugErrorEvent::ParseTableT_ ReportBugErrorEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ReportBugErrorEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::ReportBugErrorEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr ReportBugErrorEvent::ReportBugErrorEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL ReportBugErrorEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ReportBugErrorEvent(arena);
}
constexpr auto ReportBugErrorEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ReportBugErrorEvent), alignof(ReportBugErrorEvent));
}
constexpr auto ReportBugErrorEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ReportBugErrorEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<ReportBugErrorEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ReportBugErrorEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<ReportBugErrorEvent>(), &ReportBugErrorEvent::ByteSizeLong,
              &ReportBugErrorEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ReportBugErrorEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[20],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ReportBugErrorEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ReportBugErrorEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ReportBugErrorEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ReportBugErrorEvent::InternalGenerateClassData_(
            _default, &ReportBugErrorEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ReportBugErrorEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ReportBugErrorEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ReportBugErrorEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ReportBugErrorEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ReportBugErrorEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ReportBugErrorEventGlobalsTypeInternal ReportBugErrorEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ReportBugErrorEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ReportBugErrorEvent_globals_.GetClassData();
#else
  return ReportBugErrorEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class RepairStartedEvent::_Internal {
 public:
};

constexpr RepairStartedEvent::ParseTableT_ RepairStartedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(RepairStartedEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::RepairStartedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr RepairStartedEvent::RepairStartedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL RepairStartedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) RepairStartedEvent(arena);
}
constexpr auto RepairStartedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(RepairStartedEvent), alignof(RepairStartedEvent));
}
constexpr auto RepairStartedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &RepairStartedEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<RepairStartedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &RepairStartedEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<RepairStartedEvent>(), &RepairStartedEvent::ByteSizeLong,
              &RepairStartedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(RepairStartedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[26],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct RepairStartedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr RepairStartedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 RepairStartedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(RepairStartedEvent::InternalGenerateClassData_(
            _default, &RepairStartedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<RepairStartedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~RepairStartedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) RepairStartedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<RepairStartedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(RepairStartedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST RepairStartedEventGlobalsTypeInternal RepairStartedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* RepairStartedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return RepairStartedEvent_globals_.GetClassData();
#else
  return RepairStartedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class RebuildKeychainEvent::_Internal {
 public:
};

constexpr RebuildKeychainEvent::ParseTableT_ RebuildKeychainEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(RebuildKeychainEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::RebuildKeychainEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr RebuildKeychainEvent::RebuildKeychainEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL RebuildKeychainEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) RebuildKeychainEvent(arena);
}
constexpr auto RebuildKeychainEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(RebuildKeychainEvent), alignof(RebuildKeychainEvent));
}
constexpr auto RebuildKeychainEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &RebuildKeychainEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<RebuildKeychainEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &RebuildKeychainEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<RebuildKeychainEvent>(), &RebuildKeychainEvent::ByteSizeLong,
              &RebuildKeychainEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(RebuildKeychainEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[60],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct RebuildKeychainEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr RebuildKeychainEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 RebuildKeychainEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(RebuildKeychainEvent::InternalGenerateClassData_(
            _default, &RebuildKeychainEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<RebuildKeychainEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~RebuildKeychainEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) RebuildKeychainEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<RebuildKeychainEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(RebuildKeychainEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST RebuildKeychainEventGlobalsTypeInternal RebuildKeychainEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* RebuildKeychainEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return RebuildKeychainEvent_globals_.GetClassData();
#else
  return RebuildKeychainEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class MailServerSettingsErrorEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<MailServerSettingsErrorEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(MailServerSettingsErrorEvent, _impl_._has_bits_);
};

constexpr MailServerSettingsErrorEvent::ParseTableT_ MailServerSettingsErrorEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(MailServerSettingsErrorEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::MailServerSettingsErrorEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .grpc.MailServerSettingsErrorType type = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(MailServerSettingsErrorEvent, _impl_.type_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(MailServerSettingsErrorEvent, _impl_.type_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.MailServerSettingsErrorType type = 1;
      {PROTOBUF_FIELD_OFFSET(MailServerSettingsErrorEvent, _impl_.type_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr MailServerSettingsErrorEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        type_{static_cast< ::grpc::MailServerSettingsErrorType >(0)} {}

template <typename>
constexpr MailServerSettingsErrorEvent::MailServerSettingsErrorEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL MailServerSettingsErrorEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) MailServerSettingsErrorEvent(arena);
}
constexpr auto MailServerSettingsErrorEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(MailServerSettingsErrorEvent), alignof(MailServerSettingsErrorEvent));
}
constexpr auto MailServerSettingsErrorEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &MailServerSettingsErrorEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<MailServerSettingsErrorEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &MailServerSettingsErrorEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<MailServerSettingsErrorEvent>(), &MailServerSettingsErrorEvent::ByteSizeLong,
              &MailServerSettingsErrorEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(MailServerSettingsErrorEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[54],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct MailServerSettingsErrorEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr MailServerSettingsErrorEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 MailServerSettingsErrorEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(MailServerSettingsErrorEvent::InternalGenerateClassData_(
            _default, &MailServerSettingsErrorEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<MailServerSettingsErrorEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~MailServerSettingsErrorEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) MailServerSettingsErrorEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<MailServerSettingsErrorEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(MailServerSettingsErrorEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST MailServerSettingsErrorEventGlobalsTypeInternal MailServerSettingsErrorEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* MailServerSettingsErrorEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return MailServerSettingsErrorEvent_globals_.GetClassData();
#else
  return MailServerSettingsErrorEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class LoginTwoPasswordsRequestedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<LoginTwoPasswordsRequestedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(LoginTwoPasswordsRequestedEvent, _impl_._has_bits_);
};

constexpr LoginTwoPasswordsRequestedEvent::ParseTableT_ LoginTwoPasswordsRequestedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(LoginTwoPasswordsRequestedEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::LoginTwoPasswordsRequestedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string username = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(LoginTwoPasswordsRequestedEvent, _impl_.username_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string username = 1;
      {PROTOBUF_FIELD_OFFSET(LoginTwoPasswordsRequestedEvent, _impl_.username_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\44\10\0\0\0\0\0\0"
      "grpc.LoginTwoPasswordsRequestedEvent"
      "username"
    }},
  };
}


inline constexpr LoginTwoPasswordsRequestedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        username_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr LoginTwoPasswordsRequestedEvent::LoginTwoPasswordsRequestedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL LoginTwoPasswordsRequestedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) LoginTwoPasswordsRequestedEvent(arena);
}
constexpr auto LoginTwoPasswordsRequestedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(LoginTwoPasswordsRequestedEvent), alignof(LoginTwoPasswordsRequestedEvent));
}
constexpr auto LoginTwoPasswordsRequestedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &LoginTwoPasswordsRequestedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<LoginTwoPasswordsRequestedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &LoginTwoPasswordsRequestedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<LoginTwoPasswordsRequestedEvent>(), &LoginTwoPasswordsRequestedEvent::ByteSizeLong,
              &LoginTwoPasswordsRequestedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(LoginTwoPasswordsRequestedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[37],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct LoginTwoPasswordsRequestedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr LoginTwoPasswordsRequestedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 LoginTwoPasswordsRequestedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(LoginTwoPasswordsRequestedEvent::InternalGenerateClassData_(
            _default, &LoginTwoPasswordsRequestedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<LoginTwoPasswordsRequestedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~LoginTwoPasswordsRequestedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) LoginTwoPasswordsRequestedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<LoginTwoPasswordsRequestedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(LoginTwoPasswordsRequestedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST LoginTwoPasswordsRequestedEventGlobalsTypeInternal LoginTwoPasswordsRequestedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* LoginTwoPasswordsRequestedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return LoginTwoPasswordsRequestedEvent_globals_.GetClassData();
#else
  return LoginTwoPasswordsRequestedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class LoginTfaRequestedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<LoginTfaRequestedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(LoginTfaRequestedEvent, _impl_._has_bits_);
};

constexpr LoginTfaRequestedEvent::ParseTableT_ LoginTfaRequestedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(LoginTfaRequestedEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::LoginTfaRequestedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string username = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(LoginTfaRequestedEvent, _impl_.username_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string username = 1;
      {PROTOBUF_FIELD_OFFSET(LoginTfaRequestedEvent, _impl_.username_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\33\10\0\0\0\0\0\0"
      "grpc.LoginTfaRequestedEvent"
      "username"
    }},
  };
}


inline constexpr LoginTfaRequestedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        username_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr LoginTfaRequestedEvent::LoginTfaRequestedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL LoginTfaRequestedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) LoginTfaRequestedEvent(arena);
}
constexpr auto LoginTfaRequestedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(LoginTfaRequestedEvent), alignof(LoginTfaRequestedEvent));
}
constexpr auto LoginTfaRequestedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &LoginTfaRequestedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<LoginTfaRequestedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &LoginTfaRequestedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<LoginTfaRequestedEvent>(), &LoginTfaRequestedEvent::ByteSizeLong,
              &LoginTfaRequestedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(LoginTfaRequestedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[32],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct LoginTfaRequestedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr LoginTfaRequestedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 LoginTfaRequestedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(LoginTfaRequestedEvent::InternalGenerateClassData_(
            _default, &LoginTfaRequestedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<LoginTfaRequestedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~LoginTfaRequestedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) LoginTfaRequestedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<LoginTfaRequestedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(LoginTfaRequestedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST LoginTfaRequestedEventGlobalsTypeInternal LoginTfaRequestedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* LoginTfaRequestedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return LoginTfaRequestedEvent_globals_.GetClassData();
#else
  return LoginTfaRequestedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class LoginTfaOrFidoRequestedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<LoginTfaOrFidoRequestedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(LoginTfaOrFidoRequestedEvent, _impl_._has_bits_);
};

constexpr LoginTfaOrFidoRequestedEvent::ParseTableT_ LoginTfaOrFidoRequestedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(LoginTfaOrFidoRequestedEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::LoginTfaOrFidoRequestedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string username = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(LoginTfaOrFidoRequestedEvent, _impl_.username_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string username = 1;
      {PROTOBUF_FIELD_OFFSET(LoginTfaOrFidoRequestedEvent, _impl_.username_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\41\10\0\0\0\0\0\0"
      "grpc.LoginTfaOrFidoRequestedEvent"
      "username"
    }},
  };
}


inline constexpr LoginTfaOrFidoRequestedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        username_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr LoginTfaOrFidoRequestedEvent::LoginTfaOrFidoRequestedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL LoginTfaOrFidoRequestedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) LoginTfaOrFidoRequestedEvent(arena);
}
constexpr auto LoginTfaOrFidoRequestedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(LoginTfaOrFidoRequestedEvent), alignof(LoginTfaOrFidoRequestedEvent));
}
constexpr auto LoginTfaOrFidoRequestedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &LoginTfaOrFidoRequestedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<LoginTfaOrFidoRequestedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &LoginTfaOrFidoRequestedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<LoginTfaOrFidoRequestedEvent>(), &LoginTfaOrFidoRequestedEvent::ByteSizeLong,
              &LoginTfaOrFidoRequestedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(LoginTfaOrFidoRequestedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[34],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct LoginTfaOrFidoRequestedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr LoginTfaOrFidoRequestedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 LoginTfaOrFidoRequestedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(LoginTfaOrFidoRequestedEvent::InternalGenerateClassData_(
            _default, &LoginTfaOrFidoRequestedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<LoginTfaOrFidoRequestedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~LoginTfaOrFidoRequestedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) LoginTfaOrFidoRequestedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<LoginTfaOrFidoRequestedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(LoginTfaOrFidoRequestedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST LoginTfaOrFidoRequestedEventGlobalsTypeInternal LoginTfaOrFidoRequestedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* LoginTfaOrFidoRequestedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return LoginTfaOrFidoRequestedEvent_globals_.GetClassData();
#else
  return LoginTfaOrFidoRequestedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class LoginRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<LoginRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(LoginRequest, _impl_._has_bits_);
};

constexpr LoginRequest::ParseTableT_ LoginRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(LoginRequest, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::LoginRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string username = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(LoginRequest, _impl_.username_)}},
      // bytes password = 2;
      {::_pbi::TcParser::FastBS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(LoginRequest, _impl_.password_)}},
      // optional bool useHvDetails = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(LoginRequest, _impl_.usehvdetails_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(LoginRequest, _impl_.usehvdetails_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string username = 1;
      {PROTOBUF_FIELD_OFFSET(LoginRequest, _impl_.username_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // bytes password = 2;
      {PROTOBUF_FIELD_OFFSET(LoginRequest, _impl_.password_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // optional bool useHvDetails = 3;
      {PROTOBUF_FIELD_OFFSET(LoginRequest, _impl_.usehvdetails_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    // no aux_entries
    {{
      "\21\10\0\0\0\0\0\0"
      "grpc.LoginRequest"
      "username"
    }},
  };
}


inline constexpr LoginRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        username_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        password_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        usehvdetails_{false} {}

template <typename>
constexpr LoginRequest::LoginRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL LoginRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) LoginRequest(arena);
}
constexpr auto LoginRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(LoginRequest), alignof(LoginRequest));
}
constexpr auto LoginRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &LoginRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<LoginRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &LoginRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<LoginRequest>(), &LoginRequest::ByteSizeLong,
              &LoginRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(LoginRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[3],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct LoginRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr LoginRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 LoginRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(LoginRequest::InternalGenerateClassData_(
            _default, &LoginRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<LoginRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~LoginRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) LoginRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<LoginRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(LoginRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST LoginRequestGlobalsTypeInternal LoginRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* LoginRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return LoginRequest_globals_.GetClassData();
#else
  return LoginRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class LoginHvRequestedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<LoginHvRequestedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(LoginHvRequestedEvent, _impl_._has_bits_);
};

constexpr LoginHvRequestedEvent::ParseTableT_ LoginHvRequestedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(LoginHvRequestedEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::LoginHvRequestedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string hvUrl = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(LoginHvRequestedEvent, _impl_.hvurl_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string hvUrl = 1;
      {PROTOBUF_FIELD_OFFSET(LoginHvRequestedEvent, _impl_.hvurl_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\32\5\0\0\0\0\0\0"
      "grpc.LoginHvRequestedEvent"
      "hvUrl"
    }},
  };
}


inline constexpr LoginHvRequestedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        hvurl_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr LoginHvRequestedEvent::LoginHvRequestedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL LoginHvRequestedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) LoginHvRequestedEvent(arena);
}
constexpr auto LoginHvRequestedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(LoginHvRequestedEvent), alignof(LoginHvRequestedEvent));
}
constexpr auto LoginHvRequestedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &LoginHvRequestedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<LoginHvRequestedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &LoginHvRequestedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<LoginHvRequestedEvent>(), &LoginHvRequestedEvent::ByteSizeLong,
              &LoginHvRequestedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(LoginHvRequestedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[39],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct LoginHvRequestedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr LoginHvRequestedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 LoginHvRequestedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(LoginHvRequestedEvent::InternalGenerateClassData_(
            _default, &LoginHvRequestedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<LoginHvRequestedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~LoginHvRequestedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) LoginHvRequestedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<LoginHvRequestedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(LoginHvRequestedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST LoginHvRequestedEventGlobalsTypeInternal LoginHvRequestedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* LoginHvRequestedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return LoginHvRequestedEvent_globals_.GetClassData();
#else
  return LoginHvRequestedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class LoginFinishedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<LoginFinishedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(LoginFinishedEvent, _impl_._has_bits_);
};

constexpr LoginFinishedEvent::ParseTableT_ LoginFinishedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(LoginFinishedEvent, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::LoginFinishedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // bool wasSignedOut = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(LoginFinishedEvent, _impl_.wassignedout_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(LoginFinishedEvent, _impl_.wassignedout_)}},
      // string userID = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(LoginFinishedEvent, _impl_.userid_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string userID = 1;
      {PROTOBUF_FIELD_OFFSET(LoginFinishedEvent, _impl_.userid_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // bool wasSignedOut = 2;
      {PROTOBUF_FIELD_OFFSET(LoginFinishedEvent, _impl_.wassignedout_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    // no aux_entries
    {{
      "\27\6\0\0\0\0\0\0"
      "grpc.LoginFinishedEvent"
      "userID"
    }},
  };
}


inline constexpr LoginFinishedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        userid_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        wassignedout_{false} {}

template <typename>
constexpr LoginFinishedEvent::LoginFinishedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL LoginFinishedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) LoginFinishedEvent(arena);
}
constexpr auto LoginFinishedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(LoginFinishedEvent), alignof(LoginFinishedEvent));
}
constexpr auto LoginFinishedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &LoginFinishedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<LoginFinishedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &LoginFinishedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<LoginFinishedEvent>(), &LoginFinishedEvent::ByteSizeLong,
              &LoginFinishedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(LoginFinishedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[38],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct LoginFinishedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr LoginFinishedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 LoginFinishedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(LoginFinishedEvent::InternalGenerateClassData_(
            _default, &LoginFinishedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<LoginFinishedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~LoginFinishedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) LoginFinishedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<LoginFinishedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(LoginFinishedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST LoginFinishedEventGlobalsTypeInternal LoginFinishedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* LoginFinishedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return LoginFinishedEvent_globals_.GetClassData();
#else
  return LoginFinishedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class LoginFidoTouchEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<LoginFidoTouchEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(LoginFidoTouchEvent, _impl_._has_bits_);
};

constexpr LoginFidoTouchEvent::ParseTableT_ LoginFidoTouchEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(LoginFidoTouchEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::LoginFidoTouchEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string username = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(LoginFidoTouchEvent, _impl_.username_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string username = 1;
      {PROTOBUF_FIELD_OFFSET(LoginFidoTouchEvent, _impl_.username_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\30\10\0\0\0\0\0\0"
      "grpc.LoginFidoTouchEvent"
      "username"
    }},
  };
}


inline constexpr LoginFidoTouchEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        username_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr LoginFidoTouchEvent::LoginFidoTouchEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL LoginFidoTouchEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) LoginFidoTouchEvent(arena);
}
constexpr auto LoginFidoTouchEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(LoginFidoTouchEvent), alignof(LoginFidoTouchEvent));
}
constexpr auto LoginFidoTouchEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &LoginFidoTouchEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<LoginFidoTouchEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &LoginFidoTouchEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<LoginFidoTouchEvent>(), &LoginFidoTouchEvent::ByteSizeLong,
              &LoginFidoTouchEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(LoginFidoTouchEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[35],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct LoginFidoTouchEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr LoginFidoTouchEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 LoginFidoTouchEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(LoginFidoTouchEvent::InternalGenerateClassData_(
            _default, &LoginFidoTouchEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<LoginFidoTouchEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~LoginFidoTouchEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) LoginFidoTouchEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<LoginFidoTouchEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(LoginFidoTouchEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST LoginFidoTouchEventGlobalsTypeInternal LoginFidoTouchEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* LoginFidoTouchEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return LoginFidoTouchEvent_globals_.GetClassData();
#else
  return LoginFidoTouchEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class LoginFidoRequestedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<LoginFidoRequestedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(LoginFidoRequestedEvent, _impl_._has_bits_);
};

constexpr LoginFidoRequestedEvent::ParseTableT_ LoginFidoRequestedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(LoginFidoRequestedEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::LoginFidoRequestedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string username = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(LoginFidoRequestedEvent, _impl_.username_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string username = 1;
      {PROTOBUF_FIELD_OFFSET(LoginFidoRequestedEvent, _impl_.username_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\34\10\0\0\0\0\0\0"
      "grpc.LoginFidoRequestedEvent"
      "username"
    }},
  };
}


inline constexpr LoginFidoRequestedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        username_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr LoginFidoRequestedEvent::LoginFidoRequestedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL LoginFidoRequestedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) LoginFidoRequestedEvent(arena);
}
constexpr auto LoginFidoRequestedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(LoginFidoRequestedEvent), alignof(LoginFidoRequestedEvent));
}
constexpr auto LoginFidoRequestedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &LoginFidoRequestedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<LoginFidoRequestedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &LoginFidoRequestedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<LoginFidoRequestedEvent>(), &LoginFidoRequestedEvent::ByteSizeLong,
              &LoginFidoRequestedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(LoginFidoRequestedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[33],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct LoginFidoRequestedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr LoginFidoRequestedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 LoginFidoRequestedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(LoginFidoRequestedEvent::InternalGenerateClassData_(
            _default, &LoginFidoRequestedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<LoginFidoRequestedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~LoginFidoRequestedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) LoginFidoRequestedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<LoginFidoRequestedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(LoginFidoRequestedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST LoginFidoRequestedEventGlobalsTypeInternal LoginFidoRequestedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* LoginFidoRequestedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return LoginFidoRequestedEvent_globals_.GetClassData();
#else
  return LoginFidoRequestedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class LoginFidoPinRequired::_Internal {
 public:
  using HasBits = decltype(::std::declval<LoginFidoPinRequired>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(LoginFidoPinRequired, _impl_._has_bits_);
};

constexpr LoginFidoPinRequired::ParseTableT_ LoginFidoPinRequired::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(LoginFidoPinRequired, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::LoginFidoPinRequired>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string username = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(LoginFidoPinRequired, _impl_.username_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string username = 1;
      {PROTOBUF_FIELD_OFFSET(LoginFidoPinRequired, _impl_.username_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\31\10\0\0\0\0\0\0"
      "grpc.LoginFidoPinRequired"
      "username"
    }},
  };
}


inline constexpr LoginFidoPinRequired::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        username_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr LoginFidoPinRequired::LoginFidoPinRequired(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL LoginFidoPinRequired::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) LoginFidoPinRequired(arena);
}
constexpr auto LoginFidoPinRequired::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(LoginFidoPinRequired), alignof(LoginFidoPinRequired));
}
constexpr auto LoginFidoPinRequired::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &LoginFidoPinRequired::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<LoginFidoPinRequired>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &LoginFidoPinRequired::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<LoginFidoPinRequired>(), &LoginFidoPinRequired::ByteSizeLong,
              &LoginFidoPinRequired::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(LoginFidoPinRequired, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[36],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct LoginFidoPinRequiredGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr LoginFidoPinRequiredGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 LoginFidoPinRequired_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(LoginFidoPinRequired::InternalGenerateClassData_(
            _default, &LoginFidoPinRequired_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<LoginFidoPinRequired>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~LoginFidoPinRequiredGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) LoginFidoPinRequired _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<LoginFidoPinRequired>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(LoginFidoPinRequiredGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST LoginFidoPinRequiredGlobalsTypeInternal LoginFidoPinRequired_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* LoginFidoPinRequired_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return LoginFidoPinRequired_globals_.GetClassData();
#else
  return LoginFidoPinRequired_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class LoginErrorEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<LoginErrorEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(LoginErrorEvent, _impl_._has_bits_);
};

constexpr LoginErrorEvent::ParseTableT_ LoginErrorEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(LoginErrorEvent, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::LoginErrorEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string message = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(LoginErrorEvent, _impl_.message_)}},
      // .grpc.LoginErrorType type = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(LoginErrorEvent, _impl_.type_), 1>(),
       {8, 1, 0,
        PROTOBUF_FIELD_OFFSET(LoginErrorEvent, _impl_.type_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.LoginErrorType type = 1;
      {PROTOBUF_FIELD_OFFSET(LoginErrorEvent, _impl_.type_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // string message = 2;
      {PROTOBUF_FIELD_OFFSET(LoginErrorEvent, _impl_.message_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\24\0\7\0\0\0\0\0"
      "grpc.LoginErrorEvent"
      "message"
    }},
  };
}


inline constexpr LoginErrorEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        message_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        type_{static_cast< ::grpc::LoginErrorType >(0)} {}

template <typename>
constexpr LoginErrorEvent::LoginErrorEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL LoginErrorEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) LoginErrorEvent(arena);
}
constexpr auto LoginErrorEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(LoginErrorEvent), alignof(LoginErrorEvent));
}
constexpr auto LoginErrorEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &LoginErrorEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<LoginErrorEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &LoginErrorEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<LoginErrorEvent>(), &LoginErrorEvent::ByteSizeLong,
              &LoginErrorEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(LoginErrorEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[31],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct LoginErrorEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr LoginErrorEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 LoginErrorEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(LoginErrorEvent::InternalGenerateClassData_(
            _default, &LoginErrorEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<LoginErrorEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~LoginErrorEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) LoginErrorEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<LoginErrorEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(LoginErrorEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST LoginErrorEventGlobalsTypeInternal LoginErrorEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* LoginErrorEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return LoginErrorEvent_globals_.GetClassData();
#else
  return LoginErrorEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class LoginAbortRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<LoginAbortRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(LoginAbortRequest, _impl_._has_bits_);
};

constexpr LoginAbortRequest::ParseTableT_ LoginAbortRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(LoginAbortRequest, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::LoginAbortRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string username = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(LoginAbortRequest, _impl_.username_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string username = 1;
      {PROTOBUF_FIELD_OFFSET(LoginAbortRequest, _impl_.username_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\26\10\0\0\0\0\0\0"
      "grpc.LoginAbortRequest"
      "username"
    }},
  };
}


inline constexpr LoginAbortRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        username_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr LoginAbortRequest::LoginAbortRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL LoginAbortRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) LoginAbortRequest(arena);
}
constexpr auto LoginAbortRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(LoginAbortRequest), alignof(LoginAbortRequest));
}
constexpr auto LoginAbortRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &LoginAbortRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<LoginAbortRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &LoginAbortRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<LoginAbortRequest>(), &LoginAbortRequest::ByteSizeLong,
              &LoginAbortRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(LoginAbortRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[4],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct LoginAbortRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr LoginAbortRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 LoginAbortRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(LoginAbortRequest::InternalGenerateClassData_(
            _default, &LoginAbortRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<LoginAbortRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~LoginAbortRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) LoginAbortRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<LoginAbortRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(LoginAbortRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST LoginAbortRequestGlobalsTypeInternal LoginAbortRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* LoginAbortRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return LoginAbortRequest_globals_.GetClassData();
#else
  return LoginAbortRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class KnowledgeBaseSuggestion::_Internal {
 public:
  using HasBits = decltype(::std::declval<KnowledgeBaseSuggestion>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(KnowledgeBaseSuggestion, _impl_._has_bits_);
};

constexpr KnowledgeBaseSuggestion::ParseTableT_ KnowledgeBaseSuggestion::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(KnowledgeBaseSuggestion, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::KnowledgeBaseSuggestion>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string title = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(KnowledgeBaseSuggestion, _impl_.title_)}},
      // string url = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(KnowledgeBaseSuggestion, _impl_.url_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string url = 1;
      {PROTOBUF_FIELD_OFFSET(KnowledgeBaseSuggestion, _impl_.url_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string title = 2;
      {PROTOBUF_FIELD_OFFSET(KnowledgeBaseSuggestion, _impl_.title_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\34\3\5\0\0\0\0\0"
      "grpc.KnowledgeBaseSuggestion"
      "url"
      "title"
    }},
  };
}


inline constexpr KnowledgeBaseSuggestion::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        url_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        title_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr KnowledgeBaseSuggestion::KnowledgeBaseSuggestion(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL KnowledgeBaseSuggestion::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) KnowledgeBaseSuggestion(arena);
}
constexpr auto KnowledgeBaseSuggestion::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(KnowledgeBaseSuggestion), alignof(KnowledgeBaseSuggestion));
}
constexpr auto KnowledgeBaseSuggestion::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &KnowledgeBaseSuggestion::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<KnowledgeBaseSuggestion>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &KnowledgeBaseSuggestion::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<KnowledgeBaseSuggestion>(), &KnowledgeBaseSuggestion::ByteSizeLong,
              &KnowledgeBaseSuggestion::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(KnowledgeBaseSuggestion, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[28],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct KnowledgeBaseSuggestionGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr KnowledgeBaseSuggestionGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 KnowledgeBaseSuggestion_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(KnowledgeBaseSuggestion::InternalGenerateClassData_(
            _default, &KnowledgeBaseSuggestion_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<KnowledgeBaseSuggestion>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~KnowledgeBaseSuggestionGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) KnowledgeBaseSuggestion _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<KnowledgeBaseSuggestion>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(KnowledgeBaseSuggestionGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST KnowledgeBaseSuggestionGlobalsTypeInternal KnowledgeBaseSuggestion_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* KnowledgeBaseSuggestion_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return KnowledgeBaseSuggestion_globals_.GetClassData();
#else
  return KnowledgeBaseSuggestion_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class InternetStatusEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<InternetStatusEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(InternetStatusEvent, _impl_._has_bits_);
};

constexpr InternetStatusEvent::ParseTableT_ InternetStatusEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(InternetStatusEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::InternetStatusEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // bool connected = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(InternetStatusEvent, _impl_.connected_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(InternetStatusEvent, _impl_.connected_)}},
    }}, {{
      65535, 65535
    }}, {{
      // bool connected = 1;
      {PROTOBUF_FIELD_OFFSET(InternetStatusEvent, _impl_.connected_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr InternetStatusEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        connected_{false} {}

template <typename>
constexpr InternetStatusEvent::InternetStatusEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL InternetStatusEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) InternetStatusEvent(arena);
}
constexpr auto InternetStatusEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(InternetStatusEvent), alignof(InternetStatusEvent));
}
constexpr auto InternetStatusEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &InternetStatusEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<InternetStatusEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &InternetStatusEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<InternetStatusEvent>(), &InternetStatusEvent::ByteSizeLong,
              &InternetStatusEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(InternetStatusEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[15],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct InternetStatusEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr InternetStatusEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 InternetStatusEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(InternetStatusEvent::InternalGenerateClassData_(
            _default, &InternetStatusEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<InternetStatusEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~InternetStatusEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) InternetStatusEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<InternetStatusEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(InternetStatusEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST InternetStatusEventGlobalsTypeInternal InternetStatusEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* InternetStatusEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return InternetStatusEvent_globals_.GetClassData();
#else
  return InternetStatusEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ImapSmtpSettings::_Internal {
 public:
  using HasBits = decltype(::std::declval<ImapSmtpSettings>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ImapSmtpSettings, _impl_._has_bits_);
};

constexpr ImapSmtpSettings::ParseTableT_ ImapSmtpSettings::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ImapSmtpSettings, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::ImapSmtpSettings>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // bool useSSLForSmtp = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(ImapSmtpSettings, _impl_.usesslforsmtp_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(ImapSmtpSettings, _impl_.usesslforsmtp_)}},
      // int32 imapPort = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ImapSmtpSettings, _impl_.imapport_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(ImapSmtpSettings, _impl_.imapport_)}},
      // int32 smtpPort = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ImapSmtpSettings, _impl_.smtpport_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(ImapSmtpSettings, _impl_.smtpport_)}},
      // bool useSSLForImap = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(ImapSmtpSettings, _impl_.usesslforimap_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(ImapSmtpSettings, _impl_.usesslforimap_)}},
    }}, {{
      65535, 65535
    }}, {{
      // int32 imapPort = 1;
      {PROTOBUF_FIELD_OFFSET(ImapSmtpSettings, _impl_.imapport_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int32 smtpPort = 2;
      {PROTOBUF_FIELD_OFFSET(ImapSmtpSettings, _impl_.smtpport_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // bool useSSLForImap = 3;
      {PROTOBUF_FIELD_OFFSET(ImapSmtpSettings, _impl_.usesslforimap_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // bool useSSLForSmtp = 4;
      {PROTOBUF_FIELD_OFFSET(ImapSmtpSettings, _impl_.usesslforsmtp_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr ImapSmtpSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        imapport_{0},
        smtpport_{0},
        usesslforimap_{false},
        usesslforsmtp_{false} {}

template <typename>
constexpr ImapSmtpSettings::ImapSmtpSettings(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ImapSmtpSettings::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ImapSmtpSettings(arena);
}
constexpr auto ImapSmtpSettings::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ImapSmtpSettings), alignof(ImapSmtpSettings));
}
constexpr auto ImapSmtpSettings::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ImapSmtpSettings::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ImapSmtpSettings>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ImapSmtpSettings::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ImapSmtpSettings>(), &ImapSmtpSettings::ByteSizeLong,
              &ImapSmtpSettings::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ImapSmtpSettings, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[5],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ImapSmtpSettingsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ImapSmtpSettingsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ImapSmtpSettings_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ImapSmtpSettings::InternalGenerateClassData_(
            _default, &ImapSmtpSettings_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ImapSmtpSettings>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ImapSmtpSettingsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ImapSmtpSettings _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ImapSmtpSettings>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ImapSmtpSettingsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ImapSmtpSettingsGlobalsTypeInternal ImapSmtpSettings_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ImapSmtpSettings_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ImapSmtpSettings_globals_.GetClassData();
#else
  return ImapSmtpSettings_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ImapLoginFailedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<ImapLoginFailedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ImapLoginFailedEvent, _impl_._has_bits_);
};

constexpr ImapLoginFailedEvent::ParseTableT_ ImapLoginFailedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ImapLoginFailedEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::ImapLoginFailedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string username = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ImapLoginFailedEvent, _impl_.username_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string username = 1;
      {PROTOBUF_FIELD_OFFSET(ImapLoginFailedEvent, _impl_.username_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\31\10\0\0\0\0\0\0"
      "grpc.ImapLoginFailedEvent"
      "username"
    }},
  };
}


inline constexpr ImapLoginFailedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        username_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr ImapLoginFailedEvent::ImapLoginFailedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ImapLoginFailedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ImapLoginFailedEvent(arena);
}
constexpr auto ImapLoginFailedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ImapLoginFailedEvent), alignof(ImapLoginFailedEvent));
}
constexpr auto ImapLoginFailedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ImapLoginFailedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ImapLoginFailedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ImapLoginFailedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ImapLoginFailedEvent>(), &ImapLoginFailedEvent::ByteSizeLong,
              &ImapLoginFailedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ImapLoginFailedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[71],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ImapLoginFailedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ImapLoginFailedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ImapLoginFailedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ImapLoginFailedEvent::InternalGenerateClassData_(
            _default, &ImapLoginFailedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ImapLoginFailedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ImapLoginFailedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ImapLoginFailedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ImapLoginFailedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ImapLoginFailedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ImapLoginFailedEventGlobalsTypeInternal ImapLoginFailedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ImapLoginFailedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ImapLoginFailedEvent_globals_.GetClassData();
#else
  return ImapLoginFailedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class HasNoKeychainEvent::_Internal {
 public:
};

constexpr HasNoKeychainEvent::ParseTableT_ HasNoKeychainEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(HasNoKeychainEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::HasNoKeychainEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr HasNoKeychainEvent::HasNoKeychainEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL HasNoKeychainEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) HasNoKeychainEvent(arena);
}
constexpr auto HasNoKeychainEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(HasNoKeychainEvent), alignof(HasNoKeychainEvent));
}
constexpr auto HasNoKeychainEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &HasNoKeychainEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<HasNoKeychainEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &HasNoKeychainEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<HasNoKeychainEvent>(), &HasNoKeychainEvent::ByteSizeLong,
              &HasNoKeychainEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(HasNoKeychainEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[59],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct HasNoKeychainEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr HasNoKeychainEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 HasNoKeychainEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(HasNoKeychainEvent::InternalGenerateClassData_(
            _default, &HasNoKeychainEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<HasNoKeychainEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~HasNoKeychainEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) HasNoKeychainEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<HasNoKeychainEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(HasNoKeychainEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST HasNoKeychainEventGlobalsTypeInternal HasNoKeychainEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* HasNoKeychainEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return HasNoKeychainEvent_globals_.GetClassData();
#else
  return HasNoKeychainEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class GuiReadyResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<GuiReadyResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(GuiReadyResponse, _impl_._has_bits_);
};

constexpr GuiReadyResponse::ParseTableT_ GuiReadyResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(GuiReadyResponse, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::GuiReadyResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // bool showSplashScreen = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(GuiReadyResponse, _impl_.showsplashscreen_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(GuiReadyResponse, _impl_.showsplashscreen_)}},
    }}, {{
      65535, 65535
    }}, {{
      // bool showSplashScreen = 1;
      {PROTOBUF_FIELD_OFFSET(GuiReadyResponse, _impl_.showsplashscreen_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr GuiReadyResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        showsplashscreen_{false} {}

template <typename>
constexpr GuiReadyResponse::GuiReadyResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL GuiReadyResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) GuiReadyResponse(arena);
}
constexpr auto GuiReadyResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(GuiReadyResponse), alignof(GuiReadyResponse));
}
constexpr auto GuiReadyResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &GuiReadyResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<GuiReadyResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &GuiReadyResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<GuiReadyResponse>(), &GuiReadyResponse::ByteSizeLong,
              &GuiReadyResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(GuiReadyResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct GuiReadyResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr GuiReadyResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 GuiReadyResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(GuiReadyResponse::InternalGenerateClassData_(
            _default, &GuiReadyResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<GuiReadyResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~GuiReadyResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) GuiReadyResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<GuiReadyResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(GuiReadyResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST GuiReadyResponseGlobalsTypeInternal GuiReadyResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* GuiReadyResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return GuiReadyResponse_globals_.GetClassData();
#else
  return GuiReadyResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class GenericErrorEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<GenericErrorEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(GenericErrorEvent, _impl_._has_bits_);
};

constexpr GenericErrorEvent::ParseTableT_ GenericErrorEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(GenericErrorEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::GenericErrorEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .grpc.ErrorCode code = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(GenericErrorEvent, _impl_.code_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(GenericErrorEvent, _impl_.code_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.ErrorCode code = 1;
      {PROTOBUF_FIELD_OFFSET(GenericErrorEvent, _impl_.code_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr GenericErrorEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        code_{static_cast< ::grpc::ErrorCode >(0)} {}

template <typename>
constexpr GenericErrorEvent::GenericErrorEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL GenericErrorEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) GenericErrorEvent(arena);
}
constexpr auto GenericErrorEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(GenericErrorEvent), alignof(GenericErrorEvent));
}
constexpr auto GenericErrorEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &GenericErrorEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<GenericErrorEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &GenericErrorEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<GenericErrorEvent>(), &GenericErrorEvent::ByteSizeLong,
              &GenericErrorEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(GenericErrorEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[76],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct GenericErrorEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr GenericErrorEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 GenericErrorEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(GenericErrorEvent::InternalGenerateClassData_(
            _default, &GenericErrorEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<GenericErrorEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~GenericErrorEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) GenericErrorEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<GenericErrorEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(GenericErrorEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST GenericErrorEventGlobalsTypeInternal GenericErrorEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* GenericErrorEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return GenericErrorEvent_globals_.GetClassData();
#else
  return GenericErrorEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class EventStreamRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<EventStreamRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(EventStreamRequest, _impl_._has_bits_);
};

constexpr EventStreamRequest::ParseTableT_ EventStreamRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(EventStreamRequest, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::EventStreamRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string ClientPlatform = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(EventStreamRequest, _impl_.clientplatform_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string ClientPlatform = 1;
      {PROTOBUF_FIELD_OFFSET(EventStreamRequest, _impl_.clientplatform_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\27\16\0\0\0\0\0\0"
      "grpc.EventStreamRequest"
      "ClientPlatform"
    }},
  };
}


inline constexpr EventStreamRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        clientplatform_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr EventStreamRequest::EventStreamRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL EventStreamRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) EventStreamRequest(arena);
}
constexpr auto EventStreamRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(EventStreamRequest), alignof(EventStreamRequest));
}
constexpr auto EventStreamRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &EventStreamRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<EventStreamRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &EventStreamRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<EventStreamRequest>(), &EventStreamRequest::ByteSizeLong,
              &EventStreamRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(EventStreamRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[12],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct EventStreamRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr EventStreamRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 EventStreamRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(EventStreamRequest::InternalGenerateClassData_(
            _default, &EventStreamRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<EventStreamRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~EventStreamRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) EventStreamRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<EventStreamRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(EventStreamRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST EventStreamRequestGlobalsTypeInternal EventStreamRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* EventStreamRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return EventStreamRequest_globals_.GetClassData();
#else
  return EventStreamRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class DiskCachePathChangedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<DiskCachePathChangedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(DiskCachePathChangedEvent, _impl_._has_bits_);
};

constexpr DiskCachePathChangedEvent::ParseTableT_ DiskCachePathChangedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DiskCachePathChangedEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::DiskCachePathChangedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string path = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(DiskCachePathChangedEvent, _impl_.path_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string path = 1;
      {PROTOBUF_FIELD_OFFSET(DiskCachePathChangedEvent, _impl_.path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\36\4\0\0\0\0\0\0"
      "grpc.DiskCachePathChangedEvent"
      "path"
    }},
  };
}


inline constexpr DiskCachePathChangedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr DiskCachePathChangedEvent::DiskCachePathChangedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL DiskCachePathChangedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DiskCachePathChangedEvent(arena);
}
constexpr auto DiskCachePathChangedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(DiskCachePathChangedEvent), alignof(DiskCachePathChangedEvent));
}
constexpr auto DiskCachePathChangedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DiskCachePathChangedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<DiskCachePathChangedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DiskCachePathChangedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<DiskCachePathChangedEvent>(), &DiskCachePathChangedEvent::ByteSizeLong,
              &DiskCachePathChangedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DiskCachePathChangedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[51],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DiskCachePathChangedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DiskCachePathChangedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DiskCachePathChangedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DiskCachePathChangedEvent::InternalGenerateClassData_(
            _default, &DiskCachePathChangedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DiskCachePathChangedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DiskCachePathChangedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DiskCachePathChangedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DiskCachePathChangedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DiskCachePathChangedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DiskCachePathChangedEventGlobalsTypeInternal DiskCachePathChangedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DiskCachePathChangedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DiskCachePathChangedEvent_globals_.GetClassData();
#else
  return DiskCachePathChangedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class DiskCachePathChangeFinishedEvent::_Internal {
 public:
};

constexpr DiskCachePathChangeFinishedEvent::ParseTableT_ DiskCachePathChangeFinishedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DiskCachePathChangeFinishedEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::DiskCachePathChangeFinishedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr DiskCachePathChangeFinishedEvent::DiskCachePathChangeFinishedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL DiskCachePathChangeFinishedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DiskCachePathChangeFinishedEvent(arena);
}
constexpr auto DiskCachePathChangeFinishedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(DiskCachePathChangeFinishedEvent), alignof(DiskCachePathChangeFinishedEvent));
}
constexpr auto DiskCachePathChangeFinishedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DiskCachePathChangeFinishedEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<DiskCachePathChangeFinishedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DiskCachePathChangeFinishedEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<DiskCachePathChangeFinishedEvent>(), &DiskCachePathChangeFinishedEvent::ByteSizeLong,
              &DiskCachePathChangeFinishedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DiskCachePathChangeFinishedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[52],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DiskCachePathChangeFinishedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DiskCachePathChangeFinishedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DiskCachePathChangeFinishedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DiskCachePathChangeFinishedEvent::InternalGenerateClassData_(
            _default, &DiskCachePathChangeFinishedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DiskCachePathChangeFinishedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DiskCachePathChangeFinishedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DiskCachePathChangeFinishedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DiskCachePathChangeFinishedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DiskCachePathChangeFinishedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DiskCachePathChangeFinishedEventGlobalsTypeInternal DiskCachePathChangeFinishedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DiskCachePathChangeFinishedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DiskCachePathChangeFinishedEvent_globals_.GetClassData();
#else
  return DiskCachePathChangeFinishedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class DiskCacheErrorEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<DiskCacheErrorEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(DiskCacheErrorEvent, _impl_._has_bits_);
};

constexpr DiskCacheErrorEvent::ParseTableT_ DiskCacheErrorEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DiskCacheErrorEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::DiskCacheErrorEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .grpc.DiskCacheErrorType type = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(DiskCacheErrorEvent, _impl_.type_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(DiskCacheErrorEvent, _impl_.type_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.DiskCacheErrorType type = 1;
      {PROTOBUF_FIELD_OFFSET(DiskCacheErrorEvent, _impl_.type_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr DiskCacheErrorEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        type_{static_cast< ::grpc::DiskCacheErrorType >(0)} {}

template <typename>
constexpr DiskCacheErrorEvent::DiskCacheErrorEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL DiskCacheErrorEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DiskCacheErrorEvent(arena);
}
constexpr auto DiskCacheErrorEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(DiskCacheErrorEvent), alignof(DiskCacheErrorEvent));
}
constexpr auto DiskCacheErrorEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DiskCacheErrorEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<DiskCacheErrorEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DiskCacheErrorEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<DiskCacheErrorEvent>(), &DiskCacheErrorEvent::ByteSizeLong,
              &DiskCacheErrorEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DiskCacheErrorEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[50],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DiskCacheErrorEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DiskCacheErrorEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DiskCacheErrorEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DiskCacheErrorEvent::InternalGenerateClassData_(
            _default, &DiskCacheErrorEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DiskCacheErrorEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DiskCacheErrorEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DiskCacheErrorEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DiskCacheErrorEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DiskCacheErrorEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DiskCacheErrorEventGlobalsTypeInternal DiskCacheErrorEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DiskCacheErrorEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DiskCacheErrorEvent_globals_.GetClassData();
#else
  return DiskCacheErrorEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ConfigureAppleMailRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<ConfigureAppleMailRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ConfigureAppleMailRequest, _impl_._has_bits_);
};

constexpr ConfigureAppleMailRequest::ParseTableT_ ConfigureAppleMailRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ConfigureAppleMailRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::ConfigureAppleMailRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string address = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(ConfigureAppleMailRequest, _impl_.address_)}},
      // string userID = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ConfigureAppleMailRequest, _impl_.userid_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string userID = 1;
      {PROTOBUF_FIELD_OFFSET(ConfigureAppleMailRequest, _impl_.userid_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string address = 2;
      {PROTOBUF_FIELD_OFFSET(ConfigureAppleMailRequest, _impl_.address_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\36\6\7\0\0\0\0\0"
      "grpc.ConfigureAppleMailRequest"
      "userID"
      "address"
    }},
  };
}


inline constexpr ConfigureAppleMailRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        userid_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        address_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr ConfigureAppleMailRequest::ConfigureAppleMailRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ConfigureAppleMailRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ConfigureAppleMailRequest(arena);
}
constexpr auto ConfigureAppleMailRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ConfigureAppleMailRequest), alignof(ConfigureAppleMailRequest));
}
constexpr auto ConfigureAppleMailRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ConfigureAppleMailRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ConfigureAppleMailRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ConfigureAppleMailRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ConfigureAppleMailRequest>(), &ConfigureAppleMailRequest::ByteSizeLong,
              &ConfigureAppleMailRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ConfigureAppleMailRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[11],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ConfigureAppleMailRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ConfigureAppleMailRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ConfigureAppleMailRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ConfigureAppleMailRequest::InternalGenerateClassData_(
            _default, &ConfigureAppleMailRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ConfigureAppleMailRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ConfigureAppleMailRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ConfigureAppleMailRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ConfigureAppleMailRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ConfigureAppleMailRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ConfigureAppleMailRequestGlobalsTypeInternal ConfigureAppleMailRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ConfigureAppleMailRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ConfigureAppleMailRequest_globals_.GetClassData();
#else
  return ConfigureAppleMailRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ChangeMailServerSettingsFinishedEvent::_Internal {
 public:
};

constexpr ChangeMailServerSettingsFinishedEvent::ParseTableT_ ChangeMailServerSettingsFinishedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ChangeMailServerSettingsFinishedEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::ChangeMailServerSettingsFinishedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr ChangeMailServerSettingsFinishedEvent::ChangeMailServerSettingsFinishedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL ChangeMailServerSettingsFinishedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ChangeMailServerSettingsFinishedEvent(arena);
}
constexpr auto ChangeMailServerSettingsFinishedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ChangeMailServerSettingsFinishedEvent), alignof(ChangeMailServerSettingsFinishedEvent));
}
constexpr auto ChangeMailServerSettingsFinishedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ChangeMailServerSettingsFinishedEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<ChangeMailServerSettingsFinishedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ChangeMailServerSettingsFinishedEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<ChangeMailServerSettingsFinishedEvent>(), &ChangeMailServerSettingsFinishedEvent::ByteSizeLong,
              &ChangeMailServerSettingsFinishedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ChangeMailServerSettingsFinishedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[56],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ChangeMailServerSettingsFinishedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ChangeMailServerSettingsFinishedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ChangeMailServerSettingsFinishedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ChangeMailServerSettingsFinishedEvent::InternalGenerateClassData_(
            _default, &ChangeMailServerSettingsFinishedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ChangeMailServerSettingsFinishedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ChangeMailServerSettingsFinishedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ChangeMailServerSettingsFinishedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ChangeMailServerSettingsFinishedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ChangeMailServerSettingsFinishedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ChangeMailServerSettingsFinishedEventGlobalsTypeInternal ChangeMailServerSettingsFinishedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ChangeMailServerSettingsFinishedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ChangeMailServerSettingsFinishedEvent_globals_.GetClassData();
#else
  return ChangeMailServerSettingsFinishedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ChangeKeychainFinishedEvent::_Internal {
 public:
};

constexpr ChangeKeychainFinishedEvent::ParseTableT_ ChangeKeychainFinishedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ChangeKeychainFinishedEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::ChangeKeychainFinishedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr ChangeKeychainFinishedEvent::ChangeKeychainFinishedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL ChangeKeychainFinishedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ChangeKeychainFinishedEvent(arena);
}
constexpr auto ChangeKeychainFinishedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ChangeKeychainFinishedEvent), alignof(ChangeKeychainFinishedEvent));
}
constexpr auto ChangeKeychainFinishedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ChangeKeychainFinishedEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<ChangeKeychainFinishedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ChangeKeychainFinishedEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<ChangeKeychainFinishedEvent>(), &ChangeKeychainFinishedEvent::ByteSizeLong,
              &ChangeKeychainFinishedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ChangeKeychainFinishedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[58],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ChangeKeychainFinishedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ChangeKeychainFinishedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ChangeKeychainFinishedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ChangeKeychainFinishedEvent::InternalGenerateClassData_(
            _default, &ChangeKeychainFinishedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ChangeKeychainFinishedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ChangeKeychainFinishedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ChangeKeychainFinishedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ChangeKeychainFinishedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ChangeKeychainFinishedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ChangeKeychainFinishedEventGlobalsTypeInternal ChangeKeychainFinishedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ChangeKeychainFinishedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ChangeKeychainFinishedEvent_globals_.GetClassData();
#else
  return ChangeKeychainFinishedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class CertificateInstallSuccessEvent::_Internal {
 public:
};

constexpr CertificateInstallSuccessEvent::ParseTableT_ CertificateInstallSuccessEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CertificateInstallSuccessEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::CertificateInstallSuccessEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr CertificateInstallSuccessEvent::CertificateInstallSuccessEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL CertificateInstallSuccessEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CertificateInstallSuccessEvent(arena);
}
constexpr auto CertificateInstallSuccessEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(CertificateInstallSuccessEvent), alignof(CertificateInstallSuccessEvent));
}
constexpr auto CertificateInstallSuccessEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CertificateInstallSuccessEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<CertificateInstallSuccessEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CertificateInstallSuccessEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<CertificateInstallSuccessEvent>(), &CertificateInstallSuccessEvent::ByteSizeLong,
              &CertificateInstallSuccessEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CertificateInstallSuccessEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[23],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CertificateInstallSuccessEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CertificateInstallSuccessEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CertificateInstallSuccessEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CertificateInstallSuccessEvent::InternalGenerateClassData_(
            _default, &CertificateInstallSuccessEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CertificateInstallSuccessEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CertificateInstallSuccessEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CertificateInstallSuccessEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CertificateInstallSuccessEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CertificateInstallSuccessEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CertificateInstallSuccessEventGlobalsTypeInternal CertificateInstallSuccessEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CertificateInstallSuccessEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CertificateInstallSuccessEvent_globals_.GetClassData();
#else
  return CertificateInstallSuccessEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class CertificateInstallFailedEvent::_Internal {
 public:
};

constexpr CertificateInstallFailedEvent::ParseTableT_ CertificateInstallFailedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CertificateInstallFailedEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::CertificateInstallFailedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr CertificateInstallFailedEvent::CertificateInstallFailedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL CertificateInstallFailedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CertificateInstallFailedEvent(arena);
}
constexpr auto CertificateInstallFailedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(CertificateInstallFailedEvent), alignof(CertificateInstallFailedEvent));
}
constexpr auto CertificateInstallFailedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CertificateInstallFailedEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<CertificateInstallFailedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CertificateInstallFailedEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<CertificateInstallFailedEvent>(), &CertificateInstallFailedEvent::ByteSizeLong,
              &CertificateInstallFailedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CertificateInstallFailedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[25],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CertificateInstallFailedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CertificateInstallFailedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CertificateInstallFailedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CertificateInstallFailedEvent::InternalGenerateClassData_(
            _default, &CertificateInstallFailedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CertificateInstallFailedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CertificateInstallFailedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CertificateInstallFailedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CertificateInstallFailedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CertificateInstallFailedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CertificateInstallFailedEventGlobalsTypeInternal CertificateInstallFailedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CertificateInstallFailedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CertificateInstallFailedEvent_globals_.GetClassData();
#else
  return CertificateInstallFailedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class CertificateInstallCanceledEvent::_Internal {
 public:
};

constexpr CertificateInstallCanceledEvent::ParseTableT_ CertificateInstallCanceledEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CertificateInstallCanceledEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::CertificateInstallCanceledEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr CertificateInstallCanceledEvent::CertificateInstallCanceledEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL CertificateInstallCanceledEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CertificateInstallCanceledEvent(arena);
}
constexpr auto CertificateInstallCanceledEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(CertificateInstallCanceledEvent), alignof(CertificateInstallCanceledEvent));
}
constexpr auto CertificateInstallCanceledEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CertificateInstallCanceledEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<CertificateInstallCanceledEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CertificateInstallCanceledEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<CertificateInstallCanceledEvent>(), &CertificateInstallCanceledEvent::ByteSizeLong,
              &CertificateInstallCanceledEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CertificateInstallCanceledEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[24],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CertificateInstallCanceledEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CertificateInstallCanceledEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CertificateInstallCanceledEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CertificateInstallCanceledEvent::InternalGenerateClassData_(
            _default, &CertificateInstallCanceledEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CertificateInstallCanceledEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CertificateInstallCanceledEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CertificateInstallCanceledEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CertificateInstallCanceledEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CertificateInstallCanceledEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CertificateInstallCanceledEventGlobalsTypeInternal CertificateInstallCanceledEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CertificateInstallCanceledEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CertificateInstallCanceledEvent_globals_.GetClassData();
#else
  return CertificateInstallCanceledEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class AvailableKeychainsResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<AvailableKeychainsResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(AvailableKeychainsResponse, _impl_._has_bits_);
};

constexpr AvailableKeychainsResponse::ParseTableT_ AvailableKeychainsResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AvailableKeychainsResponse, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::AvailableKeychainsResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated string keychains = 1;
      {::_pbi::TcParser::FastUR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(AvailableKeychainsResponse, _impl_.keychains_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated string keychains = 1;
      {PROTOBUF_FIELD_OFFSET(AvailableKeychainsResponse, _impl_.keychains_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
    }},
    // no aux_entries
    {{
      "\37\11\0\0\0\0\0\0"
      "grpc.AvailableKeychainsResponse"
      "keychains"
    }},
  };
}


inline constexpr AvailableKeychainsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        keychains_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::grpc::AvailableKeychainsResponse,
            PROTOBUF_FIELD_OFFSET(::grpc::AvailableKeychainsResponse, _impl_.keychains_)>()
         }
     {}

template <typename>
constexpr AvailableKeychainsResponse::AvailableKeychainsResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL AvailableKeychainsResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AvailableKeychainsResponse(arena);
}
constexpr auto AvailableKeychainsResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(AvailableKeychainsResponse), alignof(AvailableKeychainsResponse));
}
constexpr auto AvailableKeychainsResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &AvailableKeychainsResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AvailableKeychainsResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AvailableKeychainsResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<AvailableKeychainsResponse>(), &AvailableKeychainsResponse::ByteSizeLong,
              &AvailableKeychainsResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AvailableKeychainsResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[6],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AvailableKeychainsResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AvailableKeychainsResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AvailableKeychainsResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AvailableKeychainsResponse::InternalGenerateClassData_(
            _default, &AvailableKeychainsResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AvailableKeychainsResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AvailableKeychainsResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AvailableKeychainsResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AvailableKeychainsResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AvailableKeychainsResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AvailableKeychainsResponseGlobalsTypeInternal AvailableKeychainsResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AvailableKeychainsResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AvailableKeychainsResponse_globals_.GetClassData();
#else
  return AvailableKeychainsResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ApiCertIssueEvent::_Internal {
 public:
};

constexpr ApiCertIssueEvent::ParseTableT_ ApiCertIssueEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ApiCertIssueEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::ApiCertIssueEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr ApiCertIssueEvent::ApiCertIssueEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL ApiCertIssueEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ApiCertIssueEvent(arena);
}
constexpr auto ApiCertIssueEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ApiCertIssueEvent), alignof(ApiCertIssueEvent));
}
constexpr auto ApiCertIssueEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ApiCertIssueEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<ApiCertIssueEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ApiCertIssueEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<ApiCertIssueEvent>(), &ApiCertIssueEvent::ByteSizeLong,
              &ApiCertIssueEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ApiCertIssueEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[64],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ApiCertIssueEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ApiCertIssueEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ApiCertIssueEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ApiCertIssueEvent::InternalGenerateClassData_(
            _default, &ApiCertIssueEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ApiCertIssueEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ApiCertIssueEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ApiCertIssueEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ApiCertIssueEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ApiCertIssueEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ApiCertIssueEventGlobalsTypeInternal ApiCertIssueEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ApiCertIssueEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ApiCertIssueEvent_globals_.GetClassData();
#else
  return ApiCertIssueEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class AllUsersLoadedEvent::_Internal {
 public:
};

constexpr AllUsersLoadedEvent::ParseTableT_ AllUsersLoadedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AllUsersLoadedEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::AllUsersLoadedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr AllUsersLoadedEvent::AllUsersLoadedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL AllUsersLoadedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AllUsersLoadedEvent(arena);
}
constexpr auto AllUsersLoadedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(AllUsersLoadedEvent), alignof(AllUsersLoadedEvent));
}
constexpr auto AllUsersLoadedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &AllUsersLoadedEvent::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<AllUsersLoadedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AllUsersLoadedEvent::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<AllUsersLoadedEvent>(), &AllUsersLoadedEvent::ByteSizeLong,
              &AllUsersLoadedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AllUsersLoadedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[27],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AllUsersLoadedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AllUsersLoadedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AllUsersLoadedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AllUsersLoadedEvent::InternalGenerateClassData_(
            _default, &AllUsersLoadedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AllUsersLoadedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AllUsersLoadedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AllUsersLoadedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AllUsersLoadedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AllUsersLoadedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AllUsersLoadedEventGlobalsTypeInternal AllUsersLoadedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AllUsersLoadedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AllUsersLoadedEvent_globals_.GetClassData();
#else
  return AllUsersLoadedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class AddressChangedLogoutEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<AddressChangedLogoutEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(AddressChangedLogoutEvent, _impl_._has_bits_);
};

constexpr AddressChangedLogoutEvent::ParseTableT_ AddressChangedLogoutEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AddressChangedLogoutEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::AddressChangedLogoutEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string address = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(AddressChangedLogoutEvent, _impl_.address_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string address = 1;
      {PROTOBUF_FIELD_OFFSET(AddressChangedLogoutEvent, _impl_.address_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\36\7\0\0\0\0\0\0"
      "grpc.AddressChangedLogoutEvent"
      "address"
    }},
  };
}


inline constexpr AddressChangedLogoutEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        address_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr AddressChangedLogoutEvent::AddressChangedLogoutEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL AddressChangedLogoutEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AddressChangedLogoutEvent(arena);
}
constexpr auto AddressChangedLogoutEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(AddressChangedLogoutEvent), alignof(AddressChangedLogoutEvent));
}
constexpr auto AddressChangedLogoutEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &AddressChangedLogoutEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AddressChangedLogoutEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AddressChangedLogoutEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<AddressChangedLogoutEvent>(), &AddressChangedLogoutEvent::ByteSizeLong,
              &AddressChangedLogoutEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AddressChangedLogoutEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[63],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AddressChangedLogoutEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AddressChangedLogoutEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AddressChangedLogoutEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AddressChangedLogoutEvent::InternalGenerateClassData_(
            _default, &AddressChangedLogoutEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AddressChangedLogoutEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AddressChangedLogoutEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AddressChangedLogoutEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AddressChangedLogoutEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AddressChangedLogoutEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AddressChangedLogoutEventGlobalsTypeInternal AddressChangedLogoutEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AddressChangedLogoutEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AddressChangedLogoutEvent_globals_.GetClassData();
#else
  return AddressChangedLogoutEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class AddressChangedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<AddressChangedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(AddressChangedEvent, _impl_._has_bits_);
};

constexpr AddressChangedEvent::ParseTableT_ AddressChangedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AddressChangedEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::AddressChangedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string address = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(AddressChangedEvent, _impl_.address_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string address = 1;
      {PROTOBUF_FIELD_OFFSET(AddressChangedEvent, _impl_.address_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\30\7\0\0\0\0\0\0"
      "grpc.AddressChangedEvent"
      "address"
    }},
  };
}


inline constexpr AddressChangedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        address_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr AddressChangedEvent::AddressChangedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL AddressChangedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AddressChangedEvent(arena);
}
constexpr auto AddressChangedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(AddressChangedEvent), alignof(AddressChangedEvent));
}
constexpr auto AddressChangedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &AddressChangedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AddressChangedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AddressChangedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<AddressChangedEvent>(), &AddressChangedEvent::ByteSizeLong,
              &AddressChangedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AddressChangedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[62],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AddressChangedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AddressChangedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AddressChangedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AddressChangedEvent::InternalGenerateClassData_(
            _default, &AddressChangedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AddressChangedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AddressChangedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AddressChangedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AddressChangedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AddressChangedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AddressChangedEventGlobalsTypeInternal AddressChangedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AddressChangedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AddressChangedEvent_globals_.GetClassData();
#else
  return AddressChangedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class AddLogEntryRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<AddLogEntryRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(AddLogEntryRequest, _impl_._has_bits_);
};

constexpr AddLogEntryRequest::ParseTableT_ AddLogEntryRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AddLogEntryRequest, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::AddLogEntryRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .grpc.LogLevel level = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(AddLogEntryRequest, _impl_.level_), 2>(),
       {8, 2, 0,
        PROTOBUF_FIELD_OFFSET(AddLogEntryRequest, _impl_.level_)}},
      // string package = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(AddLogEntryRequest, _impl_.package_)}},
      // string message = 3;
      {::_pbi::TcParser::FastUS1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(AddLogEntryRequest, _impl_.message_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.LogLevel level = 1;
      {PROTOBUF_FIELD_OFFSET(AddLogEntryRequest, _impl_.level_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // string package = 2;
      {PROTOBUF_FIELD_OFFSET(AddLogEntryRequest, _impl_.package_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string message = 3;
      {PROTOBUF_FIELD_OFFSET(AddLogEntryRequest, _impl_.message_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\27\0\7\7\0\0\0\0"
      "grpc.AddLogEntryRequest"
      "package"
      "message"
    }},
  };
}


inline constexpr AddLogEntryRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        package_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        message_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        level_{static_cast< ::grpc::LogLevel >(0)} {}

template <typename>
constexpr AddLogEntryRequest::AddLogEntryRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL AddLogEntryRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AddLogEntryRequest(arena);
}
constexpr auto AddLogEntryRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(AddLogEntryRequest), alignof(AddLogEntryRequest));
}
constexpr auto AddLogEntryRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &AddLogEntryRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AddLogEntryRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AddLogEntryRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<AddLogEntryRequest>(), &AddLogEntryRequest::ByteSizeLong,
              &AddLogEntryRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AddLogEntryRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AddLogEntryRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AddLogEntryRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AddLogEntryRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AddLogEntryRequest::InternalGenerateClassData_(
            _default, &AddLogEntryRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AddLogEntryRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AddLogEntryRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AddLogEntryRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AddLogEntryRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AddLogEntryRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AddLogEntryRequestGlobalsTypeInternal AddLogEntryRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AddLogEntryRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AddLogEntryRequest_globals_.GetClassData();
#else
  return AddLogEntryRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UserListResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<UserListResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(UserListResponse, _impl_._has_bits_);
};

constexpr UserListResponse::ParseTableT_ UserListResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UserListResponse, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UserListResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated .grpc.User users = 1;
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(UserListResponse, _impl_.users_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .grpc.User users = 1;
      {PROTOBUF_FIELD_OFFSET(UserListResponse, _impl_.users_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::User>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::User_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr UserListResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        users_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::grpc::UserListResponse,
            PROTOBUF_FIELD_OFFSET(::grpc::UserListResponse, _impl_.users_)>()
         }
     {}

template <typename>
constexpr UserListResponse::UserListResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UserListResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UserListResponse(arena);
}
constexpr auto UserListResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(UserListResponse), alignof(UserListResponse));
}
constexpr auto UserListResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UserListResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UserListResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UserListResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UserListResponse>(), &UserListResponse::ByteSizeLong,
              &UserListResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UserListResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[10],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UserListResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UserListResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UserListResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UserListResponse::InternalGenerateClassData_(
            _default, &UserListResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UserListResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UserListResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UserListResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UserListResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UserListResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UserListResponseGlobalsTypeInternal UserListResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UserListResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UserListResponse_globals_.GetClassData();
#else
  return UserListResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UserEvent::_Internal {
 public:
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::grpc::UserEvent, _impl_._oneof_case_);
};

constexpr UserEvent::ParseTableT_ UserEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UserEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      9, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294966784,  // skipmap
      offsetof(ParseTableT_, field_entries),
      9,  // num_field_entries
      9,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UserEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.ToggleSplitModeFinishedEvent toggleSplitModeFinished = 1;
      {PROTOBUF_FIELD_OFFSET(UserEvent, _impl_.event_.togglesplitmodefinished_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.UserDisconnectedEvent userDisconnected = 2;
      {PROTOBUF_FIELD_OFFSET(UserEvent, _impl_.event_.userdisconnected_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.UserChangedEvent userChanged = 3;
      {PROTOBUF_FIELD_OFFSET(UserEvent, _impl_.event_.userchanged_), _Internal::kOneofCaseOffset + 0, 2, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.UserBadEvent userBadEvent = 4;
      {PROTOBUF_FIELD_OFFSET(UserEvent, _impl_.event_.userbadevent_), _Internal::kOneofCaseOffset + 0, 3, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.UsedBytesChangedEvent usedBytesChangedEvent = 5;
      {PROTOBUF_FIELD_OFFSET(UserEvent, _impl_.event_.usedbyteschangedevent_), _Internal::kOneofCaseOffset + 0, 4, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.ImapLoginFailedEvent imapLoginFailedEvent = 6;
      {PROTOBUF_FIELD_OFFSET(UserEvent, _impl_.event_.imaploginfailedevent_), _Internal::kOneofCaseOffset + 0, 5, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.SyncStartedEvent syncStartedEvent = 7;
      {PROTOBUF_FIELD_OFFSET(UserEvent, _impl_.event_.syncstartedevent_), _Internal::kOneofCaseOffset + 0, 6, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.SyncFinishedEvent syncFinishedEvent = 8;
      {PROTOBUF_FIELD_OFFSET(UserEvent, _impl_.event_.syncfinishedevent_), _Internal::kOneofCaseOffset + 0, 7, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.SyncProgressEvent syncProgressEvent = 9;
      {PROTOBUF_FIELD_OFFSET(UserEvent, _impl_.event_.syncprogressevent_), _Internal::kOneofCaseOffset + 0, 8, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::ToggleSplitModeFinishedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::ToggleSplitModeFinishedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::UserDisconnectedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::UserDisconnectedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::UserChangedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::UserChangedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::UserBadEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::UserBadEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::UsedBytesChangedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::UsedBytesChangedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::ImapLoginFailedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::ImapLoginFailedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::SyncStartedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::SyncStartedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::SyncFinishedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::SyncFinishedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::SyncProgressEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::SyncProgressEvent_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr UserEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

template <typename>
constexpr UserEvent::UserEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UserEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UserEvent(arena);
}
constexpr auto UserEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(UserEvent), alignof(UserEvent));
}
constexpr auto UserEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UserEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UserEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UserEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UserEvent>(), &UserEvent::ByteSizeLong,
              &UserEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UserEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[65],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UserEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UserEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UserEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UserEvent::InternalGenerateClassData_(
            _default, &UserEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UserEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UserEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UserEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UserEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UserEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UserEventGlobalsTypeInternal UserEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UserEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UserEvent_globals_.GetClassData();
#else
  return UserEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UpdateEvent::_Internal {
 public:
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::grpc::UpdateEvent, _impl_._oneof_case_);
};

constexpr UpdateEvent::ParseTableT_ UpdateEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UpdateEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      8, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967040,  // skipmap
      offsetof(ParseTableT_, field_entries),
      8,  // num_field_entries
      8,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::UpdateEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.UpdateErrorEvent error = 1;
      {PROTOBUF_FIELD_OFFSET(UpdateEvent, _impl_.event_.error_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.UpdateManualReadyEvent manualReady = 2;
      {PROTOBUF_FIELD_OFFSET(UpdateEvent, _impl_.event_.manualready_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.UpdateManualRestartNeededEvent manualRestartNeeded = 3;
      {PROTOBUF_FIELD_OFFSET(UpdateEvent, _impl_.event_.manualrestartneeded_), _Internal::kOneofCaseOffset + 0, 2, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.UpdateForceEvent force = 4;
      {PROTOBUF_FIELD_OFFSET(UpdateEvent, _impl_.event_.force_), _Internal::kOneofCaseOffset + 0, 3, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.UpdateSilentRestartNeeded silentRestartNeeded = 5;
      {PROTOBUF_FIELD_OFFSET(UpdateEvent, _impl_.event_.silentrestartneeded_), _Internal::kOneofCaseOffset + 0, 4, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.UpdateIsLatestVersion isLatestVersion = 6;
      {PROTOBUF_FIELD_OFFSET(UpdateEvent, _impl_.event_.islatestversion_), _Internal::kOneofCaseOffset + 0, 5, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.UpdateCheckFinished checkFinished = 7;
      {PROTOBUF_FIELD_OFFSET(UpdateEvent, _impl_.event_.checkfinished_), _Internal::kOneofCaseOffset + 0, 6, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.UpdateVersionChanged versionChanged = 8;
      {PROTOBUF_FIELD_OFFSET(UpdateEvent, _impl_.event_.versionchanged_), _Internal::kOneofCaseOffset + 0, 7, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::UpdateErrorEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::UpdateErrorEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::UpdateManualReadyEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::UpdateManualReadyEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::UpdateManualRestartNeededEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::UpdateManualRestartNeededEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::UpdateForceEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::UpdateForceEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::UpdateSilentRestartNeeded>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::UpdateSilentRestartNeeded_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::UpdateIsLatestVersion>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::UpdateIsLatestVersion_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::UpdateCheckFinished>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::UpdateCheckFinished_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::UpdateVersionChanged>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::UpdateVersionChanged_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr UpdateEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

template <typename>
constexpr UpdateEvent::UpdateEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UpdateEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UpdateEvent(arena);
}
constexpr auto UpdateEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(UpdateEvent), alignof(UpdateEvent));
}
constexpr auto UpdateEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UpdateEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UpdateEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UpdateEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UpdateEvent>(), &UpdateEvent::ByteSizeLong,
              &UpdateEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UpdateEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[40],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UpdateEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UpdateEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UpdateEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UpdateEvent::InternalGenerateClassData_(
            _default, &UpdateEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UpdateEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UpdateEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UpdateEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UpdateEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UpdateEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UpdateEventGlobalsTypeInternal UpdateEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UpdateEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UpdateEvent_globals_.GetClassData();
#else
  return UpdateEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class MailServerSettingsChangedEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<MailServerSettingsChangedEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(MailServerSettingsChangedEvent, _impl_._has_bits_);
};

constexpr MailServerSettingsChangedEvent::ParseTableT_ MailServerSettingsChangedEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(MailServerSettingsChangedEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::MailServerSettingsChangedEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .grpc.ImapSmtpSettings settings = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(MailServerSettingsChangedEvent, _impl_.settings_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.ImapSmtpSettings settings = 1;
      {PROTOBUF_FIELD_OFFSET(MailServerSettingsChangedEvent, _impl_.settings_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::ImapSmtpSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::ImapSmtpSettings_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr MailServerSettingsChangedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        settings_{nullptr} {}

template <typename>
constexpr MailServerSettingsChangedEvent::MailServerSettingsChangedEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL MailServerSettingsChangedEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) MailServerSettingsChangedEvent(arena);
}
constexpr auto MailServerSettingsChangedEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(MailServerSettingsChangedEvent), alignof(MailServerSettingsChangedEvent));
}
constexpr auto MailServerSettingsChangedEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &MailServerSettingsChangedEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<MailServerSettingsChangedEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &MailServerSettingsChangedEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<MailServerSettingsChangedEvent>(), &MailServerSettingsChangedEvent::ByteSizeLong,
              &MailServerSettingsChangedEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(MailServerSettingsChangedEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[55],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct MailServerSettingsChangedEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr MailServerSettingsChangedEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 MailServerSettingsChangedEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(MailServerSettingsChangedEvent::InternalGenerateClassData_(
            _default, &MailServerSettingsChangedEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<MailServerSettingsChangedEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~MailServerSettingsChangedEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) MailServerSettingsChangedEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<MailServerSettingsChangedEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(MailServerSettingsChangedEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST MailServerSettingsChangedEventGlobalsTypeInternal MailServerSettingsChangedEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* MailServerSettingsChangedEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return MailServerSettingsChangedEvent_globals_.GetClassData();
#else
  return MailServerSettingsChangedEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class MailEvent::_Internal {
 public:
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::grpc::MailEvent, _impl_._oneof_case_);
};

constexpr MailEvent::ParseTableT_ MailEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(MailEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      3, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      3,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::MailEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.AddressChangedEvent addressChanged = 1;
      {PROTOBUF_FIELD_OFFSET(MailEvent, _impl_.event_.addresschanged_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.AddressChangedLogoutEvent addressChangedLogout = 2;
      {PROTOBUF_FIELD_OFFSET(MailEvent, _impl_.event_.addresschangedlogout_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.ApiCertIssueEvent apiCertIssue = 3;
      {PROTOBUF_FIELD_OFFSET(MailEvent, _impl_.event_.apicertissue_), _Internal::kOneofCaseOffset + 0, 2, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::AddressChangedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::AddressChangedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::AddressChangedLogoutEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::AddressChangedLogoutEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::ApiCertIssueEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::ApiCertIssueEvent_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr MailEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

template <typename>
constexpr MailEvent::MailEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL MailEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) MailEvent(arena);
}
constexpr auto MailEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(MailEvent), alignof(MailEvent));
}
constexpr auto MailEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &MailEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<MailEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &MailEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<MailEvent>(), &MailEvent::ByteSizeLong,
              &MailEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(MailEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[61],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct MailEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr MailEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 MailEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(MailEvent::InternalGenerateClassData_(
            _default, &MailEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<MailEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~MailEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) MailEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<MailEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(MailEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST MailEventGlobalsTypeInternal MailEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* MailEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return MailEvent_globals_.GetClassData();
#else
  return MailEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class LoginEvent::_Internal {
 public:
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::grpc::LoginEvent, _impl_._oneof_case_);
};

constexpr LoginEvent::ParseTableT_ LoginEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(LoginEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      11, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294965248,  // skipmap
      offsetof(ParseTableT_, field_entries),
      11,  // num_field_entries
      11,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::LoginEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.LoginErrorEvent error = 1;
      {PROTOBUF_FIELD_OFFSET(LoginEvent, _impl_.event_.error_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.LoginTfaRequestedEvent tfaRequested = 2;
      {PROTOBUF_FIELD_OFFSET(LoginEvent, _impl_.event_.tfarequested_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.LoginTwoPasswordsRequestedEvent twoPasswordRequested = 3;
      {PROTOBUF_FIELD_OFFSET(LoginEvent, _impl_.event_.twopasswordrequested_), _Internal::kOneofCaseOffset + 0, 2, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.LoginFinishedEvent finished = 4;
      {PROTOBUF_FIELD_OFFSET(LoginEvent, _impl_.event_.finished_), _Internal::kOneofCaseOffset + 0, 3, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.LoginFinishedEvent alreadyLoggedIn = 5;
      {PROTOBUF_FIELD_OFFSET(LoginEvent, _impl_.event_.alreadyloggedin_), _Internal::kOneofCaseOffset + 0, 4, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.LoginHvRequestedEvent hvRequested = 6;
      {PROTOBUF_FIELD_OFFSET(LoginEvent, _impl_.event_.hvrequested_), _Internal::kOneofCaseOffset + 0, 5, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.LoginFidoRequestedEvent fidoRequested = 7;
      {PROTOBUF_FIELD_OFFSET(LoginEvent, _impl_.event_.fidorequested_), _Internal::kOneofCaseOffset + 0, 6, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.LoginTfaOrFidoRequestedEvent tfaOrFidoRequested = 8;
      {PROTOBUF_FIELD_OFFSET(LoginEvent, _impl_.event_.tfaorfidorequested_), _Internal::kOneofCaseOffset + 0, 7, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.LoginFidoTouchEvent loginFidoTouchRequested = 9;
      {PROTOBUF_FIELD_OFFSET(LoginEvent, _impl_.event_.loginfidotouchrequested_), _Internal::kOneofCaseOffset + 0, 8, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.LoginFidoTouchEvent loginFidoTouchCompleted = 10;
      {PROTOBUF_FIELD_OFFSET(LoginEvent, _impl_.event_.loginfidotouchcompleted_), _Internal::kOneofCaseOffset + 0, 9, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.LoginFidoPinRequired loginFidoPinRequired = 11;
      {PROTOBUF_FIELD_OFFSET(LoginEvent, _impl_.event_.loginfidopinrequired_), _Internal::kOneofCaseOffset + 0, 10, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::LoginErrorEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::LoginErrorEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::LoginTfaRequestedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::LoginTfaRequestedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::LoginTwoPasswordsRequestedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::LoginTwoPasswordsRequestedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::LoginFinishedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::LoginFinishedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::LoginFinishedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::LoginFinishedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::LoginHvRequestedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::LoginHvRequestedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::LoginFidoRequestedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::LoginFidoRequestedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::LoginTfaOrFidoRequestedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::LoginTfaOrFidoRequestedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::LoginFidoTouchEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::LoginFidoTouchEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::LoginFidoTouchEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::LoginFidoTouchEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::LoginFidoPinRequired>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::LoginFidoPinRequired_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr LoginEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

template <typename>
constexpr LoginEvent::LoginEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL LoginEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) LoginEvent(arena);
}
constexpr auto LoginEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(LoginEvent), alignof(LoginEvent));
}
constexpr auto LoginEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &LoginEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<LoginEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &LoginEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<LoginEvent>(), &LoginEvent::ByteSizeLong,
              &LoginEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(LoginEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[30],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct LoginEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr LoginEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 LoginEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(LoginEvent::InternalGenerateClassData_(
            _default, &LoginEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<LoginEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~LoginEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) LoginEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<LoginEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(LoginEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST LoginEventGlobalsTypeInternal LoginEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* LoginEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return LoginEvent_globals_.GetClassData();
#else
  return LoginEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class KnowledgeBaseSuggestionsEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<KnowledgeBaseSuggestionsEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(KnowledgeBaseSuggestionsEvent, _impl_._has_bits_);
};

constexpr KnowledgeBaseSuggestionsEvent::ParseTableT_ KnowledgeBaseSuggestionsEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(KnowledgeBaseSuggestionsEvent, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::KnowledgeBaseSuggestionsEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated .grpc.KnowledgeBaseSuggestion suggestions = 1;
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(KnowledgeBaseSuggestionsEvent, _impl_.suggestions_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .grpc.KnowledgeBaseSuggestion suggestions = 1;
      {PROTOBUF_FIELD_OFFSET(KnowledgeBaseSuggestionsEvent, _impl_.suggestions_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::KnowledgeBaseSuggestion>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::KnowledgeBaseSuggestion_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr KnowledgeBaseSuggestionsEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        suggestions_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::grpc::KnowledgeBaseSuggestionsEvent,
            PROTOBUF_FIELD_OFFSET(::grpc::KnowledgeBaseSuggestionsEvent, _impl_.suggestions_)>()
         }
     {}

template <typename>
constexpr KnowledgeBaseSuggestionsEvent::KnowledgeBaseSuggestionsEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL KnowledgeBaseSuggestionsEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) KnowledgeBaseSuggestionsEvent(arena);
}
constexpr auto KnowledgeBaseSuggestionsEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(KnowledgeBaseSuggestionsEvent), alignof(KnowledgeBaseSuggestionsEvent));
}
constexpr auto KnowledgeBaseSuggestionsEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &KnowledgeBaseSuggestionsEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<KnowledgeBaseSuggestionsEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &KnowledgeBaseSuggestionsEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<KnowledgeBaseSuggestionsEvent>(), &KnowledgeBaseSuggestionsEvent::ByteSizeLong,
              &KnowledgeBaseSuggestionsEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(KnowledgeBaseSuggestionsEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[29],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct KnowledgeBaseSuggestionsEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr KnowledgeBaseSuggestionsEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 KnowledgeBaseSuggestionsEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(KnowledgeBaseSuggestionsEvent::InternalGenerateClassData_(
            _default, &KnowledgeBaseSuggestionsEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<KnowledgeBaseSuggestionsEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~KnowledgeBaseSuggestionsEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) KnowledgeBaseSuggestionsEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<KnowledgeBaseSuggestionsEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(KnowledgeBaseSuggestionsEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST KnowledgeBaseSuggestionsEventGlobalsTypeInternal KnowledgeBaseSuggestionsEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* KnowledgeBaseSuggestionsEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return KnowledgeBaseSuggestionsEvent_globals_.GetClassData();
#else
  return KnowledgeBaseSuggestionsEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class KeychainEvent::_Internal {
 public:
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::grpc::KeychainEvent, _impl_._oneof_case_);
};

constexpr KeychainEvent::ParseTableT_ KeychainEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(KeychainEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      3, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      3,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::KeychainEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.ChangeKeychainFinishedEvent changeKeychainFinished = 1;
      {PROTOBUF_FIELD_OFFSET(KeychainEvent, _impl_.event_.changekeychainfinished_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.HasNoKeychainEvent hasNoKeychain = 2;
      {PROTOBUF_FIELD_OFFSET(KeychainEvent, _impl_.event_.hasnokeychain_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.RebuildKeychainEvent rebuildKeychain = 3;
      {PROTOBUF_FIELD_OFFSET(KeychainEvent, _impl_.event_.rebuildkeychain_), _Internal::kOneofCaseOffset + 0, 2, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::ChangeKeychainFinishedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::ChangeKeychainFinishedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::HasNoKeychainEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::HasNoKeychainEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::RebuildKeychainEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::RebuildKeychainEvent_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr KeychainEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

template <typename>
constexpr KeychainEvent::KeychainEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL KeychainEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) KeychainEvent(arena);
}
constexpr auto KeychainEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(KeychainEvent), alignof(KeychainEvent));
}
constexpr auto KeychainEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &KeychainEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<KeychainEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &KeychainEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<KeychainEvent>(), &KeychainEvent::ByteSizeLong,
              &KeychainEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(KeychainEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[57],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct KeychainEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr KeychainEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 KeychainEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(KeychainEvent::InternalGenerateClassData_(
            _default, &KeychainEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<KeychainEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~KeychainEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) KeychainEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<KeychainEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(KeychainEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST KeychainEventGlobalsTypeInternal KeychainEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* KeychainEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return KeychainEvent_globals_.GetClassData();
#else
  return KeychainEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class DiskCacheEvent::_Internal {
 public:
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::grpc::DiskCacheEvent, _impl_._oneof_case_);
};

constexpr DiskCacheEvent::ParseTableT_ DiskCacheEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DiskCacheEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      3, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      3,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::DiskCacheEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.DiskCacheErrorEvent error = 1;
      {PROTOBUF_FIELD_OFFSET(DiskCacheEvent, _impl_.event_.error_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.DiskCachePathChangedEvent pathChanged = 2;
      {PROTOBUF_FIELD_OFFSET(DiskCacheEvent, _impl_.event_.pathchanged_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.DiskCachePathChangeFinishedEvent pathChangeFinished = 3;
      {PROTOBUF_FIELD_OFFSET(DiskCacheEvent, _impl_.event_.pathchangefinished_), _Internal::kOneofCaseOffset + 0, 2, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::DiskCacheErrorEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::DiskCacheErrorEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::DiskCachePathChangedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::DiskCachePathChangedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::DiskCachePathChangeFinishedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::DiskCachePathChangeFinishedEvent_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr DiskCacheEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

template <typename>
constexpr DiskCacheEvent::DiskCacheEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL DiskCacheEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DiskCacheEvent(arena);
}
constexpr auto DiskCacheEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(DiskCacheEvent), alignof(DiskCacheEvent));
}
constexpr auto DiskCacheEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DiskCacheEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<DiskCacheEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DiskCacheEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<DiskCacheEvent>(), &DiskCacheEvent::ByteSizeLong,
              &DiskCacheEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DiskCacheEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[49],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DiskCacheEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DiskCacheEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DiskCacheEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DiskCacheEvent::InternalGenerateClassData_(
            _default, &DiskCacheEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DiskCacheEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DiskCacheEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DiskCacheEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DiskCacheEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DiskCacheEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DiskCacheEventGlobalsTypeInternal DiskCacheEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DiskCacheEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DiskCacheEvent_globals_.GetClassData();
#else
  return DiskCacheEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class MailServerSettingsEvent::_Internal {
 public:
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::grpc::MailServerSettingsEvent, _impl_._oneof_case_);
};

constexpr MailServerSettingsEvent::ParseTableT_ MailServerSettingsEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(MailServerSettingsEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      3, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      3,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::MailServerSettingsEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.MailServerSettingsErrorEvent error = 1;
      {PROTOBUF_FIELD_OFFSET(MailServerSettingsEvent, _impl_.event_.error_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.MailServerSettingsChangedEvent mailServerSettingsChanged = 2;
      {PROTOBUF_FIELD_OFFSET(MailServerSettingsEvent, _impl_.event_.mailserversettingschanged_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.ChangeMailServerSettingsFinishedEvent changeMailServerSettingsFinished = 3;
      {PROTOBUF_FIELD_OFFSET(MailServerSettingsEvent, _impl_.event_.changemailserversettingsfinished_), _Internal::kOneofCaseOffset + 0, 2, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::MailServerSettingsErrorEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::MailServerSettingsErrorEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::MailServerSettingsChangedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::MailServerSettingsChangedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::ChangeMailServerSettingsFinishedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::ChangeMailServerSettingsFinishedEvent_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr MailServerSettingsEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

template <typename>
constexpr MailServerSettingsEvent::MailServerSettingsEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL MailServerSettingsEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) MailServerSettingsEvent(arena);
}
constexpr auto MailServerSettingsEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(MailServerSettingsEvent), alignof(MailServerSettingsEvent));
}
constexpr auto MailServerSettingsEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &MailServerSettingsEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<MailServerSettingsEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &MailServerSettingsEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<MailServerSettingsEvent>(), &MailServerSettingsEvent::ByteSizeLong,
              &MailServerSettingsEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(MailServerSettingsEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[53],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct MailServerSettingsEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr MailServerSettingsEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 MailServerSettingsEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(MailServerSettingsEvent::InternalGenerateClassData_(
            _default, &MailServerSettingsEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<MailServerSettingsEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~MailServerSettingsEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) MailServerSettingsEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<MailServerSettingsEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(MailServerSettingsEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST MailServerSettingsEventGlobalsTypeInternal MailServerSettingsEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* MailServerSettingsEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return MailServerSettingsEvent_globals_.GetClassData();
#else
  return MailServerSettingsEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class AppEvent::_Internal {
 public:
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_._oneof_case_);
};

constexpr AppEvent::ParseTableT_ AppEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AppEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      15, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294934528,  // skipmap
      offsetof(ParseTableT_, field_entries),
      15,  // num_field_entries
      15,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::AppEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.InternetStatusEvent internetStatus = 1;
      {PROTOBUF_FIELD_OFFSET(AppEvent, _impl_.event_.internetstatus_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.ToggleAutostartFinishedEvent toggleAutostartFinished = 2;
      {PROTOBUF_FIELD_OFFSET(AppEvent, _impl_.event_.toggleautostartfinished_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.ResetFinishedEvent resetFinished = 3;
      {PROTOBUF_FIELD_OFFSET(AppEvent, _impl_.event_.resetfinished_), _Internal::kOneofCaseOffset + 0, 2, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.ReportBugFinishedEvent reportBugFinished = 4;
      {PROTOBUF_FIELD_OFFSET(AppEvent, _impl_.event_.reportbugfinished_), _Internal::kOneofCaseOffset + 0, 3, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.ReportBugSuccessEvent reportBugSuccess = 5;
      {PROTOBUF_FIELD_OFFSET(AppEvent, _impl_.event_.reportbugsuccess_), _Internal::kOneofCaseOffset + 0, 4, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.ReportBugErrorEvent reportBugError = 6;
      {PROTOBUF_FIELD_OFFSET(AppEvent, _impl_.event_.reportbugerror_), _Internal::kOneofCaseOffset + 0, 5, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.ShowMainWindowEvent showMainWindow = 7;
      {PROTOBUF_FIELD_OFFSET(AppEvent, _impl_.event_.showmainwindow_), _Internal::kOneofCaseOffset + 0, 6, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.ReportBugFallbackEvent reportBugFallback = 8;
      {PROTOBUF_FIELD_OFFSET(AppEvent, _impl_.event_.reportbugfallback_), _Internal::kOneofCaseOffset + 0, 7, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.CertificateInstallSuccessEvent certificateInstallSuccess = 9;
      {PROTOBUF_FIELD_OFFSET(AppEvent, _impl_.event_.certificateinstallsuccess_), _Internal::kOneofCaseOffset + 0, 8, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.CertificateInstallCanceledEvent certificateInstallCanceled = 10;
      {PROTOBUF_FIELD_OFFSET(AppEvent, _impl_.event_.certificateinstallcanceled_), _Internal::kOneofCaseOffset + 0, 9, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.CertificateInstallFailedEvent certificateInstallFailed = 11;
      {PROTOBUF_FIELD_OFFSET(AppEvent, _impl_.event_.certificateinstallfailed_), _Internal::kOneofCaseOffset + 0, 10, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.KnowledgeBaseSuggestionsEvent knowledgeBaseSuggestions = 12;
      {PROTOBUF_FIELD_OFFSET(AppEvent, _impl_.event_.knowledgebasesuggestions_), _Internal::kOneofCaseOffset + 0, 11, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.RepairStartedEvent repairStarted = 13;
      {PROTOBUF_FIELD_OFFSET(AppEvent, _impl_.event_.repairstarted_), _Internal::kOneofCaseOffset + 0, 12, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.AllUsersLoadedEvent allUsersLoaded = 14;
      {PROTOBUF_FIELD_OFFSET(AppEvent, _impl_.event_.allusersloaded_), _Internal::kOneofCaseOffset + 0, 13, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.UserNotificationEvent userNotification = 15;
      {PROTOBUF_FIELD_OFFSET(AppEvent, _impl_.event_.usernotification_), _Internal::kOneofCaseOffset + 0, 14, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::InternetStatusEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::InternetStatusEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::ToggleAutostartFinishedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::ToggleAutostartFinishedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::ResetFinishedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::ResetFinishedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::ReportBugFinishedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::ReportBugFinishedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::ReportBugSuccessEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::ReportBugSuccessEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::ReportBugErrorEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::ReportBugErrorEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::ShowMainWindowEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::ShowMainWindowEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::ReportBugFallbackEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::ReportBugFallbackEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::CertificateInstallSuccessEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::CertificateInstallSuccessEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::CertificateInstallCanceledEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::CertificateInstallCanceledEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::CertificateInstallFailedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::CertificateInstallFailedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::KnowledgeBaseSuggestionsEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::KnowledgeBaseSuggestionsEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::RepairStartedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::RepairStartedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::AllUsersLoadedEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::AllUsersLoadedEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::UserNotificationEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::UserNotificationEvent_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr AppEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

template <typename>
constexpr AppEvent::AppEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL AppEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AppEvent(arena);
}
constexpr auto AppEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(AppEvent), alignof(AppEvent));
}
constexpr auto AppEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &AppEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AppEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AppEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<AppEvent>(), &AppEvent::ByteSizeLong,
              &AppEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AppEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[14],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AppEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AppEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AppEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AppEvent::InternalGenerateClassData_(
            _default, &AppEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AppEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AppEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AppEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AppEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AppEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AppEventGlobalsTypeInternal AppEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AppEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AppEvent_globals_.GetClassData();
#else
  return AppEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class StreamEvent::_Internal {
 public:
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::grpc::StreamEvent, _impl_._oneof_case_);
};

constexpr StreamEvent::ParseTableT_ StreamEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(StreamEvent,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      9, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294966784,  // skipmap
      offsetof(ParseTableT_, field_entries),
      9,  // num_field_entries
      9,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::grpc::StreamEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .grpc.AppEvent app = 1;
      {PROTOBUF_FIELD_OFFSET(StreamEvent, _impl_.event_.app_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.LoginEvent login = 2;
      {PROTOBUF_FIELD_OFFSET(StreamEvent, _impl_.event_.login_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.UpdateEvent update = 3;
      {PROTOBUF_FIELD_OFFSET(StreamEvent, _impl_.event_.update_), _Internal::kOneofCaseOffset + 0, 2, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.DiskCacheEvent cache = 4;
      {PROTOBUF_FIELD_OFFSET(StreamEvent, _impl_.event_.cache_), _Internal::kOneofCaseOffset + 0, 3, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.MailServerSettingsEvent mailServerSettings = 5;
      {PROTOBUF_FIELD_OFFSET(StreamEvent, _impl_.event_.mailserversettings_), _Internal::kOneofCaseOffset + 0, 4, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.KeychainEvent keychain = 6;
      {PROTOBUF_FIELD_OFFSET(StreamEvent, _impl_.event_.keychain_), _Internal::kOneofCaseOffset + 0, 5, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.MailEvent mail = 7;
      {PROTOBUF_FIELD_OFFSET(StreamEvent, _impl_.event_.mail_), _Internal::kOneofCaseOffset + 0, 6, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.UserEvent user = 8;
      {PROTOBUF_FIELD_OFFSET(StreamEvent, _impl_.event_.user_), _Internal::kOneofCaseOffset + 0, 7, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .grpc.GenericErrorEvent genericError = 9;
      {PROTOBUF_FIELD_OFFSET(StreamEvent, _impl_.event_.genericerror_), _Internal::kOneofCaseOffset + 0, 8, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::AppEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::AppEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::LoginEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::LoginEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::UpdateEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::UpdateEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::DiskCacheEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::DiskCacheEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::MailServerSettingsEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::MailServerSettingsEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::KeychainEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::KeychainEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::MailEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::MailEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::UserEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::UserEvent_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::grpc::GenericErrorEvent>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::grpc::GenericErrorEvent_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr StreamEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

template <typename>
constexpr StreamEvent::StreamEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL StreamEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) StreamEvent(arena);
}
constexpr auto StreamEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(StreamEvent), alignof(StreamEvent));
}
constexpr auto StreamEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &StreamEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<StreamEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &StreamEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<StreamEvent>(), &StreamEvent::ByteSizeLong,
              &StreamEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(StreamEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[13],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_bridge_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct StreamEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr StreamEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 StreamEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(StreamEvent::InternalGenerateClassData_(
            _default, &StreamEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<StreamEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~StreamEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) StreamEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<StreamEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(StreamEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST StreamEventGlobalsTypeInternal StreamEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* StreamEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return StreamEvent_globals_.GetClassData();
#else
  return StreamEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace grpc
static const ::_pb::EnumDescriptor* PROTOBUF_NONNULL
    file_level_enum_descriptors_bridge_2eproto[7];
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_bridge_2eproto = nullptr;
const ::uint32_t
    TableStruct_bridge_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::AddLogEntryRequest, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::AddLogEntryRequest, _impl_.level_),
        PROTOBUF_FIELD_OFFSET(::grpc::AddLogEntryRequest, _impl_.package_),
        PROTOBUF_FIELD_OFFSET(::grpc::AddLogEntryRequest, _impl_.message_),
        2,
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::GuiReadyResponse, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::GuiReadyResponse, _impl_.showsplashscreen_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::ReportBugRequest, _impl_._has_bits_),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::ReportBugRequest, _impl_.ostype_),
        PROTOBUF_FIELD_OFFSET(::grpc::ReportBugRequest, _impl_.osversion_),
        PROTOBUF_FIELD_OFFSET(::grpc::ReportBugRequest, _impl_.title_),
        PROTOBUF_FIELD_OFFSET(::grpc::ReportBugRequest, _impl_.description_),
        PROTOBUF_FIELD_OFFSET(::grpc::ReportBugRequest, _impl_.address_),
        PROTOBUF_FIELD_OFFSET(::grpc::ReportBugRequest, _impl_.emailclient_),
        PROTOBUF_FIELD_OFFSET(::grpc::ReportBugRequest, _impl_.includelogs_),
        0,
        1,
        2,
        3,
        4,
        5,
        6,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::LoginRequest, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::LoginRequest, _impl_.username_),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginRequest, _impl_.password_),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginRequest, _impl_.usehvdetails_),
        0,
        1,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::LoginAbortRequest, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::LoginAbortRequest, _impl_.username_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::ImapSmtpSettings, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::ImapSmtpSettings, _impl_.imapport_),
        PROTOBUF_FIELD_OFFSET(::grpc::ImapSmtpSettings, _impl_.smtpport_),
        PROTOBUF_FIELD_OFFSET(::grpc::ImapSmtpSettings, _impl_.usesslforimap_),
        PROTOBUF_FIELD_OFFSET(::grpc::ImapSmtpSettings, _impl_.usesslforsmtp_),
        0,
        1,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::AvailableKeychainsResponse, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::AvailableKeychainsResponse, _impl_.keychains_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::User, _impl_._has_bits_),
        12, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::User, _impl_.id_),
        PROTOBUF_FIELD_OFFSET(::grpc::User, _impl_.username_),
        PROTOBUF_FIELD_OFFSET(::grpc::User, _impl_.avatartext_),
        PROTOBUF_FIELD_OFFSET(::grpc::User, _impl_.state_),
        PROTOBUF_FIELD_OFFSET(::grpc::User, _impl_.splitmode_),
        PROTOBUF_FIELD_OFFSET(::grpc::User, _impl_.usedbytes_),
        PROTOBUF_FIELD_OFFSET(::grpc::User, _impl_.totalbytes_),
        PROTOBUF_FIELD_OFFSET(::grpc::User, _impl_.password_),
        PROTOBUF_FIELD_OFFSET(::grpc::User, _impl_.addresses_),
        1,
        2,
        3,
        5,
        6,
        7,
        8,
        4,
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::UserSplitModeRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::UserSplitModeRequest, _impl_.userid_),
        PROTOBUF_FIELD_OFFSET(::grpc::UserSplitModeRequest, _impl_.active_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::UserBadEventFeedbackRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::UserBadEventFeedbackRequest, _impl_.userid_),
        PROTOBUF_FIELD_OFFSET(::grpc::UserBadEventFeedbackRequest, _impl_.doresync_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::UserListResponse, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::UserListResponse, _impl_.users_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::ConfigureAppleMailRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::ConfigureAppleMailRequest, _impl_.userid_),
        PROTOBUF_FIELD_OFFSET(::grpc::ConfigureAppleMailRequest, _impl_.address_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::EventStreamRequest, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::EventStreamRequest, _impl_.clientplatform_),
        0,
        0x004, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::StreamEvent, _impl_._oneof_case_[0]),
        PROTOBUF_FIELD_OFFSET(::grpc::StreamEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::StreamEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::StreamEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::StreamEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::StreamEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::StreamEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::StreamEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::StreamEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::StreamEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::StreamEvent, _impl_.event_),
        0x004, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_._oneof_case_[0]),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::AppEvent, _impl_.event_),
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::InternetStatusEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::InternetStatusEvent, _impl_.connected_),
        0,
        0x000, // bitmap
        0x000, // bitmap
        0x000, // bitmap
        0x000, // bitmap
        0x000, // bitmap
        0x000, // bitmap
        0x000, // bitmap
        0x000, // bitmap
        0x000, // bitmap
        0x000, // bitmap
        0x000, // bitmap
        0x000, // bitmap
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::KnowledgeBaseSuggestion, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::KnowledgeBaseSuggestion, _impl_.url_),
        PROTOBUF_FIELD_OFFSET(::grpc::KnowledgeBaseSuggestion, _impl_.title_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::KnowledgeBaseSuggestionsEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::KnowledgeBaseSuggestionsEvent, _impl_.suggestions_),
        0,
        0x004, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::LoginEvent, _impl_._oneof_case_[0]),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginEvent, _impl_.event_),
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::LoginErrorEvent, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::LoginErrorEvent, _impl_.type_),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginErrorEvent, _impl_.message_),
        1,
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::LoginTfaRequestedEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::LoginTfaRequestedEvent, _impl_.username_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::LoginFidoRequestedEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::LoginFidoRequestedEvent, _impl_.username_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::LoginTfaOrFidoRequestedEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::LoginTfaOrFidoRequestedEvent, _impl_.username_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::LoginFidoTouchEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::LoginFidoTouchEvent, _impl_.username_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::LoginFidoPinRequired, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::LoginFidoPinRequired, _impl_.username_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::LoginTwoPasswordsRequestedEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::LoginTwoPasswordsRequestedEvent, _impl_.username_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::LoginFinishedEvent, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::LoginFinishedEvent, _impl_.userid_),
        PROTOBUF_FIELD_OFFSET(::grpc::LoginFinishedEvent, _impl_.wassignedout_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::LoginHvRequestedEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::LoginHvRequestedEvent, _impl_.hvurl_),
        0,
        0x004, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateEvent, _impl_._oneof_case_[0]),
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateEvent, _impl_.event_),
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateErrorEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateErrorEvent, _impl_.type_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateManualReadyEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateManualReadyEvent, _impl_.version_),
        0,
        0x000, // bitmap
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateForceEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::UpdateForceEvent, _impl_.version_),
        0,
        0x000, // bitmap
        0x000, // bitmap
        0x000, // bitmap
        0x000, // bitmap
        0x004, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::DiskCacheEvent, _impl_._oneof_case_[0]),
        PROTOBUF_FIELD_OFFSET(::grpc::DiskCacheEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::DiskCacheEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::DiskCacheEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::DiskCacheEvent, _impl_.event_),
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::DiskCacheErrorEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::DiskCacheErrorEvent, _impl_.type_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::DiskCachePathChangedEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::DiskCachePathChangedEvent, _impl_.path_),
        0,
        0x000, // bitmap
        0x004, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::MailServerSettingsEvent, _impl_._oneof_case_[0]),
        PROTOBUF_FIELD_OFFSET(::grpc::MailServerSettingsEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::MailServerSettingsEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::MailServerSettingsEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::MailServerSettingsEvent, _impl_.event_),
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::MailServerSettingsErrorEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::MailServerSettingsErrorEvent, _impl_.type_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::MailServerSettingsChangedEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::MailServerSettingsChangedEvent, _impl_.settings_),
        0,
        0x000, // bitmap
        0x004, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::KeychainEvent, _impl_._oneof_case_[0]),
        PROTOBUF_FIELD_OFFSET(::grpc::KeychainEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::KeychainEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::KeychainEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::KeychainEvent, _impl_.event_),
        0x000, // bitmap
        0x000, // bitmap
        0x000, // bitmap
        0x004, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::MailEvent, _impl_._oneof_case_[0]),
        PROTOBUF_FIELD_OFFSET(::grpc::MailEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::MailEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::MailEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::MailEvent, _impl_.event_),
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::AddressChangedEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::AddressChangedEvent, _impl_.address_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::AddressChangedLogoutEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::AddressChangedLogoutEvent, _impl_.address_),
        0,
        0x000, // bitmap
        0x004, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::UserEvent, _impl_._oneof_case_[0]),
        PROTOBUF_FIELD_OFFSET(::grpc::UserEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UserEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UserEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UserEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UserEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UserEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UserEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UserEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UserEvent, _impl_.event_),
        PROTOBUF_FIELD_OFFSET(::grpc::UserEvent, _impl_.event_),
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::ToggleSplitModeFinishedEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::ToggleSplitModeFinishedEvent, _impl_.userid_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::UserDisconnectedEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::UserDisconnectedEvent, _impl_.username_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::UserChangedEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::UserChangedEvent, _impl_.userid_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::UserBadEvent, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::UserBadEvent, _impl_.userid_),
        PROTOBUF_FIELD_OFFSET(::grpc::UserBadEvent, _impl_.errormessage_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::UsedBytesChangedEvent, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::UsedBytesChangedEvent, _impl_.userid_),
        PROTOBUF_FIELD_OFFSET(::grpc::UsedBytesChangedEvent, _impl_.usedbytes_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::ImapLoginFailedEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::ImapLoginFailedEvent, _impl_.username_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::SyncStartedEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::SyncStartedEvent, _impl_.userid_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::SyncFinishedEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::SyncFinishedEvent, _impl_.userid_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::SyncProgressEvent, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::SyncProgressEvent, _impl_.userid_),
        PROTOBUF_FIELD_OFFSET(::grpc::SyncProgressEvent, _impl_.progress_),
        PROTOBUF_FIELD_OFFSET(::grpc::SyncProgressEvent, _impl_.elapsedms_),
        PROTOBUF_FIELD_OFFSET(::grpc::SyncProgressEvent, _impl_.remainingms_),
        0,
        1,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::UserNotificationEvent, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::UserNotificationEvent, _impl_.title_),
        PROTOBUF_FIELD_OFFSET(::grpc::UserNotificationEvent, _impl_.subtitle_),
        PROTOBUF_FIELD_OFFSET(::grpc::UserNotificationEvent, _impl_.body_),
        PROTOBUF_FIELD_OFFSET(::grpc::UserNotificationEvent, _impl_.userid_),
        0,
        1,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::grpc::GenericErrorEvent, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::grpc::GenericErrorEvent, _impl_.code_),
        0,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::grpc::AddLogEntryRequest)},
        {9, sizeof(::grpc::GuiReadyResponse)},
        {14, sizeof(::grpc::ReportBugRequest)},
        {31, sizeof(::grpc::LoginRequest)},
        {40, sizeof(::grpc::LoginAbortRequest)},
        {45, sizeof(::grpc::ImapSmtpSettings)},
        {56, sizeof(::grpc::AvailableKeychainsResponse)},
        {61, sizeof(::grpc::User)},
        {82, sizeof(::grpc::UserSplitModeRequest)},
        {89, sizeof(::grpc::UserBadEventFeedbackRequest)},
        {96, sizeof(::grpc::UserListResponse)},
        {101, sizeof(::grpc::ConfigureAppleMailRequest)},
        {108, sizeof(::grpc::EventStreamRequest)},
        {113, sizeof(::grpc::StreamEvent)},
        {125, sizeof(::grpc::AppEvent)},
        {143, sizeof(::grpc::InternetStatusEvent)},
        {148, sizeof(::grpc::ToggleAutostartFinishedEvent)},
        {149, sizeof(::grpc::ResetFinishedEvent)},
        {150, sizeof(::grpc::ReportBugFinishedEvent)},
        {151, sizeof(::grpc::ReportBugSuccessEvent)},
        {152, sizeof(::grpc::ReportBugErrorEvent)},
        {153, sizeof(::grpc::ShowMainWindowEvent)},
        {154, sizeof(::grpc::ReportBugFallbackEvent)},
        {155, sizeof(::grpc::CertificateInstallSuccessEvent)},
        {156, sizeof(::grpc::CertificateInstallCanceledEvent)},
        {157, sizeof(::grpc::CertificateInstallFailedEvent)},
        {158, sizeof(::grpc::RepairStartedEvent)},
        {159, sizeof(::grpc::AllUsersLoadedEvent)},
        {160, sizeof(::grpc::KnowledgeBaseSuggestion)},
        {167, sizeof(::grpc::KnowledgeBaseSuggestionsEvent)},
        {172, sizeof(::grpc::LoginEvent)},
        {186, sizeof(::grpc::LoginErrorEvent)},
        {193, sizeof(::grpc::LoginTfaRequestedEvent)},
        {198, sizeof(::grpc::LoginFidoRequestedEvent)},
        {203, sizeof(::grpc::LoginTfaOrFidoRequestedEvent)},
        {208, sizeof(::grpc::LoginFidoTouchEvent)},
        {213, sizeof(::grpc::LoginFidoPinRequired)},
        {218, sizeof(::grpc::LoginTwoPasswordsRequestedEvent)},
        {223, sizeof(::grpc::LoginFinishedEvent)},
        {230, sizeof(::grpc::LoginHvRequestedEvent)},
        {235, sizeof(::grpc::UpdateEvent)},
        {246, sizeof(::grpc::UpdateErrorEvent)},
        {251, sizeof(::grpc::UpdateManualReadyEvent)},
        {256, sizeof(::grpc::UpdateManualRestartNeededEvent)},
        {257, sizeof(::grpc::UpdateForceEvent)},
        {262, sizeof(::grpc::UpdateSilentRestartNeeded)},
        {263, sizeof(::grpc::UpdateIsLatestVersion)},
        {264, sizeof(::grpc::UpdateCheckFinished)},
        {265, sizeof(::grpc::UpdateVersionChanged)},
        {266, sizeof(::grpc::DiskCacheEvent)},
        {272, sizeof(::grpc::DiskCacheErrorEvent)},
        {277, sizeof(::grpc::DiskCachePathChangedEvent)},
        {282, sizeof(::grpc::DiskCachePathChangeFinishedEvent)},
        {283, sizeof(::grpc::MailServerSettingsEvent)},
        {289, sizeof(::grpc::MailServerSettingsErrorEvent)},
        {294, sizeof(::grpc::MailServerSettingsChangedEvent)},
        {299, sizeof(::grpc::ChangeMailServerSettingsFinishedEvent)},
        {300, sizeof(::grpc::KeychainEvent)},
        {306, sizeof(::grpc::ChangeKeychainFinishedEvent)},
        {307, sizeof(::grpc::HasNoKeychainEvent)},
        {308, sizeof(::grpc::RebuildKeychainEvent)},
        {309, sizeof(::grpc::MailEvent)},
        {315, sizeof(::grpc::AddressChangedEvent)},
        {320, sizeof(::grpc::AddressChangedLogoutEvent)},
        {325, sizeof(::grpc::ApiCertIssueEvent)},
        {326, sizeof(::grpc::UserEvent)},
        {338, sizeof(::grpc::ToggleSplitModeFinishedEvent)},
        {343, sizeof(::grpc::UserDisconnectedEvent)},
        {348, sizeof(::grpc::UserChangedEvent)},
        {353, sizeof(::grpc::UserBadEvent)},
        {360, sizeof(::grpc::UsedBytesChangedEvent)},
        {367, sizeof(::grpc::ImapLoginFailedEvent)},
        {372, sizeof(::grpc::SyncStartedEvent)},
        {377, sizeof(::grpc::SyncFinishedEvent)},
        {382, sizeof(::grpc::SyncProgressEvent)},
        {393, sizeof(::grpc::UserNotificationEvent)},
        {404, sizeof(::grpc::GenericErrorEvent)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::grpc::AddLogEntryRequest_globals_,
        &::grpc::GuiReadyResponse_globals_,
        &::grpc::ReportBugRequest_globals_,
        &::grpc::LoginRequest_globals_,
        &::grpc::LoginAbortRequest_globals_,
        &::grpc::ImapSmtpSettings_globals_,
        &::grpc::AvailableKeychainsResponse_globals_,
        &::grpc::User_globals_,
        &::grpc::UserSplitModeRequest_globals_,
        &::grpc::UserBadEventFeedbackRequest_globals_,
        &::grpc::UserListResponse_globals_,
        &::grpc::ConfigureAppleMailRequest_globals_,
        &::grpc::EventStreamRequest_globals_,
        &::grpc::StreamEvent_globals_,
        &::grpc::AppEvent_globals_,
        &::grpc::InternetStatusEvent_globals_,
        &::grpc::ToggleAutostartFinishedEvent_globals_,
        &::grpc::ResetFinishedEvent_globals_,
        &::grpc::ReportBugFinishedEvent_globals_,
        &::grpc::ReportBugSuccessEvent_globals_,
        &::grpc::ReportBugErrorEvent_globals_,
        &::grpc::ShowMainWindowEvent_globals_,
        &::grpc::ReportBugFallbackEvent_globals_,
        &::grpc::CertificateInstallSuccessEvent_globals_,
        &::grpc::CertificateInstallCanceledEvent_globals_,
        &::grpc::CertificateInstallFailedEvent_globals_,
        &::grpc::RepairStartedEvent_globals_,
        &::grpc::AllUsersLoadedEvent_globals_,
        &::grpc::KnowledgeBaseSuggestion_globals_,
        &::grpc::KnowledgeBaseSuggestionsEvent_globals_,
        &::grpc::LoginEvent_globals_,
        &::grpc::LoginErrorEvent_globals_,
        &::grpc::LoginTfaRequestedEvent_globals_,
        &::grpc::LoginFidoRequestedEvent_globals_,
        &::grpc::LoginTfaOrFidoRequestedEvent_globals_,
        &::grpc::LoginFidoTouchEvent_globals_,
        &::grpc::LoginFidoPinRequired_globals_,
        &::grpc::LoginTwoPasswordsRequestedEvent_globals_,
        &::grpc::LoginFinishedEvent_globals_,
        &::grpc::LoginHvRequestedEvent_globals_,
        &::grpc::UpdateEvent_globals_,
        &::grpc::UpdateErrorEvent_globals_,
        &::grpc::UpdateManualReadyEvent_globals_,
        &::grpc::UpdateManualRestartNeededEvent_globals_,
        &::grpc::UpdateForceEvent_globals_,
        &::grpc::UpdateSilentRestartNeeded_globals_,
        &::grpc::UpdateIsLatestVersion_globals_,
        &::grpc::UpdateCheckFinished_globals_,
        &::grpc::UpdateVersionChanged_globals_,
        &::grpc::DiskCacheEvent_globals_,
        &::grpc::DiskCacheErrorEvent_globals_,
        &::grpc::DiskCachePathChangedEvent_globals_,
        &::grpc::DiskCachePathChangeFinishedEvent_globals_,
        &::grpc::MailServerSettingsEvent_globals_,
        &::grpc::MailServerSettingsErrorEvent_globals_,
        &::grpc::MailServerSettingsChangedEvent_globals_,
        &::grpc::ChangeMailServerSettingsFinishedEvent_globals_,
        &::grpc::KeychainEvent_globals_,
        &::grpc::ChangeKeychainFinishedEvent_globals_,
        &::grpc::HasNoKeychainEvent_globals_,
        &::grpc::RebuildKeychainEvent_globals_,
        &::grpc::MailEvent_globals_,
        &::grpc::AddressChangedEvent_globals_,
        &::grpc::AddressChangedLogoutEvent_globals_,
        &::grpc::ApiCertIssueEvent_globals_,
        &::grpc::UserEvent_globals_,
        &::grpc::ToggleSplitModeFinishedEvent_globals_,
        &::grpc::UserDisconnectedEvent_globals_,
        &::grpc::UserChangedEvent_globals_,
        &::grpc::UserBadEvent_globals_,
        &::grpc::UsedBytesChangedEvent_globals_,
        &::grpc::ImapLoginFailedEvent_globals_,
        &::grpc::SyncStartedEvent_globals_,
        &::grpc::SyncFinishedEvent_globals_,
        &::grpc::SyncProgressEvent_globals_,
        &::grpc::UserNotificationEvent_globals_,
        &::grpc::GenericErrorEvent_globals_,
};
const char descriptor_table_protodef_bridge_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\014bridge.proto\022\004grpc\032\033google/protobuf/em"
    "pty.proto\032\036google/protobuf/wrappers.prot"
    "o\"U\n\022AddLogEntryRequest\022\035\n\005level\030\001 \001(\0162\016"
    ".grpc.LogLevel\022\017\n\007package\030\002 \001(\t\022\017\n\007messa"
    "ge\030\003 \001(\t\",\n\020GuiReadyResponse\022\030\n\020showSpla"
    "shScreen\030\001 \001(\010\"\224\001\n\020ReportBugRequest\022\016\n\006o"
    "sType\030\001 \001(\t\022\021\n\tosVersion\030\002 \001(\t\022\r\n\005title\030"
    "\003 \001(\t\022\023\n\013description\030\004 \001(\t\022\017\n\007address\030\005 "
    "\001(\t\022\023\n\013emailClient\030\006 \001(\t\022\023\n\013includeLogs\030"
    "\007 \001(\010\"^\n\014LoginRequest\022\020\n\010username\030\001 \001(\t\022"
    "\020\n\010password\030\002 \001(\014\022\031\n\014useHvDetails\030\003 \001(\010H"
    "\000\210\001\001B\017\n\r_useHvDetails\"%\n\021LoginAbortReque"
    "st\022\020\n\010username\030\001 \001(\t\"d\n\020ImapSmtpSettings"
    "\022\020\n\010imapPort\030\001 \001(\005\022\020\n\010smtpPort\030\002 \001(\005\022\025\n\r"
    "useSSLForImap\030\003 \001(\010\022\025\n\ruseSSLForSmtp\030\004 \001"
    "(\010\"/\n\032AvailableKeychainsResponse\022\021\n\tkeyc"
    "hains\030\001 \003(\t\"\267\001\n\004User\022\n\n\002id\030\001 \001(\t\022\020\n\010user"
    "name\030\002 \001(\t\022\022\n\navatarText\030\003 \001(\t\022\036\n\005state\030"
    "\004 \001(\0162\017.grpc.UserState\022\021\n\tsplitMode\030\005 \001("
    "\010\022\021\n\tusedBytes\030\006 \001(\003\022\022\n\ntotalBytes\030\007 \001(\003"
    "\022\020\n\010password\030\010 \001(\014\022\021\n\taddresses\030\t \003(\t\"6\n"
    "\024UserSplitModeRequest\022\016\n\006userID\030\001 \001(\t\022\016\n"
    "\006active\030\002 \001(\010\"\?\n\033UserBadEventFeedbackReq"
    "uest\022\016\n\006userID\030\001 \001(\t\022\020\n\010doResync\030\002 \001(\010\"-"
    "\n\020UserListResponse\022\031\n\005users\030\001 \003(\0132\n.grpc"
    ".User\"<\n\031ConfigureAppleMailRequest\022\016\n\006us"
    "erID\030\001 \001(\t\022\017\n\007address\030\002 \001(\t\",\n\022EventStre"
    "amRequest\022\026\n\016ClientPlatform\030\001 \001(\t\"\375\002\n\013St"
    "reamEvent\022\035\n\003app\030\001 \001(\0132\016.grpc.AppEventH\000"
    "\022!\n\005login\030\002 \001(\0132\020.grpc.LoginEventH\000\022#\n\006u"
    "pdate\030\003 \001(\0132\021.grpc.UpdateEventH\000\022%\n\005cach"
    "e\030\004 \001(\0132\024.grpc.DiskCacheEventH\000\022;\n\022mailS"
    "erverSettings\030\005 \001(\0132\035.grpc.MailServerSet"
    "tingsEventH\000\022\'\n\010keychain\030\006 \001(\0132\023.grpc.Ke"
    "ychainEventH\000\022\037\n\004mail\030\007 \001(\0132\017.grpc.MailE"
    "ventH\000\022\037\n\004user\030\010 \001(\0132\017.grpc.UserEventH\000\022"
    "/\n\014genericError\030\t \001(\0132\027.grpc.GenericErro"
    "rEventH\000B\007\n\005event\"\246\007\n\010AppEvent\0223\n\016intern"
    "etStatus\030\001 \001(\0132\031.grpc.InternetStatusEven"
    "tH\000\022E\n\027toggleAutostartFinished\030\002 \001(\0132\".g"
    "rpc.ToggleAutostartFinishedEventH\000\0221\n\rre"
    "setFinished\030\003 \001(\0132\030.grpc.ResetFinishedEv"
    "entH\000\0229\n\021reportBugFinished\030\004 \001(\0132\034.grpc."
    "ReportBugFinishedEventH\000\0227\n\020reportBugSuc"
    "cess\030\005 \001(\0132\033.grpc.ReportBugSuccessEventH"
    "\000\0223\n\016reportBugError\030\006 \001(\0132\031.grpc.ReportB"
    "ugErrorEventH\000\0223\n\016showMainWindow\030\007 \001(\0132\031"
    ".grpc.ShowMainWindowEventH\000\0229\n\021reportBug"
    "Fallback\030\010 \001(\0132\034.grpc.ReportBugFallbackE"
    "ventH\000\022I\n\031certificateInstallSuccess\030\t \001("
    "\0132$.grpc.CertificateInstallSuccessEventH"
    "\000\022K\n\032certificateInstallCanceled\030\n \001(\0132%."
    "grpc.CertificateInstallCanceledEventH\000\022G"
    "\n\030certificateInstallFailed\030\013 \001(\0132#.grpc."
    "CertificateInstallFailedEventH\000\022G\n\030knowl"
    "edgeBaseSuggestions\030\014 \001(\0132#.grpc.Knowled"
    "geBaseSuggestionsEventH\000\0221\n\rrepairStarte"
    "d\030\r \001(\0132\030.grpc.RepairStartedEventH\000\0223\n\016a"
    "llUsersLoaded\030\016 \001(\0132\031.grpc.AllUsersLoade"
    "dEventH\000\0227\n\020userNotification\030\017 \001(\0132\033.grp"
    "c.UserNotificationEventH\000B\007\n\005event\"(\n\023In"
    "ternetStatusEvent\022\021\n\tconnected\030\001 \001(\010\"\036\n\034"
    "ToggleAutostartFinishedEvent\"\024\n\022ResetFin"
    "ishedEvent\"\030\n\026ReportBugFinishedEvent\"\027\n\025"
    "ReportBugSuccessEvent\"\025\n\023ReportBugErrorE"
    "vent\"\025\n\023ShowMainWindowEvent\"\030\n\026ReportBug"
    "FallbackEvent\" \n\036CertificateInstallSucce"
    "ssEvent\"!\n\037CertificateInstallCanceledEve"
    "nt\"\037\n\035CertificateInstallFailedEvent\"\024\n\022R"
    "epairStartedEvent\"\025\n\023AllUsersLoadedEvent"
    "\"5\n\027KnowledgeBaseSuggestion\022\013\n\003url\030\001 \001(\t"
    "\022\r\n\005title\030\002 \001(\t\"S\n\035KnowledgeBaseSuggesti"
    "onsEvent\0222\n\013suggestions\030\001 \003(\0132\035.grpc.Kno"
    "wledgeBaseSuggestion\"\203\005\n\nLoginEvent\022&\n\005e"
    "rror\030\001 \001(\0132\025.grpc.LoginErrorEventH\000\0224\n\014t"
    "faRequested\030\002 \001(\0132\034.grpc.LoginTfaRequest"
    "edEventH\000\022E\n\024twoPasswordRequested\030\003 \001(\0132"
    "%.grpc.LoginTwoPasswordsRequestedEventH\000"
    "\022,\n\010finished\030\004 \001(\0132\030.grpc.LoginFinishedE"
    "ventH\000\0223\n\017alreadyLoggedIn\030\005 \001(\0132\030.grpc.L"
    "oginFinishedEventH\000\0222\n\013hvRequested\030\006 \001(\013"
    "2\033.grpc.LoginHvRequestedEventH\000\0226\n\rfidoR"
    "equested\030\007 \001(\0132\035.grpc.LoginFidoRequested"
    "EventH\000\022@\n\022tfaOrFidoRequested\030\010 \001(\0132\".gr"
    "pc.LoginTfaOrFidoRequestedEventH\000\022<\n\027log"
    "inFidoTouchRequested\030\t \001(\0132\031.grpc.LoginF"
    "idoTouchEventH\000\022<\n\027loginFidoTouchComplet"
    "ed\030\n \001(\0132\031.grpc.LoginFidoTouchEventH\000\022:\n"
    "\024loginFidoPinRequired\030\013 \001(\0132\032.grpc.Login"
    "FidoPinRequiredH\000B\007\n\005event\"F\n\017LoginError"
    "Event\022\"\n\004type\030\001 \001(\0162\024.grpc.LoginErrorTyp"
    "e\022\017\n\007message\030\002 \001(\t\"*\n\026LoginTfaRequestedE"
    "vent\022\020\n\010username\030\001 \001(\t\"+\n\027LoginFidoReque"
    "stedEvent\022\020\n\010username\030\001 \001(\t\"0\n\034LoginTfaO"
    "rFidoRequestedEvent\022\020\n\010username\030\001 \001(\t\"\'\n"
    "\023LoginFidoTouchEvent\022\020\n\010username\030\001 \001(\t\"("
    "\n\024LoginFidoPinRequired\022\020\n\010username\030\001 \001(\t"
    "\"3\n\037LoginTwoPasswordsRequestedEvent\022\020\n\010u"
    "sername\030\001 \001(\t\":\n\022LoginFinishedEvent\022\016\n\006u"
    "serID\030\001 \001(\t\022\024\n\014wasSignedOut\030\002 \001(\010\"&\n\025Log"
    "inHvRequestedEvent\022\r\n\005hvUrl\030\001 \001(\t\"\304\003\n\013Up"
    "dateEvent\022\'\n\005error\030\001 \001(\0132\026.grpc.UpdateEr"
    "rorEventH\000\0223\n\013manualReady\030\002 \001(\0132\034.grpc.U"
    "pdateManualReadyEventH\000\022C\n\023manualRestart"
    "Needed\030\003 \001(\0132$.grpc.UpdateManualRestartN"
    "eededEventH\000\022\'\n\005force\030\004 \001(\0132\026.grpc.Updat"
    "eForceEventH\000\022>\n\023silentRestartNeeded\030\005 \001"
    "(\0132\037.grpc.UpdateSilentRestartNeededH\000\0226\n"
    "\017isLatestVersion\030\006 \001(\0132\033.grpc.UpdateIsLa"
    "testVersionH\000\0222\n\rcheckFinished\030\007 \001(\0132\031.g"
    "rpc.UpdateCheckFinishedH\000\0224\n\016versionChan"
    "ged\030\010 \001(\0132\032.grpc.UpdateVersionChangedH\000B"
    "\007\n\005event\"7\n\020UpdateErrorEvent\022#\n\004type\030\001 \001"
    "(\0162\025.grpc.UpdateErrorType\")\n\026UpdateManua"
    "lReadyEvent\022\017\n\007version\030\001 \001(\t\" \n\036UpdateMa"
    "nualRestartNeededEvent\"#\n\020UpdateForceEve"
    "nt\022\017\n\007version\030\001 \001(\t\"\033\n\031UpdateSilentResta"
    "rtNeeded\"\027\n\025UpdateIsLatestVersion\"\025\n\023Upd"
    "ateCheckFinished\"\026\n\024UpdateVersionChanged"
    "\"\303\001\n\016DiskCacheEvent\022*\n\005error\030\001 \001(\0132\031.grp"
    "c.DiskCacheErrorEventH\000\0226\n\013pathChanged\030\002"
    " \001(\0132\037.grpc.DiskCachePathChangedEventH\000\022"
    "D\n\022pathChangeFinished\030\003 \001(\0132&.grpc.DiskC"
    "achePathChangeFinishedEventH\000B\007\n\005event\"="
    "\n\023DiskCacheErrorEvent\022&\n\004type\030\001 \001(\0162\030.gr"
    "pc.DiskCacheErrorType\")\n\031DiskCachePathCh"
    "angedEvent\022\014\n\004path\030\001 \001(\t\"\"\n DiskCachePat"
    "hChangeFinishedEvent\"\373\001\n\027MailServerSetti"
    "ngsEvent\0223\n\005error\030\001 \001(\0132\".grpc.MailServe"
    "rSettingsErrorEventH\000\022I\n\031mailServerSetti"
    "ngsChanged\030\002 \001(\0132$.grpc.MailServerSettin"
    "gsChangedEventH\000\022W\n changeMailServerSett"
    "ingsFinished\030\003 \001(\0132+.grpc.ChangeMailServ"
    "erSettingsFinishedEventH\000B\007\n\005event\"O\n\034Ma"
    "ilServerSettingsErrorEvent\022/\n\004type\030\001 \001(\016"
    "2!.grpc.MailServerSettingsErrorType\"J\n\036M"
    "ailServerSettingsChangedEvent\022(\n\010setting"
    "s\030\001 \001(\0132\026.grpc.ImapSmtpSettings\"\'\n%Chang"
    "eMailServerSettingsFinishedEvent\"\307\001\n\rKey"
    "chainEvent\022C\n\026changeKeychainFinished\030\001 \001"
    "(\0132!.grpc.ChangeKeychainFinishedEventH\000\022"
    "1\n\rhasNoKeychain\030\002 \001(\0132\030.grpc.HasNoKeych"
    "ainEventH\000\0225\n\017rebuildKeychain\030\003 \001(\0132\032.gr"
    "pc.RebuildKeychainEventH\000B\007\n\005event\"\035\n\033Ch"
    "angeKeychainFinishedEvent\"\024\n\022HasNoKeycha"
    "inEvent\"\026\n\024RebuildKeychainEvent\"\273\001\n\tMail"
    "Event\0223\n\016addressChanged\030\001 \001(\0132\031.grpc.Add"
    "ressChangedEventH\000\022\?\n\024addressChangedLogo"
    "ut\030\002 \001(\0132\037.grpc.AddressChangedLogoutEven"
    "tH\000\022/\n\014apiCertIssue\030\003 \001(\0132\027.grpc.ApiCert"
    "IssueEventH\000B\007\n\005event\"&\n\023AddressChangedE"
    "vent\022\017\n\007address\030\001 \001(\t\",\n\031AddressChangedL"
    "ogoutEvent\022\017\n\007address\030\001 \001(\t\"\023\n\021ApiCertIs"
    "sueEvent\"\211\004\n\tUserEvent\022E\n\027toggleSplitMod"
    "eFinished\030\001 \001(\0132\".grpc.ToggleSplitModeFi"
    "nishedEventH\000\0227\n\020userDisconnected\030\002 \001(\0132"
    "\033.grpc.UserDisconnectedEventH\000\022-\n\013userCh"
    "anged\030\003 \001(\0132\026.grpc.UserChangedEventH\000\022*\n"
    "\014userBadEvent\030\004 \001(\0132\022.grpc.UserBadEventH"
    "\000\022<\n\025usedBytesChangedEvent\030\005 \001(\0132\033.grpc."
    "UsedBytesChangedEventH\000\022:\n\024imapLoginFail"
    "edEvent\030\006 \001(\0132\032.grpc.ImapLoginFailedEven"
    "tH\000\0222\n\020syncStartedEvent\030\007 \001(\0132\026.grpc.Syn"
    "cStartedEventH\000\0224\n\021syncFinishedEvent\030\010 \001"
    "(\0132\027.grpc.SyncFinishedEventH\000\0224\n\021syncPro"
    "gressEvent\030\t \001(\0132\027.grpc.SyncProgressEven"
    "tH\000B\007\n\005event\".\n\034ToggleSplitModeFinishedE"
    "vent\022\016\n\006userID\030\001 \001(\t\")\n\025UserDisconnected"
    "Event\022\020\n\010username\030\001 \001(\t\"\"\n\020UserChangedEv"
    "ent\022\016\n\006userID\030\001 \001(\t\"4\n\014UserBadEvent\022\016\n\006u"
    "serID\030\001 \001(\t\022\024\n\014errorMessage\030\002 \001(\t\":\n\025Use"
    "dBytesChangedEvent\022\016\n\006userID\030\001 \001(\t\022\021\n\tus"
    "edBytes\030\002 \001(\003\"(\n\024ImapLoginFailedEvent\022\020\n"
    "\010username\030\001 \001(\t\"\"\n\020SyncStartedEvent\022\016\n\006u"
    "serID\030\001 \001(\t\"#\n\021SyncFinishedEvent\022\016\n\006user"
    "ID\030\001 \001(\t\"]\n\021SyncProgressEvent\022\016\n\006userID\030"
    "\001 \001(\t\022\020\n\010progress\030\002 \001(\001\022\021\n\telapsedMs\030\003 \001"
    "(\003\022\023\n\013remainingMs\030\004 \001(\003\"V\n\025UserNotificat"
    "ionEvent\022\r\n\005title\030\001 \001(\t\022\020\n\010subtitle\030\002 \001("
    "\t\022\014\n\004body\030\003 \001(\t\022\016\n\006userID\030\004 \001(\t\"2\n\021Gener"
    "icErrorEvent\022\035\n\004code\030\001 \001(\0162\017.grpc.ErrorC"
    "ode*q\n\010LogLevel\022\r\n\tLOG_PANIC\020\000\022\r\n\tLOG_FA"
    "TAL\020\001\022\r\n\tLOG_ERROR\020\002\022\014\n\010LOG_WARN\020\003\022\014\n\010LO"
    "G_INFO\020\004\022\r\n\tLOG_DEBUG\020\005\022\r\n\tLOG_TRACE\020\006*6"
    "\n\tUserState\022\016\n\nSIGNED_OUT\020\000\022\n\n\006LOCKED\020\001\022"
    "\r\n\tCONNECTED\020\002*\354\001\n\016LoginErrorType\022\033\n\027USE"
    "RNAME_PASSWORD_ERROR\020\000\022\r\n\tFREE_USER\020\001\022\024\n"
    "\020CONNECTION_ERROR\020\002\022\r\n\tTFA_ERROR\020\003\022\r\n\tTF"
    "A_ABORT\020\004\022\027\n\023TWO_PASSWORDS_ERROR\020\005\022\027\n\023TW"
    "O_PASSWORDS_ABORT\020\006\022\014\n\010HV_ERROR\020\007\022\024\n\020FID"
    "O_PIN_INVALID\020\010\022\024\n\020FIDO_PIN_BLOCKED\020\t\022\016\n"
    "\nFIDO_ERROR\020\n*[\n\017UpdateErrorType\022\027\n\023UPDA"
    "TE_MANUAL_ERROR\020\000\022\026\n\022UPDATE_FORCE_ERROR\020"
    "\001\022\027\n\023UPDATE_SILENT_ERROR\020\002*4\n\022DiskCacheE"
    "rrorType\022\036\n\032CANT_MOVE_DISK_CACHE_ERROR\020\000"
    "*\335\001\n\033MailServerSettingsErrorType\022\033\n\027IMAP"
    "_PORT_STARTUP_ERROR\020\000\022\033\n\027SMTP_PORT_START"
    "UP_ERROR\020\001\022\032\n\026IMAP_PORT_CHANGE_ERROR\020\002\022\032"
    "\n\026SMTP_PORT_CHANGE_ERROR\020\003\022%\n!IMAP_CONNE"
    "CTION_MODE_CHANGE_ERROR\020\004\022%\n!SMTP_CONNEC"
    "TION_MODE_CHANGE_ERROR\020\005*S\n\tErrorCode\022\021\n"
    "\rUNKNOWN_ERROR\020\000\022\031\n\025TLS_CERT_EXPORT_ERRO"
    "R\020\001\022\030\n\024TLS_KEY_EXPORT_ERROR\020\0022\275\"\n\006Bridge"
    "\022I\n\013CheckTokens\022\034.google.protobuf.String"
    "Value\032\034.google.protobuf.StringValue\022\?\n\013A"
    "ddLogEntry\022\030.grpc.AddLogEntryRequest\032\026.g"
    "oogle.protobuf.Empty\022:\n\010GuiReady\022\026.googl"
    "e.protobuf.Empty\032\026.grpc.GuiReadyResponse"
    "\0226\n\004Quit\022\026.google.protobuf.Empty\032\026.googl"
    "e.protobuf.Empty\0229\n\007Restart\022\026.google.pro"
    "tobuf.Empty\032\026.google.protobuf.Empty\022C\n\rS"
    "howOnStartup\022\026.google.protobuf.Empty\032\032.g"
    "oogle.protobuf.BoolValue\022F\n\020SetIsAutosta"
    "rtOn\022\032.google.protobuf.BoolValue\032\026.googl"
    "e.protobuf.Empty\022C\n\rIsAutostartOn\022\026.goog"
    "le.protobuf.Empty\032\032.google.protobuf.Bool"
    "Value\022F\n\020SetIsBetaEnabled\022\032.google.proto"
    "buf.BoolValue\032\026.google.protobuf.Empty\022C\n"
    "\rIsBetaEnabled\022\026.google.protobuf.Empty\032\032"
    ".google.protobuf.BoolValue\022I\n\023SetIsAllMa"
    "ilVisible\022\032.google.protobuf.BoolValue\032\026."
    "google.protobuf.Empty\022F\n\020IsAllMailVisibl"
    "e\022\026.google.protobuf.Empty\032\032.google.proto"
    "buf.BoolValue\022L\n\026SetIsTelemetryDisabled\022"
    "\032.google.protobuf.BoolValue\032\026.google.pro"
    "tobuf.Empty\022I\n\023IsTelemetryDisabled\022\026.goo"
    "gle.protobuf.Empty\032\032.google.protobuf.Boo"
    "lValue\022<\n\004GoOs\022\026.google.protobuf.Empty\032\034"
    ".google.protobuf.StringValue\022>\n\014TriggerR"
    "eset\022\026.google.protobuf.Empty\032\026.google.pr"
    "otobuf.Empty\022\?\n\007Version\022\026.google.protobu"
    "f.Empty\032\034.google.protobuf.StringValue\022@\n"
    "\010LogsPath\022\026.google.protobuf.Empty\032\034.goog"
    "le.protobuf.StringValue\022C\n\013LicensePath\022\026"
    ".google.protobuf.Empty\032\034.google.protobuf"
    ".StringValue\022L\n\024ReleaseNotesPageLink\022\026.g"
    "oogle.protobuf.Empty\032\034.google.protobuf.S"
    "tringValue\022N\n\026DependencyLicensesLink\022\026.g"
    "oogle.protobuf.Empty\032\034.google.protobuf.S"
    "tringValue\022G\n\017LandingPageLink\022\026.google.p"
    "rotobuf.Empty\032\034.google.protobuf.StringVa"
    "lue\022J\n\022SetColorSchemeName\022\034.google.proto"
    "buf.StringValue\032\026.google.protobuf.Empty\022"
    "G\n\017ColorSchemeName\022\026.google.protobuf.Emp"
    "ty\032\034.google.protobuf.StringValue\022J\n\022Curr"
    "entEmailClient\022\026.google.protobuf.Empty\032\034"
    ".google.protobuf.StringValue\022;\n\tReportBu"
    "g\022\026.grpc.ReportBugRequest\032\026.google.proto"
    "buf.Empty\022E\n\rForceLauncher\022\034.google.prot"
    "obuf.StringValue\032\026.google.protobuf.Empty"
    "\022I\n\021SetMainExecutable\022\034.google.protobuf."
    "StringValue\032\026.google.protobuf.Empty\022W\n\037R"
    "equestKnowledgeBaseSuggestions\022\034.google."
    "protobuf.StringValue\032\026.google.protobuf.E"
    "mpty\0223\n\005Login\022\022.grpc.LoginRequest\032\026.goog"
    "le.protobuf.Empty\0226\n\010Login2FA\022\022.grpc.Log"
    "inRequest\032\026.google.protobuf.Empty\0227\n\tLog"
    "inFido\022\022.grpc.LoginRequest\032\026.google.prot"
    "obuf.Empty\022=\n\017Login2Passwords\022\022.grpc.Log"
    "inRequest\032\026.google.protobuf.Empty\022=\n\nLog"
    "inAbort\022\027.grpc.LoginAbortRequest\032\026.googl"
    "e.protobuf.Empty\022E\n\022FidoAssertionAbort\022\027"
    ".grpc.LoginAbortRequest\032\026.google.protobu"
    "f.Empty\022=\n\013CheckUpdate\022\026.google.protobuf"
    ".Empty\032\026.google.protobuf.Empty\022\?\n\rInstal"
    "lUpdate\022\026.google.protobuf.Empty\032\026.google"
    ".protobuf.Empty\022L\n\026SetIsAutomaticUpdateO"
    "n\022\032.google.protobuf.BoolValue\032\026.google.p"
    "rotobuf.Empty\022I\n\023IsAutomaticUpdateOn\022\026.g"
    "oogle.protobuf.Empty\032\032.google.protobuf.B"
    "oolValue\022E\n\rDiskCachePath\022\026.google.proto"
    "buf.Empty\032\034.google.protobuf.StringValue\022"
    "H\n\020SetDiskCachePath\022\034.google.protobuf.St"
    "ringValue\032\026.google.protobuf.Empty\022E\n\017Set"
    "IsDoHEnabled\022\032.google.protobuf.BoolValue"
    "\032\026.google.protobuf.Empty\022B\n\014IsDoHEnabled"
    "\022\026.google.protobuf.Empty\032\032.google.protob"
    "uf.BoolValue\022D\n\022MailServerSettings\022\026.goo"
    "gle.protobuf.Empty\032\026.grpc.ImapSmtpSettin"
    "gs\022G\n\025SetMailServerSettings\022\026.grpc.ImapS"
    "mtpSettings\032\026.google.protobuf.Empty\022@\n\010H"
    "ostname\022\026.google.protobuf.Empty\032\034.google"
    ".protobuf.StringValue\022E\n\nIsPortFree\022\033.go"
    "ogle.protobuf.Int32Value\032\032.google.protob"
    "uf.BoolValue\022N\n\022AvailableKeychains\022\026.goo"
    "gle.protobuf.Empty\032 .grpc.AvailableKeych"
    "ainsResponse\022J\n\022SetCurrentKeychain\022\034.goo"
    "gle.protobuf.StringValue\032\026.google.protob"
    "uf.Empty\022G\n\017CurrentKeychain\022\026.google.pro"
    "tobuf.Empty\032\034.google.protobuf.StringValu"
    "e\022=\n\013GetUserList\022\026.google.protobuf.Empty"
    "\032\026.grpc.UserListResponse\0223\n\007GetUser\022\034.go"
    "ogle.protobuf.StringValue\032\n.grpc.User\022F\n"
    "\020SetUserSplitMode\022\032.grpc.UserSplitModeRe"
    "quest\032\026.google.protobuf.Empty\022U\n\030SendBad"
    "EventUserFeedback\022!.grpc.UserBadEventFee"
    "dbackRequest\032\026.google.protobuf.Empty\022B\n\n"
    "LogoutUser\022\034.google.protobuf.StringValue"
    "\032\026.google.protobuf.Empty\022B\n\nRemoveUser\022\034"
    ".google.protobuf.StringValue\032\026.google.pr"
    "otobuf.Empty\022Q\n\026ConfigureUserAppleMail\022\037"
    ".grpc.ConfigureAppleMailRequest\032\026.google"
    ".protobuf.Empty\022O\n\031IsTLSCertificateInsta"
    "lled\022\026.google.protobuf.Empty\032\032.google.pr"
    "otobuf.BoolValue\022G\n\025InstallTLSCertificat"
    "e\022\026.google.protobuf.Empty\032\026.google.proto"
    "buf.Empty\022M\n\025ExportTLSCertificates\022\034.goo"
    "gle.protobuf.StringValue\032\026.google.protob"
    "uf.Empty\022\?\n\016RunEventStream\022\030.grpc.EventS"
    "treamRequest\032\021.grpc.StreamEvent0\001\022A\n\017Sto"
    "pEventStream\022\026.google.protobuf.Empty\032\026.g"
    "oogle.protobuf.Empty\022\?\n\rTriggerRepair\022\026."
    "google.protobuf.Empty\032\026.google.protobuf."
    "EmptyB6Z4github.com/ProtonMail/proton-br"
    "idge/v3/internal/grpcb\006proto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_bridge_2eproto_deps[2] = {
        &::descriptor_table_google_2fprotobuf_2fempty_2eproto,
        &::descriptor_table_google_2fprotobuf_2fwrappers_2eproto,
};
static ::absl::once_flag descriptor_table_bridge_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_bridge_2eproto = {
    false,
    false,
    12589,
    descriptor_table_protodef_bridge_2eproto,
    "bridge.proto",
    &descriptor_table_bridge_2eproto_once,
    descriptor_table_bridge_2eproto_deps,
    2,
    77,
    schemas,
    file_message_globals,
    TableStruct_bridge_2eproto::offsets,
    file_level_enum_descriptors_bridge_2eproto,
    file_level_service_descriptors_bridge_2eproto,
};
namespace grpc {
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
LogLevel_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_bridge_2eproto);
  return file_level_enum_descriptors_bridge_2eproto[0];
}
PROTOBUF_CONSTINIT const uint32_t LogLevel_internal_data_[] = {
    458752u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
UserState_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_bridge_2eproto);
  return file_level_enum_descriptors_bridge_2eproto[1];
}
PROTOBUF_CONSTINIT const uint32_t UserState_internal_data_[] = {
    196608u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
LoginErrorType_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_bridge_2eproto);
  return file_level_enum_descriptors_bridge_2eproto[2];
}
PROTOBUF_CONSTINIT const uint32_t LoginErrorType_internal_data_[] = {
    720896u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
UpdateErrorType_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_bridge_2eproto);
  return file_level_enum_descriptors_bridge_2eproto[3];
}
PROTOBUF_CONSTINIT const uint32_t UpdateErrorType_internal_data_[] = {
    196608u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
DiskCacheErrorType_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_bridge_2eproto);
  return file_level_enum_descriptors_bridge_2eproto[4];
}
PROTOBUF_CONSTINIT const uint32_t DiskCacheErrorType_internal_data_[] = {
    65536u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
MailServerSettingsErrorType_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_bridge_2eproto);
  return file_level_enum_descriptors_bridge_2eproto[5];
}
PROTOBUF_CONSTINIT const uint32_t MailServerSettingsErrorType_internal_data_[] = {
    393216u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
ErrorCode_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_bridge_2eproto);
  return file_level_enum_descriptors_bridge_2eproto[6];
}
PROTOBUF_CONSTINIT const uint32_t ErrorCode_internal_data_[] = {
    196608u, 0u, };
// ===================================================================

AddLogEntryRequest::AddLogEntryRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AddLogEntryRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.AddLogEntryRequest)
}
PROTOBUF_NDEBUG_INLINE AddLogEntryRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::AddLogEntryRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        package_(arena, from.package_),
        message_(arena, from.message_) {}

AddLogEntryRequest::AddLogEntryRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const AddLogEntryRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AddLogEntryRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  AddLogEntryRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.level_ = from._impl_.level_;

  // @@protoc_insertion_point(copy_constructor:grpc.AddLogEntryRequest)
}
PROTOBUF_NDEBUG_INLINE AddLogEntryRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        package_(arena),
        message_(arena) {}

inline void AddLogEntryRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.level_ = {};
}
AddLogEntryRequest::~AddLogEntryRequest() {
  // @@protoc_insertion_point(destructor:grpc.AddLogEntryRequest)
  SharedDtor(*this);
}
inline void AddLogEntryRequest::SharedDtor(MessageLite& self) {
  AddLogEntryRequest& this_ = static_cast<AddLogEntryRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.package_.Destroy();
  this_._impl_.message_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AddLogEntryRequest_class_data_ =
        AddLogEntryRequest::InternalGenerateClassData_(AddLogEntryRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AddLogEntryRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AddLogEntryRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AddLogEntryRequest_class_data_.tc_table);
  return AddLogEntryRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AddLogEntryRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AddLogEntryRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AddLogEntryRequest_globals_));
  return AddLogEntryRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AddLogEntryRequest::ParseTableT_
    AddLogEntryRequest::_table_ =
        AddLogEntryRequest::InternalGenerateParseTable_(AddLogEntryRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void AddLogEntryRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.AddLogEntryRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.package_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.message_.ClearNonDefaultToEmpty();
    }
  }
  _impl_.level_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL AddLogEntryRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const AddLogEntryRequest& this_ = static_cast<const AddLogEntryRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL AddLogEntryRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const AddLogEntryRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.AddLogEntryRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .grpc.LogLevel level = 1;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_level() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_level(), target);
    }
  }

  // string package = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_package().empty()) {
      const ::std::string& _s = this_._internal_package();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.AddLogEntryRequest.package");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // string message = 3;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_message().empty()) {
      const ::std::string& _s = this_._internal_message();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.AddLogEntryRequest.message");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.AddLogEntryRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t AddLogEntryRequest::ByteSizeLong(const MessageLite& base) {
  const AddLogEntryRequest& this_ = static_cast<const AddLogEntryRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t AddLogEntryRequest::ByteSizeLong() const {
  const AddLogEntryRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.AddLogEntryRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // string package = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_package().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_package());
      }
    }
    // string message = 3;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_message().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_message());
      }
    }
    // .grpc.LogLevel level = 1;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_level() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_level());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void AddLogEntryRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<AddLogEntryRequest*>(&to_msg);
  auto& from = static_cast<const AddLogEntryRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.AddLogEntryRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_package().empty()) {
        _this->_internal_set_package(from._internal_package());
      } else {
        if (_this->_impl_.package_.IsDefault()) {
          _this->_internal_set_package("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_message().empty()) {
        _this->_internal_set_message(from._internal_message());
      } else {
        if (_this->_impl_.message_.IsDefault()) {
          _this->_internal_set_message("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_level() != 0) {
        _this->_impl_.level_ = from._impl_.level_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void AddLogEntryRequest::CopyFrom(const AddLogEntryRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.AddLogEntryRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void AddLogEntryRequest::InternalSwap(AddLogEntryRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.package_, &other->_impl_.package_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.message_, &other->_impl_.message_, arena);
  swap(_impl_.level_, other->_impl_.level_);
}

::google::protobuf::Metadata AddLogEntryRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

GuiReadyResponse::GuiReadyResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GuiReadyResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.GuiReadyResponse)
}
GuiReadyResponse::GuiReadyResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const GuiReadyResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GuiReadyResponse_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE GuiReadyResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void GuiReadyResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.showsplashscreen_ = {};
}
GuiReadyResponse::~GuiReadyResponse() {
  // @@protoc_insertion_point(destructor:grpc.GuiReadyResponse)
  SharedDtor(*this);
}
inline void GuiReadyResponse::SharedDtor(MessageLite& self) {
  GuiReadyResponse& this_ = static_cast<GuiReadyResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull GuiReadyResponse_class_data_ =
        GuiReadyResponse::InternalGenerateClassData_(GuiReadyResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GuiReadyResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GuiReadyResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(GuiReadyResponse_class_data_.tc_table);
  return GuiReadyResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GuiReadyResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GuiReadyResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&GuiReadyResponse_globals_));
  return GuiReadyResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const GuiReadyResponse::ParseTableT_
    GuiReadyResponse::_table_ =
        GuiReadyResponse::InternalGenerateParseTable_(GuiReadyResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void GuiReadyResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.GuiReadyResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.showsplashscreen_ = false;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL GuiReadyResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const GuiReadyResponse& this_ = static_cast<const GuiReadyResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL GuiReadyResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const GuiReadyResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.GuiReadyResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // bool showSplashScreen = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_showsplashscreen() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          1, this_._internal_showsplashscreen(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.GuiReadyResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t GuiReadyResponse::ByteSizeLong(const MessageLite& base) {
  const GuiReadyResponse& this_ = static_cast<const GuiReadyResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t GuiReadyResponse::ByteSizeLong() const {
  const GuiReadyResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.GuiReadyResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // bool showSplashScreen = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_showsplashscreen() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void GuiReadyResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<GuiReadyResponse*>(&to_msg);
  auto& from = static_cast<const GuiReadyResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.GuiReadyResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (from._internal_showsplashscreen() != 0) {
      _this->_impl_.showsplashscreen_ = from._impl_.showsplashscreen_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void GuiReadyResponse::CopyFrom(const GuiReadyResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.GuiReadyResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void GuiReadyResponse::InternalSwap(GuiReadyResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.showsplashscreen_, other->_impl_.showsplashscreen_);
}

::google::protobuf::Metadata GuiReadyResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ReportBugRequest::ReportBugRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ReportBugRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.ReportBugRequest)
}
PROTOBUF_NDEBUG_INLINE ReportBugRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::ReportBugRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        ostype_(arena, from.ostype_),
        osversion_(arena, from.osversion_),
        title_(arena, from.title_),
        description_(arena, from.description_),
        address_(arena, from.address_),
        emailclient_(arena, from.emailclient_) {}

ReportBugRequest::ReportBugRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ReportBugRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ReportBugRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ReportBugRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.includelogs_ = from._impl_.includelogs_;

  // @@protoc_insertion_point(copy_constructor:grpc.ReportBugRequest)
}
PROTOBUF_NDEBUG_INLINE ReportBugRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        ostype_(arena),
        osversion_(arena),
        title_(arena),
        description_(arena),
        address_(arena),
        emailclient_(arena) {}

inline void ReportBugRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.includelogs_ = {};
}
ReportBugRequest::~ReportBugRequest() {
  // @@protoc_insertion_point(destructor:grpc.ReportBugRequest)
  SharedDtor(*this);
}
inline void ReportBugRequest::SharedDtor(MessageLite& self) {
  ReportBugRequest& this_ = static_cast<ReportBugRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.ostype_.Destroy();
  this_._impl_.osversion_.Destroy();
  this_._impl_.title_.Destroy();
  this_._impl_.description_.Destroy();
  this_._impl_.address_.Destroy();
  this_._impl_.emailclient_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ReportBugRequest_class_data_ =
        ReportBugRequest::InternalGenerateClassData_(ReportBugRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReportBugRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReportBugRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ReportBugRequest_class_data_.tc_table);
  return ReportBugRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReportBugRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReportBugRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ReportBugRequest_globals_));
  return ReportBugRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ReportBugRequest::ParseTableT_
    ReportBugRequest::_table_ =
        ReportBugRequest::InternalGenerateParseTable_(ReportBugRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ReportBugRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.ReportBugRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.ostype_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.osversion_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.title_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.description_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _impl_.address_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      _impl_.emailclient_.ClearNonDefaultToEmpty();
    }
  }
  _impl_.includelogs_ = false;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ReportBugRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ReportBugRequest& this_ = static_cast<const ReportBugRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ReportBugRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ReportBugRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.ReportBugRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string osType = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_ostype().empty()) {
      const ::std::string& _s = this_._internal_ostype();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.ReportBugRequest.osType");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string osVersion = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_osversion().empty()) {
      const ::std::string& _s = this_._internal_osversion();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.ReportBugRequest.osVersion");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // string title = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_title().empty()) {
      const ::std::string& _s = this_._internal_title();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.ReportBugRequest.title");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  // string description = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_description().empty()) {
      const ::std::string& _s = this_._internal_description();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.ReportBugRequest.description");
      target = stream->WriteStringMaybeAliased(4, _s, target);
    }
  }

  // string address = 5;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (!this_._internal_address().empty()) {
      const ::std::string& _s = this_._internal_address();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.ReportBugRequest.address");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // string emailClient = 6;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (!this_._internal_emailclient().empty()) {
      const ::std::string& _s = this_._internal_emailclient();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.ReportBugRequest.emailClient");
      target = stream->WriteStringMaybeAliased(6, _s, target);
    }
  }

  // bool includeLogs = 7;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_includelogs() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          7, this_._internal_includelogs(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.ReportBugRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ReportBugRequest::ByteSizeLong(const MessageLite& base) {
  const ReportBugRequest& this_ = static_cast<const ReportBugRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ReportBugRequest::ByteSizeLong() const {
  const ReportBugRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.ReportBugRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    // string osType = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_ostype().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_ostype());
      }
    }
    // string osVersion = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_osversion().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_osversion());
      }
    }
    // string title = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_title().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_title());
      }
    }
    // string description = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!this_._internal_description().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_description());
      }
    }
    // string address = 5;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!this_._internal_address().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_address());
      }
    }
    // string emailClient = 6;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (!this_._internal_emailclient().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_emailclient());
      }
    }
    // bool includeLogs = 7;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_includelogs() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ReportBugRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ReportBugRequest*>(&to_msg);
  auto& from = static_cast<const ReportBugRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.ReportBugRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_ostype().empty()) {
        _this->_internal_set_ostype(from._internal_ostype());
      } else {
        if (_this->_impl_.ostype_.IsDefault()) {
          _this->_internal_set_ostype("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_osversion().empty()) {
        _this->_internal_set_osversion(from._internal_osversion());
      } else {
        if (_this->_impl_.osversion_.IsDefault()) {
          _this->_internal_set_osversion("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_title().empty()) {
        _this->_internal_set_title(from._internal_title());
      } else {
        if (_this->_impl_.title_.IsDefault()) {
          _this->_internal_set_title("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!from._internal_description().empty()) {
        _this->_internal_set_description(from._internal_description());
      } else {
        if (_this->_impl_.description_.IsDefault()) {
          _this->_internal_set_description("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!from._internal_address().empty()) {
        _this->_internal_set_address(from._internal_address());
      } else {
        if (_this->_impl_.address_.IsDefault()) {
          _this->_internal_set_address("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (!from._internal_emailclient().empty()) {
        _this->_internal_set_emailclient(from._internal_emailclient());
      } else {
        if (_this->_impl_.emailclient_.IsDefault()) {
          _this->_internal_set_emailclient("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_includelogs() != 0) {
        _this->_impl_.includelogs_ = from._impl_.includelogs_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ReportBugRequest::CopyFrom(const ReportBugRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.ReportBugRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ReportBugRequest::InternalSwap(ReportBugRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.ostype_, &other->_impl_.ostype_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.osversion_, &other->_impl_.osversion_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.title_, &other->_impl_.title_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.description_, &other->_impl_.description_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.address_, &other->_impl_.address_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.emailclient_, &other->_impl_.emailclient_, arena);
  swap(_impl_.includelogs_, other->_impl_.includelogs_);
}

::google::protobuf::Metadata ReportBugRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

LoginRequest::LoginRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.LoginRequest)
}
PROTOBUF_NDEBUG_INLINE LoginRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::LoginRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        username_(arena, from.username_),
        password_(arena, from.password_) {}

LoginRequest::LoginRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const LoginRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  LoginRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.usehvdetails_ = from._impl_.usehvdetails_;

  // @@protoc_insertion_point(copy_constructor:grpc.LoginRequest)
}
PROTOBUF_NDEBUG_INLINE LoginRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        username_(arena),
        password_(arena) {}

inline void LoginRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.usehvdetails_ = {};
}
LoginRequest::~LoginRequest() {
  // @@protoc_insertion_point(destructor:grpc.LoginRequest)
  SharedDtor(*this);
}
inline void LoginRequest::SharedDtor(MessageLite& self) {
  LoginRequest& this_ = static_cast<LoginRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.username_.Destroy();
  this_._impl_.password_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull LoginRequest_class_data_ =
        LoginRequest::InternalGenerateClassData_(LoginRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(LoginRequest_class_data_.tc_table);
  return LoginRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&LoginRequest_globals_));
  return LoginRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const LoginRequest::ParseTableT_
    LoginRequest::_table_ =
        LoginRequest::InternalGenerateParseTable_(LoginRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void LoginRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.LoginRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.username_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.password_.ClearNonDefaultToEmpty();
    }
  }
  _impl_.usehvdetails_ = false;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL LoginRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const LoginRequest& this_ = static_cast<const LoginRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL LoginRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const LoginRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.LoginRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string username = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_username().empty()) {
      const ::std::string& _s = this_._internal_username();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.LoginRequest.username");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // bytes password = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_password().empty()) {
      const ::std::string& _s = this_._internal_password();
      target = stream->WriteBytesMaybeAliased(2, _s, target);
    }
  }

  // optional bool useHvDetails = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(
        3, this_._internal_usehvdetails(), target);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.LoginRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t LoginRequest::ByteSizeLong(const MessageLite& base) {
  const LoginRequest& this_ = static_cast<const LoginRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t LoginRequest::ByteSizeLong() const {
  const LoginRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.LoginRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  total_size += static_cast<bool>(0x00000004U & cached_has_bits) * 2;
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string username = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_username().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_username());
      }
    }
    // bytes password = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_password().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_password());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void LoginRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<LoginRequest*>(&to_msg);
  auto& from = static_cast<const LoginRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.LoginRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_username().empty()) {
        _this->_internal_set_username(from._internal_username());
      } else {
        if (_this->_impl_.username_.IsDefault()) {
          _this->_internal_set_username("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_password().empty()) {
        _this->_internal_set_password(from._internal_password());
      } else {
        if (_this->_impl_.password_.IsDefault()) {
          _this->_internal_set_password("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_impl_.usehvdetails_ = from._impl_.usehvdetails_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void LoginRequest::CopyFrom(const LoginRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.LoginRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void LoginRequest::InternalSwap(LoginRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.username_, &other->_impl_.username_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.password_, &other->_impl_.password_, arena);
  swap(_impl_.usehvdetails_, other->_impl_.usehvdetails_);
}

::google::protobuf::Metadata LoginRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

LoginAbortRequest::LoginAbortRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginAbortRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.LoginAbortRequest)
}
PROTOBUF_NDEBUG_INLINE LoginAbortRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::LoginAbortRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        username_(arena, from.username_) {}

LoginAbortRequest::LoginAbortRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const LoginAbortRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginAbortRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  LoginAbortRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.LoginAbortRequest)
}
PROTOBUF_NDEBUG_INLINE LoginAbortRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        username_(arena) {}

inline void LoginAbortRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
LoginAbortRequest::~LoginAbortRequest() {
  // @@protoc_insertion_point(destructor:grpc.LoginAbortRequest)
  SharedDtor(*this);
}
inline void LoginAbortRequest::SharedDtor(MessageLite& self) {
  LoginAbortRequest& this_ = static_cast<LoginAbortRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.username_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull LoginAbortRequest_class_data_ =
        LoginAbortRequest::InternalGenerateClassData_(LoginAbortRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginAbortRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginAbortRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(LoginAbortRequest_class_data_.tc_table);
  return LoginAbortRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginAbortRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginAbortRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&LoginAbortRequest_globals_));
  return LoginAbortRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const LoginAbortRequest::ParseTableT_
    LoginAbortRequest::_table_ =
        LoginAbortRequest::InternalGenerateParseTable_(LoginAbortRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void LoginAbortRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.LoginAbortRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.username_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL LoginAbortRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const LoginAbortRequest& this_ = static_cast<const LoginAbortRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL LoginAbortRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const LoginAbortRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.LoginAbortRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string username = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_username().empty()) {
      const ::std::string& _s = this_._internal_username();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.LoginAbortRequest.username");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.LoginAbortRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t LoginAbortRequest::ByteSizeLong(const MessageLite& base) {
  const LoginAbortRequest& this_ = static_cast<const LoginAbortRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t LoginAbortRequest::ByteSizeLong() const {
  const LoginAbortRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.LoginAbortRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string username = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_username().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_username());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void LoginAbortRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<LoginAbortRequest*>(&to_msg);
  auto& from = static_cast<const LoginAbortRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.LoginAbortRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_username().empty()) {
      _this->_internal_set_username(from._internal_username());
    } else {
      if (_this->_impl_.username_.IsDefault()) {
        _this->_internal_set_username("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void LoginAbortRequest::CopyFrom(const LoginAbortRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.LoginAbortRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void LoginAbortRequest::InternalSwap(LoginAbortRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.username_, &other->_impl_.username_, arena);
}

::google::protobuf::Metadata LoginAbortRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ImapSmtpSettings::ImapSmtpSettings(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ImapSmtpSettings_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.ImapSmtpSettings)
}
ImapSmtpSettings::ImapSmtpSettings(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ImapSmtpSettings& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ImapSmtpSettings_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE ImapSmtpSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void ImapSmtpSettings::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, imapport_),
           0,
           offsetof(Impl_, usesslforsmtp_) -
               offsetof(Impl_, imapport_) +
               sizeof(Impl_::usesslforsmtp_));
}
ImapSmtpSettings::~ImapSmtpSettings() {
  // @@protoc_insertion_point(destructor:grpc.ImapSmtpSettings)
  SharedDtor(*this);
}
inline void ImapSmtpSettings::SharedDtor(MessageLite& self) {
  ImapSmtpSettings& this_ = static_cast<ImapSmtpSettings&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ImapSmtpSettings_class_data_ =
        ImapSmtpSettings::InternalGenerateClassData_(ImapSmtpSettings_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ImapSmtpSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ImapSmtpSettings_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ImapSmtpSettings_class_data_.tc_table);
  return ImapSmtpSettings_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ImapSmtpSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ImapSmtpSettings_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ImapSmtpSettings_globals_));
  return ImapSmtpSettings_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ImapSmtpSettings::ParseTableT_
    ImapSmtpSettings::_table_ =
        ImapSmtpSettings::InternalGenerateParseTable_(ImapSmtpSettings_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ImapSmtpSettings::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.ImapSmtpSettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    ::memset(&_impl_.imapport_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.usesslforsmtp_) -
        reinterpret_cast<char*>(&_impl_.imapport_)) + sizeof(_impl_.usesslforsmtp_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ImapSmtpSettings::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ImapSmtpSettings& this_ = static_cast<const ImapSmtpSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ImapSmtpSettings::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ImapSmtpSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.ImapSmtpSettings)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // int32 imapPort = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_imapport() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<1>(
              stream, this_._internal_imapport(), target);
    }
  }

  // int32 smtpPort = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_smtpport() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<2>(
              stream, this_._internal_smtpport(), target);
    }
  }

  // bool useSSLForImap = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_usesslforimap() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          3, this_._internal_usesslforimap(), target);
    }
  }

  // bool useSSLForSmtp = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_usesslforsmtp() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          4, this_._internal_usesslforsmtp(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.ImapSmtpSettings)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ImapSmtpSettings::ByteSizeLong(const MessageLite& base) {
  const ImapSmtpSettings& this_ = static_cast<const ImapSmtpSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ImapSmtpSettings::ByteSizeLong() const {
  const ImapSmtpSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.ImapSmtpSettings)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // int32 imapPort = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_imapport() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_imapport());
      }
    }
    // int32 smtpPort = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_smtpport() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_smtpport());
      }
    }
    // bool useSSLForImap = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_usesslforimap() != 0) {
        total_size += 2;
      }
    }
    // bool useSSLForSmtp = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_usesslforsmtp() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ImapSmtpSettings::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ImapSmtpSettings*>(&to_msg);
  auto& from = static_cast<const ImapSmtpSettings&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.ImapSmtpSettings)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_imapport() != 0) {
        _this->_impl_.imapport_ = from._impl_.imapport_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_smtpport() != 0) {
        _this->_impl_.smtpport_ = from._impl_.smtpport_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_usesslforimap() != 0) {
        _this->_impl_.usesslforimap_ = from._impl_.usesslforimap_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_usesslforsmtp() != 0) {
        _this->_impl_.usesslforsmtp_ = from._impl_.usesslforsmtp_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ImapSmtpSettings::CopyFrom(const ImapSmtpSettings& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.ImapSmtpSettings)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ImapSmtpSettings::InternalSwap(ImapSmtpSettings* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(ImapSmtpSettings, _impl_.usesslforsmtp_)
      + sizeof(ImapSmtpSettings::_impl_.usesslforsmtp_)
      - PROTOBUF_FIELD_OFFSET(ImapSmtpSettings, _impl_.imapport_)>(
          reinterpret_cast<char*>(&_impl_.imapport_),
          reinterpret_cast<char*>(&other->_impl_.imapport_));
}

::google::protobuf::Metadata ImapSmtpSettings::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

AvailableKeychainsResponse::AvailableKeychainsResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AvailableKeychainsResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.AvailableKeychainsResponse)
}
PROTOBUF_NDEBUG_INLINE AvailableKeychainsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::AvailableKeychainsResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        keychains_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::grpc::AvailableKeychainsResponse,
              PROTOBUF_FIELD_OFFSET(::grpc::AvailableKeychainsResponse, _impl_.keychains_)>()
          , from.keychains_
        }
     {}

AvailableKeychainsResponse::AvailableKeychainsResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const AvailableKeychainsResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AvailableKeychainsResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  AvailableKeychainsResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.AvailableKeychainsResponse)
}
PROTOBUF_NDEBUG_INLINE AvailableKeychainsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        keychains_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::grpc::AvailableKeychainsResponse,
            PROTOBUF_FIELD_OFFSET(::grpc::AvailableKeychainsResponse, _impl_.keychains_)>()
         }
     {}

inline void AvailableKeychainsResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
AvailableKeychainsResponse::~AvailableKeychainsResponse() {
  // @@protoc_insertion_point(destructor:grpc.AvailableKeychainsResponse)
  SharedDtor(*this);
}
inline void AvailableKeychainsResponse::SharedDtor(MessageLite& self) {
  AvailableKeychainsResponse& this_ = static_cast<AvailableKeychainsResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AvailableKeychainsResponse_class_data_ =
        AvailableKeychainsResponse::InternalGenerateClassData_(AvailableKeychainsResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AvailableKeychainsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AvailableKeychainsResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AvailableKeychainsResponse_class_data_.tc_table);
  return AvailableKeychainsResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AvailableKeychainsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AvailableKeychainsResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AvailableKeychainsResponse_globals_));
  return AvailableKeychainsResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AvailableKeychainsResponse::ParseTableT_
    AvailableKeychainsResponse::_table_ =
        AvailableKeychainsResponse::InternalGenerateParseTable_(AvailableKeychainsResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void AvailableKeychainsResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.AvailableKeychainsResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.keychains_.Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL AvailableKeychainsResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const AvailableKeychainsResponse& this_ = static_cast<const AvailableKeychainsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL AvailableKeychainsResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const AvailableKeychainsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.AvailableKeychainsResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated string keychains = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_keychains_size(); i < n; ++i) {
      const auto& s = this_._internal_keychains().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.AvailableKeychainsResponse.keychains");
      target = stream->WriteString(1, s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.AvailableKeychainsResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t AvailableKeychainsResponse::ByteSizeLong(const MessageLite& base) {
  const AvailableKeychainsResponse& this_ = static_cast<const AvailableKeychainsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t AvailableKeychainsResponse::ByteSizeLong() const {
  const AvailableKeychainsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.AvailableKeychainsResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
   {
    // repeated string keychains = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_keychains().size());
      for (int i = 0, n = this_._internal_keychains().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_keychains().Get(i));
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void AvailableKeychainsResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<AvailableKeychainsResponse*>(&to_msg);
  auto& from = static_cast<const AvailableKeychainsResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.AvailableKeychainsResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _this->_internal_mutable_keychains()->InternalMergeFromWithArena(
        ::google::protobuf::MessageLite::internal_visibility(), arena,
        from._internal_keychains());
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void AvailableKeychainsResponse::CopyFrom(const AvailableKeychainsResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.AvailableKeychainsResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void AvailableKeychainsResponse::InternalSwap(AvailableKeychainsResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.keychains_.InternalSwap(&other->_impl_.keychains_);
}

::google::protobuf::Metadata AvailableKeychainsResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

User::User(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, User_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.User)
}
PROTOBUF_NDEBUG_INLINE User::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::User& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        addresses_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::grpc::User,
              PROTOBUF_FIELD_OFFSET(::grpc::User, _impl_.addresses_)>()
          , from.addresses_
        }
        ,
        id_(arena, from.id_),
        username_(arena, from.username_),
        avatartext_(arena, from.avatartext_),
        password_(arena, from.password_) {}

User::User(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const User& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, User_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  User* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, state_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, state_),
           offsetof(Impl_, totalbytes_) -
               offsetof(Impl_, state_) +
               sizeof(Impl_::totalbytes_));

  // @@protoc_insertion_point(copy_constructor:grpc.User)
}
PROTOBUF_NDEBUG_INLINE User::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        addresses_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::grpc::User,
            PROTOBUF_FIELD_OFFSET(::grpc::User, _impl_.addresses_)>()
         }
        ,
        id_(arena),
        username_(arena),
        avatartext_(arena),
        password_(arena) {}

inline void User::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, state_),
           0,
           offsetof(Impl_, totalbytes_) -
               offsetof(Impl_, state_) +
               sizeof(Impl_::totalbytes_));
}
User::~User() {
  // @@protoc_insertion_point(destructor:grpc.User)
  SharedDtor(*this);
}
inline void User::SharedDtor(MessageLite& self) {
  User& this_ = static_cast<User&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.id_.Destroy();
  this_._impl_.username_.Destroy();
  this_._impl_.avatartext_.Destroy();
  this_._impl_.password_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull User_class_data_ =
        User::InternalGenerateClassData_(User_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
User::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&User_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(User_class_data_.tc_table);
  return User_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
User::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&User_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&User_globals_));
  return User_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const User::ParseTableT_
    User::_table_ =
        User::InternalGenerateParseTable_(User_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void User::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.User)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.addresses_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.id_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.username_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.avatartext_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _impl_.password_.ClearNonDefaultToEmpty();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x000000e0U)) {
    ::memset(&_impl_.state_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.usedbytes_) -
        reinterpret_cast<char*>(&_impl_.state_)) + sizeof(_impl_.usedbytes_));
  }
  _impl_.totalbytes_ = ::int64_t{0};
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL User::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const User& this_ = static_cast<const User&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL User::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const User& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.User)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string id = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_id().empty()) {
      const ::std::string& _s = this_._internal_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.User.id");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string username = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_username().empty()) {
      const ::std::string& _s = this_._internal_username();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.User.username");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // string avatarText = 3;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_avatartext().empty()) {
      const ::std::string& _s = this_._internal_avatartext();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.User.avatarText");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  // .grpc.UserState state = 4;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_state() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          4, this_._internal_state(), target);
    }
  }

  // bool splitMode = 5;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_splitmode() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          5, this_._internal_splitmode(), target);
    }
  }

  // int64 usedBytes = 6;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (this_._internal_usedbytes() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<6>(
              stream, this_._internal_usedbytes(), target);
    }
  }

  // int64 totalBytes = 7;
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (this_._internal_totalbytes() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<7>(
              stream, this_._internal_totalbytes(), target);
    }
  }

  // bytes password = 8;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (!this_._internal_password().empty()) {
      const ::std::string& _s = this_._internal_password();
      target = stream->WriteBytesMaybeAliased(8, _s, target);
    }
  }

  // repeated string addresses = 9;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_addresses_size(); i < n; ++i) {
      const auto& s = this_._internal_addresses().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.User.addresses");
      target = stream->WriteString(9, s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.User)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t User::ByteSizeLong(const MessageLite& base) {
  const User& this_ = static_cast<const User&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t User::ByteSizeLong() const {
  const User& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.User)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // repeated string addresses = 9;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_addresses().size());
      for (int i = 0, n = this_._internal_addresses().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_addresses().Get(i));
      }
    }
    // string id = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_id());
      }
    }
    // string username = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_username().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_username());
      }
    }
    // string avatarText = 3;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!this_._internal_avatartext().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_avatartext());
      }
    }
    // bytes password = 8;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!this_._internal_password().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_password());
      }
    }
    // .grpc.UserState state = 4;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_state() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_state());
      }
    }
    // bool splitMode = 5;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_splitmode() != 0) {
        total_size += 2;
      }
    }
    // int64 usedBytes = 6;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (this_._internal_usedbytes() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_usedbytes());
      }
    }
  }
   {
    // int64 totalBytes = 7;
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      if (this_._internal_totalbytes() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_totalbytes());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void User::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<User*>(&to_msg);
  auto& from = static_cast<const User&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.User)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_addresses()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_addresses());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_id().empty()) {
        _this->_internal_set_id(from._internal_id());
      } else {
        if (_this->_impl_.id_.IsDefault()) {
          _this->_internal_set_id("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_username().empty()) {
        _this->_internal_set_username(from._internal_username());
      } else {
        if (_this->_impl_.username_.IsDefault()) {
          _this->_internal_set_username("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!from._internal_avatartext().empty()) {
        _this->_internal_set_avatartext(from._internal_avatartext());
      } else {
        if (_this->_impl_.avatartext_.IsDefault()) {
          _this->_internal_set_avatartext("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!from._internal_password().empty()) {
        _this->_internal_set_password(from._internal_password());
      } else {
        if (_this->_impl_.password_.IsDefault()) {
          _this->_internal_set_password("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_state() != 0) {
        _this->_impl_.state_ = from._impl_.state_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_splitmode() != 0) {
        _this->_impl_.splitmode_ = from._impl_.splitmode_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (from._internal_usedbytes() != 0) {
        _this->_impl_.usedbytes_ = from._impl_.usedbytes_;
      }
    }
  }
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (from._internal_totalbytes() != 0) {
      _this->_impl_.totalbytes_ = from._impl_.totalbytes_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void User::CopyFrom(const User& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.User)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void User::InternalSwap(User* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.addresses_.InternalSwap(&other->_impl_.addresses_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.id_, &other->_impl_.id_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.username_, &other->_impl_.username_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.avatartext_, &other->_impl_.avatartext_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.password_, &other->_impl_.password_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(User, _impl_.totalbytes_)
      + sizeof(User::_impl_.totalbytes_)
      - PROTOBUF_FIELD_OFFSET(User, _impl_.state_)>(
          reinterpret_cast<char*>(&_impl_.state_),
          reinterpret_cast<char*>(&other->_impl_.state_));
}

::google::protobuf::Metadata User::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UserSplitModeRequest::UserSplitModeRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserSplitModeRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.UserSplitModeRequest)
}
PROTOBUF_NDEBUG_INLINE UserSplitModeRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::UserSplitModeRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        userid_(arena, from.userid_) {}

UserSplitModeRequest::UserSplitModeRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UserSplitModeRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserSplitModeRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UserSplitModeRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.active_ = from._impl_.active_;

  // @@protoc_insertion_point(copy_constructor:grpc.UserSplitModeRequest)
}
PROTOBUF_NDEBUG_INLINE UserSplitModeRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        userid_(arena) {}

inline void UserSplitModeRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.active_ = {};
}
UserSplitModeRequest::~UserSplitModeRequest() {
  // @@protoc_insertion_point(destructor:grpc.UserSplitModeRequest)
  SharedDtor(*this);
}
inline void UserSplitModeRequest::SharedDtor(MessageLite& self) {
  UserSplitModeRequest& this_ = static_cast<UserSplitModeRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.userid_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UserSplitModeRequest_class_data_ =
        UserSplitModeRequest::InternalGenerateClassData_(UserSplitModeRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserSplitModeRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserSplitModeRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UserSplitModeRequest_class_data_.tc_table);
  return UserSplitModeRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserSplitModeRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserSplitModeRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UserSplitModeRequest_globals_));
  return UserSplitModeRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UserSplitModeRequest::ParseTableT_
    UserSplitModeRequest::_table_ =
        UserSplitModeRequest::InternalGenerateParseTable_(UserSplitModeRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UserSplitModeRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.UserSplitModeRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.userid_.ClearNonDefaultToEmpty();
  }
  _impl_.active_ = false;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UserSplitModeRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UserSplitModeRequest& this_ = static_cast<const UserSplitModeRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UserSplitModeRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UserSplitModeRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.UserSplitModeRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string userID = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_userid().empty()) {
      const ::std::string& _s = this_._internal_userid();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.UserSplitModeRequest.userID");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // bool active = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_active() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          2, this_._internal_active(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.UserSplitModeRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UserSplitModeRequest::ByteSizeLong(const MessageLite& base) {
  const UserSplitModeRequest& this_ = static_cast<const UserSplitModeRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UserSplitModeRequest::ByteSizeLong() const {
  const UserSplitModeRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.UserSplitModeRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string userID = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_userid().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_userid());
      }
    }
    // bool active = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_active() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UserSplitModeRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UserSplitModeRequest*>(&to_msg);
  auto& from = static_cast<const UserSplitModeRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.UserSplitModeRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_userid().empty()) {
        _this->_internal_set_userid(from._internal_userid());
      } else {
        if (_this->_impl_.userid_.IsDefault()) {
          _this->_internal_set_userid("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_active() != 0) {
        _this->_impl_.active_ = from._impl_.active_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UserSplitModeRequest::CopyFrom(const UserSplitModeRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.UserSplitModeRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UserSplitModeRequest::InternalSwap(UserSplitModeRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.userid_, &other->_impl_.userid_, arena);
  swap(_impl_.active_, other->_impl_.active_);
}

::google::protobuf::Metadata UserSplitModeRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UserBadEventFeedbackRequest::UserBadEventFeedbackRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserBadEventFeedbackRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.UserBadEventFeedbackRequest)
}
PROTOBUF_NDEBUG_INLINE UserBadEventFeedbackRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::UserBadEventFeedbackRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        userid_(arena, from.userid_) {}

UserBadEventFeedbackRequest::UserBadEventFeedbackRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UserBadEventFeedbackRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserBadEventFeedbackRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UserBadEventFeedbackRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.doresync_ = from._impl_.doresync_;

  // @@protoc_insertion_point(copy_constructor:grpc.UserBadEventFeedbackRequest)
}
PROTOBUF_NDEBUG_INLINE UserBadEventFeedbackRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        userid_(arena) {}

inline void UserBadEventFeedbackRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.doresync_ = {};
}
UserBadEventFeedbackRequest::~UserBadEventFeedbackRequest() {
  // @@protoc_insertion_point(destructor:grpc.UserBadEventFeedbackRequest)
  SharedDtor(*this);
}
inline void UserBadEventFeedbackRequest::SharedDtor(MessageLite& self) {
  UserBadEventFeedbackRequest& this_ = static_cast<UserBadEventFeedbackRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.userid_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UserBadEventFeedbackRequest_class_data_ =
        UserBadEventFeedbackRequest::InternalGenerateClassData_(UserBadEventFeedbackRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserBadEventFeedbackRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserBadEventFeedbackRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UserBadEventFeedbackRequest_class_data_.tc_table);
  return UserBadEventFeedbackRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserBadEventFeedbackRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserBadEventFeedbackRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UserBadEventFeedbackRequest_globals_));
  return UserBadEventFeedbackRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UserBadEventFeedbackRequest::ParseTableT_
    UserBadEventFeedbackRequest::_table_ =
        UserBadEventFeedbackRequest::InternalGenerateParseTable_(UserBadEventFeedbackRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UserBadEventFeedbackRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.UserBadEventFeedbackRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.userid_.ClearNonDefaultToEmpty();
  }
  _impl_.doresync_ = false;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UserBadEventFeedbackRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UserBadEventFeedbackRequest& this_ = static_cast<const UserBadEventFeedbackRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UserBadEventFeedbackRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UserBadEventFeedbackRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.UserBadEventFeedbackRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string userID = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_userid().empty()) {
      const ::std::string& _s = this_._internal_userid();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.UserBadEventFeedbackRequest.userID");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // bool doResync = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_doresync() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          2, this_._internal_doresync(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.UserBadEventFeedbackRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UserBadEventFeedbackRequest::ByteSizeLong(const MessageLite& base) {
  const UserBadEventFeedbackRequest& this_ = static_cast<const UserBadEventFeedbackRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UserBadEventFeedbackRequest::ByteSizeLong() const {
  const UserBadEventFeedbackRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.UserBadEventFeedbackRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string userID = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_userid().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_userid());
      }
    }
    // bool doResync = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_doresync() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UserBadEventFeedbackRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UserBadEventFeedbackRequest*>(&to_msg);
  auto& from = static_cast<const UserBadEventFeedbackRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.UserBadEventFeedbackRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_userid().empty()) {
        _this->_internal_set_userid(from._internal_userid());
      } else {
        if (_this->_impl_.userid_.IsDefault()) {
          _this->_internal_set_userid("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_doresync() != 0) {
        _this->_impl_.doresync_ = from._impl_.doresync_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UserBadEventFeedbackRequest::CopyFrom(const UserBadEventFeedbackRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.UserBadEventFeedbackRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UserBadEventFeedbackRequest::InternalSwap(UserBadEventFeedbackRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.userid_, &other->_impl_.userid_, arena);
  swap(_impl_.doresync_, other->_impl_.doresync_);
}

::google::protobuf::Metadata UserBadEventFeedbackRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UserListResponse::UserListResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserListResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.UserListResponse)
}
PROTOBUF_NDEBUG_INLINE UserListResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::UserListResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        users_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::grpc::UserListResponse,
              PROTOBUF_FIELD_OFFSET(::grpc::UserListResponse, _impl_.users_)>()
          , from.users_
        }
     {}

UserListResponse::UserListResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UserListResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserListResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UserListResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.UserListResponse)
}
PROTOBUF_NDEBUG_INLINE UserListResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        users_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::grpc::UserListResponse,
            PROTOBUF_FIELD_OFFSET(::grpc::UserListResponse, _impl_.users_)>()
         }
     {}

inline void UserListResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
UserListResponse::~UserListResponse() {
  // @@protoc_insertion_point(destructor:grpc.UserListResponse)
  SharedDtor(*this);
}
inline void UserListResponse::SharedDtor(MessageLite& self) {
  UserListResponse& this_ = static_cast<UserListResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UserListResponse_class_data_ =
        UserListResponse::InternalGenerateClassData_(UserListResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserListResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserListResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UserListResponse_class_data_.tc_table);
  return UserListResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserListResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserListResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UserListResponse_globals_));
  return UserListResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UserListResponse::ParseTableT_
    UserListResponse::_table_ =
        UserListResponse::InternalGenerateParseTable_(UserListResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UserListResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.UserListResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.users_.Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UserListResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UserListResponse& this_ = static_cast<const UserListResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UserListResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UserListResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.UserListResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .grpc.User users = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_users_size());
         i < n; i++) {
      const auto& repfield = this_._internal_users().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              1, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.UserListResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UserListResponse::ByteSizeLong(const MessageLite& base) {
  const UserListResponse& this_ = static_cast<const UserListResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UserListResponse::ByteSizeLong() const {
  const UserListResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.UserListResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
   {
    // repeated .grpc.User users = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_users_size();
      for (const auto& msg : this_._internal_users()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UserListResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UserListResponse*>(&to_msg);
  auto& from = static_cast<const UserListResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.UserListResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _this->_internal_mutable_users()->InternalMergeFromWithArena(
        ::google::protobuf::MessageLite::internal_visibility(), arena,
        from._internal_users());
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UserListResponse::CopyFrom(const UserListResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.UserListResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UserListResponse::InternalSwap(UserListResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.users_.InternalSwap(&other->_impl_.users_);
}

::google::protobuf::Metadata UserListResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ConfigureAppleMailRequest::ConfigureAppleMailRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ConfigureAppleMailRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.ConfigureAppleMailRequest)
}
PROTOBUF_NDEBUG_INLINE ConfigureAppleMailRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::ConfigureAppleMailRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        userid_(arena, from.userid_),
        address_(arena, from.address_) {}

ConfigureAppleMailRequest::ConfigureAppleMailRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ConfigureAppleMailRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ConfigureAppleMailRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ConfigureAppleMailRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.ConfigureAppleMailRequest)
}
PROTOBUF_NDEBUG_INLINE ConfigureAppleMailRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        userid_(arena),
        address_(arena) {}

inline void ConfigureAppleMailRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ConfigureAppleMailRequest::~ConfigureAppleMailRequest() {
  // @@protoc_insertion_point(destructor:grpc.ConfigureAppleMailRequest)
  SharedDtor(*this);
}
inline void ConfigureAppleMailRequest::SharedDtor(MessageLite& self) {
  ConfigureAppleMailRequest& this_ = static_cast<ConfigureAppleMailRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.userid_.Destroy();
  this_._impl_.address_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ConfigureAppleMailRequest_class_data_ =
        ConfigureAppleMailRequest::InternalGenerateClassData_(ConfigureAppleMailRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ConfigureAppleMailRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ConfigureAppleMailRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ConfigureAppleMailRequest_class_data_.tc_table);
  return ConfigureAppleMailRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ConfigureAppleMailRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ConfigureAppleMailRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ConfigureAppleMailRequest_globals_));
  return ConfigureAppleMailRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ConfigureAppleMailRequest::ParseTableT_
    ConfigureAppleMailRequest::_table_ =
        ConfigureAppleMailRequest::InternalGenerateParseTable_(ConfigureAppleMailRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ConfigureAppleMailRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.ConfigureAppleMailRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.userid_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.address_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ConfigureAppleMailRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ConfigureAppleMailRequest& this_ = static_cast<const ConfigureAppleMailRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ConfigureAppleMailRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ConfigureAppleMailRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.ConfigureAppleMailRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string userID = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_userid().empty()) {
      const ::std::string& _s = this_._internal_userid();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.ConfigureAppleMailRequest.userID");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string address = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_address().empty()) {
      const ::std::string& _s = this_._internal_address();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.ConfigureAppleMailRequest.address");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.ConfigureAppleMailRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ConfigureAppleMailRequest::ByteSizeLong(const MessageLite& base) {
  const ConfigureAppleMailRequest& this_ = static_cast<const ConfigureAppleMailRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ConfigureAppleMailRequest::ByteSizeLong() const {
  const ConfigureAppleMailRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.ConfigureAppleMailRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string userID = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_userid().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_userid());
      }
    }
    // string address = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_address().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_address());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ConfigureAppleMailRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ConfigureAppleMailRequest*>(&to_msg);
  auto& from = static_cast<const ConfigureAppleMailRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.ConfigureAppleMailRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_userid().empty()) {
        _this->_internal_set_userid(from._internal_userid());
      } else {
        if (_this->_impl_.userid_.IsDefault()) {
          _this->_internal_set_userid("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_address().empty()) {
        _this->_internal_set_address(from._internal_address());
      } else {
        if (_this->_impl_.address_.IsDefault()) {
          _this->_internal_set_address("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ConfigureAppleMailRequest::CopyFrom(const ConfigureAppleMailRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.ConfigureAppleMailRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ConfigureAppleMailRequest::InternalSwap(ConfigureAppleMailRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.userid_, &other->_impl_.userid_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.address_, &other->_impl_.address_, arena);
}

::google::protobuf::Metadata ConfigureAppleMailRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

EventStreamRequest::EventStreamRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, EventStreamRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.EventStreamRequest)
}
PROTOBUF_NDEBUG_INLINE EventStreamRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::EventStreamRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        clientplatform_(arena, from.clientplatform_) {}

EventStreamRequest::EventStreamRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const EventStreamRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, EventStreamRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  EventStreamRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.EventStreamRequest)
}
PROTOBUF_NDEBUG_INLINE EventStreamRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        clientplatform_(arena) {}

inline void EventStreamRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
EventStreamRequest::~EventStreamRequest() {
  // @@protoc_insertion_point(destructor:grpc.EventStreamRequest)
  SharedDtor(*this);
}
inline void EventStreamRequest::SharedDtor(MessageLite& self) {
  EventStreamRequest& this_ = static_cast<EventStreamRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.clientplatform_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull EventStreamRequest_class_data_ =
        EventStreamRequest::InternalGenerateClassData_(EventStreamRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
EventStreamRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&EventStreamRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(EventStreamRequest_class_data_.tc_table);
  return EventStreamRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
EventStreamRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&EventStreamRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&EventStreamRequest_globals_));
  return EventStreamRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const EventStreamRequest::ParseTableT_
    EventStreamRequest::_table_ =
        EventStreamRequest::InternalGenerateParseTable_(EventStreamRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void EventStreamRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.EventStreamRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.clientplatform_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL EventStreamRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const EventStreamRequest& this_ = static_cast<const EventStreamRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL EventStreamRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const EventStreamRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.EventStreamRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string ClientPlatform = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_clientplatform().empty()) {
      const ::std::string& _s = this_._internal_clientplatform();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.EventStreamRequest.ClientPlatform");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.EventStreamRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t EventStreamRequest::ByteSizeLong(const MessageLite& base) {
  const EventStreamRequest& this_ = static_cast<const EventStreamRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t EventStreamRequest::ByteSizeLong() const {
  const EventStreamRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.EventStreamRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string ClientPlatform = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_clientplatform().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_clientplatform());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void EventStreamRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<EventStreamRequest*>(&to_msg);
  auto& from = static_cast<const EventStreamRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.EventStreamRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_clientplatform().empty()) {
      _this->_internal_set_clientplatform(from._internal_clientplatform());
    } else {
      if (_this->_impl_.clientplatform_.IsDefault()) {
        _this->_internal_set_clientplatform("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void EventStreamRequest::CopyFrom(const EventStreamRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.EventStreamRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void EventStreamRequest::InternalSwap(EventStreamRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.clientplatform_, &other->_impl_.clientplatform_, arena);
}

::google::protobuf::Metadata EventStreamRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void StreamEvent::set_allocated_app(::grpc::AppEvent* PROTOBUF_NULLABLE app) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (app) {
    ::google::protobuf::Arena* submessage_arena = app->GetArena();
    if (message_arena != submessage_arena) {
      app = ::google::protobuf::internal::GetOwnedMessage(message_arena, app, submessage_arena);
    }
    set_has_app();
    _impl_.event_.app_ = app;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.StreamEvent.app)
}
void StreamEvent::set_allocated_login(::grpc::LoginEvent* PROTOBUF_NULLABLE login) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (login) {
    ::google::protobuf::Arena* submessage_arena = login->GetArena();
    if (message_arena != submessage_arena) {
      login = ::google::protobuf::internal::GetOwnedMessage(message_arena, login, submessage_arena);
    }
    set_has_login();
    _impl_.event_.login_ = login;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.StreamEvent.login)
}
void StreamEvent::set_allocated_update(::grpc::UpdateEvent* PROTOBUF_NULLABLE update) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (update) {
    ::google::protobuf::Arena* submessage_arena = update->GetArena();
    if (message_arena != submessage_arena) {
      update = ::google::protobuf::internal::GetOwnedMessage(message_arena, update, submessage_arena);
    }
    set_has_update();
    _impl_.event_.update_ = update;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.StreamEvent.update)
}
void StreamEvent::set_allocated_cache(::grpc::DiskCacheEvent* PROTOBUF_NULLABLE cache) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (cache) {
    ::google::protobuf::Arena* submessage_arena = cache->GetArena();
    if (message_arena != submessage_arena) {
      cache = ::google::protobuf::internal::GetOwnedMessage(message_arena, cache, submessage_arena);
    }
    set_has_cache();
    _impl_.event_.cache_ = cache;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.StreamEvent.cache)
}
void StreamEvent::set_allocated_mailserversettings(::grpc::MailServerSettingsEvent* PROTOBUF_NULLABLE mailserversettings) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (mailserversettings) {
    ::google::protobuf::Arena* submessage_arena = mailserversettings->GetArena();
    if (message_arena != submessage_arena) {
      mailserversettings = ::google::protobuf::internal::GetOwnedMessage(message_arena, mailserversettings, submessage_arena);
    }
    set_has_mailserversettings();
    _impl_.event_.mailserversettings_ = mailserversettings;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.StreamEvent.mailServerSettings)
}
void StreamEvent::set_allocated_keychain(::grpc::KeychainEvent* PROTOBUF_NULLABLE keychain) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (keychain) {
    ::google::protobuf::Arena* submessage_arena = keychain->GetArena();
    if (message_arena != submessage_arena) {
      keychain = ::google::protobuf::internal::GetOwnedMessage(message_arena, keychain, submessage_arena);
    }
    set_has_keychain();
    _impl_.event_.keychain_ = keychain;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.StreamEvent.keychain)
}
void StreamEvent::set_allocated_mail(::grpc::MailEvent* PROTOBUF_NULLABLE mail) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (mail) {
    ::google::protobuf::Arena* submessage_arena = mail->GetArena();
    if (message_arena != submessage_arena) {
      mail = ::google::protobuf::internal::GetOwnedMessage(message_arena, mail, submessage_arena);
    }
    set_has_mail();
    _impl_.event_.mail_ = mail;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.StreamEvent.mail)
}
void StreamEvent::set_allocated_user(::grpc::UserEvent* PROTOBUF_NULLABLE user) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (user) {
    ::google::protobuf::Arena* submessage_arena = user->GetArena();
    if (message_arena != submessage_arena) {
      user = ::google::protobuf::internal::GetOwnedMessage(message_arena, user, submessage_arena);
    }
    set_has_user();
    _impl_.event_.user_ = user;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.StreamEvent.user)
}
void StreamEvent::set_allocated_genericerror(::grpc::GenericErrorEvent* PROTOBUF_NULLABLE genericerror) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (genericerror) {
    ::google::protobuf::Arena* submessage_arena = genericerror->GetArena();
    if (message_arena != submessage_arena) {
      genericerror = ::google::protobuf::internal::GetOwnedMessage(message_arena, genericerror, submessage_arena);
    }
    set_has_genericerror();
    _impl_.event_.genericerror_ = genericerror;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.StreamEvent.genericError)
}
StreamEvent::StreamEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.StreamEvent)
}
PROTOBUF_NDEBUG_INLINE StreamEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::StreamEvent& from_msg)
      : event_{},
        _cached_size_{0},
        _oneof_case_{from._oneof_case_[0]} {}

StreamEvent::StreamEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const StreamEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  StreamEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  switch (event_case()) {
    case EVENT_NOT_SET:
      break;
      case kApp:
        _impl_.event_.app_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.app_);
        break;
      case kLogin:
        _impl_.event_.login_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.login_);
        break;
      case kUpdate:
        _impl_.event_.update_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.update_);
        break;
      case kCache:
        _impl_.event_.cache_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.cache_);
        break;
      case kMailServerSettings:
        _impl_.event_.mailserversettings_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.mailserversettings_);
        break;
      case kKeychain:
        _impl_.event_.keychain_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.keychain_);
        break;
      case kMail:
        _impl_.event_.mail_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.mail_);
        break;
      case kUser:
        _impl_.event_.user_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.user_);
        break;
      case kGenericError:
        _impl_.event_.genericerror_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.genericerror_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:grpc.StreamEvent)
}
PROTOBUF_NDEBUG_INLINE StreamEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

inline void StreamEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
StreamEvent::~StreamEvent() {
  // @@protoc_insertion_point(destructor:grpc.StreamEvent)
  SharedDtor(*this);
}
inline void StreamEvent::SharedDtor(MessageLite& self) {
  StreamEvent& this_ = static_cast<StreamEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  if (this_.has_event()) {
    this_.clear_event();
  }
  this_._impl_.~Impl_();
}

void StreamEvent::clear_event() {
// @@protoc_insertion_point(one_of_clear_start:grpc.StreamEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (event_case()) {
    case kApp: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.app_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.app_);
      }
      break;
    }
    case kLogin: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.login_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.login_);
      }
      break;
    }
    case kUpdate: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.update_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.update_);
      }
      break;
    }
    case kCache: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.cache_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.cache_);
      }
      break;
    }
    case kMailServerSettings: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.mailserversettings_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.mailserversettings_);
      }
      break;
    }
    case kKeychain: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.keychain_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.keychain_);
      }
      break;
    }
    case kMail: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.mail_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.mail_);
      }
      break;
    }
    case kUser: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.user_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.user_);
      }
      break;
    }
    case kGenericError: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.genericerror_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.genericerror_);
      }
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = EVENT_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull StreamEvent_class_data_ =
        StreamEvent::InternalGenerateClassData_(StreamEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(StreamEvent_class_data_.tc_table);
  return StreamEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&StreamEvent_globals_));
  return StreamEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const StreamEvent::ParseTableT_
    StreamEvent::_table_ =
        StreamEvent::InternalGenerateParseTable_(StreamEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void StreamEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.StreamEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  clear_event();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL StreamEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const StreamEvent& this_ = static_cast<const StreamEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL StreamEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const StreamEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.StreamEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  switch (this_.event_case()) {
    case kApp: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          1, *this_._impl_.event_.app_, this_._impl_.event_.app_->GetCachedSize(), target,
          stream);
      break;
    }
    case kLogin: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          2, *this_._impl_.event_.login_, this_._impl_.event_.login_->GetCachedSize(), target,
          stream);
      break;
    }
    case kUpdate: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          3, *this_._impl_.event_.update_, this_._impl_.event_.update_->GetCachedSize(), target,
          stream);
      break;
    }
    case kCache: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          4, *this_._impl_.event_.cache_, this_._impl_.event_.cache_->GetCachedSize(), target,
          stream);
      break;
    }
    case kMailServerSettings: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          5, *this_._impl_.event_.mailserversettings_, this_._impl_.event_.mailserversettings_->GetCachedSize(), target,
          stream);
      break;
    }
    case kKeychain: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          6, *this_._impl_.event_.keychain_, this_._impl_.event_.keychain_->GetCachedSize(), target,
          stream);
      break;
    }
    case kMail: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          7, *this_._impl_.event_.mail_, this_._impl_.event_.mail_->GetCachedSize(), target,
          stream);
      break;
    }
    case kUser: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          8, *this_._impl_.event_.user_, this_._impl_.event_.user_->GetCachedSize(), target,
          stream);
      break;
    }
    case kGenericError: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          9, *this_._impl_.event_.genericerror_, this_._impl_.event_.genericerror_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.StreamEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t StreamEvent::ByteSizeLong(const MessageLite& base) {
  const StreamEvent& this_ = static_cast<const StreamEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t StreamEvent::ByteSizeLong() const {
  const StreamEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.StreamEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  switch (this_.event_case()) {
    // .grpc.AppEvent app = 1;
    case kApp: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.app_);
      break;
    }
    // .grpc.LoginEvent login = 2;
    case kLogin: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.login_);
      break;
    }
    // .grpc.UpdateEvent update = 3;
    case kUpdate: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.update_);
      break;
    }
    // .grpc.DiskCacheEvent cache = 4;
    case kCache: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.cache_);
      break;
    }
    // .grpc.MailServerSettingsEvent mailServerSettings = 5;
    case kMailServerSettings: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.mailserversettings_);
      break;
    }
    // .grpc.KeychainEvent keychain = 6;
    case kKeychain: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.keychain_);
      break;
    }
    // .grpc.MailEvent mail = 7;
    case kMail: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.mail_);
      break;
    }
    // .grpc.UserEvent user = 8;
    case kUser: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.user_);
      break;
    }
    // .grpc.GenericErrorEvent genericError = 9;
    case kGenericError: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.genericerror_);
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void StreamEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<StreamEvent*>(&to_msg);
  auto& from = static_cast<const StreamEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.StreamEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_event();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kApp: {
        if (oneof_needs_init) {
          _this->_impl_.event_.app_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.app_);
        } else {
          _this->_impl_.event_.app_->MergeFrom(*from._impl_.event_.app_);
        }
        break;
      }
      case kLogin: {
        if (oneof_needs_init) {
          _this->_impl_.event_.login_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.login_);
        } else {
          _this->_impl_.event_.login_->MergeFrom(*from._impl_.event_.login_);
        }
        break;
      }
      case kUpdate: {
        if (oneof_needs_init) {
          _this->_impl_.event_.update_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.update_);
        } else {
          _this->_impl_.event_.update_->MergeFrom(*from._impl_.event_.update_);
        }
        break;
      }
      case kCache: {
        if (oneof_needs_init) {
          _this->_impl_.event_.cache_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.cache_);
        } else {
          _this->_impl_.event_.cache_->MergeFrom(*from._impl_.event_.cache_);
        }
        break;
      }
      case kMailServerSettings: {
        if (oneof_needs_init) {
          _this->_impl_.event_.mailserversettings_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.mailserversettings_);
        } else {
          _this->_impl_.event_.mailserversettings_->MergeFrom(*from._impl_.event_.mailserversettings_);
        }
        break;
      }
      case kKeychain: {
        if (oneof_needs_init) {
          _this->_impl_.event_.keychain_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.keychain_);
        } else {
          _this->_impl_.event_.keychain_->MergeFrom(*from._impl_.event_.keychain_);
        }
        break;
      }
      case kMail: {
        if (oneof_needs_init) {
          _this->_impl_.event_.mail_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.mail_);
        } else {
          _this->_impl_.event_.mail_->MergeFrom(*from._impl_.event_.mail_);
        }
        break;
      }
      case kUser: {
        if (oneof_needs_init) {
          _this->_impl_.event_.user_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.user_);
        } else {
          _this->_impl_.event_.user_->MergeFrom(*from._impl_.event_.user_);
        }
        break;
      }
      case kGenericError: {
        if (oneof_needs_init) {
          _this->_impl_.event_.genericerror_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.genericerror_);
        } else {
          _this->_impl_.event_.genericerror_->MergeFrom(*from._impl_.event_.genericerror_);
        }
        break;
      }
      case EVENT_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void StreamEvent::CopyFrom(const StreamEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.StreamEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void StreamEvent::InternalSwap(StreamEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_.event_, other->_impl_.event_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata StreamEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void AppEvent::set_allocated_internetstatus(::grpc::InternetStatusEvent* PROTOBUF_NULLABLE internetstatus) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (internetstatus) {
    ::google::protobuf::Arena* submessage_arena = internetstatus->GetArena();
    if (message_arena != submessage_arena) {
      internetstatus = ::google::protobuf::internal::GetOwnedMessage(message_arena, internetstatus, submessage_arena);
    }
    set_has_internetstatus();
    _impl_.event_.internetstatus_ = internetstatus;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.AppEvent.internetStatus)
}
void AppEvent::set_allocated_toggleautostartfinished(::grpc::ToggleAutostartFinishedEvent* PROTOBUF_NULLABLE toggleautostartfinished) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (toggleautostartfinished) {
    ::google::protobuf::Arena* submessage_arena = toggleautostartfinished->GetArena();
    if (message_arena != submessage_arena) {
      toggleautostartfinished = ::google::protobuf::internal::GetOwnedMessage(message_arena, toggleautostartfinished, submessage_arena);
    }
    set_has_toggleautostartfinished();
    _impl_.event_.toggleautostartfinished_ = toggleautostartfinished;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.AppEvent.toggleAutostartFinished)
}
void AppEvent::set_allocated_resetfinished(::grpc::ResetFinishedEvent* PROTOBUF_NULLABLE resetfinished) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (resetfinished) {
    ::google::protobuf::Arena* submessage_arena = resetfinished->GetArena();
    if (message_arena != submessage_arena) {
      resetfinished = ::google::protobuf::internal::GetOwnedMessage(message_arena, resetfinished, submessage_arena);
    }
    set_has_resetfinished();
    _impl_.event_.resetfinished_ = resetfinished;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.AppEvent.resetFinished)
}
void AppEvent::set_allocated_reportbugfinished(::grpc::ReportBugFinishedEvent* PROTOBUF_NULLABLE reportbugfinished) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (reportbugfinished) {
    ::google::protobuf::Arena* submessage_arena = reportbugfinished->GetArena();
    if (message_arena != submessage_arena) {
      reportbugfinished = ::google::protobuf::internal::GetOwnedMessage(message_arena, reportbugfinished, submessage_arena);
    }
    set_has_reportbugfinished();
    _impl_.event_.reportbugfinished_ = reportbugfinished;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.AppEvent.reportBugFinished)
}
void AppEvent::set_allocated_reportbugsuccess(::grpc::ReportBugSuccessEvent* PROTOBUF_NULLABLE reportbugsuccess) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (reportbugsuccess) {
    ::google::protobuf::Arena* submessage_arena = reportbugsuccess->GetArena();
    if (message_arena != submessage_arena) {
      reportbugsuccess = ::google::protobuf::internal::GetOwnedMessage(message_arena, reportbugsuccess, submessage_arena);
    }
    set_has_reportbugsuccess();
    _impl_.event_.reportbugsuccess_ = reportbugsuccess;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.AppEvent.reportBugSuccess)
}
void AppEvent::set_allocated_reportbugerror(::grpc::ReportBugErrorEvent* PROTOBUF_NULLABLE reportbugerror) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (reportbugerror) {
    ::google::protobuf::Arena* submessage_arena = reportbugerror->GetArena();
    if (message_arena != submessage_arena) {
      reportbugerror = ::google::protobuf::internal::GetOwnedMessage(message_arena, reportbugerror, submessage_arena);
    }
    set_has_reportbugerror();
    _impl_.event_.reportbugerror_ = reportbugerror;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.AppEvent.reportBugError)
}
void AppEvent::set_allocated_showmainwindow(::grpc::ShowMainWindowEvent* PROTOBUF_NULLABLE showmainwindow) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (showmainwindow) {
    ::google::protobuf::Arena* submessage_arena = showmainwindow->GetArena();
    if (message_arena != submessage_arena) {
      showmainwindow = ::google::protobuf::internal::GetOwnedMessage(message_arena, showmainwindow, submessage_arena);
    }
    set_has_showmainwindow();
    _impl_.event_.showmainwindow_ = showmainwindow;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.AppEvent.showMainWindow)
}
void AppEvent::set_allocated_reportbugfallback(::grpc::ReportBugFallbackEvent* PROTOBUF_NULLABLE reportbugfallback) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (reportbugfallback) {
    ::google::protobuf::Arena* submessage_arena = reportbugfallback->GetArena();
    if (message_arena != submessage_arena) {
      reportbugfallback = ::google::protobuf::internal::GetOwnedMessage(message_arena, reportbugfallback, submessage_arena);
    }
    set_has_reportbugfallback();
    _impl_.event_.reportbugfallback_ = reportbugfallback;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.AppEvent.reportBugFallback)
}
void AppEvent::set_allocated_certificateinstallsuccess(::grpc::CertificateInstallSuccessEvent* PROTOBUF_NULLABLE certificateinstallsuccess) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (certificateinstallsuccess) {
    ::google::protobuf::Arena* submessage_arena = certificateinstallsuccess->GetArena();
    if (message_arena != submessage_arena) {
      certificateinstallsuccess = ::google::protobuf::internal::GetOwnedMessage(message_arena, certificateinstallsuccess, submessage_arena);
    }
    set_has_certificateinstallsuccess();
    _impl_.event_.certificateinstallsuccess_ = certificateinstallsuccess;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.AppEvent.certificateInstallSuccess)
}
void AppEvent::set_allocated_certificateinstallcanceled(::grpc::CertificateInstallCanceledEvent* PROTOBUF_NULLABLE certificateinstallcanceled) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (certificateinstallcanceled) {
    ::google::protobuf::Arena* submessage_arena = certificateinstallcanceled->GetArena();
    if (message_arena != submessage_arena) {
      certificateinstallcanceled = ::google::protobuf::internal::GetOwnedMessage(message_arena, certificateinstallcanceled, submessage_arena);
    }
    set_has_certificateinstallcanceled();
    _impl_.event_.certificateinstallcanceled_ = certificateinstallcanceled;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.AppEvent.certificateInstallCanceled)
}
void AppEvent::set_allocated_certificateinstallfailed(::grpc::CertificateInstallFailedEvent* PROTOBUF_NULLABLE certificateinstallfailed) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (certificateinstallfailed) {
    ::google::protobuf::Arena* submessage_arena = certificateinstallfailed->GetArena();
    if (message_arena != submessage_arena) {
      certificateinstallfailed = ::google::protobuf::internal::GetOwnedMessage(message_arena, certificateinstallfailed, submessage_arena);
    }
    set_has_certificateinstallfailed();
    _impl_.event_.certificateinstallfailed_ = certificateinstallfailed;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.AppEvent.certificateInstallFailed)
}
void AppEvent::set_allocated_knowledgebasesuggestions(::grpc::KnowledgeBaseSuggestionsEvent* PROTOBUF_NULLABLE knowledgebasesuggestions) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (knowledgebasesuggestions) {
    ::google::protobuf::Arena* submessage_arena = knowledgebasesuggestions->GetArena();
    if (message_arena != submessage_arena) {
      knowledgebasesuggestions = ::google::protobuf::internal::GetOwnedMessage(message_arena, knowledgebasesuggestions, submessage_arena);
    }
    set_has_knowledgebasesuggestions();
    _impl_.event_.knowledgebasesuggestions_ = knowledgebasesuggestions;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.AppEvent.knowledgeBaseSuggestions)
}
void AppEvent::set_allocated_repairstarted(::grpc::RepairStartedEvent* PROTOBUF_NULLABLE repairstarted) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (repairstarted) {
    ::google::protobuf::Arena* submessage_arena = repairstarted->GetArena();
    if (message_arena != submessage_arena) {
      repairstarted = ::google::protobuf::internal::GetOwnedMessage(message_arena, repairstarted, submessage_arena);
    }
    set_has_repairstarted();
    _impl_.event_.repairstarted_ = repairstarted;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.AppEvent.repairStarted)
}
void AppEvent::set_allocated_allusersloaded(::grpc::AllUsersLoadedEvent* PROTOBUF_NULLABLE allusersloaded) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (allusersloaded) {
    ::google::protobuf::Arena* submessage_arena = allusersloaded->GetArena();
    if (message_arena != submessage_arena) {
      allusersloaded = ::google::protobuf::internal::GetOwnedMessage(message_arena, allusersloaded, submessage_arena);
    }
    set_has_allusersloaded();
    _impl_.event_.allusersloaded_ = allusersloaded;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.AppEvent.allUsersLoaded)
}
void AppEvent::set_allocated_usernotification(::grpc::UserNotificationEvent* PROTOBUF_NULLABLE usernotification) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (usernotification) {
    ::google::protobuf::Arena* submessage_arena = usernotification->GetArena();
    if (message_arena != submessage_arena) {
      usernotification = ::google::protobuf::internal::GetOwnedMessage(message_arena, usernotification, submessage_arena);
    }
    set_has_usernotification();
    _impl_.event_.usernotification_ = usernotification;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.AppEvent.userNotification)
}
AppEvent::AppEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AppEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.AppEvent)
}
PROTOBUF_NDEBUG_INLINE AppEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::AppEvent& from_msg)
      : event_{},
        _cached_size_{0},
        _oneof_case_{from._oneof_case_[0]} {}

AppEvent::AppEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const AppEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AppEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  AppEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  switch (event_case()) {
    case EVENT_NOT_SET:
      break;
      case kInternetStatus:
        _impl_.event_.internetstatus_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.internetstatus_);
        break;
      case kToggleAutostartFinished:
        _impl_.event_.toggleautostartfinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.toggleautostartfinished_);
        break;
      case kResetFinished:
        _impl_.event_.resetfinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.resetfinished_);
        break;
      case kReportBugFinished:
        _impl_.event_.reportbugfinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.reportbugfinished_);
        break;
      case kReportBugSuccess:
        _impl_.event_.reportbugsuccess_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.reportbugsuccess_);
        break;
      case kReportBugError:
        _impl_.event_.reportbugerror_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.reportbugerror_);
        break;
      case kShowMainWindow:
        _impl_.event_.showmainwindow_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.showmainwindow_);
        break;
      case kReportBugFallback:
        _impl_.event_.reportbugfallback_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.reportbugfallback_);
        break;
      case kCertificateInstallSuccess:
        _impl_.event_.certificateinstallsuccess_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.certificateinstallsuccess_);
        break;
      case kCertificateInstallCanceled:
        _impl_.event_.certificateinstallcanceled_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.certificateinstallcanceled_);
        break;
      case kCertificateInstallFailed:
        _impl_.event_.certificateinstallfailed_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.certificateinstallfailed_);
        break;
      case kKnowledgeBaseSuggestions:
        _impl_.event_.knowledgebasesuggestions_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.knowledgebasesuggestions_);
        break;
      case kRepairStarted:
        _impl_.event_.repairstarted_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.repairstarted_);
        break;
      case kAllUsersLoaded:
        _impl_.event_.allusersloaded_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.allusersloaded_);
        break;
      case kUserNotification:
        _impl_.event_.usernotification_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.usernotification_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:grpc.AppEvent)
}
PROTOBUF_NDEBUG_INLINE AppEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

inline void AppEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
AppEvent::~AppEvent() {
  // @@protoc_insertion_point(destructor:grpc.AppEvent)
  SharedDtor(*this);
}
inline void AppEvent::SharedDtor(MessageLite& self) {
  AppEvent& this_ = static_cast<AppEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  if (this_.has_event()) {
    this_.clear_event();
  }
  this_._impl_.~Impl_();
}

void AppEvent::clear_event() {
// @@protoc_insertion_point(one_of_clear_start:grpc.AppEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (event_case()) {
    case kInternetStatus: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.internetstatus_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.internetstatus_);
      }
      break;
    }
    case kToggleAutostartFinished: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.toggleautostartfinished_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.toggleautostartfinished_);
      }
      break;
    }
    case kResetFinished: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.resetfinished_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.resetfinished_);
      }
      break;
    }
    case kReportBugFinished: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.reportbugfinished_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.reportbugfinished_);
      }
      break;
    }
    case kReportBugSuccess: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.reportbugsuccess_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.reportbugsuccess_);
      }
      break;
    }
    case kReportBugError: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.reportbugerror_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.reportbugerror_);
      }
      break;
    }
    case kShowMainWindow: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.showmainwindow_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.showmainwindow_);
      }
      break;
    }
    case kReportBugFallback: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.reportbugfallback_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.reportbugfallback_);
      }
      break;
    }
    case kCertificateInstallSuccess: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.certificateinstallsuccess_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.certificateinstallsuccess_);
      }
      break;
    }
    case kCertificateInstallCanceled: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.certificateinstallcanceled_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.certificateinstallcanceled_);
      }
      break;
    }
    case kCertificateInstallFailed: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.certificateinstallfailed_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.certificateinstallfailed_);
      }
      break;
    }
    case kKnowledgeBaseSuggestions: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.knowledgebasesuggestions_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.knowledgebasesuggestions_);
      }
      break;
    }
    case kRepairStarted: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.repairstarted_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.repairstarted_);
      }
      break;
    }
    case kAllUsersLoaded: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.allusersloaded_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.allusersloaded_);
      }
      break;
    }
    case kUserNotification: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.usernotification_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.usernotification_);
      }
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = EVENT_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AppEvent_class_data_ =
        AppEvent::InternalGenerateClassData_(AppEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AppEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AppEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AppEvent_class_data_.tc_table);
  return AppEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AppEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AppEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AppEvent_globals_));
  return AppEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AppEvent::ParseTableT_
    AppEvent::_table_ =
        AppEvent::InternalGenerateParseTable_(AppEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void AppEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.AppEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  clear_event();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL AppEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const AppEvent& this_ = static_cast<const AppEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL AppEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const AppEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.AppEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  switch (this_.event_case()) {
    case kInternetStatus: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          1, *this_._impl_.event_.internetstatus_, this_._impl_.event_.internetstatus_->GetCachedSize(), target,
          stream);
      break;
    }
    case kToggleAutostartFinished: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          2, *this_._impl_.event_.toggleautostartfinished_, this_._impl_.event_.toggleautostartfinished_->GetCachedSize(), target,
          stream);
      break;
    }
    case kResetFinished: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          3, *this_._impl_.event_.resetfinished_, this_._impl_.event_.resetfinished_->GetCachedSize(), target,
          stream);
      break;
    }
    case kReportBugFinished: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          4, *this_._impl_.event_.reportbugfinished_, this_._impl_.event_.reportbugfinished_->GetCachedSize(), target,
          stream);
      break;
    }
    case kReportBugSuccess: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          5, *this_._impl_.event_.reportbugsuccess_, this_._impl_.event_.reportbugsuccess_->GetCachedSize(), target,
          stream);
      break;
    }
    case kReportBugError: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          6, *this_._impl_.event_.reportbugerror_, this_._impl_.event_.reportbugerror_->GetCachedSize(), target,
          stream);
      break;
    }
    case kShowMainWindow: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          7, *this_._impl_.event_.showmainwindow_, this_._impl_.event_.showmainwindow_->GetCachedSize(), target,
          stream);
      break;
    }
    case kReportBugFallback: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          8, *this_._impl_.event_.reportbugfallback_, this_._impl_.event_.reportbugfallback_->GetCachedSize(), target,
          stream);
      break;
    }
    case kCertificateInstallSuccess: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          9, *this_._impl_.event_.certificateinstallsuccess_, this_._impl_.event_.certificateinstallsuccess_->GetCachedSize(), target,
          stream);
      break;
    }
    case kCertificateInstallCanceled: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          10, *this_._impl_.event_.certificateinstallcanceled_, this_._impl_.event_.certificateinstallcanceled_->GetCachedSize(), target,
          stream);
      break;
    }
    case kCertificateInstallFailed: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          11, *this_._impl_.event_.certificateinstallfailed_, this_._impl_.event_.certificateinstallfailed_->GetCachedSize(), target,
          stream);
      break;
    }
    case kKnowledgeBaseSuggestions: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          12, *this_._impl_.event_.knowledgebasesuggestions_, this_._impl_.event_.knowledgebasesuggestions_->GetCachedSize(), target,
          stream);
      break;
    }
    case kRepairStarted: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          13, *this_._impl_.event_.repairstarted_, this_._impl_.event_.repairstarted_->GetCachedSize(), target,
          stream);
      break;
    }
    case kAllUsersLoaded: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          14, *this_._impl_.event_.allusersloaded_, this_._impl_.event_.allusersloaded_->GetCachedSize(), target,
          stream);
      break;
    }
    case kUserNotification: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          15, *this_._impl_.event_.usernotification_, this_._impl_.event_.usernotification_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.AppEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t AppEvent::ByteSizeLong(const MessageLite& base) {
  const AppEvent& this_ = static_cast<const AppEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t AppEvent::ByteSizeLong() const {
  const AppEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.AppEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  switch (this_.event_case()) {
    // .grpc.InternetStatusEvent internetStatus = 1;
    case kInternetStatus: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.internetstatus_);
      break;
    }
    // .grpc.ToggleAutostartFinishedEvent toggleAutostartFinished = 2;
    case kToggleAutostartFinished: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.toggleautostartfinished_);
      break;
    }
    // .grpc.ResetFinishedEvent resetFinished = 3;
    case kResetFinished: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.resetfinished_);
      break;
    }
    // .grpc.ReportBugFinishedEvent reportBugFinished = 4;
    case kReportBugFinished: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.reportbugfinished_);
      break;
    }
    // .grpc.ReportBugSuccessEvent reportBugSuccess = 5;
    case kReportBugSuccess: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.reportbugsuccess_);
      break;
    }
    // .grpc.ReportBugErrorEvent reportBugError = 6;
    case kReportBugError: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.reportbugerror_);
      break;
    }
    // .grpc.ShowMainWindowEvent showMainWindow = 7;
    case kShowMainWindow: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.showmainwindow_);
      break;
    }
    // .grpc.ReportBugFallbackEvent reportBugFallback = 8;
    case kReportBugFallback: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.reportbugfallback_);
      break;
    }
    // .grpc.CertificateInstallSuccessEvent certificateInstallSuccess = 9;
    case kCertificateInstallSuccess: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.certificateinstallsuccess_);
      break;
    }
    // .grpc.CertificateInstallCanceledEvent certificateInstallCanceled = 10;
    case kCertificateInstallCanceled: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.certificateinstallcanceled_);
      break;
    }
    // .grpc.CertificateInstallFailedEvent certificateInstallFailed = 11;
    case kCertificateInstallFailed: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.certificateinstallfailed_);
      break;
    }
    // .grpc.KnowledgeBaseSuggestionsEvent knowledgeBaseSuggestions = 12;
    case kKnowledgeBaseSuggestions: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.knowledgebasesuggestions_);
      break;
    }
    // .grpc.RepairStartedEvent repairStarted = 13;
    case kRepairStarted: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.repairstarted_);
      break;
    }
    // .grpc.AllUsersLoadedEvent allUsersLoaded = 14;
    case kAllUsersLoaded: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.allusersloaded_);
      break;
    }
    // .grpc.UserNotificationEvent userNotification = 15;
    case kUserNotification: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.usernotification_);
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void AppEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<AppEvent*>(&to_msg);
  auto& from = static_cast<const AppEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.AppEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_event();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kInternetStatus: {
        if (oneof_needs_init) {
          _this->_impl_.event_.internetstatus_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.internetstatus_);
        } else {
          _this->_impl_.event_.internetstatus_->MergeFrom(*from._impl_.event_.internetstatus_);
        }
        break;
      }
      case kToggleAutostartFinished: {
        if (oneof_needs_init) {
          _this->_impl_.event_.toggleautostartfinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.toggleautostartfinished_);
        } else {
          _this->_impl_.event_.toggleautostartfinished_->MergeFrom(*from._impl_.event_.toggleautostartfinished_);
        }
        break;
      }
      case kResetFinished: {
        if (oneof_needs_init) {
          _this->_impl_.event_.resetfinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.resetfinished_);
        } else {
          _this->_impl_.event_.resetfinished_->MergeFrom(*from._impl_.event_.resetfinished_);
        }
        break;
      }
      case kReportBugFinished: {
        if (oneof_needs_init) {
          _this->_impl_.event_.reportbugfinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.reportbugfinished_);
        } else {
          _this->_impl_.event_.reportbugfinished_->MergeFrom(*from._impl_.event_.reportbugfinished_);
        }
        break;
      }
      case kReportBugSuccess: {
        if (oneof_needs_init) {
          _this->_impl_.event_.reportbugsuccess_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.reportbugsuccess_);
        } else {
          _this->_impl_.event_.reportbugsuccess_->MergeFrom(*from._impl_.event_.reportbugsuccess_);
        }
        break;
      }
      case kReportBugError: {
        if (oneof_needs_init) {
          _this->_impl_.event_.reportbugerror_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.reportbugerror_);
        } else {
          _this->_impl_.event_.reportbugerror_->MergeFrom(*from._impl_.event_.reportbugerror_);
        }
        break;
      }
      case kShowMainWindow: {
        if (oneof_needs_init) {
          _this->_impl_.event_.showmainwindow_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.showmainwindow_);
        } else {
          _this->_impl_.event_.showmainwindow_->MergeFrom(*from._impl_.event_.showmainwindow_);
        }
        break;
      }
      case kReportBugFallback: {
        if (oneof_needs_init) {
          _this->_impl_.event_.reportbugfallback_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.reportbugfallback_);
        } else {
          _this->_impl_.event_.reportbugfallback_->MergeFrom(*from._impl_.event_.reportbugfallback_);
        }
        break;
      }
      case kCertificateInstallSuccess: {
        if (oneof_needs_init) {
          _this->_impl_.event_.certificateinstallsuccess_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.certificateinstallsuccess_);
        } else {
          _this->_impl_.event_.certificateinstallsuccess_->MergeFrom(*from._impl_.event_.certificateinstallsuccess_);
        }
        break;
      }
      case kCertificateInstallCanceled: {
        if (oneof_needs_init) {
          _this->_impl_.event_.certificateinstallcanceled_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.certificateinstallcanceled_);
        } else {
          _this->_impl_.event_.certificateinstallcanceled_->MergeFrom(*from._impl_.event_.certificateinstallcanceled_);
        }
        break;
      }
      case kCertificateInstallFailed: {
        if (oneof_needs_init) {
          _this->_impl_.event_.certificateinstallfailed_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.certificateinstallfailed_);
        } else {
          _this->_impl_.event_.certificateinstallfailed_->MergeFrom(*from._impl_.event_.certificateinstallfailed_);
        }
        break;
      }
      case kKnowledgeBaseSuggestions: {
        if (oneof_needs_init) {
          _this->_impl_.event_.knowledgebasesuggestions_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.knowledgebasesuggestions_);
        } else {
          _this->_impl_.event_.knowledgebasesuggestions_->MergeFrom(*from._impl_.event_.knowledgebasesuggestions_);
        }
        break;
      }
      case kRepairStarted: {
        if (oneof_needs_init) {
          _this->_impl_.event_.repairstarted_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.repairstarted_);
        } else {
          _this->_impl_.event_.repairstarted_->MergeFrom(*from._impl_.event_.repairstarted_);
        }
        break;
      }
      case kAllUsersLoaded: {
        if (oneof_needs_init) {
          _this->_impl_.event_.allusersloaded_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.allusersloaded_);
        } else {
          _this->_impl_.event_.allusersloaded_->MergeFrom(*from._impl_.event_.allusersloaded_);
        }
        break;
      }
      case kUserNotification: {
        if (oneof_needs_init) {
          _this->_impl_.event_.usernotification_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.usernotification_);
        } else {
          _this->_impl_.event_.usernotification_->MergeFrom(*from._impl_.event_.usernotification_);
        }
        break;
      }
      case EVENT_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void AppEvent::CopyFrom(const AppEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.AppEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void AppEvent::InternalSwap(AppEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_.event_, other->_impl_.event_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata AppEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

InternetStatusEvent::InternetStatusEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, InternetStatusEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.InternetStatusEvent)
}
InternetStatusEvent::InternetStatusEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const InternetStatusEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, InternetStatusEvent_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE InternetStatusEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void InternetStatusEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.connected_ = {};
}
InternetStatusEvent::~InternetStatusEvent() {
  // @@protoc_insertion_point(destructor:grpc.InternetStatusEvent)
  SharedDtor(*this);
}
inline void InternetStatusEvent::SharedDtor(MessageLite& self) {
  InternetStatusEvent& this_ = static_cast<InternetStatusEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull InternetStatusEvent_class_data_ =
        InternetStatusEvent::InternalGenerateClassData_(InternetStatusEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
InternetStatusEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&InternetStatusEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(InternetStatusEvent_class_data_.tc_table);
  return InternetStatusEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
InternetStatusEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&InternetStatusEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&InternetStatusEvent_globals_));
  return InternetStatusEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const InternetStatusEvent::ParseTableT_
    InternetStatusEvent::_table_ =
        InternetStatusEvent::InternalGenerateParseTable_(InternetStatusEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void InternetStatusEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.InternetStatusEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.connected_ = false;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL InternetStatusEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const InternetStatusEvent& this_ = static_cast<const InternetStatusEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL InternetStatusEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const InternetStatusEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.InternetStatusEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // bool connected = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_connected() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          1, this_._internal_connected(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.InternetStatusEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t InternetStatusEvent::ByteSizeLong(const MessageLite& base) {
  const InternetStatusEvent& this_ = static_cast<const InternetStatusEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t InternetStatusEvent::ByteSizeLong() const {
  const InternetStatusEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.InternetStatusEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // bool connected = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_connected() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void InternetStatusEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<InternetStatusEvent*>(&to_msg);
  auto& from = static_cast<const InternetStatusEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.InternetStatusEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (from._internal_connected() != 0) {
      _this->_impl_.connected_ = from._impl_.connected_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void InternetStatusEvent::CopyFrom(const InternetStatusEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.InternetStatusEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void InternetStatusEvent::InternalSwap(InternetStatusEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.connected_, other->_impl_.connected_);
}

::google::protobuf::Metadata InternetStatusEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ToggleAutostartFinishedEvent::ToggleAutostartFinishedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ToggleAutostartFinishedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.ToggleAutostartFinishedEvent)
}
ToggleAutostartFinishedEvent::ToggleAutostartFinishedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ToggleAutostartFinishedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ToggleAutostartFinishedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ToggleAutostartFinishedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.ToggleAutostartFinishedEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ToggleAutostartFinishedEvent_class_data_ =
        ToggleAutostartFinishedEvent::InternalGenerateClassData_(ToggleAutostartFinishedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ToggleAutostartFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ToggleAutostartFinishedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ToggleAutostartFinishedEvent_class_data_.tc_table);
  return ToggleAutostartFinishedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ToggleAutostartFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ToggleAutostartFinishedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ToggleAutostartFinishedEvent_globals_));
  return ToggleAutostartFinishedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ToggleAutostartFinishedEvent::ParseTableT_
    ToggleAutostartFinishedEvent::_table_ =
        ToggleAutostartFinishedEvent::InternalGenerateParseTable_(ToggleAutostartFinishedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata ToggleAutostartFinishedEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ResetFinishedEvent::ResetFinishedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ResetFinishedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.ResetFinishedEvent)
}
ResetFinishedEvent::ResetFinishedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ResetFinishedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ResetFinishedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ResetFinishedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.ResetFinishedEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ResetFinishedEvent_class_data_ =
        ResetFinishedEvent::InternalGenerateClassData_(ResetFinishedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ResetFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ResetFinishedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ResetFinishedEvent_class_data_.tc_table);
  return ResetFinishedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ResetFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ResetFinishedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ResetFinishedEvent_globals_));
  return ResetFinishedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ResetFinishedEvent::ParseTableT_
    ResetFinishedEvent::_table_ =
        ResetFinishedEvent::InternalGenerateParseTable_(ResetFinishedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata ResetFinishedEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ReportBugFinishedEvent::ReportBugFinishedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ReportBugFinishedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.ReportBugFinishedEvent)
}
ReportBugFinishedEvent::ReportBugFinishedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ReportBugFinishedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ReportBugFinishedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ReportBugFinishedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.ReportBugFinishedEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ReportBugFinishedEvent_class_data_ =
        ReportBugFinishedEvent::InternalGenerateClassData_(ReportBugFinishedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReportBugFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReportBugFinishedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ReportBugFinishedEvent_class_data_.tc_table);
  return ReportBugFinishedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReportBugFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReportBugFinishedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ReportBugFinishedEvent_globals_));
  return ReportBugFinishedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ReportBugFinishedEvent::ParseTableT_
    ReportBugFinishedEvent::_table_ =
        ReportBugFinishedEvent::InternalGenerateParseTable_(ReportBugFinishedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata ReportBugFinishedEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ReportBugSuccessEvent::ReportBugSuccessEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ReportBugSuccessEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.ReportBugSuccessEvent)
}
ReportBugSuccessEvent::ReportBugSuccessEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ReportBugSuccessEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ReportBugSuccessEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ReportBugSuccessEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.ReportBugSuccessEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ReportBugSuccessEvent_class_data_ =
        ReportBugSuccessEvent::InternalGenerateClassData_(ReportBugSuccessEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReportBugSuccessEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReportBugSuccessEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ReportBugSuccessEvent_class_data_.tc_table);
  return ReportBugSuccessEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReportBugSuccessEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReportBugSuccessEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ReportBugSuccessEvent_globals_));
  return ReportBugSuccessEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ReportBugSuccessEvent::ParseTableT_
    ReportBugSuccessEvent::_table_ =
        ReportBugSuccessEvent::InternalGenerateParseTable_(ReportBugSuccessEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata ReportBugSuccessEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ReportBugErrorEvent::ReportBugErrorEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ReportBugErrorEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.ReportBugErrorEvent)
}
ReportBugErrorEvent::ReportBugErrorEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ReportBugErrorEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ReportBugErrorEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ReportBugErrorEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.ReportBugErrorEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ReportBugErrorEvent_class_data_ =
        ReportBugErrorEvent::InternalGenerateClassData_(ReportBugErrorEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReportBugErrorEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReportBugErrorEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ReportBugErrorEvent_class_data_.tc_table);
  return ReportBugErrorEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReportBugErrorEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReportBugErrorEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ReportBugErrorEvent_globals_));
  return ReportBugErrorEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ReportBugErrorEvent::ParseTableT_
    ReportBugErrorEvent::_table_ =
        ReportBugErrorEvent::InternalGenerateParseTable_(ReportBugErrorEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata ReportBugErrorEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ShowMainWindowEvent::ShowMainWindowEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ShowMainWindowEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.ShowMainWindowEvent)
}
ShowMainWindowEvent::ShowMainWindowEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ShowMainWindowEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ShowMainWindowEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ShowMainWindowEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.ShowMainWindowEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ShowMainWindowEvent_class_data_ =
        ShowMainWindowEvent::InternalGenerateClassData_(ShowMainWindowEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ShowMainWindowEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ShowMainWindowEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ShowMainWindowEvent_class_data_.tc_table);
  return ShowMainWindowEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ShowMainWindowEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ShowMainWindowEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ShowMainWindowEvent_globals_));
  return ShowMainWindowEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ShowMainWindowEvent::ParseTableT_
    ShowMainWindowEvent::_table_ =
        ShowMainWindowEvent::InternalGenerateParseTable_(ShowMainWindowEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata ShowMainWindowEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ReportBugFallbackEvent::ReportBugFallbackEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ReportBugFallbackEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.ReportBugFallbackEvent)
}
ReportBugFallbackEvent::ReportBugFallbackEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ReportBugFallbackEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ReportBugFallbackEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ReportBugFallbackEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.ReportBugFallbackEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ReportBugFallbackEvent_class_data_ =
        ReportBugFallbackEvent::InternalGenerateClassData_(ReportBugFallbackEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReportBugFallbackEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReportBugFallbackEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ReportBugFallbackEvent_class_data_.tc_table);
  return ReportBugFallbackEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReportBugFallbackEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReportBugFallbackEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ReportBugFallbackEvent_globals_));
  return ReportBugFallbackEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ReportBugFallbackEvent::ParseTableT_
    ReportBugFallbackEvent::_table_ =
        ReportBugFallbackEvent::InternalGenerateParseTable_(ReportBugFallbackEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata ReportBugFallbackEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

CertificateInstallSuccessEvent::CertificateInstallSuccessEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, CertificateInstallSuccessEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.CertificateInstallSuccessEvent)
}
CertificateInstallSuccessEvent::CertificateInstallSuccessEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const CertificateInstallSuccessEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, CertificateInstallSuccessEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  CertificateInstallSuccessEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.CertificateInstallSuccessEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CertificateInstallSuccessEvent_class_data_ =
        CertificateInstallSuccessEvent::InternalGenerateClassData_(CertificateInstallSuccessEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CertificateInstallSuccessEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CertificateInstallSuccessEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CertificateInstallSuccessEvent_class_data_.tc_table);
  return CertificateInstallSuccessEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CertificateInstallSuccessEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CertificateInstallSuccessEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CertificateInstallSuccessEvent_globals_));
  return CertificateInstallSuccessEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CertificateInstallSuccessEvent::ParseTableT_
    CertificateInstallSuccessEvent::_table_ =
        CertificateInstallSuccessEvent::InternalGenerateParseTable_(CertificateInstallSuccessEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata CertificateInstallSuccessEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

CertificateInstallCanceledEvent::CertificateInstallCanceledEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, CertificateInstallCanceledEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.CertificateInstallCanceledEvent)
}
CertificateInstallCanceledEvent::CertificateInstallCanceledEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const CertificateInstallCanceledEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, CertificateInstallCanceledEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  CertificateInstallCanceledEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.CertificateInstallCanceledEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CertificateInstallCanceledEvent_class_data_ =
        CertificateInstallCanceledEvent::InternalGenerateClassData_(CertificateInstallCanceledEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CertificateInstallCanceledEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CertificateInstallCanceledEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CertificateInstallCanceledEvent_class_data_.tc_table);
  return CertificateInstallCanceledEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CertificateInstallCanceledEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CertificateInstallCanceledEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CertificateInstallCanceledEvent_globals_));
  return CertificateInstallCanceledEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CertificateInstallCanceledEvent::ParseTableT_
    CertificateInstallCanceledEvent::_table_ =
        CertificateInstallCanceledEvent::InternalGenerateParseTable_(CertificateInstallCanceledEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata CertificateInstallCanceledEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

CertificateInstallFailedEvent::CertificateInstallFailedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, CertificateInstallFailedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.CertificateInstallFailedEvent)
}
CertificateInstallFailedEvent::CertificateInstallFailedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const CertificateInstallFailedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, CertificateInstallFailedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  CertificateInstallFailedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.CertificateInstallFailedEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CertificateInstallFailedEvent_class_data_ =
        CertificateInstallFailedEvent::InternalGenerateClassData_(CertificateInstallFailedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CertificateInstallFailedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CertificateInstallFailedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CertificateInstallFailedEvent_class_data_.tc_table);
  return CertificateInstallFailedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CertificateInstallFailedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CertificateInstallFailedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CertificateInstallFailedEvent_globals_));
  return CertificateInstallFailedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CertificateInstallFailedEvent::ParseTableT_
    CertificateInstallFailedEvent::_table_ =
        CertificateInstallFailedEvent::InternalGenerateParseTable_(CertificateInstallFailedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata CertificateInstallFailedEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

RepairStartedEvent::RepairStartedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, RepairStartedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.RepairStartedEvent)
}
RepairStartedEvent::RepairStartedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const RepairStartedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, RepairStartedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  RepairStartedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.RepairStartedEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull RepairStartedEvent_class_data_ =
        RepairStartedEvent::InternalGenerateClassData_(RepairStartedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
RepairStartedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&RepairStartedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(RepairStartedEvent_class_data_.tc_table);
  return RepairStartedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
RepairStartedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&RepairStartedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&RepairStartedEvent_globals_));
  return RepairStartedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const RepairStartedEvent::ParseTableT_
    RepairStartedEvent::_table_ =
        RepairStartedEvent::InternalGenerateParseTable_(RepairStartedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata RepairStartedEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

AllUsersLoadedEvent::AllUsersLoadedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, AllUsersLoadedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.AllUsersLoadedEvent)
}
AllUsersLoadedEvent::AllUsersLoadedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const AllUsersLoadedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, AllUsersLoadedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  AllUsersLoadedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.AllUsersLoadedEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AllUsersLoadedEvent_class_data_ =
        AllUsersLoadedEvent::InternalGenerateClassData_(AllUsersLoadedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AllUsersLoadedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AllUsersLoadedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AllUsersLoadedEvent_class_data_.tc_table);
  return AllUsersLoadedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AllUsersLoadedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AllUsersLoadedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AllUsersLoadedEvent_globals_));
  return AllUsersLoadedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AllUsersLoadedEvent::ParseTableT_
    AllUsersLoadedEvent::_table_ =
        AllUsersLoadedEvent::InternalGenerateParseTable_(AllUsersLoadedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata AllUsersLoadedEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

KnowledgeBaseSuggestion::KnowledgeBaseSuggestion(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, KnowledgeBaseSuggestion_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.KnowledgeBaseSuggestion)
}
PROTOBUF_NDEBUG_INLINE KnowledgeBaseSuggestion::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::KnowledgeBaseSuggestion& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        url_(arena, from.url_),
        title_(arena, from.title_) {}

KnowledgeBaseSuggestion::KnowledgeBaseSuggestion(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const KnowledgeBaseSuggestion& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, KnowledgeBaseSuggestion_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  KnowledgeBaseSuggestion* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.KnowledgeBaseSuggestion)
}
PROTOBUF_NDEBUG_INLINE KnowledgeBaseSuggestion::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        url_(arena),
        title_(arena) {}

inline void KnowledgeBaseSuggestion::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
KnowledgeBaseSuggestion::~KnowledgeBaseSuggestion() {
  // @@protoc_insertion_point(destructor:grpc.KnowledgeBaseSuggestion)
  SharedDtor(*this);
}
inline void KnowledgeBaseSuggestion::SharedDtor(MessageLite& self) {
  KnowledgeBaseSuggestion& this_ = static_cast<KnowledgeBaseSuggestion&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.url_.Destroy();
  this_._impl_.title_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull KnowledgeBaseSuggestion_class_data_ =
        KnowledgeBaseSuggestion::InternalGenerateClassData_(KnowledgeBaseSuggestion_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
KnowledgeBaseSuggestion::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&KnowledgeBaseSuggestion_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(KnowledgeBaseSuggestion_class_data_.tc_table);
  return KnowledgeBaseSuggestion_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
KnowledgeBaseSuggestion::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&KnowledgeBaseSuggestion_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&KnowledgeBaseSuggestion_globals_));
  return KnowledgeBaseSuggestion_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const KnowledgeBaseSuggestion::ParseTableT_
    KnowledgeBaseSuggestion::_table_ =
        KnowledgeBaseSuggestion::InternalGenerateParseTable_(KnowledgeBaseSuggestion_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void KnowledgeBaseSuggestion::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.KnowledgeBaseSuggestion)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.url_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.title_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL KnowledgeBaseSuggestion::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const KnowledgeBaseSuggestion& this_ = static_cast<const KnowledgeBaseSuggestion&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL KnowledgeBaseSuggestion::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const KnowledgeBaseSuggestion& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.KnowledgeBaseSuggestion)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string url = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_url().empty()) {
      const ::std::string& _s = this_._internal_url();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.KnowledgeBaseSuggestion.url");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string title = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_title().empty()) {
      const ::std::string& _s = this_._internal_title();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.KnowledgeBaseSuggestion.title");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.KnowledgeBaseSuggestion)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t KnowledgeBaseSuggestion::ByteSizeLong(const MessageLite& base) {
  const KnowledgeBaseSuggestion& this_ = static_cast<const KnowledgeBaseSuggestion&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t KnowledgeBaseSuggestion::ByteSizeLong() const {
  const KnowledgeBaseSuggestion& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.KnowledgeBaseSuggestion)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string url = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_url().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_url());
      }
    }
    // string title = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_title().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_title());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void KnowledgeBaseSuggestion::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<KnowledgeBaseSuggestion*>(&to_msg);
  auto& from = static_cast<const KnowledgeBaseSuggestion&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.KnowledgeBaseSuggestion)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_url().empty()) {
        _this->_internal_set_url(from._internal_url());
      } else {
        if (_this->_impl_.url_.IsDefault()) {
          _this->_internal_set_url("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_title().empty()) {
        _this->_internal_set_title(from._internal_title());
      } else {
        if (_this->_impl_.title_.IsDefault()) {
          _this->_internal_set_title("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void KnowledgeBaseSuggestion::CopyFrom(const KnowledgeBaseSuggestion& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.KnowledgeBaseSuggestion)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void KnowledgeBaseSuggestion::InternalSwap(KnowledgeBaseSuggestion* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.url_, &other->_impl_.url_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.title_, &other->_impl_.title_, arena);
}

::google::protobuf::Metadata KnowledgeBaseSuggestion::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

KnowledgeBaseSuggestionsEvent::KnowledgeBaseSuggestionsEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, KnowledgeBaseSuggestionsEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.KnowledgeBaseSuggestionsEvent)
}
PROTOBUF_NDEBUG_INLINE KnowledgeBaseSuggestionsEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::KnowledgeBaseSuggestionsEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        suggestions_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::grpc::KnowledgeBaseSuggestionsEvent,
              PROTOBUF_FIELD_OFFSET(::grpc::KnowledgeBaseSuggestionsEvent, _impl_.suggestions_)>()
          , from.suggestions_
        }
     {}

KnowledgeBaseSuggestionsEvent::KnowledgeBaseSuggestionsEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const KnowledgeBaseSuggestionsEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, KnowledgeBaseSuggestionsEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  KnowledgeBaseSuggestionsEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.KnowledgeBaseSuggestionsEvent)
}
PROTOBUF_NDEBUG_INLINE KnowledgeBaseSuggestionsEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        suggestions_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::grpc::KnowledgeBaseSuggestionsEvent,
            PROTOBUF_FIELD_OFFSET(::grpc::KnowledgeBaseSuggestionsEvent, _impl_.suggestions_)>()
         }
     {}

inline void KnowledgeBaseSuggestionsEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
KnowledgeBaseSuggestionsEvent::~KnowledgeBaseSuggestionsEvent() {
  // @@protoc_insertion_point(destructor:grpc.KnowledgeBaseSuggestionsEvent)
  SharedDtor(*this);
}
inline void KnowledgeBaseSuggestionsEvent::SharedDtor(MessageLite& self) {
  KnowledgeBaseSuggestionsEvent& this_ = static_cast<KnowledgeBaseSuggestionsEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull KnowledgeBaseSuggestionsEvent_class_data_ =
        KnowledgeBaseSuggestionsEvent::InternalGenerateClassData_(KnowledgeBaseSuggestionsEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
KnowledgeBaseSuggestionsEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&KnowledgeBaseSuggestionsEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(KnowledgeBaseSuggestionsEvent_class_data_.tc_table);
  return KnowledgeBaseSuggestionsEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
KnowledgeBaseSuggestionsEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&KnowledgeBaseSuggestionsEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&KnowledgeBaseSuggestionsEvent_globals_));
  return KnowledgeBaseSuggestionsEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const KnowledgeBaseSuggestionsEvent::ParseTableT_
    KnowledgeBaseSuggestionsEvent::_table_ =
        KnowledgeBaseSuggestionsEvent::InternalGenerateParseTable_(KnowledgeBaseSuggestionsEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void KnowledgeBaseSuggestionsEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.KnowledgeBaseSuggestionsEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.suggestions_.Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL KnowledgeBaseSuggestionsEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const KnowledgeBaseSuggestionsEvent& this_ = static_cast<const KnowledgeBaseSuggestionsEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL KnowledgeBaseSuggestionsEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const KnowledgeBaseSuggestionsEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.KnowledgeBaseSuggestionsEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .grpc.KnowledgeBaseSuggestion suggestions = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_suggestions_size());
         i < n; i++) {
      const auto& repfield = this_._internal_suggestions().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              1, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.KnowledgeBaseSuggestionsEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t KnowledgeBaseSuggestionsEvent::ByteSizeLong(const MessageLite& base) {
  const KnowledgeBaseSuggestionsEvent& this_ = static_cast<const KnowledgeBaseSuggestionsEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t KnowledgeBaseSuggestionsEvent::ByteSizeLong() const {
  const KnowledgeBaseSuggestionsEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.KnowledgeBaseSuggestionsEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
   {
    // repeated .grpc.KnowledgeBaseSuggestion suggestions = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_suggestions_size();
      for (const auto& msg : this_._internal_suggestions()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void KnowledgeBaseSuggestionsEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<KnowledgeBaseSuggestionsEvent*>(&to_msg);
  auto& from = static_cast<const KnowledgeBaseSuggestionsEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.KnowledgeBaseSuggestionsEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _this->_internal_mutable_suggestions()->InternalMergeFromWithArena(
        ::google::protobuf::MessageLite::internal_visibility(), arena,
        from._internal_suggestions());
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void KnowledgeBaseSuggestionsEvent::CopyFrom(const KnowledgeBaseSuggestionsEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.KnowledgeBaseSuggestionsEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void KnowledgeBaseSuggestionsEvent::InternalSwap(KnowledgeBaseSuggestionsEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.suggestions_.InternalSwap(&other->_impl_.suggestions_);
}

::google::protobuf::Metadata KnowledgeBaseSuggestionsEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void LoginEvent::set_allocated_error(::grpc::LoginErrorEvent* PROTOBUF_NULLABLE error) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (error) {
    ::google::protobuf::Arena* submessage_arena = error->GetArena();
    if (message_arena != submessage_arena) {
      error = ::google::protobuf::internal::GetOwnedMessage(message_arena, error, submessage_arena);
    }
    set_has_error();
    _impl_.event_.error_ = error;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.LoginEvent.error)
}
void LoginEvent::set_allocated_tfarequested(::grpc::LoginTfaRequestedEvent* PROTOBUF_NULLABLE tfarequested) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (tfarequested) {
    ::google::protobuf::Arena* submessage_arena = tfarequested->GetArena();
    if (message_arena != submessage_arena) {
      tfarequested = ::google::protobuf::internal::GetOwnedMessage(message_arena, tfarequested, submessage_arena);
    }
    set_has_tfarequested();
    _impl_.event_.tfarequested_ = tfarequested;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.LoginEvent.tfaRequested)
}
void LoginEvent::set_allocated_twopasswordrequested(::grpc::LoginTwoPasswordsRequestedEvent* PROTOBUF_NULLABLE twopasswordrequested) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (twopasswordrequested) {
    ::google::protobuf::Arena* submessage_arena = twopasswordrequested->GetArena();
    if (message_arena != submessage_arena) {
      twopasswordrequested = ::google::protobuf::internal::GetOwnedMessage(message_arena, twopasswordrequested, submessage_arena);
    }
    set_has_twopasswordrequested();
    _impl_.event_.twopasswordrequested_ = twopasswordrequested;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.LoginEvent.twoPasswordRequested)
}
void LoginEvent::set_allocated_finished(::grpc::LoginFinishedEvent* PROTOBUF_NULLABLE finished) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (finished) {
    ::google::protobuf::Arena* submessage_arena = finished->GetArena();
    if (message_arena != submessage_arena) {
      finished = ::google::protobuf::internal::GetOwnedMessage(message_arena, finished, submessage_arena);
    }
    set_has_finished();
    _impl_.event_.finished_ = finished;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.LoginEvent.finished)
}
void LoginEvent::set_allocated_alreadyloggedin(::grpc::LoginFinishedEvent* PROTOBUF_NULLABLE alreadyloggedin) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (alreadyloggedin) {
    ::google::protobuf::Arena* submessage_arena = alreadyloggedin->GetArena();
    if (message_arena != submessage_arena) {
      alreadyloggedin = ::google::protobuf::internal::GetOwnedMessage(message_arena, alreadyloggedin, submessage_arena);
    }
    set_has_alreadyloggedin();
    _impl_.event_.alreadyloggedin_ = alreadyloggedin;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.LoginEvent.alreadyLoggedIn)
}
void LoginEvent::set_allocated_hvrequested(::grpc::LoginHvRequestedEvent* PROTOBUF_NULLABLE hvrequested) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (hvrequested) {
    ::google::protobuf::Arena* submessage_arena = hvrequested->GetArena();
    if (message_arena != submessage_arena) {
      hvrequested = ::google::protobuf::internal::GetOwnedMessage(message_arena, hvrequested, submessage_arena);
    }
    set_has_hvrequested();
    _impl_.event_.hvrequested_ = hvrequested;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.LoginEvent.hvRequested)
}
void LoginEvent::set_allocated_fidorequested(::grpc::LoginFidoRequestedEvent* PROTOBUF_NULLABLE fidorequested) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (fidorequested) {
    ::google::protobuf::Arena* submessage_arena = fidorequested->GetArena();
    if (message_arena != submessage_arena) {
      fidorequested = ::google::protobuf::internal::GetOwnedMessage(message_arena, fidorequested, submessage_arena);
    }
    set_has_fidorequested();
    _impl_.event_.fidorequested_ = fidorequested;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.LoginEvent.fidoRequested)
}
void LoginEvent::set_allocated_tfaorfidorequested(::grpc::LoginTfaOrFidoRequestedEvent* PROTOBUF_NULLABLE tfaorfidorequested) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (tfaorfidorequested) {
    ::google::protobuf::Arena* submessage_arena = tfaorfidorequested->GetArena();
    if (message_arena != submessage_arena) {
      tfaorfidorequested = ::google::protobuf::internal::GetOwnedMessage(message_arena, tfaorfidorequested, submessage_arena);
    }
    set_has_tfaorfidorequested();
    _impl_.event_.tfaorfidorequested_ = tfaorfidorequested;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.LoginEvent.tfaOrFidoRequested)
}
void LoginEvent::set_allocated_loginfidotouchrequested(::grpc::LoginFidoTouchEvent* PROTOBUF_NULLABLE loginfidotouchrequested) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (loginfidotouchrequested) {
    ::google::protobuf::Arena* submessage_arena = loginfidotouchrequested->GetArena();
    if (message_arena != submessage_arena) {
      loginfidotouchrequested = ::google::protobuf::internal::GetOwnedMessage(message_arena, loginfidotouchrequested, submessage_arena);
    }
    set_has_loginfidotouchrequested();
    _impl_.event_.loginfidotouchrequested_ = loginfidotouchrequested;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.LoginEvent.loginFidoTouchRequested)
}
void LoginEvent::set_allocated_loginfidotouchcompleted(::grpc::LoginFidoTouchEvent* PROTOBUF_NULLABLE loginfidotouchcompleted) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (loginfidotouchcompleted) {
    ::google::protobuf::Arena* submessage_arena = loginfidotouchcompleted->GetArena();
    if (message_arena != submessage_arena) {
      loginfidotouchcompleted = ::google::protobuf::internal::GetOwnedMessage(message_arena, loginfidotouchcompleted, submessage_arena);
    }
    set_has_loginfidotouchcompleted();
    _impl_.event_.loginfidotouchcompleted_ = loginfidotouchcompleted;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.LoginEvent.loginFidoTouchCompleted)
}
void LoginEvent::set_allocated_loginfidopinrequired(::grpc::LoginFidoPinRequired* PROTOBUF_NULLABLE loginfidopinrequired) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (loginfidopinrequired) {
    ::google::protobuf::Arena* submessage_arena = loginfidopinrequired->GetArena();
    if (message_arena != submessage_arena) {
      loginfidopinrequired = ::google::protobuf::internal::GetOwnedMessage(message_arena, loginfidopinrequired, submessage_arena);
    }
    set_has_loginfidopinrequired();
    _impl_.event_.loginfidopinrequired_ = loginfidopinrequired;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.LoginEvent.loginFidoPinRequired)
}
LoginEvent::LoginEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.LoginEvent)
}
PROTOBUF_NDEBUG_INLINE LoginEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::LoginEvent& from_msg)
      : event_{},
        _cached_size_{0},
        _oneof_case_{from._oneof_case_[0]} {}

LoginEvent::LoginEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const LoginEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  LoginEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  switch (event_case()) {
    case EVENT_NOT_SET:
      break;
      case kError:
        _impl_.event_.error_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.error_);
        break;
      case kTfaRequested:
        _impl_.event_.tfarequested_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.tfarequested_);
        break;
      case kTwoPasswordRequested:
        _impl_.event_.twopasswordrequested_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.twopasswordrequested_);
        break;
      case kFinished:
        _impl_.event_.finished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.finished_);
        break;
      case kAlreadyLoggedIn:
        _impl_.event_.alreadyloggedin_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.alreadyloggedin_);
        break;
      case kHvRequested:
        _impl_.event_.hvrequested_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.hvrequested_);
        break;
      case kFidoRequested:
        _impl_.event_.fidorequested_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.fidorequested_);
        break;
      case kTfaOrFidoRequested:
        _impl_.event_.tfaorfidorequested_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.tfaorfidorequested_);
        break;
      case kLoginFidoTouchRequested:
        _impl_.event_.loginfidotouchrequested_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.loginfidotouchrequested_);
        break;
      case kLoginFidoTouchCompleted:
        _impl_.event_.loginfidotouchcompleted_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.loginfidotouchcompleted_);
        break;
      case kLoginFidoPinRequired:
        _impl_.event_.loginfidopinrequired_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.loginfidopinrequired_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:grpc.LoginEvent)
}
PROTOBUF_NDEBUG_INLINE LoginEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

inline void LoginEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
LoginEvent::~LoginEvent() {
  // @@protoc_insertion_point(destructor:grpc.LoginEvent)
  SharedDtor(*this);
}
inline void LoginEvent::SharedDtor(MessageLite& self) {
  LoginEvent& this_ = static_cast<LoginEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  if (this_.has_event()) {
    this_.clear_event();
  }
  this_._impl_.~Impl_();
}

void LoginEvent::clear_event() {
// @@protoc_insertion_point(one_of_clear_start:grpc.LoginEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (event_case()) {
    case kError: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.error_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.error_);
      }
      break;
    }
    case kTfaRequested: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.tfarequested_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.tfarequested_);
      }
      break;
    }
    case kTwoPasswordRequested: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.twopasswordrequested_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.twopasswordrequested_);
      }
      break;
    }
    case kFinished: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.finished_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.finished_);
      }
      break;
    }
    case kAlreadyLoggedIn: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.alreadyloggedin_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.alreadyloggedin_);
      }
      break;
    }
    case kHvRequested: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.hvrequested_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.hvrequested_);
      }
      break;
    }
    case kFidoRequested: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.fidorequested_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.fidorequested_);
      }
      break;
    }
    case kTfaOrFidoRequested: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.tfaorfidorequested_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.tfaorfidorequested_);
      }
      break;
    }
    case kLoginFidoTouchRequested: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.loginfidotouchrequested_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.loginfidotouchrequested_);
      }
      break;
    }
    case kLoginFidoTouchCompleted: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.loginfidotouchcompleted_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.loginfidotouchcompleted_);
      }
      break;
    }
    case kLoginFidoPinRequired: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.loginfidopinrequired_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.loginfidopinrequired_);
      }
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = EVENT_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull LoginEvent_class_data_ =
        LoginEvent::InternalGenerateClassData_(LoginEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(LoginEvent_class_data_.tc_table);
  return LoginEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&LoginEvent_globals_));
  return LoginEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const LoginEvent::ParseTableT_
    LoginEvent::_table_ =
        LoginEvent::InternalGenerateParseTable_(LoginEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void LoginEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.LoginEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  clear_event();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL LoginEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const LoginEvent& this_ = static_cast<const LoginEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL LoginEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const LoginEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.LoginEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  switch (this_.event_case()) {
    case kError: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          1, *this_._impl_.event_.error_, this_._impl_.event_.error_->GetCachedSize(), target,
          stream);
      break;
    }
    case kTfaRequested: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          2, *this_._impl_.event_.tfarequested_, this_._impl_.event_.tfarequested_->GetCachedSize(), target,
          stream);
      break;
    }
    case kTwoPasswordRequested: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          3, *this_._impl_.event_.twopasswordrequested_, this_._impl_.event_.twopasswordrequested_->GetCachedSize(), target,
          stream);
      break;
    }
    case kFinished: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          4, *this_._impl_.event_.finished_, this_._impl_.event_.finished_->GetCachedSize(), target,
          stream);
      break;
    }
    case kAlreadyLoggedIn: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          5, *this_._impl_.event_.alreadyloggedin_, this_._impl_.event_.alreadyloggedin_->GetCachedSize(), target,
          stream);
      break;
    }
    case kHvRequested: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          6, *this_._impl_.event_.hvrequested_, this_._impl_.event_.hvrequested_->GetCachedSize(), target,
          stream);
      break;
    }
    case kFidoRequested: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          7, *this_._impl_.event_.fidorequested_, this_._impl_.event_.fidorequested_->GetCachedSize(), target,
          stream);
      break;
    }
    case kTfaOrFidoRequested: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          8, *this_._impl_.event_.tfaorfidorequested_, this_._impl_.event_.tfaorfidorequested_->GetCachedSize(), target,
          stream);
      break;
    }
    case kLoginFidoTouchRequested: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          9, *this_._impl_.event_.loginfidotouchrequested_, this_._impl_.event_.loginfidotouchrequested_->GetCachedSize(), target,
          stream);
      break;
    }
    case kLoginFidoTouchCompleted: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          10, *this_._impl_.event_.loginfidotouchcompleted_, this_._impl_.event_.loginfidotouchcompleted_->GetCachedSize(), target,
          stream);
      break;
    }
    case kLoginFidoPinRequired: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          11, *this_._impl_.event_.loginfidopinrequired_, this_._impl_.event_.loginfidopinrequired_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.LoginEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t LoginEvent::ByteSizeLong(const MessageLite& base) {
  const LoginEvent& this_ = static_cast<const LoginEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t LoginEvent::ByteSizeLong() const {
  const LoginEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.LoginEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  switch (this_.event_case()) {
    // .grpc.LoginErrorEvent error = 1;
    case kError: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.error_);
      break;
    }
    // .grpc.LoginTfaRequestedEvent tfaRequested = 2;
    case kTfaRequested: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.tfarequested_);
      break;
    }
    // .grpc.LoginTwoPasswordsRequestedEvent twoPasswordRequested = 3;
    case kTwoPasswordRequested: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.twopasswordrequested_);
      break;
    }
    // .grpc.LoginFinishedEvent finished = 4;
    case kFinished: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.finished_);
      break;
    }
    // .grpc.LoginFinishedEvent alreadyLoggedIn = 5;
    case kAlreadyLoggedIn: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.alreadyloggedin_);
      break;
    }
    // .grpc.LoginHvRequestedEvent hvRequested = 6;
    case kHvRequested: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.hvrequested_);
      break;
    }
    // .grpc.LoginFidoRequestedEvent fidoRequested = 7;
    case kFidoRequested: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.fidorequested_);
      break;
    }
    // .grpc.LoginTfaOrFidoRequestedEvent tfaOrFidoRequested = 8;
    case kTfaOrFidoRequested: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.tfaorfidorequested_);
      break;
    }
    // .grpc.LoginFidoTouchEvent loginFidoTouchRequested = 9;
    case kLoginFidoTouchRequested: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.loginfidotouchrequested_);
      break;
    }
    // .grpc.LoginFidoTouchEvent loginFidoTouchCompleted = 10;
    case kLoginFidoTouchCompleted: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.loginfidotouchcompleted_);
      break;
    }
    // .grpc.LoginFidoPinRequired loginFidoPinRequired = 11;
    case kLoginFidoPinRequired: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.loginfidopinrequired_);
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void LoginEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<LoginEvent*>(&to_msg);
  auto& from = static_cast<const LoginEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.LoginEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_event();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kError: {
        if (oneof_needs_init) {
          _this->_impl_.event_.error_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.error_);
        } else {
          _this->_impl_.event_.error_->MergeFrom(*from._impl_.event_.error_);
        }
        break;
      }
      case kTfaRequested: {
        if (oneof_needs_init) {
          _this->_impl_.event_.tfarequested_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.tfarequested_);
        } else {
          _this->_impl_.event_.tfarequested_->MergeFrom(*from._impl_.event_.tfarequested_);
        }
        break;
      }
      case kTwoPasswordRequested: {
        if (oneof_needs_init) {
          _this->_impl_.event_.twopasswordrequested_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.twopasswordrequested_);
        } else {
          _this->_impl_.event_.twopasswordrequested_->MergeFrom(*from._impl_.event_.twopasswordrequested_);
        }
        break;
      }
      case kFinished: {
        if (oneof_needs_init) {
          _this->_impl_.event_.finished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.finished_);
        } else {
          _this->_impl_.event_.finished_->MergeFrom(*from._impl_.event_.finished_);
        }
        break;
      }
      case kAlreadyLoggedIn: {
        if (oneof_needs_init) {
          _this->_impl_.event_.alreadyloggedin_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.alreadyloggedin_);
        } else {
          _this->_impl_.event_.alreadyloggedin_->MergeFrom(*from._impl_.event_.alreadyloggedin_);
        }
        break;
      }
      case kHvRequested: {
        if (oneof_needs_init) {
          _this->_impl_.event_.hvrequested_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.hvrequested_);
        } else {
          _this->_impl_.event_.hvrequested_->MergeFrom(*from._impl_.event_.hvrequested_);
        }
        break;
      }
      case kFidoRequested: {
        if (oneof_needs_init) {
          _this->_impl_.event_.fidorequested_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.fidorequested_);
        } else {
          _this->_impl_.event_.fidorequested_->MergeFrom(*from._impl_.event_.fidorequested_);
        }
        break;
      }
      case kTfaOrFidoRequested: {
        if (oneof_needs_init) {
          _this->_impl_.event_.tfaorfidorequested_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.tfaorfidorequested_);
        } else {
          _this->_impl_.event_.tfaorfidorequested_->MergeFrom(*from._impl_.event_.tfaorfidorequested_);
        }
        break;
      }
      case kLoginFidoTouchRequested: {
        if (oneof_needs_init) {
          _this->_impl_.event_.loginfidotouchrequested_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.loginfidotouchrequested_);
        } else {
          _this->_impl_.event_.loginfidotouchrequested_->MergeFrom(*from._impl_.event_.loginfidotouchrequested_);
        }
        break;
      }
      case kLoginFidoTouchCompleted: {
        if (oneof_needs_init) {
          _this->_impl_.event_.loginfidotouchcompleted_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.loginfidotouchcompleted_);
        } else {
          _this->_impl_.event_.loginfidotouchcompleted_->MergeFrom(*from._impl_.event_.loginfidotouchcompleted_);
        }
        break;
      }
      case kLoginFidoPinRequired: {
        if (oneof_needs_init) {
          _this->_impl_.event_.loginfidopinrequired_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.loginfidopinrequired_);
        } else {
          _this->_impl_.event_.loginfidopinrequired_->MergeFrom(*from._impl_.event_.loginfidopinrequired_);
        }
        break;
      }
      case EVENT_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void LoginEvent::CopyFrom(const LoginEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.LoginEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void LoginEvent::InternalSwap(LoginEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_.event_, other->_impl_.event_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata LoginEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

LoginErrorEvent::LoginErrorEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginErrorEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.LoginErrorEvent)
}
PROTOBUF_NDEBUG_INLINE LoginErrorEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::LoginErrorEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        message_(arena, from.message_) {}

LoginErrorEvent::LoginErrorEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const LoginErrorEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginErrorEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  LoginErrorEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.type_ = from._impl_.type_;

  // @@protoc_insertion_point(copy_constructor:grpc.LoginErrorEvent)
}
PROTOBUF_NDEBUG_INLINE LoginErrorEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        message_(arena) {}

inline void LoginErrorEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.type_ = {};
}
LoginErrorEvent::~LoginErrorEvent() {
  // @@protoc_insertion_point(destructor:grpc.LoginErrorEvent)
  SharedDtor(*this);
}
inline void LoginErrorEvent::SharedDtor(MessageLite& self) {
  LoginErrorEvent& this_ = static_cast<LoginErrorEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.message_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull LoginErrorEvent_class_data_ =
        LoginErrorEvent::InternalGenerateClassData_(LoginErrorEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginErrorEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginErrorEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(LoginErrorEvent_class_data_.tc_table);
  return LoginErrorEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginErrorEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginErrorEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&LoginErrorEvent_globals_));
  return LoginErrorEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const LoginErrorEvent::ParseTableT_
    LoginErrorEvent::_table_ =
        LoginErrorEvent::InternalGenerateParseTable_(LoginErrorEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void LoginErrorEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.LoginErrorEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.message_.ClearNonDefaultToEmpty();
  }
  _impl_.type_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL LoginErrorEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const LoginErrorEvent& this_ = static_cast<const LoginErrorEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL LoginErrorEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const LoginErrorEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.LoginErrorEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .grpc.LoginErrorType type = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_type() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_type(), target);
    }
  }

  // string message = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_message().empty()) {
      const ::std::string& _s = this_._internal_message();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.LoginErrorEvent.message");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.LoginErrorEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t LoginErrorEvent::ByteSizeLong(const MessageLite& base) {
  const LoginErrorEvent& this_ = static_cast<const LoginErrorEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t LoginErrorEvent::ByteSizeLong() const {
  const LoginErrorEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.LoginErrorEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string message = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_message().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_message());
      }
    }
    // .grpc.LoginErrorType type = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_type() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_type());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void LoginErrorEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<LoginErrorEvent*>(&to_msg);
  auto& from = static_cast<const LoginErrorEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.LoginErrorEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_message().empty()) {
        _this->_internal_set_message(from._internal_message());
      } else {
        if (_this->_impl_.message_.IsDefault()) {
          _this->_internal_set_message("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_type() != 0) {
        _this->_impl_.type_ = from._impl_.type_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void LoginErrorEvent::CopyFrom(const LoginErrorEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.LoginErrorEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void LoginErrorEvent::InternalSwap(LoginErrorEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.message_, &other->_impl_.message_, arena);
  swap(_impl_.type_, other->_impl_.type_);
}

::google::protobuf::Metadata LoginErrorEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

LoginTfaRequestedEvent::LoginTfaRequestedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginTfaRequestedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.LoginTfaRequestedEvent)
}
PROTOBUF_NDEBUG_INLINE LoginTfaRequestedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::LoginTfaRequestedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        username_(arena, from.username_) {}

LoginTfaRequestedEvent::LoginTfaRequestedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const LoginTfaRequestedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginTfaRequestedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  LoginTfaRequestedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.LoginTfaRequestedEvent)
}
PROTOBUF_NDEBUG_INLINE LoginTfaRequestedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        username_(arena) {}

inline void LoginTfaRequestedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
LoginTfaRequestedEvent::~LoginTfaRequestedEvent() {
  // @@protoc_insertion_point(destructor:grpc.LoginTfaRequestedEvent)
  SharedDtor(*this);
}
inline void LoginTfaRequestedEvent::SharedDtor(MessageLite& self) {
  LoginTfaRequestedEvent& this_ = static_cast<LoginTfaRequestedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.username_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull LoginTfaRequestedEvent_class_data_ =
        LoginTfaRequestedEvent::InternalGenerateClassData_(LoginTfaRequestedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginTfaRequestedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginTfaRequestedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(LoginTfaRequestedEvent_class_data_.tc_table);
  return LoginTfaRequestedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginTfaRequestedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginTfaRequestedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&LoginTfaRequestedEvent_globals_));
  return LoginTfaRequestedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const LoginTfaRequestedEvent::ParseTableT_
    LoginTfaRequestedEvent::_table_ =
        LoginTfaRequestedEvent::InternalGenerateParseTable_(LoginTfaRequestedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void LoginTfaRequestedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.LoginTfaRequestedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.username_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL LoginTfaRequestedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const LoginTfaRequestedEvent& this_ = static_cast<const LoginTfaRequestedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL LoginTfaRequestedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const LoginTfaRequestedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.LoginTfaRequestedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string username = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_username().empty()) {
      const ::std::string& _s = this_._internal_username();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.LoginTfaRequestedEvent.username");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.LoginTfaRequestedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t LoginTfaRequestedEvent::ByteSizeLong(const MessageLite& base) {
  const LoginTfaRequestedEvent& this_ = static_cast<const LoginTfaRequestedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t LoginTfaRequestedEvent::ByteSizeLong() const {
  const LoginTfaRequestedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.LoginTfaRequestedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string username = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_username().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_username());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void LoginTfaRequestedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<LoginTfaRequestedEvent*>(&to_msg);
  auto& from = static_cast<const LoginTfaRequestedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.LoginTfaRequestedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_username().empty()) {
      _this->_internal_set_username(from._internal_username());
    } else {
      if (_this->_impl_.username_.IsDefault()) {
        _this->_internal_set_username("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void LoginTfaRequestedEvent::CopyFrom(const LoginTfaRequestedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.LoginTfaRequestedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void LoginTfaRequestedEvent::InternalSwap(LoginTfaRequestedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.username_, &other->_impl_.username_, arena);
}

::google::protobuf::Metadata LoginTfaRequestedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

LoginFidoRequestedEvent::LoginFidoRequestedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginFidoRequestedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.LoginFidoRequestedEvent)
}
PROTOBUF_NDEBUG_INLINE LoginFidoRequestedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::LoginFidoRequestedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        username_(arena, from.username_) {}

LoginFidoRequestedEvent::LoginFidoRequestedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const LoginFidoRequestedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginFidoRequestedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  LoginFidoRequestedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.LoginFidoRequestedEvent)
}
PROTOBUF_NDEBUG_INLINE LoginFidoRequestedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        username_(arena) {}

inline void LoginFidoRequestedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
LoginFidoRequestedEvent::~LoginFidoRequestedEvent() {
  // @@protoc_insertion_point(destructor:grpc.LoginFidoRequestedEvent)
  SharedDtor(*this);
}
inline void LoginFidoRequestedEvent::SharedDtor(MessageLite& self) {
  LoginFidoRequestedEvent& this_ = static_cast<LoginFidoRequestedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.username_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull LoginFidoRequestedEvent_class_data_ =
        LoginFidoRequestedEvent::InternalGenerateClassData_(LoginFidoRequestedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginFidoRequestedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginFidoRequestedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(LoginFidoRequestedEvent_class_data_.tc_table);
  return LoginFidoRequestedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginFidoRequestedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginFidoRequestedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&LoginFidoRequestedEvent_globals_));
  return LoginFidoRequestedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const LoginFidoRequestedEvent::ParseTableT_
    LoginFidoRequestedEvent::_table_ =
        LoginFidoRequestedEvent::InternalGenerateParseTable_(LoginFidoRequestedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void LoginFidoRequestedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.LoginFidoRequestedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.username_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL LoginFidoRequestedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const LoginFidoRequestedEvent& this_ = static_cast<const LoginFidoRequestedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL LoginFidoRequestedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const LoginFidoRequestedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.LoginFidoRequestedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string username = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_username().empty()) {
      const ::std::string& _s = this_._internal_username();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.LoginFidoRequestedEvent.username");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.LoginFidoRequestedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t LoginFidoRequestedEvent::ByteSizeLong(const MessageLite& base) {
  const LoginFidoRequestedEvent& this_ = static_cast<const LoginFidoRequestedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t LoginFidoRequestedEvent::ByteSizeLong() const {
  const LoginFidoRequestedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.LoginFidoRequestedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string username = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_username().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_username());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void LoginFidoRequestedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<LoginFidoRequestedEvent*>(&to_msg);
  auto& from = static_cast<const LoginFidoRequestedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.LoginFidoRequestedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_username().empty()) {
      _this->_internal_set_username(from._internal_username());
    } else {
      if (_this->_impl_.username_.IsDefault()) {
        _this->_internal_set_username("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void LoginFidoRequestedEvent::CopyFrom(const LoginFidoRequestedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.LoginFidoRequestedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void LoginFidoRequestedEvent::InternalSwap(LoginFidoRequestedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.username_, &other->_impl_.username_, arena);
}

::google::protobuf::Metadata LoginFidoRequestedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

LoginTfaOrFidoRequestedEvent::LoginTfaOrFidoRequestedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginTfaOrFidoRequestedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.LoginTfaOrFidoRequestedEvent)
}
PROTOBUF_NDEBUG_INLINE LoginTfaOrFidoRequestedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::LoginTfaOrFidoRequestedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        username_(arena, from.username_) {}

LoginTfaOrFidoRequestedEvent::LoginTfaOrFidoRequestedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const LoginTfaOrFidoRequestedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginTfaOrFidoRequestedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  LoginTfaOrFidoRequestedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.LoginTfaOrFidoRequestedEvent)
}
PROTOBUF_NDEBUG_INLINE LoginTfaOrFidoRequestedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        username_(arena) {}

inline void LoginTfaOrFidoRequestedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
LoginTfaOrFidoRequestedEvent::~LoginTfaOrFidoRequestedEvent() {
  // @@protoc_insertion_point(destructor:grpc.LoginTfaOrFidoRequestedEvent)
  SharedDtor(*this);
}
inline void LoginTfaOrFidoRequestedEvent::SharedDtor(MessageLite& self) {
  LoginTfaOrFidoRequestedEvent& this_ = static_cast<LoginTfaOrFidoRequestedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.username_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull LoginTfaOrFidoRequestedEvent_class_data_ =
        LoginTfaOrFidoRequestedEvent::InternalGenerateClassData_(LoginTfaOrFidoRequestedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginTfaOrFidoRequestedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginTfaOrFidoRequestedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(LoginTfaOrFidoRequestedEvent_class_data_.tc_table);
  return LoginTfaOrFidoRequestedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginTfaOrFidoRequestedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginTfaOrFidoRequestedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&LoginTfaOrFidoRequestedEvent_globals_));
  return LoginTfaOrFidoRequestedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const LoginTfaOrFidoRequestedEvent::ParseTableT_
    LoginTfaOrFidoRequestedEvent::_table_ =
        LoginTfaOrFidoRequestedEvent::InternalGenerateParseTable_(LoginTfaOrFidoRequestedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void LoginTfaOrFidoRequestedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.LoginTfaOrFidoRequestedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.username_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL LoginTfaOrFidoRequestedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const LoginTfaOrFidoRequestedEvent& this_ = static_cast<const LoginTfaOrFidoRequestedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL LoginTfaOrFidoRequestedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const LoginTfaOrFidoRequestedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.LoginTfaOrFidoRequestedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string username = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_username().empty()) {
      const ::std::string& _s = this_._internal_username();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.LoginTfaOrFidoRequestedEvent.username");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.LoginTfaOrFidoRequestedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t LoginTfaOrFidoRequestedEvent::ByteSizeLong(const MessageLite& base) {
  const LoginTfaOrFidoRequestedEvent& this_ = static_cast<const LoginTfaOrFidoRequestedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t LoginTfaOrFidoRequestedEvent::ByteSizeLong() const {
  const LoginTfaOrFidoRequestedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.LoginTfaOrFidoRequestedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string username = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_username().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_username());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void LoginTfaOrFidoRequestedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<LoginTfaOrFidoRequestedEvent*>(&to_msg);
  auto& from = static_cast<const LoginTfaOrFidoRequestedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.LoginTfaOrFidoRequestedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_username().empty()) {
      _this->_internal_set_username(from._internal_username());
    } else {
      if (_this->_impl_.username_.IsDefault()) {
        _this->_internal_set_username("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void LoginTfaOrFidoRequestedEvent::CopyFrom(const LoginTfaOrFidoRequestedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.LoginTfaOrFidoRequestedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void LoginTfaOrFidoRequestedEvent::InternalSwap(LoginTfaOrFidoRequestedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.username_, &other->_impl_.username_, arena);
}

::google::protobuf::Metadata LoginTfaOrFidoRequestedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

LoginFidoTouchEvent::LoginFidoTouchEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginFidoTouchEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.LoginFidoTouchEvent)
}
PROTOBUF_NDEBUG_INLINE LoginFidoTouchEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::LoginFidoTouchEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        username_(arena, from.username_) {}

LoginFidoTouchEvent::LoginFidoTouchEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const LoginFidoTouchEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginFidoTouchEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  LoginFidoTouchEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.LoginFidoTouchEvent)
}
PROTOBUF_NDEBUG_INLINE LoginFidoTouchEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        username_(arena) {}

inline void LoginFidoTouchEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
LoginFidoTouchEvent::~LoginFidoTouchEvent() {
  // @@protoc_insertion_point(destructor:grpc.LoginFidoTouchEvent)
  SharedDtor(*this);
}
inline void LoginFidoTouchEvent::SharedDtor(MessageLite& self) {
  LoginFidoTouchEvent& this_ = static_cast<LoginFidoTouchEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.username_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull LoginFidoTouchEvent_class_data_ =
        LoginFidoTouchEvent::InternalGenerateClassData_(LoginFidoTouchEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginFidoTouchEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginFidoTouchEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(LoginFidoTouchEvent_class_data_.tc_table);
  return LoginFidoTouchEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginFidoTouchEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginFidoTouchEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&LoginFidoTouchEvent_globals_));
  return LoginFidoTouchEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const LoginFidoTouchEvent::ParseTableT_
    LoginFidoTouchEvent::_table_ =
        LoginFidoTouchEvent::InternalGenerateParseTable_(LoginFidoTouchEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void LoginFidoTouchEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.LoginFidoTouchEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.username_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL LoginFidoTouchEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const LoginFidoTouchEvent& this_ = static_cast<const LoginFidoTouchEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL LoginFidoTouchEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const LoginFidoTouchEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.LoginFidoTouchEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string username = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_username().empty()) {
      const ::std::string& _s = this_._internal_username();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.LoginFidoTouchEvent.username");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.LoginFidoTouchEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t LoginFidoTouchEvent::ByteSizeLong(const MessageLite& base) {
  const LoginFidoTouchEvent& this_ = static_cast<const LoginFidoTouchEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t LoginFidoTouchEvent::ByteSizeLong() const {
  const LoginFidoTouchEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.LoginFidoTouchEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string username = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_username().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_username());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void LoginFidoTouchEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<LoginFidoTouchEvent*>(&to_msg);
  auto& from = static_cast<const LoginFidoTouchEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.LoginFidoTouchEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_username().empty()) {
      _this->_internal_set_username(from._internal_username());
    } else {
      if (_this->_impl_.username_.IsDefault()) {
        _this->_internal_set_username("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void LoginFidoTouchEvent::CopyFrom(const LoginFidoTouchEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.LoginFidoTouchEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void LoginFidoTouchEvent::InternalSwap(LoginFidoTouchEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.username_, &other->_impl_.username_, arena);
}

::google::protobuf::Metadata LoginFidoTouchEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

LoginFidoPinRequired::LoginFidoPinRequired(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginFidoPinRequired_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.LoginFidoPinRequired)
}
PROTOBUF_NDEBUG_INLINE LoginFidoPinRequired::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::LoginFidoPinRequired& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        username_(arena, from.username_) {}

LoginFidoPinRequired::LoginFidoPinRequired(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const LoginFidoPinRequired& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginFidoPinRequired_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  LoginFidoPinRequired* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.LoginFidoPinRequired)
}
PROTOBUF_NDEBUG_INLINE LoginFidoPinRequired::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        username_(arena) {}

inline void LoginFidoPinRequired::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
LoginFidoPinRequired::~LoginFidoPinRequired() {
  // @@protoc_insertion_point(destructor:grpc.LoginFidoPinRequired)
  SharedDtor(*this);
}
inline void LoginFidoPinRequired::SharedDtor(MessageLite& self) {
  LoginFidoPinRequired& this_ = static_cast<LoginFidoPinRequired&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.username_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull LoginFidoPinRequired_class_data_ =
        LoginFidoPinRequired::InternalGenerateClassData_(LoginFidoPinRequired_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginFidoPinRequired::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginFidoPinRequired_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(LoginFidoPinRequired_class_data_.tc_table);
  return LoginFidoPinRequired_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginFidoPinRequired::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginFidoPinRequired_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&LoginFidoPinRequired_globals_));
  return LoginFidoPinRequired_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const LoginFidoPinRequired::ParseTableT_
    LoginFidoPinRequired::_table_ =
        LoginFidoPinRequired::InternalGenerateParseTable_(LoginFidoPinRequired_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void LoginFidoPinRequired::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.LoginFidoPinRequired)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.username_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL LoginFidoPinRequired::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const LoginFidoPinRequired& this_ = static_cast<const LoginFidoPinRequired&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL LoginFidoPinRequired::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const LoginFidoPinRequired& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.LoginFidoPinRequired)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string username = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_username().empty()) {
      const ::std::string& _s = this_._internal_username();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.LoginFidoPinRequired.username");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.LoginFidoPinRequired)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t LoginFidoPinRequired::ByteSizeLong(const MessageLite& base) {
  const LoginFidoPinRequired& this_ = static_cast<const LoginFidoPinRequired&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t LoginFidoPinRequired::ByteSizeLong() const {
  const LoginFidoPinRequired& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.LoginFidoPinRequired)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string username = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_username().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_username());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void LoginFidoPinRequired::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<LoginFidoPinRequired*>(&to_msg);
  auto& from = static_cast<const LoginFidoPinRequired&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.LoginFidoPinRequired)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_username().empty()) {
      _this->_internal_set_username(from._internal_username());
    } else {
      if (_this->_impl_.username_.IsDefault()) {
        _this->_internal_set_username("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void LoginFidoPinRequired::CopyFrom(const LoginFidoPinRequired& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.LoginFidoPinRequired)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void LoginFidoPinRequired::InternalSwap(LoginFidoPinRequired* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.username_, &other->_impl_.username_, arena);
}

::google::protobuf::Metadata LoginFidoPinRequired::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

LoginTwoPasswordsRequestedEvent::LoginTwoPasswordsRequestedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginTwoPasswordsRequestedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.LoginTwoPasswordsRequestedEvent)
}
PROTOBUF_NDEBUG_INLINE LoginTwoPasswordsRequestedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::LoginTwoPasswordsRequestedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        username_(arena, from.username_) {}

LoginTwoPasswordsRequestedEvent::LoginTwoPasswordsRequestedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const LoginTwoPasswordsRequestedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginTwoPasswordsRequestedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  LoginTwoPasswordsRequestedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.LoginTwoPasswordsRequestedEvent)
}
PROTOBUF_NDEBUG_INLINE LoginTwoPasswordsRequestedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        username_(arena) {}

inline void LoginTwoPasswordsRequestedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
LoginTwoPasswordsRequestedEvent::~LoginTwoPasswordsRequestedEvent() {
  // @@protoc_insertion_point(destructor:grpc.LoginTwoPasswordsRequestedEvent)
  SharedDtor(*this);
}
inline void LoginTwoPasswordsRequestedEvent::SharedDtor(MessageLite& self) {
  LoginTwoPasswordsRequestedEvent& this_ = static_cast<LoginTwoPasswordsRequestedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.username_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull LoginTwoPasswordsRequestedEvent_class_data_ =
        LoginTwoPasswordsRequestedEvent::InternalGenerateClassData_(LoginTwoPasswordsRequestedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginTwoPasswordsRequestedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginTwoPasswordsRequestedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(LoginTwoPasswordsRequestedEvent_class_data_.tc_table);
  return LoginTwoPasswordsRequestedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginTwoPasswordsRequestedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginTwoPasswordsRequestedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&LoginTwoPasswordsRequestedEvent_globals_));
  return LoginTwoPasswordsRequestedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const LoginTwoPasswordsRequestedEvent::ParseTableT_
    LoginTwoPasswordsRequestedEvent::_table_ =
        LoginTwoPasswordsRequestedEvent::InternalGenerateParseTable_(LoginTwoPasswordsRequestedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void LoginTwoPasswordsRequestedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.LoginTwoPasswordsRequestedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.username_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL LoginTwoPasswordsRequestedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const LoginTwoPasswordsRequestedEvent& this_ = static_cast<const LoginTwoPasswordsRequestedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL LoginTwoPasswordsRequestedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const LoginTwoPasswordsRequestedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.LoginTwoPasswordsRequestedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string username = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_username().empty()) {
      const ::std::string& _s = this_._internal_username();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.LoginTwoPasswordsRequestedEvent.username");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.LoginTwoPasswordsRequestedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t LoginTwoPasswordsRequestedEvent::ByteSizeLong(const MessageLite& base) {
  const LoginTwoPasswordsRequestedEvent& this_ = static_cast<const LoginTwoPasswordsRequestedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t LoginTwoPasswordsRequestedEvent::ByteSizeLong() const {
  const LoginTwoPasswordsRequestedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.LoginTwoPasswordsRequestedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string username = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_username().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_username());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void LoginTwoPasswordsRequestedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<LoginTwoPasswordsRequestedEvent*>(&to_msg);
  auto& from = static_cast<const LoginTwoPasswordsRequestedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.LoginTwoPasswordsRequestedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_username().empty()) {
      _this->_internal_set_username(from._internal_username());
    } else {
      if (_this->_impl_.username_.IsDefault()) {
        _this->_internal_set_username("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void LoginTwoPasswordsRequestedEvent::CopyFrom(const LoginTwoPasswordsRequestedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.LoginTwoPasswordsRequestedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void LoginTwoPasswordsRequestedEvent::InternalSwap(LoginTwoPasswordsRequestedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.username_, &other->_impl_.username_, arena);
}

::google::protobuf::Metadata LoginTwoPasswordsRequestedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

LoginFinishedEvent::LoginFinishedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginFinishedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.LoginFinishedEvent)
}
PROTOBUF_NDEBUG_INLINE LoginFinishedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::LoginFinishedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        userid_(arena, from.userid_) {}

LoginFinishedEvent::LoginFinishedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const LoginFinishedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginFinishedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  LoginFinishedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.wassignedout_ = from._impl_.wassignedout_;

  // @@protoc_insertion_point(copy_constructor:grpc.LoginFinishedEvent)
}
PROTOBUF_NDEBUG_INLINE LoginFinishedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        userid_(arena) {}

inline void LoginFinishedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.wassignedout_ = {};
}
LoginFinishedEvent::~LoginFinishedEvent() {
  // @@protoc_insertion_point(destructor:grpc.LoginFinishedEvent)
  SharedDtor(*this);
}
inline void LoginFinishedEvent::SharedDtor(MessageLite& self) {
  LoginFinishedEvent& this_ = static_cast<LoginFinishedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.userid_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull LoginFinishedEvent_class_data_ =
        LoginFinishedEvent::InternalGenerateClassData_(LoginFinishedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginFinishedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(LoginFinishedEvent_class_data_.tc_table);
  return LoginFinishedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginFinishedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&LoginFinishedEvent_globals_));
  return LoginFinishedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const LoginFinishedEvent::ParseTableT_
    LoginFinishedEvent::_table_ =
        LoginFinishedEvent::InternalGenerateParseTable_(LoginFinishedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void LoginFinishedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.LoginFinishedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.userid_.ClearNonDefaultToEmpty();
  }
  _impl_.wassignedout_ = false;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL LoginFinishedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const LoginFinishedEvent& this_ = static_cast<const LoginFinishedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL LoginFinishedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const LoginFinishedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.LoginFinishedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string userID = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_userid().empty()) {
      const ::std::string& _s = this_._internal_userid();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.LoginFinishedEvent.userID");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // bool wasSignedOut = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_wassignedout() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          2, this_._internal_wassignedout(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.LoginFinishedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t LoginFinishedEvent::ByteSizeLong(const MessageLite& base) {
  const LoginFinishedEvent& this_ = static_cast<const LoginFinishedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t LoginFinishedEvent::ByteSizeLong() const {
  const LoginFinishedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.LoginFinishedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string userID = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_userid().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_userid());
      }
    }
    // bool wasSignedOut = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_wassignedout() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void LoginFinishedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<LoginFinishedEvent*>(&to_msg);
  auto& from = static_cast<const LoginFinishedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.LoginFinishedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_userid().empty()) {
        _this->_internal_set_userid(from._internal_userid());
      } else {
        if (_this->_impl_.userid_.IsDefault()) {
          _this->_internal_set_userid("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_wassignedout() != 0) {
        _this->_impl_.wassignedout_ = from._impl_.wassignedout_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void LoginFinishedEvent::CopyFrom(const LoginFinishedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.LoginFinishedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void LoginFinishedEvent::InternalSwap(LoginFinishedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.userid_, &other->_impl_.userid_, arena);
  swap(_impl_.wassignedout_, other->_impl_.wassignedout_);
}

::google::protobuf::Metadata LoginFinishedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

LoginHvRequestedEvent::LoginHvRequestedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginHvRequestedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.LoginHvRequestedEvent)
}
PROTOBUF_NDEBUG_INLINE LoginHvRequestedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::LoginHvRequestedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        hvurl_(arena, from.hvurl_) {}

LoginHvRequestedEvent::LoginHvRequestedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const LoginHvRequestedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LoginHvRequestedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  LoginHvRequestedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.LoginHvRequestedEvent)
}
PROTOBUF_NDEBUG_INLINE LoginHvRequestedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        hvurl_(arena) {}

inline void LoginHvRequestedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
LoginHvRequestedEvent::~LoginHvRequestedEvent() {
  // @@protoc_insertion_point(destructor:grpc.LoginHvRequestedEvent)
  SharedDtor(*this);
}
inline void LoginHvRequestedEvent::SharedDtor(MessageLite& self) {
  LoginHvRequestedEvent& this_ = static_cast<LoginHvRequestedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.hvurl_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull LoginHvRequestedEvent_class_data_ =
        LoginHvRequestedEvent::InternalGenerateClassData_(LoginHvRequestedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginHvRequestedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginHvRequestedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(LoginHvRequestedEvent_class_data_.tc_table);
  return LoginHvRequestedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LoginHvRequestedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LoginHvRequestedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&LoginHvRequestedEvent_globals_));
  return LoginHvRequestedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const LoginHvRequestedEvent::ParseTableT_
    LoginHvRequestedEvent::_table_ =
        LoginHvRequestedEvent::InternalGenerateParseTable_(LoginHvRequestedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void LoginHvRequestedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.LoginHvRequestedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.hvurl_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL LoginHvRequestedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const LoginHvRequestedEvent& this_ = static_cast<const LoginHvRequestedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL LoginHvRequestedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const LoginHvRequestedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.LoginHvRequestedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string hvUrl = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_hvurl().empty()) {
      const ::std::string& _s = this_._internal_hvurl();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.LoginHvRequestedEvent.hvUrl");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.LoginHvRequestedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t LoginHvRequestedEvent::ByteSizeLong(const MessageLite& base) {
  const LoginHvRequestedEvent& this_ = static_cast<const LoginHvRequestedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t LoginHvRequestedEvent::ByteSizeLong() const {
  const LoginHvRequestedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.LoginHvRequestedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string hvUrl = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_hvurl().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_hvurl());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void LoginHvRequestedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<LoginHvRequestedEvent*>(&to_msg);
  auto& from = static_cast<const LoginHvRequestedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.LoginHvRequestedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_hvurl().empty()) {
      _this->_internal_set_hvurl(from._internal_hvurl());
    } else {
      if (_this->_impl_.hvurl_.IsDefault()) {
        _this->_internal_set_hvurl("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void LoginHvRequestedEvent::CopyFrom(const LoginHvRequestedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.LoginHvRequestedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void LoginHvRequestedEvent::InternalSwap(LoginHvRequestedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.hvurl_, &other->_impl_.hvurl_, arena);
}

::google::protobuf::Metadata LoginHvRequestedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void UpdateEvent::set_allocated_error(::grpc::UpdateErrorEvent* PROTOBUF_NULLABLE error) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (error) {
    ::google::protobuf::Arena* submessage_arena = error->GetArena();
    if (message_arena != submessage_arena) {
      error = ::google::protobuf::internal::GetOwnedMessage(message_arena, error, submessage_arena);
    }
    set_has_error();
    _impl_.event_.error_ = error;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UpdateEvent.error)
}
void UpdateEvent::set_allocated_manualready(::grpc::UpdateManualReadyEvent* PROTOBUF_NULLABLE manualready) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (manualready) {
    ::google::protobuf::Arena* submessage_arena = manualready->GetArena();
    if (message_arena != submessage_arena) {
      manualready = ::google::protobuf::internal::GetOwnedMessage(message_arena, manualready, submessage_arena);
    }
    set_has_manualready();
    _impl_.event_.manualready_ = manualready;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UpdateEvent.manualReady)
}
void UpdateEvent::set_allocated_manualrestartneeded(::grpc::UpdateManualRestartNeededEvent* PROTOBUF_NULLABLE manualrestartneeded) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (manualrestartneeded) {
    ::google::protobuf::Arena* submessage_arena = manualrestartneeded->GetArena();
    if (message_arena != submessage_arena) {
      manualrestartneeded = ::google::protobuf::internal::GetOwnedMessage(message_arena, manualrestartneeded, submessage_arena);
    }
    set_has_manualrestartneeded();
    _impl_.event_.manualrestartneeded_ = manualrestartneeded;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UpdateEvent.manualRestartNeeded)
}
void UpdateEvent::set_allocated_force(::grpc::UpdateForceEvent* PROTOBUF_NULLABLE force) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (force) {
    ::google::protobuf::Arena* submessage_arena = force->GetArena();
    if (message_arena != submessage_arena) {
      force = ::google::protobuf::internal::GetOwnedMessage(message_arena, force, submessage_arena);
    }
    set_has_force();
    _impl_.event_.force_ = force;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UpdateEvent.force)
}
void UpdateEvent::set_allocated_silentrestartneeded(::grpc::UpdateSilentRestartNeeded* PROTOBUF_NULLABLE silentrestartneeded) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (silentrestartneeded) {
    ::google::protobuf::Arena* submessage_arena = silentrestartneeded->GetArena();
    if (message_arena != submessage_arena) {
      silentrestartneeded = ::google::protobuf::internal::GetOwnedMessage(message_arena, silentrestartneeded, submessage_arena);
    }
    set_has_silentrestartneeded();
    _impl_.event_.silentrestartneeded_ = silentrestartneeded;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UpdateEvent.silentRestartNeeded)
}
void UpdateEvent::set_allocated_islatestversion(::grpc::UpdateIsLatestVersion* PROTOBUF_NULLABLE islatestversion) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (islatestversion) {
    ::google::protobuf::Arena* submessage_arena = islatestversion->GetArena();
    if (message_arena != submessage_arena) {
      islatestversion = ::google::protobuf::internal::GetOwnedMessage(message_arena, islatestversion, submessage_arena);
    }
    set_has_islatestversion();
    _impl_.event_.islatestversion_ = islatestversion;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UpdateEvent.isLatestVersion)
}
void UpdateEvent::set_allocated_checkfinished(::grpc::UpdateCheckFinished* PROTOBUF_NULLABLE checkfinished) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (checkfinished) {
    ::google::protobuf::Arena* submessage_arena = checkfinished->GetArena();
    if (message_arena != submessage_arena) {
      checkfinished = ::google::protobuf::internal::GetOwnedMessage(message_arena, checkfinished, submessage_arena);
    }
    set_has_checkfinished();
    _impl_.event_.checkfinished_ = checkfinished;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UpdateEvent.checkFinished)
}
void UpdateEvent::set_allocated_versionchanged(::grpc::UpdateVersionChanged* PROTOBUF_NULLABLE versionchanged) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (versionchanged) {
    ::google::protobuf::Arena* submessage_arena = versionchanged->GetArena();
    if (message_arena != submessage_arena) {
      versionchanged = ::google::protobuf::internal::GetOwnedMessage(message_arena, versionchanged, submessage_arena);
    }
    set_has_versionchanged();
    _impl_.event_.versionchanged_ = versionchanged;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UpdateEvent.versionChanged)
}
UpdateEvent::UpdateEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UpdateEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.UpdateEvent)
}
PROTOBUF_NDEBUG_INLINE UpdateEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::UpdateEvent& from_msg)
      : event_{},
        _cached_size_{0},
        _oneof_case_{from._oneof_case_[0]} {}

UpdateEvent::UpdateEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UpdateEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UpdateEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UpdateEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  switch (event_case()) {
    case EVENT_NOT_SET:
      break;
      case kError:
        _impl_.event_.error_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.error_);
        break;
      case kManualReady:
        _impl_.event_.manualready_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.manualready_);
        break;
      case kManualRestartNeeded:
        _impl_.event_.manualrestartneeded_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.manualrestartneeded_);
        break;
      case kForce:
        _impl_.event_.force_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.force_);
        break;
      case kSilentRestartNeeded:
        _impl_.event_.silentrestartneeded_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.silentrestartneeded_);
        break;
      case kIsLatestVersion:
        _impl_.event_.islatestversion_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.islatestversion_);
        break;
      case kCheckFinished:
        _impl_.event_.checkfinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.checkfinished_);
        break;
      case kVersionChanged:
        _impl_.event_.versionchanged_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.versionchanged_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:grpc.UpdateEvent)
}
PROTOBUF_NDEBUG_INLINE UpdateEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

inline void UpdateEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
UpdateEvent::~UpdateEvent() {
  // @@protoc_insertion_point(destructor:grpc.UpdateEvent)
  SharedDtor(*this);
}
inline void UpdateEvent::SharedDtor(MessageLite& self) {
  UpdateEvent& this_ = static_cast<UpdateEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  if (this_.has_event()) {
    this_.clear_event();
  }
  this_._impl_.~Impl_();
}

void UpdateEvent::clear_event() {
// @@protoc_insertion_point(one_of_clear_start:grpc.UpdateEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (event_case()) {
    case kError: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.error_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.error_);
      }
      break;
    }
    case kManualReady: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.manualready_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.manualready_);
      }
      break;
    }
    case kManualRestartNeeded: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.manualrestartneeded_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.manualrestartneeded_);
      }
      break;
    }
    case kForce: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.force_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.force_);
      }
      break;
    }
    case kSilentRestartNeeded: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.silentrestartneeded_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.silentrestartneeded_);
      }
      break;
    }
    case kIsLatestVersion: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.islatestversion_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.islatestversion_);
      }
      break;
    }
    case kCheckFinished: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.checkfinished_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.checkfinished_);
      }
      break;
    }
    case kVersionChanged: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.versionchanged_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.versionchanged_);
      }
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = EVENT_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UpdateEvent_class_data_ =
        UpdateEvent::InternalGenerateClassData_(UpdateEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UpdateEvent_class_data_.tc_table);
  return UpdateEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UpdateEvent_globals_));
  return UpdateEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UpdateEvent::ParseTableT_
    UpdateEvent::_table_ =
        UpdateEvent::InternalGenerateParseTable_(UpdateEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UpdateEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.UpdateEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  clear_event();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UpdateEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UpdateEvent& this_ = static_cast<const UpdateEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UpdateEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UpdateEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.UpdateEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  switch (this_.event_case()) {
    case kError: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          1, *this_._impl_.event_.error_, this_._impl_.event_.error_->GetCachedSize(), target,
          stream);
      break;
    }
    case kManualReady: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          2, *this_._impl_.event_.manualready_, this_._impl_.event_.manualready_->GetCachedSize(), target,
          stream);
      break;
    }
    case kManualRestartNeeded: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          3, *this_._impl_.event_.manualrestartneeded_, this_._impl_.event_.manualrestartneeded_->GetCachedSize(), target,
          stream);
      break;
    }
    case kForce: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          4, *this_._impl_.event_.force_, this_._impl_.event_.force_->GetCachedSize(), target,
          stream);
      break;
    }
    case kSilentRestartNeeded: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          5, *this_._impl_.event_.silentrestartneeded_, this_._impl_.event_.silentrestartneeded_->GetCachedSize(), target,
          stream);
      break;
    }
    case kIsLatestVersion: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          6, *this_._impl_.event_.islatestversion_, this_._impl_.event_.islatestversion_->GetCachedSize(), target,
          stream);
      break;
    }
    case kCheckFinished: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          7, *this_._impl_.event_.checkfinished_, this_._impl_.event_.checkfinished_->GetCachedSize(), target,
          stream);
      break;
    }
    case kVersionChanged: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          8, *this_._impl_.event_.versionchanged_, this_._impl_.event_.versionchanged_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.UpdateEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UpdateEvent::ByteSizeLong(const MessageLite& base) {
  const UpdateEvent& this_ = static_cast<const UpdateEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UpdateEvent::ByteSizeLong() const {
  const UpdateEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.UpdateEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  switch (this_.event_case()) {
    // .grpc.UpdateErrorEvent error = 1;
    case kError: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.error_);
      break;
    }
    // .grpc.UpdateManualReadyEvent manualReady = 2;
    case kManualReady: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.manualready_);
      break;
    }
    // .grpc.UpdateManualRestartNeededEvent manualRestartNeeded = 3;
    case kManualRestartNeeded: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.manualrestartneeded_);
      break;
    }
    // .grpc.UpdateForceEvent force = 4;
    case kForce: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.force_);
      break;
    }
    // .grpc.UpdateSilentRestartNeeded silentRestartNeeded = 5;
    case kSilentRestartNeeded: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.silentrestartneeded_);
      break;
    }
    // .grpc.UpdateIsLatestVersion isLatestVersion = 6;
    case kIsLatestVersion: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.islatestversion_);
      break;
    }
    // .grpc.UpdateCheckFinished checkFinished = 7;
    case kCheckFinished: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.checkfinished_);
      break;
    }
    // .grpc.UpdateVersionChanged versionChanged = 8;
    case kVersionChanged: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.versionchanged_);
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UpdateEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UpdateEvent*>(&to_msg);
  auto& from = static_cast<const UpdateEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.UpdateEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_event();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kError: {
        if (oneof_needs_init) {
          _this->_impl_.event_.error_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.error_);
        } else {
          _this->_impl_.event_.error_->MergeFrom(*from._impl_.event_.error_);
        }
        break;
      }
      case kManualReady: {
        if (oneof_needs_init) {
          _this->_impl_.event_.manualready_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.manualready_);
        } else {
          _this->_impl_.event_.manualready_->MergeFrom(*from._impl_.event_.manualready_);
        }
        break;
      }
      case kManualRestartNeeded: {
        if (oneof_needs_init) {
          _this->_impl_.event_.manualrestartneeded_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.manualrestartneeded_);
        } else {
          _this->_impl_.event_.manualrestartneeded_->MergeFrom(*from._impl_.event_.manualrestartneeded_);
        }
        break;
      }
      case kForce: {
        if (oneof_needs_init) {
          _this->_impl_.event_.force_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.force_);
        } else {
          _this->_impl_.event_.force_->MergeFrom(*from._impl_.event_.force_);
        }
        break;
      }
      case kSilentRestartNeeded: {
        if (oneof_needs_init) {
          _this->_impl_.event_.silentrestartneeded_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.silentrestartneeded_);
        } else {
          _this->_impl_.event_.silentrestartneeded_->MergeFrom(*from._impl_.event_.silentrestartneeded_);
        }
        break;
      }
      case kIsLatestVersion: {
        if (oneof_needs_init) {
          _this->_impl_.event_.islatestversion_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.islatestversion_);
        } else {
          _this->_impl_.event_.islatestversion_->MergeFrom(*from._impl_.event_.islatestversion_);
        }
        break;
      }
      case kCheckFinished: {
        if (oneof_needs_init) {
          _this->_impl_.event_.checkfinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.checkfinished_);
        } else {
          _this->_impl_.event_.checkfinished_->MergeFrom(*from._impl_.event_.checkfinished_);
        }
        break;
      }
      case kVersionChanged: {
        if (oneof_needs_init) {
          _this->_impl_.event_.versionchanged_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.versionchanged_);
        } else {
          _this->_impl_.event_.versionchanged_->MergeFrom(*from._impl_.event_.versionchanged_);
        }
        break;
      }
      case EVENT_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UpdateEvent::CopyFrom(const UpdateEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.UpdateEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UpdateEvent::InternalSwap(UpdateEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_.event_, other->_impl_.event_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata UpdateEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UpdateErrorEvent::UpdateErrorEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UpdateErrorEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.UpdateErrorEvent)
}
UpdateErrorEvent::UpdateErrorEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const UpdateErrorEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UpdateErrorEvent_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE UpdateErrorEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void UpdateErrorEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.type_ = {};
}
UpdateErrorEvent::~UpdateErrorEvent() {
  // @@protoc_insertion_point(destructor:grpc.UpdateErrorEvent)
  SharedDtor(*this);
}
inline void UpdateErrorEvent::SharedDtor(MessageLite& self) {
  UpdateErrorEvent& this_ = static_cast<UpdateErrorEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UpdateErrorEvent_class_data_ =
        UpdateErrorEvent::InternalGenerateClassData_(UpdateErrorEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateErrorEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateErrorEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UpdateErrorEvent_class_data_.tc_table);
  return UpdateErrorEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateErrorEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateErrorEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UpdateErrorEvent_globals_));
  return UpdateErrorEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UpdateErrorEvent::ParseTableT_
    UpdateErrorEvent::_table_ =
        UpdateErrorEvent::InternalGenerateParseTable_(UpdateErrorEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UpdateErrorEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.UpdateErrorEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.type_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UpdateErrorEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UpdateErrorEvent& this_ = static_cast<const UpdateErrorEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UpdateErrorEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UpdateErrorEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.UpdateErrorEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .grpc.UpdateErrorType type = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_type() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_type(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.UpdateErrorEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UpdateErrorEvent::ByteSizeLong(const MessageLite& base) {
  const UpdateErrorEvent& this_ = static_cast<const UpdateErrorEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UpdateErrorEvent::ByteSizeLong() const {
  const UpdateErrorEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.UpdateErrorEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .grpc.UpdateErrorType type = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_type() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_type());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UpdateErrorEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UpdateErrorEvent*>(&to_msg);
  auto& from = static_cast<const UpdateErrorEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.UpdateErrorEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (from._internal_type() != 0) {
      _this->_impl_.type_ = from._impl_.type_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UpdateErrorEvent::CopyFrom(const UpdateErrorEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.UpdateErrorEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UpdateErrorEvent::InternalSwap(UpdateErrorEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.type_, other->_impl_.type_);
}

::google::protobuf::Metadata UpdateErrorEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UpdateManualReadyEvent::UpdateManualReadyEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UpdateManualReadyEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.UpdateManualReadyEvent)
}
PROTOBUF_NDEBUG_INLINE UpdateManualReadyEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::UpdateManualReadyEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        version_(arena, from.version_) {}

UpdateManualReadyEvent::UpdateManualReadyEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UpdateManualReadyEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UpdateManualReadyEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UpdateManualReadyEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.UpdateManualReadyEvent)
}
PROTOBUF_NDEBUG_INLINE UpdateManualReadyEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        version_(arena) {}

inline void UpdateManualReadyEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
UpdateManualReadyEvent::~UpdateManualReadyEvent() {
  // @@protoc_insertion_point(destructor:grpc.UpdateManualReadyEvent)
  SharedDtor(*this);
}
inline void UpdateManualReadyEvent::SharedDtor(MessageLite& self) {
  UpdateManualReadyEvent& this_ = static_cast<UpdateManualReadyEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.version_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UpdateManualReadyEvent_class_data_ =
        UpdateManualReadyEvent::InternalGenerateClassData_(UpdateManualReadyEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateManualReadyEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateManualReadyEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UpdateManualReadyEvent_class_data_.tc_table);
  return UpdateManualReadyEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateManualReadyEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateManualReadyEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UpdateManualReadyEvent_globals_));
  return UpdateManualReadyEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UpdateManualReadyEvent::ParseTableT_
    UpdateManualReadyEvent::_table_ =
        UpdateManualReadyEvent::InternalGenerateParseTable_(UpdateManualReadyEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UpdateManualReadyEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.UpdateManualReadyEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.version_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UpdateManualReadyEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UpdateManualReadyEvent& this_ = static_cast<const UpdateManualReadyEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UpdateManualReadyEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UpdateManualReadyEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.UpdateManualReadyEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string version = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_version().empty()) {
      const ::std::string& _s = this_._internal_version();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.UpdateManualReadyEvent.version");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.UpdateManualReadyEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UpdateManualReadyEvent::ByteSizeLong(const MessageLite& base) {
  const UpdateManualReadyEvent& this_ = static_cast<const UpdateManualReadyEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UpdateManualReadyEvent::ByteSizeLong() const {
  const UpdateManualReadyEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.UpdateManualReadyEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string version = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_version().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_version());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UpdateManualReadyEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UpdateManualReadyEvent*>(&to_msg);
  auto& from = static_cast<const UpdateManualReadyEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.UpdateManualReadyEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_version().empty()) {
      _this->_internal_set_version(from._internal_version());
    } else {
      if (_this->_impl_.version_.IsDefault()) {
        _this->_internal_set_version("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UpdateManualReadyEvent::CopyFrom(const UpdateManualReadyEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.UpdateManualReadyEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UpdateManualReadyEvent::InternalSwap(UpdateManualReadyEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.version_, &other->_impl_.version_, arena);
}

::google::protobuf::Metadata UpdateManualReadyEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UpdateManualRestartNeededEvent::UpdateManualRestartNeededEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, UpdateManualRestartNeededEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.UpdateManualRestartNeededEvent)
}
UpdateManualRestartNeededEvent::UpdateManualRestartNeededEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UpdateManualRestartNeededEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, UpdateManualRestartNeededEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UpdateManualRestartNeededEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.UpdateManualRestartNeededEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UpdateManualRestartNeededEvent_class_data_ =
        UpdateManualRestartNeededEvent::InternalGenerateClassData_(UpdateManualRestartNeededEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateManualRestartNeededEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateManualRestartNeededEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UpdateManualRestartNeededEvent_class_data_.tc_table);
  return UpdateManualRestartNeededEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateManualRestartNeededEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateManualRestartNeededEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UpdateManualRestartNeededEvent_globals_));
  return UpdateManualRestartNeededEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UpdateManualRestartNeededEvent::ParseTableT_
    UpdateManualRestartNeededEvent::_table_ =
        UpdateManualRestartNeededEvent::InternalGenerateParseTable_(UpdateManualRestartNeededEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata UpdateManualRestartNeededEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UpdateForceEvent::UpdateForceEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UpdateForceEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.UpdateForceEvent)
}
PROTOBUF_NDEBUG_INLINE UpdateForceEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::UpdateForceEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        version_(arena, from.version_) {}

UpdateForceEvent::UpdateForceEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UpdateForceEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UpdateForceEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UpdateForceEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.UpdateForceEvent)
}
PROTOBUF_NDEBUG_INLINE UpdateForceEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        version_(arena) {}

inline void UpdateForceEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
UpdateForceEvent::~UpdateForceEvent() {
  // @@protoc_insertion_point(destructor:grpc.UpdateForceEvent)
  SharedDtor(*this);
}
inline void UpdateForceEvent::SharedDtor(MessageLite& self) {
  UpdateForceEvent& this_ = static_cast<UpdateForceEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.version_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UpdateForceEvent_class_data_ =
        UpdateForceEvent::InternalGenerateClassData_(UpdateForceEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateForceEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateForceEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UpdateForceEvent_class_data_.tc_table);
  return UpdateForceEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateForceEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateForceEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UpdateForceEvent_globals_));
  return UpdateForceEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UpdateForceEvent::ParseTableT_
    UpdateForceEvent::_table_ =
        UpdateForceEvent::InternalGenerateParseTable_(UpdateForceEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UpdateForceEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.UpdateForceEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.version_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UpdateForceEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UpdateForceEvent& this_ = static_cast<const UpdateForceEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UpdateForceEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UpdateForceEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.UpdateForceEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string version = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_version().empty()) {
      const ::std::string& _s = this_._internal_version();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.UpdateForceEvent.version");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.UpdateForceEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UpdateForceEvent::ByteSizeLong(const MessageLite& base) {
  const UpdateForceEvent& this_ = static_cast<const UpdateForceEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UpdateForceEvent::ByteSizeLong() const {
  const UpdateForceEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.UpdateForceEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string version = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_version().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_version());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UpdateForceEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UpdateForceEvent*>(&to_msg);
  auto& from = static_cast<const UpdateForceEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.UpdateForceEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_version().empty()) {
      _this->_internal_set_version(from._internal_version());
    } else {
      if (_this->_impl_.version_.IsDefault()) {
        _this->_internal_set_version("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UpdateForceEvent::CopyFrom(const UpdateForceEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.UpdateForceEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UpdateForceEvent::InternalSwap(UpdateForceEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.version_, &other->_impl_.version_, arena);
}

::google::protobuf::Metadata UpdateForceEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UpdateSilentRestartNeeded::UpdateSilentRestartNeeded(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, UpdateSilentRestartNeeded_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.UpdateSilentRestartNeeded)
}
UpdateSilentRestartNeeded::UpdateSilentRestartNeeded(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UpdateSilentRestartNeeded& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, UpdateSilentRestartNeeded_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UpdateSilentRestartNeeded* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.UpdateSilentRestartNeeded)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UpdateSilentRestartNeeded_class_data_ =
        UpdateSilentRestartNeeded::InternalGenerateClassData_(UpdateSilentRestartNeeded_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateSilentRestartNeeded::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateSilentRestartNeeded_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UpdateSilentRestartNeeded_class_data_.tc_table);
  return UpdateSilentRestartNeeded_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateSilentRestartNeeded::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateSilentRestartNeeded_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UpdateSilentRestartNeeded_globals_));
  return UpdateSilentRestartNeeded_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UpdateSilentRestartNeeded::ParseTableT_
    UpdateSilentRestartNeeded::_table_ =
        UpdateSilentRestartNeeded::InternalGenerateParseTable_(UpdateSilentRestartNeeded_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata UpdateSilentRestartNeeded::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UpdateIsLatestVersion::UpdateIsLatestVersion(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, UpdateIsLatestVersion_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.UpdateIsLatestVersion)
}
UpdateIsLatestVersion::UpdateIsLatestVersion(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UpdateIsLatestVersion& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, UpdateIsLatestVersion_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UpdateIsLatestVersion* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.UpdateIsLatestVersion)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UpdateIsLatestVersion_class_data_ =
        UpdateIsLatestVersion::InternalGenerateClassData_(UpdateIsLatestVersion_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateIsLatestVersion::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateIsLatestVersion_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UpdateIsLatestVersion_class_data_.tc_table);
  return UpdateIsLatestVersion_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateIsLatestVersion::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateIsLatestVersion_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UpdateIsLatestVersion_globals_));
  return UpdateIsLatestVersion_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UpdateIsLatestVersion::ParseTableT_
    UpdateIsLatestVersion::_table_ =
        UpdateIsLatestVersion::InternalGenerateParseTable_(UpdateIsLatestVersion_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata UpdateIsLatestVersion::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UpdateCheckFinished::UpdateCheckFinished(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, UpdateCheckFinished_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.UpdateCheckFinished)
}
UpdateCheckFinished::UpdateCheckFinished(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UpdateCheckFinished& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, UpdateCheckFinished_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UpdateCheckFinished* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.UpdateCheckFinished)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UpdateCheckFinished_class_data_ =
        UpdateCheckFinished::InternalGenerateClassData_(UpdateCheckFinished_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateCheckFinished::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateCheckFinished_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UpdateCheckFinished_class_data_.tc_table);
  return UpdateCheckFinished_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateCheckFinished::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateCheckFinished_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UpdateCheckFinished_globals_));
  return UpdateCheckFinished_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UpdateCheckFinished::ParseTableT_
    UpdateCheckFinished::_table_ =
        UpdateCheckFinished::InternalGenerateParseTable_(UpdateCheckFinished_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata UpdateCheckFinished::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UpdateVersionChanged::UpdateVersionChanged(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, UpdateVersionChanged_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.UpdateVersionChanged)
}
UpdateVersionChanged::UpdateVersionChanged(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UpdateVersionChanged& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, UpdateVersionChanged_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UpdateVersionChanged* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.UpdateVersionChanged)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UpdateVersionChanged_class_data_ =
        UpdateVersionChanged::InternalGenerateClassData_(UpdateVersionChanged_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateVersionChanged::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateVersionChanged_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UpdateVersionChanged_class_data_.tc_table);
  return UpdateVersionChanged_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateVersionChanged::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateVersionChanged_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UpdateVersionChanged_globals_));
  return UpdateVersionChanged_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UpdateVersionChanged::ParseTableT_
    UpdateVersionChanged::_table_ =
        UpdateVersionChanged::InternalGenerateParseTable_(UpdateVersionChanged_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata UpdateVersionChanged::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void DiskCacheEvent::set_allocated_error(::grpc::DiskCacheErrorEvent* PROTOBUF_NULLABLE error) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (error) {
    ::google::protobuf::Arena* submessage_arena = error->GetArena();
    if (message_arena != submessage_arena) {
      error = ::google::protobuf::internal::GetOwnedMessage(message_arena, error, submessage_arena);
    }
    set_has_error();
    _impl_.event_.error_ = error;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.DiskCacheEvent.error)
}
void DiskCacheEvent::set_allocated_pathchanged(::grpc::DiskCachePathChangedEvent* PROTOBUF_NULLABLE pathchanged) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (pathchanged) {
    ::google::protobuf::Arena* submessage_arena = pathchanged->GetArena();
    if (message_arena != submessage_arena) {
      pathchanged = ::google::protobuf::internal::GetOwnedMessage(message_arena, pathchanged, submessage_arena);
    }
    set_has_pathchanged();
    _impl_.event_.pathchanged_ = pathchanged;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.DiskCacheEvent.pathChanged)
}
void DiskCacheEvent::set_allocated_pathchangefinished(::grpc::DiskCachePathChangeFinishedEvent* PROTOBUF_NULLABLE pathchangefinished) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (pathchangefinished) {
    ::google::protobuf::Arena* submessage_arena = pathchangefinished->GetArena();
    if (message_arena != submessage_arena) {
      pathchangefinished = ::google::protobuf::internal::GetOwnedMessage(message_arena, pathchangefinished, submessage_arena);
    }
    set_has_pathchangefinished();
    _impl_.event_.pathchangefinished_ = pathchangefinished;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.DiskCacheEvent.pathChangeFinished)
}
DiskCacheEvent::DiskCacheEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DiskCacheEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.DiskCacheEvent)
}
PROTOBUF_NDEBUG_INLINE DiskCacheEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::DiskCacheEvent& from_msg)
      : event_{},
        _cached_size_{0},
        _oneof_case_{from._oneof_case_[0]} {}

DiskCacheEvent::DiskCacheEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const DiskCacheEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DiskCacheEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  DiskCacheEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  switch (event_case()) {
    case EVENT_NOT_SET:
      break;
      case kError:
        _impl_.event_.error_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.error_);
        break;
      case kPathChanged:
        _impl_.event_.pathchanged_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.pathchanged_);
        break;
      case kPathChangeFinished:
        _impl_.event_.pathchangefinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.pathchangefinished_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:grpc.DiskCacheEvent)
}
PROTOBUF_NDEBUG_INLINE DiskCacheEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

inline void DiskCacheEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
DiskCacheEvent::~DiskCacheEvent() {
  // @@protoc_insertion_point(destructor:grpc.DiskCacheEvent)
  SharedDtor(*this);
}
inline void DiskCacheEvent::SharedDtor(MessageLite& self) {
  DiskCacheEvent& this_ = static_cast<DiskCacheEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  if (this_.has_event()) {
    this_.clear_event();
  }
  this_._impl_.~Impl_();
}

void DiskCacheEvent::clear_event() {
// @@protoc_insertion_point(one_of_clear_start:grpc.DiskCacheEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (event_case()) {
    case kError: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.error_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.error_);
      }
      break;
    }
    case kPathChanged: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.pathchanged_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.pathchanged_);
      }
      break;
    }
    case kPathChangeFinished: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.pathchangefinished_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.pathchangefinished_);
      }
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = EVENT_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DiskCacheEvent_class_data_ =
        DiskCacheEvent::InternalGenerateClassData_(DiskCacheEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DiskCacheEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DiskCacheEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DiskCacheEvent_class_data_.tc_table);
  return DiskCacheEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DiskCacheEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DiskCacheEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DiskCacheEvent_globals_));
  return DiskCacheEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DiskCacheEvent::ParseTableT_
    DiskCacheEvent::_table_ =
        DiskCacheEvent::InternalGenerateParseTable_(DiskCacheEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void DiskCacheEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.DiskCacheEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  clear_event();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL DiskCacheEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const DiskCacheEvent& this_ = static_cast<const DiskCacheEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL DiskCacheEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const DiskCacheEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.DiskCacheEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  switch (this_.event_case()) {
    case kError: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          1, *this_._impl_.event_.error_, this_._impl_.event_.error_->GetCachedSize(), target,
          stream);
      break;
    }
    case kPathChanged: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          2, *this_._impl_.event_.pathchanged_, this_._impl_.event_.pathchanged_->GetCachedSize(), target,
          stream);
      break;
    }
    case kPathChangeFinished: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          3, *this_._impl_.event_.pathchangefinished_, this_._impl_.event_.pathchangefinished_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.DiskCacheEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t DiskCacheEvent::ByteSizeLong(const MessageLite& base) {
  const DiskCacheEvent& this_ = static_cast<const DiskCacheEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t DiskCacheEvent::ByteSizeLong() const {
  const DiskCacheEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.DiskCacheEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  switch (this_.event_case()) {
    // .grpc.DiskCacheErrorEvent error = 1;
    case kError: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.error_);
      break;
    }
    // .grpc.DiskCachePathChangedEvent pathChanged = 2;
    case kPathChanged: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.pathchanged_);
      break;
    }
    // .grpc.DiskCachePathChangeFinishedEvent pathChangeFinished = 3;
    case kPathChangeFinished: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.pathchangefinished_);
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void DiskCacheEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<DiskCacheEvent*>(&to_msg);
  auto& from = static_cast<const DiskCacheEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.DiskCacheEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_event();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kError: {
        if (oneof_needs_init) {
          _this->_impl_.event_.error_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.error_);
        } else {
          _this->_impl_.event_.error_->MergeFrom(*from._impl_.event_.error_);
        }
        break;
      }
      case kPathChanged: {
        if (oneof_needs_init) {
          _this->_impl_.event_.pathchanged_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.pathchanged_);
        } else {
          _this->_impl_.event_.pathchanged_->MergeFrom(*from._impl_.event_.pathchanged_);
        }
        break;
      }
      case kPathChangeFinished: {
        if (oneof_needs_init) {
          _this->_impl_.event_.pathchangefinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.pathchangefinished_);
        } else {
          _this->_impl_.event_.pathchangefinished_->MergeFrom(*from._impl_.event_.pathchangefinished_);
        }
        break;
      }
      case EVENT_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void DiskCacheEvent::CopyFrom(const DiskCacheEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.DiskCacheEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void DiskCacheEvent::InternalSwap(DiskCacheEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_.event_, other->_impl_.event_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata DiskCacheEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

DiskCacheErrorEvent::DiskCacheErrorEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DiskCacheErrorEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.DiskCacheErrorEvent)
}
DiskCacheErrorEvent::DiskCacheErrorEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const DiskCacheErrorEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DiskCacheErrorEvent_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE DiskCacheErrorEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void DiskCacheErrorEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.type_ = {};
}
DiskCacheErrorEvent::~DiskCacheErrorEvent() {
  // @@protoc_insertion_point(destructor:grpc.DiskCacheErrorEvent)
  SharedDtor(*this);
}
inline void DiskCacheErrorEvent::SharedDtor(MessageLite& self) {
  DiskCacheErrorEvent& this_ = static_cast<DiskCacheErrorEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DiskCacheErrorEvent_class_data_ =
        DiskCacheErrorEvent::InternalGenerateClassData_(DiskCacheErrorEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DiskCacheErrorEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DiskCacheErrorEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DiskCacheErrorEvent_class_data_.tc_table);
  return DiskCacheErrorEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DiskCacheErrorEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DiskCacheErrorEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DiskCacheErrorEvent_globals_));
  return DiskCacheErrorEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DiskCacheErrorEvent::ParseTableT_
    DiskCacheErrorEvent::_table_ =
        DiskCacheErrorEvent::InternalGenerateParseTable_(DiskCacheErrorEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void DiskCacheErrorEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.DiskCacheErrorEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.type_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL DiskCacheErrorEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const DiskCacheErrorEvent& this_ = static_cast<const DiskCacheErrorEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL DiskCacheErrorEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const DiskCacheErrorEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.DiskCacheErrorEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .grpc.DiskCacheErrorType type = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_type() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_type(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.DiskCacheErrorEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t DiskCacheErrorEvent::ByteSizeLong(const MessageLite& base) {
  const DiskCacheErrorEvent& this_ = static_cast<const DiskCacheErrorEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t DiskCacheErrorEvent::ByteSizeLong() const {
  const DiskCacheErrorEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.DiskCacheErrorEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .grpc.DiskCacheErrorType type = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_type() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_type());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void DiskCacheErrorEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<DiskCacheErrorEvent*>(&to_msg);
  auto& from = static_cast<const DiskCacheErrorEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.DiskCacheErrorEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (from._internal_type() != 0) {
      _this->_impl_.type_ = from._impl_.type_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void DiskCacheErrorEvent::CopyFrom(const DiskCacheErrorEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.DiskCacheErrorEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void DiskCacheErrorEvent::InternalSwap(DiskCacheErrorEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.type_, other->_impl_.type_);
}

::google::protobuf::Metadata DiskCacheErrorEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

DiskCachePathChangedEvent::DiskCachePathChangedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DiskCachePathChangedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.DiskCachePathChangedEvent)
}
PROTOBUF_NDEBUG_INLINE DiskCachePathChangedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::DiskCachePathChangedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        path_(arena, from.path_) {}

DiskCachePathChangedEvent::DiskCachePathChangedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const DiskCachePathChangedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DiskCachePathChangedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  DiskCachePathChangedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.DiskCachePathChangedEvent)
}
PROTOBUF_NDEBUG_INLINE DiskCachePathChangedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        path_(arena) {}

inline void DiskCachePathChangedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
DiskCachePathChangedEvent::~DiskCachePathChangedEvent() {
  // @@protoc_insertion_point(destructor:grpc.DiskCachePathChangedEvent)
  SharedDtor(*this);
}
inline void DiskCachePathChangedEvent::SharedDtor(MessageLite& self) {
  DiskCachePathChangedEvent& this_ = static_cast<DiskCachePathChangedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.path_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DiskCachePathChangedEvent_class_data_ =
        DiskCachePathChangedEvent::InternalGenerateClassData_(DiskCachePathChangedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DiskCachePathChangedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DiskCachePathChangedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DiskCachePathChangedEvent_class_data_.tc_table);
  return DiskCachePathChangedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DiskCachePathChangedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DiskCachePathChangedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DiskCachePathChangedEvent_globals_));
  return DiskCachePathChangedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DiskCachePathChangedEvent::ParseTableT_
    DiskCachePathChangedEvent::_table_ =
        DiskCachePathChangedEvent::InternalGenerateParseTable_(DiskCachePathChangedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void DiskCachePathChangedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.DiskCachePathChangedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.path_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL DiskCachePathChangedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const DiskCachePathChangedEvent& this_ = static_cast<const DiskCachePathChangedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL DiskCachePathChangedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const DiskCachePathChangedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.DiskCachePathChangedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string path = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_path().empty()) {
      const ::std::string& _s = this_._internal_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.DiskCachePathChangedEvent.path");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.DiskCachePathChangedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t DiskCachePathChangedEvent::ByteSizeLong(const MessageLite& base) {
  const DiskCachePathChangedEvent& this_ = static_cast<const DiskCachePathChangedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t DiskCachePathChangedEvent::ByteSizeLong() const {
  const DiskCachePathChangedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.DiskCachePathChangedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string path = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_path());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void DiskCachePathChangedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<DiskCachePathChangedEvent*>(&to_msg);
  auto& from = static_cast<const DiskCachePathChangedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.DiskCachePathChangedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_path().empty()) {
      _this->_internal_set_path(from._internal_path());
    } else {
      if (_this->_impl_.path_.IsDefault()) {
        _this->_internal_set_path("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void DiskCachePathChangedEvent::CopyFrom(const DiskCachePathChangedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.DiskCachePathChangedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void DiskCachePathChangedEvent::InternalSwap(DiskCachePathChangedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.path_, &other->_impl_.path_, arena);
}

::google::protobuf::Metadata DiskCachePathChangedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

DiskCachePathChangeFinishedEvent::DiskCachePathChangeFinishedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, DiskCachePathChangeFinishedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.DiskCachePathChangeFinishedEvent)
}
DiskCachePathChangeFinishedEvent::DiskCachePathChangeFinishedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const DiskCachePathChangeFinishedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, DiskCachePathChangeFinishedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  DiskCachePathChangeFinishedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.DiskCachePathChangeFinishedEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DiskCachePathChangeFinishedEvent_class_data_ =
        DiskCachePathChangeFinishedEvent::InternalGenerateClassData_(DiskCachePathChangeFinishedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DiskCachePathChangeFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DiskCachePathChangeFinishedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DiskCachePathChangeFinishedEvent_class_data_.tc_table);
  return DiskCachePathChangeFinishedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DiskCachePathChangeFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DiskCachePathChangeFinishedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DiskCachePathChangeFinishedEvent_globals_));
  return DiskCachePathChangeFinishedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DiskCachePathChangeFinishedEvent::ParseTableT_
    DiskCachePathChangeFinishedEvent::_table_ =
        DiskCachePathChangeFinishedEvent::InternalGenerateParseTable_(DiskCachePathChangeFinishedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata DiskCachePathChangeFinishedEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void MailServerSettingsEvent::set_allocated_error(::grpc::MailServerSettingsErrorEvent* PROTOBUF_NULLABLE error) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (error) {
    ::google::protobuf::Arena* submessage_arena = error->GetArena();
    if (message_arena != submessage_arena) {
      error = ::google::protobuf::internal::GetOwnedMessage(message_arena, error, submessage_arena);
    }
    set_has_error();
    _impl_.event_.error_ = error;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.MailServerSettingsEvent.error)
}
void MailServerSettingsEvent::set_allocated_mailserversettingschanged(::grpc::MailServerSettingsChangedEvent* PROTOBUF_NULLABLE mailserversettingschanged) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (mailserversettingschanged) {
    ::google::protobuf::Arena* submessage_arena = mailserversettingschanged->GetArena();
    if (message_arena != submessage_arena) {
      mailserversettingschanged = ::google::protobuf::internal::GetOwnedMessage(message_arena, mailserversettingschanged, submessage_arena);
    }
    set_has_mailserversettingschanged();
    _impl_.event_.mailserversettingschanged_ = mailserversettingschanged;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.MailServerSettingsEvent.mailServerSettingsChanged)
}
void MailServerSettingsEvent::set_allocated_changemailserversettingsfinished(::grpc::ChangeMailServerSettingsFinishedEvent* PROTOBUF_NULLABLE changemailserversettingsfinished) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (changemailserversettingsfinished) {
    ::google::protobuf::Arena* submessage_arena = changemailserversettingsfinished->GetArena();
    if (message_arena != submessage_arena) {
      changemailserversettingsfinished = ::google::protobuf::internal::GetOwnedMessage(message_arena, changemailserversettingsfinished, submessage_arena);
    }
    set_has_changemailserversettingsfinished();
    _impl_.event_.changemailserversettingsfinished_ = changemailserversettingsfinished;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.MailServerSettingsEvent.changeMailServerSettingsFinished)
}
MailServerSettingsEvent::MailServerSettingsEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, MailServerSettingsEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.MailServerSettingsEvent)
}
PROTOBUF_NDEBUG_INLINE MailServerSettingsEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::MailServerSettingsEvent& from_msg)
      : event_{},
        _cached_size_{0},
        _oneof_case_{from._oneof_case_[0]} {}

MailServerSettingsEvent::MailServerSettingsEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const MailServerSettingsEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, MailServerSettingsEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  MailServerSettingsEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  switch (event_case()) {
    case EVENT_NOT_SET:
      break;
      case kError:
        _impl_.event_.error_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.error_);
        break;
      case kMailServerSettingsChanged:
        _impl_.event_.mailserversettingschanged_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.mailserversettingschanged_);
        break;
      case kChangeMailServerSettingsFinished:
        _impl_.event_.changemailserversettingsfinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.changemailserversettingsfinished_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:grpc.MailServerSettingsEvent)
}
PROTOBUF_NDEBUG_INLINE MailServerSettingsEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

inline void MailServerSettingsEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
MailServerSettingsEvent::~MailServerSettingsEvent() {
  // @@protoc_insertion_point(destructor:grpc.MailServerSettingsEvent)
  SharedDtor(*this);
}
inline void MailServerSettingsEvent::SharedDtor(MessageLite& self) {
  MailServerSettingsEvent& this_ = static_cast<MailServerSettingsEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  if (this_.has_event()) {
    this_.clear_event();
  }
  this_._impl_.~Impl_();
}

void MailServerSettingsEvent::clear_event() {
// @@protoc_insertion_point(one_of_clear_start:grpc.MailServerSettingsEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (event_case()) {
    case kError: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.error_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.error_);
      }
      break;
    }
    case kMailServerSettingsChanged: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.mailserversettingschanged_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.mailserversettingschanged_);
      }
      break;
    }
    case kChangeMailServerSettingsFinished: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.changemailserversettingsfinished_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.changemailserversettingsfinished_);
      }
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = EVENT_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull MailServerSettingsEvent_class_data_ =
        MailServerSettingsEvent::InternalGenerateClassData_(MailServerSettingsEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
MailServerSettingsEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&MailServerSettingsEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(MailServerSettingsEvent_class_data_.tc_table);
  return MailServerSettingsEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
MailServerSettingsEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&MailServerSettingsEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&MailServerSettingsEvent_globals_));
  return MailServerSettingsEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const MailServerSettingsEvent::ParseTableT_
    MailServerSettingsEvent::_table_ =
        MailServerSettingsEvent::InternalGenerateParseTable_(MailServerSettingsEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void MailServerSettingsEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.MailServerSettingsEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  clear_event();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL MailServerSettingsEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const MailServerSettingsEvent& this_ = static_cast<const MailServerSettingsEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL MailServerSettingsEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const MailServerSettingsEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.MailServerSettingsEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  switch (this_.event_case()) {
    case kError: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          1, *this_._impl_.event_.error_, this_._impl_.event_.error_->GetCachedSize(), target,
          stream);
      break;
    }
    case kMailServerSettingsChanged: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          2, *this_._impl_.event_.mailserversettingschanged_, this_._impl_.event_.mailserversettingschanged_->GetCachedSize(), target,
          stream);
      break;
    }
    case kChangeMailServerSettingsFinished: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          3, *this_._impl_.event_.changemailserversettingsfinished_, this_._impl_.event_.changemailserversettingsfinished_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.MailServerSettingsEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t MailServerSettingsEvent::ByteSizeLong(const MessageLite& base) {
  const MailServerSettingsEvent& this_ = static_cast<const MailServerSettingsEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t MailServerSettingsEvent::ByteSizeLong() const {
  const MailServerSettingsEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.MailServerSettingsEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  switch (this_.event_case()) {
    // .grpc.MailServerSettingsErrorEvent error = 1;
    case kError: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.error_);
      break;
    }
    // .grpc.MailServerSettingsChangedEvent mailServerSettingsChanged = 2;
    case kMailServerSettingsChanged: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.mailserversettingschanged_);
      break;
    }
    // .grpc.ChangeMailServerSettingsFinishedEvent changeMailServerSettingsFinished = 3;
    case kChangeMailServerSettingsFinished: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.changemailserversettingsfinished_);
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void MailServerSettingsEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<MailServerSettingsEvent*>(&to_msg);
  auto& from = static_cast<const MailServerSettingsEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.MailServerSettingsEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_event();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kError: {
        if (oneof_needs_init) {
          _this->_impl_.event_.error_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.error_);
        } else {
          _this->_impl_.event_.error_->MergeFrom(*from._impl_.event_.error_);
        }
        break;
      }
      case kMailServerSettingsChanged: {
        if (oneof_needs_init) {
          _this->_impl_.event_.mailserversettingschanged_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.mailserversettingschanged_);
        } else {
          _this->_impl_.event_.mailserversettingschanged_->MergeFrom(*from._impl_.event_.mailserversettingschanged_);
        }
        break;
      }
      case kChangeMailServerSettingsFinished: {
        if (oneof_needs_init) {
          _this->_impl_.event_.changemailserversettingsfinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.changemailserversettingsfinished_);
        } else {
          _this->_impl_.event_.changemailserversettingsfinished_->MergeFrom(*from._impl_.event_.changemailserversettingsfinished_);
        }
        break;
      }
      case EVENT_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void MailServerSettingsEvent::CopyFrom(const MailServerSettingsEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.MailServerSettingsEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void MailServerSettingsEvent::InternalSwap(MailServerSettingsEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_.event_, other->_impl_.event_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata MailServerSettingsEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

MailServerSettingsErrorEvent::MailServerSettingsErrorEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, MailServerSettingsErrorEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.MailServerSettingsErrorEvent)
}
MailServerSettingsErrorEvent::MailServerSettingsErrorEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const MailServerSettingsErrorEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, MailServerSettingsErrorEvent_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE MailServerSettingsErrorEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void MailServerSettingsErrorEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.type_ = {};
}
MailServerSettingsErrorEvent::~MailServerSettingsErrorEvent() {
  // @@protoc_insertion_point(destructor:grpc.MailServerSettingsErrorEvent)
  SharedDtor(*this);
}
inline void MailServerSettingsErrorEvent::SharedDtor(MessageLite& self) {
  MailServerSettingsErrorEvent& this_ = static_cast<MailServerSettingsErrorEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull MailServerSettingsErrorEvent_class_data_ =
        MailServerSettingsErrorEvent::InternalGenerateClassData_(MailServerSettingsErrorEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
MailServerSettingsErrorEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&MailServerSettingsErrorEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(MailServerSettingsErrorEvent_class_data_.tc_table);
  return MailServerSettingsErrorEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
MailServerSettingsErrorEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&MailServerSettingsErrorEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&MailServerSettingsErrorEvent_globals_));
  return MailServerSettingsErrorEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const MailServerSettingsErrorEvent::ParseTableT_
    MailServerSettingsErrorEvent::_table_ =
        MailServerSettingsErrorEvent::InternalGenerateParseTable_(MailServerSettingsErrorEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void MailServerSettingsErrorEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.MailServerSettingsErrorEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.type_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL MailServerSettingsErrorEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const MailServerSettingsErrorEvent& this_ = static_cast<const MailServerSettingsErrorEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL MailServerSettingsErrorEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const MailServerSettingsErrorEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.MailServerSettingsErrorEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .grpc.MailServerSettingsErrorType type = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_type() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_type(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.MailServerSettingsErrorEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t MailServerSettingsErrorEvent::ByteSizeLong(const MessageLite& base) {
  const MailServerSettingsErrorEvent& this_ = static_cast<const MailServerSettingsErrorEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t MailServerSettingsErrorEvent::ByteSizeLong() const {
  const MailServerSettingsErrorEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.MailServerSettingsErrorEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .grpc.MailServerSettingsErrorType type = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_type() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_type());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void MailServerSettingsErrorEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<MailServerSettingsErrorEvent*>(&to_msg);
  auto& from = static_cast<const MailServerSettingsErrorEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.MailServerSettingsErrorEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (from._internal_type() != 0) {
      _this->_impl_.type_ = from._impl_.type_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void MailServerSettingsErrorEvent::CopyFrom(const MailServerSettingsErrorEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.MailServerSettingsErrorEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void MailServerSettingsErrorEvent::InternalSwap(MailServerSettingsErrorEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.type_, other->_impl_.type_);
}

::google::protobuf::Metadata MailServerSettingsErrorEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

MailServerSettingsChangedEvent::MailServerSettingsChangedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, MailServerSettingsChangedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.MailServerSettingsChangedEvent)
}
PROTOBUF_NDEBUG_INLINE MailServerSettingsChangedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::MailServerSettingsChangedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

MailServerSettingsChangedEvent::MailServerSettingsChangedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const MailServerSettingsChangedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, MailServerSettingsChangedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  MailServerSettingsChangedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.settings_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.settings_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:grpc.MailServerSettingsChangedEvent)
}
PROTOBUF_NDEBUG_INLINE MailServerSettingsChangedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void MailServerSettingsChangedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.settings_ = {};
}
MailServerSettingsChangedEvent::~MailServerSettingsChangedEvent() {
  // @@protoc_insertion_point(destructor:grpc.MailServerSettingsChangedEvent)
  SharedDtor(*this);
}
inline void MailServerSettingsChangedEvent::SharedDtor(MessageLite& self) {
  MailServerSettingsChangedEvent& this_ = static_cast<MailServerSettingsChangedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.settings_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull MailServerSettingsChangedEvent_class_data_ =
        MailServerSettingsChangedEvent::InternalGenerateClassData_(MailServerSettingsChangedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
MailServerSettingsChangedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&MailServerSettingsChangedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(MailServerSettingsChangedEvent_class_data_.tc_table);
  return MailServerSettingsChangedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
MailServerSettingsChangedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&MailServerSettingsChangedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&MailServerSettingsChangedEvent_globals_));
  return MailServerSettingsChangedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const MailServerSettingsChangedEvent::ParseTableT_
    MailServerSettingsChangedEvent::_table_ =
        MailServerSettingsChangedEvent::InternalGenerateParseTable_(MailServerSettingsChangedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void MailServerSettingsChangedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.MailServerSettingsChangedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(_impl_.settings_ != nullptr);
    _impl_.settings_->Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL MailServerSettingsChangedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const MailServerSettingsChangedEvent& this_ = static_cast<const MailServerSettingsChangedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL MailServerSettingsChangedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const MailServerSettingsChangedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.MailServerSettingsChangedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .grpc.ImapSmtpSettings settings = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.settings_, this_._impl_.settings_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.MailServerSettingsChangedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t MailServerSettingsChangedEvent::ByteSizeLong(const MessageLite& base) {
  const MailServerSettingsChangedEvent& this_ = static_cast<const MailServerSettingsChangedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t MailServerSettingsChangedEvent::ByteSizeLong() const {
  const MailServerSettingsChangedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.MailServerSettingsChangedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .grpc.ImapSmtpSettings settings = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.settings_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void MailServerSettingsChangedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<MailServerSettingsChangedEvent*>(&to_msg);
  auto& from = static_cast<const MailServerSettingsChangedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.MailServerSettingsChangedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(from._impl_.settings_ != nullptr);
    if (_this->_impl_.settings_ == nullptr) {
      _this->_impl_.settings_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.settings_);
    } else {
      _this->_impl_.settings_->MergeFrom(*from._impl_.settings_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void MailServerSettingsChangedEvent::CopyFrom(const MailServerSettingsChangedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.MailServerSettingsChangedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void MailServerSettingsChangedEvent::InternalSwap(MailServerSettingsChangedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.settings_, other->_impl_.settings_);
}

::google::protobuf::Metadata MailServerSettingsChangedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ChangeMailServerSettingsFinishedEvent::ChangeMailServerSettingsFinishedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ChangeMailServerSettingsFinishedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.ChangeMailServerSettingsFinishedEvent)
}
ChangeMailServerSettingsFinishedEvent::ChangeMailServerSettingsFinishedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ChangeMailServerSettingsFinishedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ChangeMailServerSettingsFinishedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ChangeMailServerSettingsFinishedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.ChangeMailServerSettingsFinishedEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ChangeMailServerSettingsFinishedEvent_class_data_ =
        ChangeMailServerSettingsFinishedEvent::InternalGenerateClassData_(ChangeMailServerSettingsFinishedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ChangeMailServerSettingsFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ChangeMailServerSettingsFinishedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ChangeMailServerSettingsFinishedEvent_class_data_.tc_table);
  return ChangeMailServerSettingsFinishedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ChangeMailServerSettingsFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ChangeMailServerSettingsFinishedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ChangeMailServerSettingsFinishedEvent_globals_));
  return ChangeMailServerSettingsFinishedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ChangeMailServerSettingsFinishedEvent::ParseTableT_
    ChangeMailServerSettingsFinishedEvent::_table_ =
        ChangeMailServerSettingsFinishedEvent::InternalGenerateParseTable_(ChangeMailServerSettingsFinishedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata ChangeMailServerSettingsFinishedEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void KeychainEvent::set_allocated_changekeychainfinished(::grpc::ChangeKeychainFinishedEvent* PROTOBUF_NULLABLE changekeychainfinished) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (changekeychainfinished) {
    ::google::protobuf::Arena* submessage_arena = changekeychainfinished->GetArena();
    if (message_arena != submessage_arena) {
      changekeychainfinished = ::google::protobuf::internal::GetOwnedMessage(message_arena, changekeychainfinished, submessage_arena);
    }
    set_has_changekeychainfinished();
    _impl_.event_.changekeychainfinished_ = changekeychainfinished;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.KeychainEvent.changeKeychainFinished)
}
void KeychainEvent::set_allocated_hasnokeychain(::grpc::HasNoKeychainEvent* PROTOBUF_NULLABLE hasnokeychain) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (hasnokeychain) {
    ::google::protobuf::Arena* submessage_arena = hasnokeychain->GetArena();
    if (message_arena != submessage_arena) {
      hasnokeychain = ::google::protobuf::internal::GetOwnedMessage(message_arena, hasnokeychain, submessage_arena);
    }
    set_has_hasnokeychain();
    _impl_.event_.hasnokeychain_ = hasnokeychain;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.KeychainEvent.hasNoKeychain)
}
void KeychainEvent::set_allocated_rebuildkeychain(::grpc::RebuildKeychainEvent* PROTOBUF_NULLABLE rebuildkeychain) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (rebuildkeychain) {
    ::google::protobuf::Arena* submessage_arena = rebuildkeychain->GetArena();
    if (message_arena != submessage_arena) {
      rebuildkeychain = ::google::protobuf::internal::GetOwnedMessage(message_arena, rebuildkeychain, submessage_arena);
    }
    set_has_rebuildkeychain();
    _impl_.event_.rebuildkeychain_ = rebuildkeychain;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.KeychainEvent.rebuildKeychain)
}
KeychainEvent::KeychainEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, KeychainEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.KeychainEvent)
}
PROTOBUF_NDEBUG_INLINE KeychainEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::KeychainEvent& from_msg)
      : event_{},
        _cached_size_{0},
        _oneof_case_{from._oneof_case_[0]} {}

KeychainEvent::KeychainEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const KeychainEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, KeychainEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  KeychainEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  switch (event_case()) {
    case EVENT_NOT_SET:
      break;
      case kChangeKeychainFinished:
        _impl_.event_.changekeychainfinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.changekeychainfinished_);
        break;
      case kHasNoKeychain:
        _impl_.event_.hasnokeychain_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.hasnokeychain_);
        break;
      case kRebuildKeychain:
        _impl_.event_.rebuildkeychain_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.rebuildkeychain_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:grpc.KeychainEvent)
}
PROTOBUF_NDEBUG_INLINE KeychainEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

inline void KeychainEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
KeychainEvent::~KeychainEvent() {
  // @@protoc_insertion_point(destructor:grpc.KeychainEvent)
  SharedDtor(*this);
}
inline void KeychainEvent::SharedDtor(MessageLite& self) {
  KeychainEvent& this_ = static_cast<KeychainEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  if (this_.has_event()) {
    this_.clear_event();
  }
  this_._impl_.~Impl_();
}

void KeychainEvent::clear_event() {
// @@protoc_insertion_point(one_of_clear_start:grpc.KeychainEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (event_case()) {
    case kChangeKeychainFinished: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.changekeychainfinished_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.changekeychainfinished_);
      }
      break;
    }
    case kHasNoKeychain: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.hasnokeychain_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.hasnokeychain_);
      }
      break;
    }
    case kRebuildKeychain: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.rebuildkeychain_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.rebuildkeychain_);
      }
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = EVENT_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull KeychainEvent_class_data_ =
        KeychainEvent::InternalGenerateClassData_(KeychainEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
KeychainEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&KeychainEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(KeychainEvent_class_data_.tc_table);
  return KeychainEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
KeychainEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&KeychainEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&KeychainEvent_globals_));
  return KeychainEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const KeychainEvent::ParseTableT_
    KeychainEvent::_table_ =
        KeychainEvent::InternalGenerateParseTable_(KeychainEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void KeychainEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.KeychainEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  clear_event();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL KeychainEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const KeychainEvent& this_ = static_cast<const KeychainEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL KeychainEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const KeychainEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.KeychainEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  switch (this_.event_case()) {
    case kChangeKeychainFinished: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          1, *this_._impl_.event_.changekeychainfinished_, this_._impl_.event_.changekeychainfinished_->GetCachedSize(), target,
          stream);
      break;
    }
    case kHasNoKeychain: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          2, *this_._impl_.event_.hasnokeychain_, this_._impl_.event_.hasnokeychain_->GetCachedSize(), target,
          stream);
      break;
    }
    case kRebuildKeychain: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          3, *this_._impl_.event_.rebuildkeychain_, this_._impl_.event_.rebuildkeychain_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.KeychainEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t KeychainEvent::ByteSizeLong(const MessageLite& base) {
  const KeychainEvent& this_ = static_cast<const KeychainEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t KeychainEvent::ByteSizeLong() const {
  const KeychainEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.KeychainEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  switch (this_.event_case()) {
    // .grpc.ChangeKeychainFinishedEvent changeKeychainFinished = 1;
    case kChangeKeychainFinished: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.changekeychainfinished_);
      break;
    }
    // .grpc.HasNoKeychainEvent hasNoKeychain = 2;
    case kHasNoKeychain: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.hasnokeychain_);
      break;
    }
    // .grpc.RebuildKeychainEvent rebuildKeychain = 3;
    case kRebuildKeychain: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.rebuildkeychain_);
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void KeychainEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<KeychainEvent*>(&to_msg);
  auto& from = static_cast<const KeychainEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.KeychainEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_event();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kChangeKeychainFinished: {
        if (oneof_needs_init) {
          _this->_impl_.event_.changekeychainfinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.changekeychainfinished_);
        } else {
          _this->_impl_.event_.changekeychainfinished_->MergeFrom(*from._impl_.event_.changekeychainfinished_);
        }
        break;
      }
      case kHasNoKeychain: {
        if (oneof_needs_init) {
          _this->_impl_.event_.hasnokeychain_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.hasnokeychain_);
        } else {
          _this->_impl_.event_.hasnokeychain_->MergeFrom(*from._impl_.event_.hasnokeychain_);
        }
        break;
      }
      case kRebuildKeychain: {
        if (oneof_needs_init) {
          _this->_impl_.event_.rebuildkeychain_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.rebuildkeychain_);
        } else {
          _this->_impl_.event_.rebuildkeychain_->MergeFrom(*from._impl_.event_.rebuildkeychain_);
        }
        break;
      }
      case EVENT_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void KeychainEvent::CopyFrom(const KeychainEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.KeychainEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void KeychainEvent::InternalSwap(KeychainEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_.event_, other->_impl_.event_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata KeychainEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ChangeKeychainFinishedEvent::ChangeKeychainFinishedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ChangeKeychainFinishedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.ChangeKeychainFinishedEvent)
}
ChangeKeychainFinishedEvent::ChangeKeychainFinishedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ChangeKeychainFinishedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ChangeKeychainFinishedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ChangeKeychainFinishedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.ChangeKeychainFinishedEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ChangeKeychainFinishedEvent_class_data_ =
        ChangeKeychainFinishedEvent::InternalGenerateClassData_(ChangeKeychainFinishedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ChangeKeychainFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ChangeKeychainFinishedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ChangeKeychainFinishedEvent_class_data_.tc_table);
  return ChangeKeychainFinishedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ChangeKeychainFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ChangeKeychainFinishedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ChangeKeychainFinishedEvent_globals_));
  return ChangeKeychainFinishedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ChangeKeychainFinishedEvent::ParseTableT_
    ChangeKeychainFinishedEvent::_table_ =
        ChangeKeychainFinishedEvent::InternalGenerateParseTable_(ChangeKeychainFinishedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata ChangeKeychainFinishedEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

HasNoKeychainEvent::HasNoKeychainEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, HasNoKeychainEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.HasNoKeychainEvent)
}
HasNoKeychainEvent::HasNoKeychainEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const HasNoKeychainEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, HasNoKeychainEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  HasNoKeychainEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.HasNoKeychainEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull HasNoKeychainEvent_class_data_ =
        HasNoKeychainEvent::InternalGenerateClassData_(HasNoKeychainEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
HasNoKeychainEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&HasNoKeychainEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(HasNoKeychainEvent_class_data_.tc_table);
  return HasNoKeychainEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
HasNoKeychainEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&HasNoKeychainEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&HasNoKeychainEvent_globals_));
  return HasNoKeychainEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const HasNoKeychainEvent::ParseTableT_
    HasNoKeychainEvent::_table_ =
        HasNoKeychainEvent::InternalGenerateParseTable_(HasNoKeychainEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata HasNoKeychainEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

RebuildKeychainEvent::RebuildKeychainEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, RebuildKeychainEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.RebuildKeychainEvent)
}
RebuildKeychainEvent::RebuildKeychainEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const RebuildKeychainEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, RebuildKeychainEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  RebuildKeychainEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.RebuildKeychainEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull RebuildKeychainEvent_class_data_ =
        RebuildKeychainEvent::InternalGenerateClassData_(RebuildKeychainEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
RebuildKeychainEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&RebuildKeychainEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(RebuildKeychainEvent_class_data_.tc_table);
  return RebuildKeychainEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
RebuildKeychainEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&RebuildKeychainEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&RebuildKeychainEvent_globals_));
  return RebuildKeychainEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const RebuildKeychainEvent::ParseTableT_
    RebuildKeychainEvent::_table_ =
        RebuildKeychainEvent::InternalGenerateParseTable_(RebuildKeychainEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata RebuildKeychainEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void MailEvent::set_allocated_addresschanged(::grpc::AddressChangedEvent* PROTOBUF_NULLABLE addresschanged) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (addresschanged) {
    ::google::protobuf::Arena* submessage_arena = addresschanged->GetArena();
    if (message_arena != submessage_arena) {
      addresschanged = ::google::protobuf::internal::GetOwnedMessage(message_arena, addresschanged, submessage_arena);
    }
    set_has_addresschanged();
    _impl_.event_.addresschanged_ = addresschanged;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.MailEvent.addressChanged)
}
void MailEvent::set_allocated_addresschangedlogout(::grpc::AddressChangedLogoutEvent* PROTOBUF_NULLABLE addresschangedlogout) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (addresschangedlogout) {
    ::google::protobuf::Arena* submessage_arena = addresschangedlogout->GetArena();
    if (message_arena != submessage_arena) {
      addresschangedlogout = ::google::protobuf::internal::GetOwnedMessage(message_arena, addresschangedlogout, submessage_arena);
    }
    set_has_addresschangedlogout();
    _impl_.event_.addresschangedlogout_ = addresschangedlogout;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.MailEvent.addressChangedLogout)
}
void MailEvent::set_allocated_apicertissue(::grpc::ApiCertIssueEvent* PROTOBUF_NULLABLE apicertissue) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (apicertissue) {
    ::google::protobuf::Arena* submessage_arena = apicertissue->GetArena();
    if (message_arena != submessage_arena) {
      apicertissue = ::google::protobuf::internal::GetOwnedMessage(message_arena, apicertissue, submessage_arena);
    }
    set_has_apicertissue();
    _impl_.event_.apicertissue_ = apicertissue;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.MailEvent.apiCertIssue)
}
MailEvent::MailEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, MailEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.MailEvent)
}
PROTOBUF_NDEBUG_INLINE MailEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::MailEvent& from_msg)
      : event_{},
        _cached_size_{0},
        _oneof_case_{from._oneof_case_[0]} {}

MailEvent::MailEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const MailEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, MailEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  MailEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  switch (event_case()) {
    case EVENT_NOT_SET:
      break;
      case kAddressChanged:
        _impl_.event_.addresschanged_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.addresschanged_);
        break;
      case kAddressChangedLogout:
        _impl_.event_.addresschangedlogout_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.addresschangedlogout_);
        break;
      case kApiCertIssue:
        _impl_.event_.apicertissue_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.apicertissue_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:grpc.MailEvent)
}
PROTOBUF_NDEBUG_INLINE MailEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

inline void MailEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
MailEvent::~MailEvent() {
  // @@protoc_insertion_point(destructor:grpc.MailEvent)
  SharedDtor(*this);
}
inline void MailEvent::SharedDtor(MessageLite& self) {
  MailEvent& this_ = static_cast<MailEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  if (this_.has_event()) {
    this_.clear_event();
  }
  this_._impl_.~Impl_();
}

void MailEvent::clear_event() {
// @@protoc_insertion_point(one_of_clear_start:grpc.MailEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (event_case()) {
    case kAddressChanged: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.addresschanged_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.addresschanged_);
      }
      break;
    }
    case kAddressChangedLogout: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.addresschangedlogout_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.addresschangedlogout_);
      }
      break;
    }
    case kApiCertIssue: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.apicertissue_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.apicertissue_);
      }
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = EVENT_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull MailEvent_class_data_ =
        MailEvent::InternalGenerateClassData_(MailEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
MailEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&MailEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(MailEvent_class_data_.tc_table);
  return MailEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
MailEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&MailEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&MailEvent_globals_));
  return MailEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const MailEvent::ParseTableT_
    MailEvent::_table_ =
        MailEvent::InternalGenerateParseTable_(MailEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void MailEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.MailEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  clear_event();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL MailEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const MailEvent& this_ = static_cast<const MailEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL MailEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const MailEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.MailEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  switch (this_.event_case()) {
    case kAddressChanged: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          1, *this_._impl_.event_.addresschanged_, this_._impl_.event_.addresschanged_->GetCachedSize(), target,
          stream);
      break;
    }
    case kAddressChangedLogout: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          2, *this_._impl_.event_.addresschangedlogout_, this_._impl_.event_.addresschangedlogout_->GetCachedSize(), target,
          stream);
      break;
    }
    case kApiCertIssue: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          3, *this_._impl_.event_.apicertissue_, this_._impl_.event_.apicertissue_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.MailEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t MailEvent::ByteSizeLong(const MessageLite& base) {
  const MailEvent& this_ = static_cast<const MailEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t MailEvent::ByteSizeLong() const {
  const MailEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.MailEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  switch (this_.event_case()) {
    // .grpc.AddressChangedEvent addressChanged = 1;
    case kAddressChanged: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.addresschanged_);
      break;
    }
    // .grpc.AddressChangedLogoutEvent addressChangedLogout = 2;
    case kAddressChangedLogout: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.addresschangedlogout_);
      break;
    }
    // .grpc.ApiCertIssueEvent apiCertIssue = 3;
    case kApiCertIssue: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.apicertissue_);
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void MailEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<MailEvent*>(&to_msg);
  auto& from = static_cast<const MailEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.MailEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_event();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kAddressChanged: {
        if (oneof_needs_init) {
          _this->_impl_.event_.addresschanged_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.addresschanged_);
        } else {
          _this->_impl_.event_.addresschanged_->MergeFrom(*from._impl_.event_.addresschanged_);
        }
        break;
      }
      case kAddressChangedLogout: {
        if (oneof_needs_init) {
          _this->_impl_.event_.addresschangedlogout_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.addresschangedlogout_);
        } else {
          _this->_impl_.event_.addresschangedlogout_->MergeFrom(*from._impl_.event_.addresschangedlogout_);
        }
        break;
      }
      case kApiCertIssue: {
        if (oneof_needs_init) {
          _this->_impl_.event_.apicertissue_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.apicertissue_);
        } else {
          _this->_impl_.event_.apicertissue_->MergeFrom(*from._impl_.event_.apicertissue_);
        }
        break;
      }
      case EVENT_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void MailEvent::CopyFrom(const MailEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.MailEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void MailEvent::InternalSwap(MailEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_.event_, other->_impl_.event_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata MailEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

AddressChangedEvent::AddressChangedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AddressChangedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.AddressChangedEvent)
}
PROTOBUF_NDEBUG_INLINE AddressChangedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::AddressChangedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        address_(arena, from.address_) {}

AddressChangedEvent::AddressChangedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const AddressChangedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AddressChangedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  AddressChangedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.AddressChangedEvent)
}
PROTOBUF_NDEBUG_INLINE AddressChangedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        address_(arena) {}

inline void AddressChangedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
AddressChangedEvent::~AddressChangedEvent() {
  // @@protoc_insertion_point(destructor:grpc.AddressChangedEvent)
  SharedDtor(*this);
}
inline void AddressChangedEvent::SharedDtor(MessageLite& self) {
  AddressChangedEvent& this_ = static_cast<AddressChangedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.address_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AddressChangedEvent_class_data_ =
        AddressChangedEvent::InternalGenerateClassData_(AddressChangedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AddressChangedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AddressChangedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AddressChangedEvent_class_data_.tc_table);
  return AddressChangedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AddressChangedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AddressChangedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AddressChangedEvent_globals_));
  return AddressChangedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AddressChangedEvent::ParseTableT_
    AddressChangedEvent::_table_ =
        AddressChangedEvent::InternalGenerateParseTable_(AddressChangedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void AddressChangedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.AddressChangedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.address_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL AddressChangedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const AddressChangedEvent& this_ = static_cast<const AddressChangedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL AddressChangedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const AddressChangedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.AddressChangedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string address = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_address().empty()) {
      const ::std::string& _s = this_._internal_address();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.AddressChangedEvent.address");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.AddressChangedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t AddressChangedEvent::ByteSizeLong(const MessageLite& base) {
  const AddressChangedEvent& this_ = static_cast<const AddressChangedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t AddressChangedEvent::ByteSizeLong() const {
  const AddressChangedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.AddressChangedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string address = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_address().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_address());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void AddressChangedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<AddressChangedEvent*>(&to_msg);
  auto& from = static_cast<const AddressChangedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.AddressChangedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_address().empty()) {
      _this->_internal_set_address(from._internal_address());
    } else {
      if (_this->_impl_.address_.IsDefault()) {
        _this->_internal_set_address("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void AddressChangedEvent::CopyFrom(const AddressChangedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.AddressChangedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void AddressChangedEvent::InternalSwap(AddressChangedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.address_, &other->_impl_.address_, arena);
}

::google::protobuf::Metadata AddressChangedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

AddressChangedLogoutEvent::AddressChangedLogoutEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AddressChangedLogoutEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.AddressChangedLogoutEvent)
}
PROTOBUF_NDEBUG_INLINE AddressChangedLogoutEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::AddressChangedLogoutEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        address_(arena, from.address_) {}

AddressChangedLogoutEvent::AddressChangedLogoutEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const AddressChangedLogoutEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AddressChangedLogoutEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  AddressChangedLogoutEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.AddressChangedLogoutEvent)
}
PROTOBUF_NDEBUG_INLINE AddressChangedLogoutEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        address_(arena) {}

inline void AddressChangedLogoutEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
AddressChangedLogoutEvent::~AddressChangedLogoutEvent() {
  // @@protoc_insertion_point(destructor:grpc.AddressChangedLogoutEvent)
  SharedDtor(*this);
}
inline void AddressChangedLogoutEvent::SharedDtor(MessageLite& self) {
  AddressChangedLogoutEvent& this_ = static_cast<AddressChangedLogoutEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.address_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AddressChangedLogoutEvent_class_data_ =
        AddressChangedLogoutEvent::InternalGenerateClassData_(AddressChangedLogoutEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AddressChangedLogoutEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AddressChangedLogoutEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AddressChangedLogoutEvent_class_data_.tc_table);
  return AddressChangedLogoutEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AddressChangedLogoutEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AddressChangedLogoutEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AddressChangedLogoutEvent_globals_));
  return AddressChangedLogoutEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AddressChangedLogoutEvent::ParseTableT_
    AddressChangedLogoutEvent::_table_ =
        AddressChangedLogoutEvent::InternalGenerateParseTable_(AddressChangedLogoutEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void AddressChangedLogoutEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.AddressChangedLogoutEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.address_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL AddressChangedLogoutEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const AddressChangedLogoutEvent& this_ = static_cast<const AddressChangedLogoutEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL AddressChangedLogoutEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const AddressChangedLogoutEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.AddressChangedLogoutEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string address = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_address().empty()) {
      const ::std::string& _s = this_._internal_address();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.AddressChangedLogoutEvent.address");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.AddressChangedLogoutEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t AddressChangedLogoutEvent::ByteSizeLong(const MessageLite& base) {
  const AddressChangedLogoutEvent& this_ = static_cast<const AddressChangedLogoutEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t AddressChangedLogoutEvent::ByteSizeLong() const {
  const AddressChangedLogoutEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.AddressChangedLogoutEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string address = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_address().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_address());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void AddressChangedLogoutEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<AddressChangedLogoutEvent*>(&to_msg);
  auto& from = static_cast<const AddressChangedLogoutEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.AddressChangedLogoutEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_address().empty()) {
      _this->_internal_set_address(from._internal_address());
    } else {
      if (_this->_impl_.address_.IsDefault()) {
        _this->_internal_set_address("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void AddressChangedLogoutEvent::CopyFrom(const AddressChangedLogoutEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.AddressChangedLogoutEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void AddressChangedLogoutEvent::InternalSwap(AddressChangedLogoutEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.address_, &other->_impl_.address_, arena);
}

::google::protobuf::Metadata AddressChangedLogoutEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ApiCertIssueEvent::ApiCertIssueEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ApiCertIssueEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:grpc.ApiCertIssueEvent)
}
ApiCertIssueEvent::ApiCertIssueEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ApiCertIssueEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ApiCertIssueEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ApiCertIssueEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:grpc.ApiCertIssueEvent)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ApiCertIssueEvent_class_data_ =
        ApiCertIssueEvent::InternalGenerateClassData_(ApiCertIssueEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ApiCertIssueEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ApiCertIssueEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ApiCertIssueEvent_class_data_.tc_table);
  return ApiCertIssueEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ApiCertIssueEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ApiCertIssueEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ApiCertIssueEvent_globals_));
  return ApiCertIssueEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ApiCertIssueEvent::ParseTableT_
    ApiCertIssueEvent::_table_ =
        ApiCertIssueEvent::InternalGenerateParseTable_(ApiCertIssueEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata ApiCertIssueEvent::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void UserEvent::set_allocated_togglesplitmodefinished(::grpc::ToggleSplitModeFinishedEvent* PROTOBUF_NULLABLE togglesplitmodefinished) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (togglesplitmodefinished) {
    ::google::protobuf::Arena* submessage_arena = togglesplitmodefinished->GetArena();
    if (message_arena != submessage_arena) {
      togglesplitmodefinished = ::google::protobuf::internal::GetOwnedMessage(message_arena, togglesplitmodefinished, submessage_arena);
    }
    set_has_togglesplitmodefinished();
    _impl_.event_.togglesplitmodefinished_ = togglesplitmodefinished;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UserEvent.toggleSplitModeFinished)
}
void UserEvent::set_allocated_userdisconnected(::grpc::UserDisconnectedEvent* PROTOBUF_NULLABLE userdisconnected) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (userdisconnected) {
    ::google::protobuf::Arena* submessage_arena = userdisconnected->GetArena();
    if (message_arena != submessage_arena) {
      userdisconnected = ::google::protobuf::internal::GetOwnedMessage(message_arena, userdisconnected, submessage_arena);
    }
    set_has_userdisconnected();
    _impl_.event_.userdisconnected_ = userdisconnected;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UserEvent.userDisconnected)
}
void UserEvent::set_allocated_userchanged(::grpc::UserChangedEvent* PROTOBUF_NULLABLE userchanged) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (userchanged) {
    ::google::protobuf::Arena* submessage_arena = userchanged->GetArena();
    if (message_arena != submessage_arena) {
      userchanged = ::google::protobuf::internal::GetOwnedMessage(message_arena, userchanged, submessage_arena);
    }
    set_has_userchanged();
    _impl_.event_.userchanged_ = userchanged;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UserEvent.userChanged)
}
void UserEvent::set_allocated_userbadevent(::grpc::UserBadEvent* PROTOBUF_NULLABLE userbadevent) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (userbadevent) {
    ::google::protobuf::Arena* submessage_arena = userbadevent->GetArena();
    if (message_arena != submessage_arena) {
      userbadevent = ::google::protobuf::internal::GetOwnedMessage(message_arena, userbadevent, submessage_arena);
    }
    set_has_userbadevent();
    _impl_.event_.userbadevent_ = userbadevent;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UserEvent.userBadEvent)
}
void UserEvent::set_allocated_usedbyteschangedevent(::grpc::UsedBytesChangedEvent* PROTOBUF_NULLABLE usedbyteschangedevent) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (usedbyteschangedevent) {
    ::google::protobuf::Arena* submessage_arena = usedbyteschangedevent->GetArena();
    if (message_arena != submessage_arena) {
      usedbyteschangedevent = ::google::protobuf::internal::GetOwnedMessage(message_arena, usedbyteschangedevent, submessage_arena);
    }
    set_has_usedbyteschangedevent();
    _impl_.event_.usedbyteschangedevent_ = usedbyteschangedevent;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UserEvent.usedBytesChangedEvent)
}
void UserEvent::set_allocated_imaploginfailedevent(::grpc::ImapLoginFailedEvent* PROTOBUF_NULLABLE imaploginfailedevent) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (imaploginfailedevent) {
    ::google::protobuf::Arena* submessage_arena = imaploginfailedevent->GetArena();
    if (message_arena != submessage_arena) {
      imaploginfailedevent = ::google::protobuf::internal::GetOwnedMessage(message_arena, imaploginfailedevent, submessage_arena);
    }
    set_has_imaploginfailedevent();
    _impl_.event_.imaploginfailedevent_ = imaploginfailedevent;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UserEvent.imapLoginFailedEvent)
}
void UserEvent::set_allocated_syncstartedevent(::grpc::SyncStartedEvent* PROTOBUF_NULLABLE syncstartedevent) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (syncstartedevent) {
    ::google::protobuf::Arena* submessage_arena = syncstartedevent->GetArena();
    if (message_arena != submessage_arena) {
      syncstartedevent = ::google::protobuf::internal::GetOwnedMessage(message_arena, syncstartedevent, submessage_arena);
    }
    set_has_syncstartedevent();
    _impl_.event_.syncstartedevent_ = syncstartedevent;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UserEvent.syncStartedEvent)
}
void UserEvent::set_allocated_syncfinishedevent(::grpc::SyncFinishedEvent* PROTOBUF_NULLABLE syncfinishedevent) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (syncfinishedevent) {
    ::google::protobuf::Arena* submessage_arena = syncfinishedevent->GetArena();
    if (message_arena != submessage_arena) {
      syncfinishedevent = ::google::protobuf::internal::GetOwnedMessage(message_arena, syncfinishedevent, submessage_arena);
    }
    set_has_syncfinishedevent();
    _impl_.event_.syncfinishedevent_ = syncfinishedevent;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UserEvent.syncFinishedEvent)
}
void UserEvent::set_allocated_syncprogressevent(::grpc::SyncProgressEvent* PROTOBUF_NULLABLE syncprogressevent) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_event();
  if (syncprogressevent) {
    ::google::protobuf::Arena* submessage_arena = syncprogressevent->GetArena();
    if (message_arena != submessage_arena) {
      syncprogressevent = ::google::protobuf::internal::GetOwnedMessage(message_arena, syncprogressevent, submessage_arena);
    }
    set_has_syncprogressevent();
    _impl_.event_.syncprogressevent_ = syncprogressevent;
  }
  // @@protoc_insertion_point(field_set_allocated:grpc.UserEvent.syncProgressEvent)
}
UserEvent::UserEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.UserEvent)
}
PROTOBUF_NDEBUG_INLINE UserEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::UserEvent& from_msg)
      : event_{},
        _cached_size_{0},
        _oneof_case_{from._oneof_case_[0]} {}

UserEvent::UserEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UserEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UserEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  switch (event_case()) {
    case EVENT_NOT_SET:
      break;
      case kToggleSplitModeFinished:
        _impl_.event_.togglesplitmodefinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.togglesplitmodefinished_);
        break;
      case kUserDisconnected:
        _impl_.event_.userdisconnected_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.userdisconnected_);
        break;
      case kUserChanged:
        _impl_.event_.userchanged_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.userchanged_);
        break;
      case kUserBadEvent:
        _impl_.event_.userbadevent_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.userbadevent_);
        break;
      case kUsedBytesChangedEvent:
        _impl_.event_.usedbyteschangedevent_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.usedbyteschangedevent_);
        break;
      case kImapLoginFailedEvent:
        _impl_.event_.imaploginfailedevent_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.imaploginfailedevent_);
        break;
      case kSyncStartedEvent:
        _impl_.event_.syncstartedevent_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.syncstartedevent_);
        break;
      case kSyncFinishedEvent:
        _impl_.event_.syncfinishedevent_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.syncfinishedevent_);
        break;
      case kSyncProgressEvent:
        _impl_.event_.syncprogressevent_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.syncprogressevent_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:grpc.UserEvent)
}
PROTOBUF_NDEBUG_INLINE UserEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : event_{},
        _cached_size_{0},
        _oneof_case_{} {}

inline void UserEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
UserEvent::~UserEvent() {
  // @@protoc_insertion_point(destructor:grpc.UserEvent)
  SharedDtor(*this);
}
inline void UserEvent::SharedDtor(MessageLite& self) {
  UserEvent& this_ = static_cast<UserEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  if (this_.has_event()) {
    this_.clear_event();
  }
  this_._impl_.~Impl_();
}

void UserEvent::clear_event() {
// @@protoc_insertion_point(one_of_clear_start:grpc.UserEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (event_case()) {
    case kToggleSplitModeFinished: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.togglesplitmodefinished_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.togglesplitmodefinished_);
      }
      break;
    }
    case kUserDisconnected: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.userdisconnected_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.userdisconnected_);
      }
      break;
    }
    case kUserChanged: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.userchanged_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.userchanged_);
      }
      break;
    }
    case kUserBadEvent: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.userbadevent_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.userbadevent_);
      }
      break;
    }
    case kUsedBytesChangedEvent: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.usedbyteschangedevent_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.usedbyteschangedevent_);
      }
      break;
    }
    case kImapLoginFailedEvent: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.imaploginfailedevent_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.imaploginfailedevent_);
      }
      break;
    }
    case kSyncStartedEvent: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.syncstartedevent_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.syncstartedevent_);
      }
      break;
    }
    case kSyncFinishedEvent: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.syncfinishedevent_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.syncfinishedevent_);
      }
      break;
    }
    case kSyncProgressEvent: {
      if (GetArena() == nullptr) {
        delete _impl_.event_.syncprogressevent_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.event_.syncprogressevent_);
      }
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = EVENT_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UserEvent_class_data_ =
        UserEvent::InternalGenerateClassData_(UserEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UserEvent_class_data_.tc_table);
  return UserEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UserEvent_globals_));
  return UserEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UserEvent::ParseTableT_
    UserEvent::_table_ =
        UserEvent::InternalGenerateParseTable_(UserEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UserEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.UserEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  clear_event();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UserEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UserEvent& this_ = static_cast<const UserEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UserEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UserEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.UserEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  switch (this_.event_case()) {
    case kToggleSplitModeFinished: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          1, *this_._impl_.event_.togglesplitmodefinished_, this_._impl_.event_.togglesplitmodefinished_->GetCachedSize(), target,
          stream);
      break;
    }
    case kUserDisconnected: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          2, *this_._impl_.event_.userdisconnected_, this_._impl_.event_.userdisconnected_->GetCachedSize(), target,
          stream);
      break;
    }
    case kUserChanged: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          3, *this_._impl_.event_.userchanged_, this_._impl_.event_.userchanged_->GetCachedSize(), target,
          stream);
      break;
    }
    case kUserBadEvent: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          4, *this_._impl_.event_.userbadevent_, this_._impl_.event_.userbadevent_->GetCachedSize(), target,
          stream);
      break;
    }
    case kUsedBytesChangedEvent: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          5, *this_._impl_.event_.usedbyteschangedevent_, this_._impl_.event_.usedbyteschangedevent_->GetCachedSize(), target,
          stream);
      break;
    }
    case kImapLoginFailedEvent: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          6, *this_._impl_.event_.imaploginfailedevent_, this_._impl_.event_.imaploginfailedevent_->GetCachedSize(), target,
          stream);
      break;
    }
    case kSyncStartedEvent: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          7, *this_._impl_.event_.syncstartedevent_, this_._impl_.event_.syncstartedevent_->GetCachedSize(), target,
          stream);
      break;
    }
    case kSyncFinishedEvent: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          8, *this_._impl_.event_.syncfinishedevent_, this_._impl_.event_.syncfinishedevent_->GetCachedSize(), target,
          stream);
      break;
    }
    case kSyncProgressEvent: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          9, *this_._impl_.event_.syncprogressevent_, this_._impl_.event_.syncprogressevent_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.UserEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UserEvent::ByteSizeLong(const MessageLite& base) {
  const UserEvent& this_ = static_cast<const UserEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UserEvent::ByteSizeLong() const {
  const UserEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.UserEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  switch (this_.event_case()) {
    // .grpc.ToggleSplitModeFinishedEvent toggleSplitModeFinished = 1;
    case kToggleSplitModeFinished: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.togglesplitmodefinished_);
      break;
    }
    // .grpc.UserDisconnectedEvent userDisconnected = 2;
    case kUserDisconnected: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.userdisconnected_);
      break;
    }
    // .grpc.UserChangedEvent userChanged = 3;
    case kUserChanged: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.userchanged_);
      break;
    }
    // .grpc.UserBadEvent userBadEvent = 4;
    case kUserBadEvent: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.userbadevent_);
      break;
    }
    // .grpc.UsedBytesChangedEvent usedBytesChangedEvent = 5;
    case kUsedBytesChangedEvent: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.usedbyteschangedevent_);
      break;
    }
    // .grpc.ImapLoginFailedEvent imapLoginFailedEvent = 6;
    case kImapLoginFailedEvent: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.imaploginfailedevent_);
      break;
    }
    // .grpc.SyncStartedEvent syncStartedEvent = 7;
    case kSyncStartedEvent: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.syncstartedevent_);
      break;
    }
    // .grpc.SyncFinishedEvent syncFinishedEvent = 8;
    case kSyncFinishedEvent: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.syncfinishedevent_);
      break;
    }
    // .grpc.SyncProgressEvent syncProgressEvent = 9;
    case kSyncProgressEvent: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.event_.syncprogressevent_);
      break;
    }
    case EVENT_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UserEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UserEvent*>(&to_msg);
  auto& from = static_cast<const UserEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.UserEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_event();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kToggleSplitModeFinished: {
        if (oneof_needs_init) {
          _this->_impl_.event_.togglesplitmodefinished_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.togglesplitmodefinished_);
        } else {
          _this->_impl_.event_.togglesplitmodefinished_->MergeFrom(*from._impl_.event_.togglesplitmodefinished_);
        }
        break;
      }
      case kUserDisconnected: {
        if (oneof_needs_init) {
          _this->_impl_.event_.userdisconnected_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.userdisconnected_);
        } else {
          _this->_impl_.event_.userdisconnected_->MergeFrom(*from._impl_.event_.userdisconnected_);
        }
        break;
      }
      case kUserChanged: {
        if (oneof_needs_init) {
          _this->_impl_.event_.userchanged_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.userchanged_);
        } else {
          _this->_impl_.event_.userchanged_->MergeFrom(*from._impl_.event_.userchanged_);
        }
        break;
      }
      case kUserBadEvent: {
        if (oneof_needs_init) {
          _this->_impl_.event_.userbadevent_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.userbadevent_);
        } else {
          _this->_impl_.event_.userbadevent_->MergeFrom(*from._impl_.event_.userbadevent_);
        }
        break;
      }
      case kUsedBytesChangedEvent: {
        if (oneof_needs_init) {
          _this->_impl_.event_.usedbyteschangedevent_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.usedbyteschangedevent_);
        } else {
          _this->_impl_.event_.usedbyteschangedevent_->MergeFrom(*from._impl_.event_.usedbyteschangedevent_);
        }
        break;
      }
      case kImapLoginFailedEvent: {
        if (oneof_needs_init) {
          _this->_impl_.event_.imaploginfailedevent_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.imaploginfailedevent_);
        } else {
          _this->_impl_.event_.imaploginfailedevent_->MergeFrom(*from._impl_.event_.imaploginfailedevent_);
        }
        break;
      }
      case kSyncStartedEvent: {
        if (oneof_needs_init) {
          _this->_impl_.event_.syncstartedevent_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.syncstartedevent_);
        } else {
          _this->_impl_.event_.syncstartedevent_->MergeFrom(*from._impl_.event_.syncstartedevent_);
        }
        break;
      }
      case kSyncFinishedEvent: {
        if (oneof_needs_init) {
          _this->_impl_.event_.syncfinishedevent_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.syncfinishedevent_);
        } else {
          _this->_impl_.event_.syncfinishedevent_->MergeFrom(*from._impl_.event_.syncfinishedevent_);
        }
        break;
      }
      case kSyncProgressEvent: {
        if (oneof_needs_init) {
          _this->_impl_.event_.syncprogressevent_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.event_.syncprogressevent_);
        } else {
          _this->_impl_.event_.syncprogressevent_->MergeFrom(*from._impl_.event_.syncprogressevent_);
        }
        break;
      }
      case EVENT_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UserEvent::CopyFrom(const UserEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.UserEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UserEvent::InternalSwap(UserEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_.event_, other->_impl_.event_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata UserEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ToggleSplitModeFinishedEvent::ToggleSplitModeFinishedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ToggleSplitModeFinishedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.ToggleSplitModeFinishedEvent)
}
PROTOBUF_NDEBUG_INLINE ToggleSplitModeFinishedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::ToggleSplitModeFinishedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        userid_(arena, from.userid_) {}

ToggleSplitModeFinishedEvent::ToggleSplitModeFinishedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ToggleSplitModeFinishedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ToggleSplitModeFinishedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ToggleSplitModeFinishedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.ToggleSplitModeFinishedEvent)
}
PROTOBUF_NDEBUG_INLINE ToggleSplitModeFinishedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        userid_(arena) {}

inline void ToggleSplitModeFinishedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ToggleSplitModeFinishedEvent::~ToggleSplitModeFinishedEvent() {
  // @@protoc_insertion_point(destructor:grpc.ToggleSplitModeFinishedEvent)
  SharedDtor(*this);
}
inline void ToggleSplitModeFinishedEvent::SharedDtor(MessageLite& self) {
  ToggleSplitModeFinishedEvent& this_ = static_cast<ToggleSplitModeFinishedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.userid_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ToggleSplitModeFinishedEvent_class_data_ =
        ToggleSplitModeFinishedEvent::InternalGenerateClassData_(ToggleSplitModeFinishedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ToggleSplitModeFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ToggleSplitModeFinishedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ToggleSplitModeFinishedEvent_class_data_.tc_table);
  return ToggleSplitModeFinishedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ToggleSplitModeFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ToggleSplitModeFinishedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ToggleSplitModeFinishedEvent_globals_));
  return ToggleSplitModeFinishedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ToggleSplitModeFinishedEvent::ParseTableT_
    ToggleSplitModeFinishedEvent::_table_ =
        ToggleSplitModeFinishedEvent::InternalGenerateParseTable_(ToggleSplitModeFinishedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ToggleSplitModeFinishedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.ToggleSplitModeFinishedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.userid_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ToggleSplitModeFinishedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ToggleSplitModeFinishedEvent& this_ = static_cast<const ToggleSplitModeFinishedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ToggleSplitModeFinishedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ToggleSplitModeFinishedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.ToggleSplitModeFinishedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string userID = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_userid().empty()) {
      const ::std::string& _s = this_._internal_userid();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.ToggleSplitModeFinishedEvent.userID");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.ToggleSplitModeFinishedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ToggleSplitModeFinishedEvent::ByteSizeLong(const MessageLite& base) {
  const ToggleSplitModeFinishedEvent& this_ = static_cast<const ToggleSplitModeFinishedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ToggleSplitModeFinishedEvent::ByteSizeLong() const {
  const ToggleSplitModeFinishedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.ToggleSplitModeFinishedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string userID = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_userid().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_userid());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ToggleSplitModeFinishedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ToggleSplitModeFinishedEvent*>(&to_msg);
  auto& from = static_cast<const ToggleSplitModeFinishedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.ToggleSplitModeFinishedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_userid().empty()) {
      _this->_internal_set_userid(from._internal_userid());
    } else {
      if (_this->_impl_.userid_.IsDefault()) {
        _this->_internal_set_userid("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ToggleSplitModeFinishedEvent::CopyFrom(const ToggleSplitModeFinishedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.ToggleSplitModeFinishedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ToggleSplitModeFinishedEvent::InternalSwap(ToggleSplitModeFinishedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.userid_, &other->_impl_.userid_, arena);
}

::google::protobuf::Metadata ToggleSplitModeFinishedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UserDisconnectedEvent::UserDisconnectedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserDisconnectedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.UserDisconnectedEvent)
}
PROTOBUF_NDEBUG_INLINE UserDisconnectedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::UserDisconnectedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        username_(arena, from.username_) {}

UserDisconnectedEvent::UserDisconnectedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UserDisconnectedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserDisconnectedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UserDisconnectedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.UserDisconnectedEvent)
}
PROTOBUF_NDEBUG_INLINE UserDisconnectedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        username_(arena) {}

inline void UserDisconnectedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
UserDisconnectedEvent::~UserDisconnectedEvent() {
  // @@protoc_insertion_point(destructor:grpc.UserDisconnectedEvent)
  SharedDtor(*this);
}
inline void UserDisconnectedEvent::SharedDtor(MessageLite& self) {
  UserDisconnectedEvent& this_ = static_cast<UserDisconnectedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.username_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UserDisconnectedEvent_class_data_ =
        UserDisconnectedEvent::InternalGenerateClassData_(UserDisconnectedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserDisconnectedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserDisconnectedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UserDisconnectedEvent_class_data_.tc_table);
  return UserDisconnectedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserDisconnectedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserDisconnectedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UserDisconnectedEvent_globals_));
  return UserDisconnectedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UserDisconnectedEvent::ParseTableT_
    UserDisconnectedEvent::_table_ =
        UserDisconnectedEvent::InternalGenerateParseTable_(UserDisconnectedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UserDisconnectedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.UserDisconnectedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.username_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UserDisconnectedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UserDisconnectedEvent& this_ = static_cast<const UserDisconnectedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UserDisconnectedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UserDisconnectedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.UserDisconnectedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string username = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_username().empty()) {
      const ::std::string& _s = this_._internal_username();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.UserDisconnectedEvent.username");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.UserDisconnectedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UserDisconnectedEvent::ByteSizeLong(const MessageLite& base) {
  const UserDisconnectedEvent& this_ = static_cast<const UserDisconnectedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UserDisconnectedEvent::ByteSizeLong() const {
  const UserDisconnectedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.UserDisconnectedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string username = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_username().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_username());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UserDisconnectedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UserDisconnectedEvent*>(&to_msg);
  auto& from = static_cast<const UserDisconnectedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.UserDisconnectedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_username().empty()) {
      _this->_internal_set_username(from._internal_username());
    } else {
      if (_this->_impl_.username_.IsDefault()) {
        _this->_internal_set_username("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UserDisconnectedEvent::CopyFrom(const UserDisconnectedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.UserDisconnectedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UserDisconnectedEvent::InternalSwap(UserDisconnectedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.username_, &other->_impl_.username_, arena);
}

::google::protobuf::Metadata UserDisconnectedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UserChangedEvent::UserChangedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserChangedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.UserChangedEvent)
}
PROTOBUF_NDEBUG_INLINE UserChangedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::UserChangedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        userid_(arena, from.userid_) {}

UserChangedEvent::UserChangedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UserChangedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserChangedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UserChangedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.UserChangedEvent)
}
PROTOBUF_NDEBUG_INLINE UserChangedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        userid_(arena) {}

inline void UserChangedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
UserChangedEvent::~UserChangedEvent() {
  // @@protoc_insertion_point(destructor:grpc.UserChangedEvent)
  SharedDtor(*this);
}
inline void UserChangedEvent::SharedDtor(MessageLite& self) {
  UserChangedEvent& this_ = static_cast<UserChangedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.userid_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UserChangedEvent_class_data_ =
        UserChangedEvent::InternalGenerateClassData_(UserChangedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserChangedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserChangedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UserChangedEvent_class_data_.tc_table);
  return UserChangedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserChangedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserChangedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UserChangedEvent_globals_));
  return UserChangedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UserChangedEvent::ParseTableT_
    UserChangedEvent::_table_ =
        UserChangedEvent::InternalGenerateParseTable_(UserChangedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UserChangedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.UserChangedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.userid_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UserChangedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UserChangedEvent& this_ = static_cast<const UserChangedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UserChangedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UserChangedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.UserChangedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string userID = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_userid().empty()) {
      const ::std::string& _s = this_._internal_userid();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.UserChangedEvent.userID");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.UserChangedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UserChangedEvent::ByteSizeLong(const MessageLite& base) {
  const UserChangedEvent& this_ = static_cast<const UserChangedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UserChangedEvent::ByteSizeLong() const {
  const UserChangedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.UserChangedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string userID = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_userid().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_userid());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UserChangedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UserChangedEvent*>(&to_msg);
  auto& from = static_cast<const UserChangedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.UserChangedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_userid().empty()) {
      _this->_internal_set_userid(from._internal_userid());
    } else {
      if (_this->_impl_.userid_.IsDefault()) {
        _this->_internal_set_userid("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UserChangedEvent::CopyFrom(const UserChangedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.UserChangedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UserChangedEvent::InternalSwap(UserChangedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.userid_, &other->_impl_.userid_, arena);
}

::google::protobuf::Metadata UserChangedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UserBadEvent::UserBadEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserBadEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.UserBadEvent)
}
PROTOBUF_NDEBUG_INLINE UserBadEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::UserBadEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        userid_(arena, from.userid_),
        errormessage_(arena, from.errormessage_) {}

UserBadEvent::UserBadEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UserBadEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserBadEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UserBadEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.UserBadEvent)
}
PROTOBUF_NDEBUG_INLINE UserBadEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        userid_(arena),
        errormessage_(arena) {}

inline void UserBadEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
UserBadEvent::~UserBadEvent() {
  // @@protoc_insertion_point(destructor:grpc.UserBadEvent)
  SharedDtor(*this);
}
inline void UserBadEvent::SharedDtor(MessageLite& self) {
  UserBadEvent& this_ = static_cast<UserBadEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.userid_.Destroy();
  this_._impl_.errormessage_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UserBadEvent_class_data_ =
        UserBadEvent::InternalGenerateClassData_(UserBadEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserBadEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserBadEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UserBadEvent_class_data_.tc_table);
  return UserBadEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserBadEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserBadEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UserBadEvent_globals_));
  return UserBadEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UserBadEvent::ParseTableT_
    UserBadEvent::_table_ =
        UserBadEvent::InternalGenerateParseTable_(UserBadEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UserBadEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.UserBadEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.userid_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.errormessage_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UserBadEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UserBadEvent& this_ = static_cast<const UserBadEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UserBadEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UserBadEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.UserBadEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string userID = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_userid().empty()) {
      const ::std::string& _s = this_._internal_userid();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.UserBadEvent.userID");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string errorMessage = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_errormessage().empty()) {
      const ::std::string& _s = this_._internal_errormessage();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.UserBadEvent.errorMessage");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.UserBadEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UserBadEvent::ByteSizeLong(const MessageLite& base) {
  const UserBadEvent& this_ = static_cast<const UserBadEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UserBadEvent::ByteSizeLong() const {
  const UserBadEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.UserBadEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string userID = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_userid().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_userid());
      }
    }
    // string errorMessage = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_errormessage().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_errormessage());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UserBadEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UserBadEvent*>(&to_msg);
  auto& from = static_cast<const UserBadEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.UserBadEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_userid().empty()) {
        _this->_internal_set_userid(from._internal_userid());
      } else {
        if (_this->_impl_.userid_.IsDefault()) {
          _this->_internal_set_userid("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_errormessage().empty()) {
        _this->_internal_set_errormessage(from._internal_errormessage());
      } else {
        if (_this->_impl_.errormessage_.IsDefault()) {
          _this->_internal_set_errormessage("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UserBadEvent::CopyFrom(const UserBadEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.UserBadEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UserBadEvent::InternalSwap(UserBadEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.userid_, &other->_impl_.userid_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.errormessage_, &other->_impl_.errormessage_, arena);
}

::google::protobuf::Metadata UserBadEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UsedBytesChangedEvent::UsedBytesChangedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UsedBytesChangedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.UsedBytesChangedEvent)
}
PROTOBUF_NDEBUG_INLINE UsedBytesChangedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::UsedBytesChangedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        userid_(arena, from.userid_) {}

UsedBytesChangedEvent::UsedBytesChangedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UsedBytesChangedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UsedBytesChangedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UsedBytesChangedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.usedbytes_ = from._impl_.usedbytes_;

  // @@protoc_insertion_point(copy_constructor:grpc.UsedBytesChangedEvent)
}
PROTOBUF_NDEBUG_INLINE UsedBytesChangedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        userid_(arena) {}

inline void UsedBytesChangedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.usedbytes_ = {};
}
UsedBytesChangedEvent::~UsedBytesChangedEvent() {
  // @@protoc_insertion_point(destructor:grpc.UsedBytesChangedEvent)
  SharedDtor(*this);
}
inline void UsedBytesChangedEvent::SharedDtor(MessageLite& self) {
  UsedBytesChangedEvent& this_ = static_cast<UsedBytesChangedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.userid_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UsedBytesChangedEvent_class_data_ =
        UsedBytesChangedEvent::InternalGenerateClassData_(UsedBytesChangedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UsedBytesChangedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UsedBytesChangedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UsedBytesChangedEvent_class_data_.tc_table);
  return UsedBytesChangedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UsedBytesChangedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UsedBytesChangedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UsedBytesChangedEvent_globals_));
  return UsedBytesChangedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UsedBytesChangedEvent::ParseTableT_
    UsedBytesChangedEvent::_table_ =
        UsedBytesChangedEvent::InternalGenerateParseTable_(UsedBytesChangedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UsedBytesChangedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.UsedBytesChangedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.userid_.ClearNonDefaultToEmpty();
  }
  _impl_.usedbytes_ = ::int64_t{0};
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UsedBytesChangedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UsedBytesChangedEvent& this_ = static_cast<const UsedBytesChangedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UsedBytesChangedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UsedBytesChangedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.UsedBytesChangedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string userID = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_userid().empty()) {
      const ::std::string& _s = this_._internal_userid();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.UsedBytesChangedEvent.userID");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // int64 usedBytes = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_usedbytes() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<2>(
              stream, this_._internal_usedbytes(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.UsedBytesChangedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UsedBytesChangedEvent::ByteSizeLong(const MessageLite& base) {
  const UsedBytesChangedEvent& this_ = static_cast<const UsedBytesChangedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UsedBytesChangedEvent::ByteSizeLong() const {
  const UsedBytesChangedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.UsedBytesChangedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string userID = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_userid().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_userid());
      }
    }
    // int64 usedBytes = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_usedbytes() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_usedbytes());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UsedBytesChangedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UsedBytesChangedEvent*>(&to_msg);
  auto& from = static_cast<const UsedBytesChangedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.UsedBytesChangedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_userid().empty()) {
        _this->_internal_set_userid(from._internal_userid());
      } else {
        if (_this->_impl_.userid_.IsDefault()) {
          _this->_internal_set_userid("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_usedbytes() != 0) {
        _this->_impl_.usedbytes_ = from._impl_.usedbytes_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UsedBytesChangedEvent::CopyFrom(const UsedBytesChangedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.UsedBytesChangedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UsedBytesChangedEvent::InternalSwap(UsedBytesChangedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.userid_, &other->_impl_.userid_, arena);
  swap(_impl_.usedbytes_, other->_impl_.usedbytes_);
}

::google::protobuf::Metadata UsedBytesChangedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ImapLoginFailedEvent::ImapLoginFailedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ImapLoginFailedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.ImapLoginFailedEvent)
}
PROTOBUF_NDEBUG_INLINE ImapLoginFailedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::ImapLoginFailedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        username_(arena, from.username_) {}

ImapLoginFailedEvent::ImapLoginFailedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ImapLoginFailedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ImapLoginFailedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ImapLoginFailedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.ImapLoginFailedEvent)
}
PROTOBUF_NDEBUG_INLINE ImapLoginFailedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        username_(arena) {}

inline void ImapLoginFailedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ImapLoginFailedEvent::~ImapLoginFailedEvent() {
  // @@protoc_insertion_point(destructor:grpc.ImapLoginFailedEvent)
  SharedDtor(*this);
}
inline void ImapLoginFailedEvent::SharedDtor(MessageLite& self) {
  ImapLoginFailedEvent& this_ = static_cast<ImapLoginFailedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.username_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ImapLoginFailedEvent_class_data_ =
        ImapLoginFailedEvent::InternalGenerateClassData_(ImapLoginFailedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ImapLoginFailedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ImapLoginFailedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ImapLoginFailedEvent_class_data_.tc_table);
  return ImapLoginFailedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ImapLoginFailedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ImapLoginFailedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ImapLoginFailedEvent_globals_));
  return ImapLoginFailedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ImapLoginFailedEvent::ParseTableT_
    ImapLoginFailedEvent::_table_ =
        ImapLoginFailedEvent::InternalGenerateParseTable_(ImapLoginFailedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ImapLoginFailedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.ImapLoginFailedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.username_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ImapLoginFailedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ImapLoginFailedEvent& this_ = static_cast<const ImapLoginFailedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ImapLoginFailedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ImapLoginFailedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.ImapLoginFailedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string username = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_username().empty()) {
      const ::std::string& _s = this_._internal_username();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.ImapLoginFailedEvent.username");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.ImapLoginFailedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ImapLoginFailedEvent::ByteSizeLong(const MessageLite& base) {
  const ImapLoginFailedEvent& this_ = static_cast<const ImapLoginFailedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ImapLoginFailedEvent::ByteSizeLong() const {
  const ImapLoginFailedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.ImapLoginFailedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string username = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_username().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_username());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ImapLoginFailedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ImapLoginFailedEvent*>(&to_msg);
  auto& from = static_cast<const ImapLoginFailedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.ImapLoginFailedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_username().empty()) {
      _this->_internal_set_username(from._internal_username());
    } else {
      if (_this->_impl_.username_.IsDefault()) {
        _this->_internal_set_username("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ImapLoginFailedEvent::CopyFrom(const ImapLoginFailedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.ImapLoginFailedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ImapLoginFailedEvent::InternalSwap(ImapLoginFailedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.username_, &other->_impl_.username_, arena);
}

::google::protobuf::Metadata ImapLoginFailedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

SyncStartedEvent::SyncStartedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SyncStartedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.SyncStartedEvent)
}
PROTOBUF_NDEBUG_INLINE SyncStartedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::SyncStartedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        userid_(arena, from.userid_) {}

SyncStartedEvent::SyncStartedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const SyncStartedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SyncStartedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SyncStartedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.SyncStartedEvent)
}
PROTOBUF_NDEBUG_INLINE SyncStartedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        userid_(arena) {}

inline void SyncStartedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
SyncStartedEvent::~SyncStartedEvent() {
  // @@protoc_insertion_point(destructor:grpc.SyncStartedEvent)
  SharedDtor(*this);
}
inline void SyncStartedEvent::SharedDtor(MessageLite& self) {
  SyncStartedEvent& this_ = static_cast<SyncStartedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.userid_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull SyncStartedEvent_class_data_ =
        SyncStartedEvent::InternalGenerateClassData_(SyncStartedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SyncStartedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SyncStartedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(SyncStartedEvent_class_data_.tc_table);
  return SyncStartedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SyncStartedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SyncStartedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&SyncStartedEvent_globals_));
  return SyncStartedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const SyncStartedEvent::ParseTableT_
    SyncStartedEvent::_table_ =
        SyncStartedEvent::InternalGenerateParseTable_(SyncStartedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void SyncStartedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.SyncStartedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.userid_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL SyncStartedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const SyncStartedEvent& this_ = static_cast<const SyncStartedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL SyncStartedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const SyncStartedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.SyncStartedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string userID = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_userid().empty()) {
      const ::std::string& _s = this_._internal_userid();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.SyncStartedEvent.userID");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.SyncStartedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t SyncStartedEvent::ByteSizeLong(const MessageLite& base) {
  const SyncStartedEvent& this_ = static_cast<const SyncStartedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t SyncStartedEvent::ByteSizeLong() const {
  const SyncStartedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.SyncStartedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string userID = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_userid().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_userid());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void SyncStartedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<SyncStartedEvent*>(&to_msg);
  auto& from = static_cast<const SyncStartedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.SyncStartedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_userid().empty()) {
      _this->_internal_set_userid(from._internal_userid());
    } else {
      if (_this->_impl_.userid_.IsDefault()) {
        _this->_internal_set_userid("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void SyncStartedEvent::CopyFrom(const SyncStartedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.SyncStartedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void SyncStartedEvent::InternalSwap(SyncStartedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.userid_, &other->_impl_.userid_, arena);
}

::google::protobuf::Metadata SyncStartedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

SyncFinishedEvent::SyncFinishedEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SyncFinishedEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.SyncFinishedEvent)
}
PROTOBUF_NDEBUG_INLINE SyncFinishedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::SyncFinishedEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        userid_(arena, from.userid_) {}

SyncFinishedEvent::SyncFinishedEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const SyncFinishedEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SyncFinishedEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SyncFinishedEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.SyncFinishedEvent)
}
PROTOBUF_NDEBUG_INLINE SyncFinishedEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        userid_(arena) {}

inline void SyncFinishedEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
SyncFinishedEvent::~SyncFinishedEvent() {
  // @@protoc_insertion_point(destructor:grpc.SyncFinishedEvent)
  SharedDtor(*this);
}
inline void SyncFinishedEvent::SharedDtor(MessageLite& self) {
  SyncFinishedEvent& this_ = static_cast<SyncFinishedEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.userid_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull SyncFinishedEvent_class_data_ =
        SyncFinishedEvent::InternalGenerateClassData_(SyncFinishedEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SyncFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SyncFinishedEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(SyncFinishedEvent_class_data_.tc_table);
  return SyncFinishedEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SyncFinishedEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SyncFinishedEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&SyncFinishedEvent_globals_));
  return SyncFinishedEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const SyncFinishedEvent::ParseTableT_
    SyncFinishedEvent::_table_ =
        SyncFinishedEvent::InternalGenerateParseTable_(SyncFinishedEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void SyncFinishedEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.SyncFinishedEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.userid_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL SyncFinishedEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const SyncFinishedEvent& this_ = static_cast<const SyncFinishedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL SyncFinishedEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const SyncFinishedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.SyncFinishedEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string userID = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_userid().empty()) {
      const ::std::string& _s = this_._internal_userid();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.SyncFinishedEvent.userID");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.SyncFinishedEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t SyncFinishedEvent::ByteSizeLong(const MessageLite& base) {
  const SyncFinishedEvent& this_ = static_cast<const SyncFinishedEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t SyncFinishedEvent::ByteSizeLong() const {
  const SyncFinishedEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.SyncFinishedEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string userID = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_userid().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_userid());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void SyncFinishedEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<SyncFinishedEvent*>(&to_msg);
  auto& from = static_cast<const SyncFinishedEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.SyncFinishedEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_userid().empty()) {
      _this->_internal_set_userid(from._internal_userid());
    } else {
      if (_this->_impl_.userid_.IsDefault()) {
        _this->_internal_set_userid("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void SyncFinishedEvent::CopyFrom(const SyncFinishedEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.SyncFinishedEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void SyncFinishedEvent::InternalSwap(SyncFinishedEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.userid_, &other->_impl_.userid_, arena);
}

::google::protobuf::Metadata SyncFinishedEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

SyncProgressEvent::SyncProgressEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SyncProgressEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.SyncProgressEvent)
}
PROTOBUF_NDEBUG_INLINE SyncProgressEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::SyncProgressEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        userid_(arena, from.userid_) {}

SyncProgressEvent::SyncProgressEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const SyncProgressEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SyncProgressEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SyncProgressEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, progress_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, progress_),
           offsetof(Impl_, remainingms_) -
               offsetof(Impl_, progress_) +
               sizeof(Impl_::remainingms_));

  // @@protoc_insertion_point(copy_constructor:grpc.SyncProgressEvent)
}
PROTOBUF_NDEBUG_INLINE SyncProgressEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        userid_(arena) {}

inline void SyncProgressEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, progress_),
           0,
           offsetof(Impl_, remainingms_) -
               offsetof(Impl_, progress_) +
               sizeof(Impl_::remainingms_));
}
SyncProgressEvent::~SyncProgressEvent() {
  // @@protoc_insertion_point(destructor:grpc.SyncProgressEvent)
  SharedDtor(*this);
}
inline void SyncProgressEvent::SharedDtor(MessageLite& self) {
  SyncProgressEvent& this_ = static_cast<SyncProgressEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.userid_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull SyncProgressEvent_class_data_ =
        SyncProgressEvent::InternalGenerateClassData_(SyncProgressEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SyncProgressEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SyncProgressEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(SyncProgressEvent_class_data_.tc_table);
  return SyncProgressEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SyncProgressEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SyncProgressEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&SyncProgressEvent_globals_));
  return SyncProgressEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const SyncProgressEvent::ParseTableT_
    SyncProgressEvent::_table_ =
        SyncProgressEvent::InternalGenerateParseTable_(SyncProgressEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void SyncProgressEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.SyncProgressEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.userid_.ClearNonDefaultToEmpty();
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000000eU)) {
    ::memset(&_impl_.progress_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.remainingms_) -
        reinterpret_cast<char*>(&_impl_.progress_)) + sizeof(_impl_.remainingms_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL SyncProgressEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const SyncProgressEvent& this_ = static_cast<const SyncProgressEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL SyncProgressEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const SyncProgressEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.SyncProgressEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string userID = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_userid().empty()) {
      const ::std::string& _s = this_._internal_userid();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.SyncProgressEvent.userID");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // double progress = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (::absl::bit_cast<::uint64_t>(this_._internal_progress()) != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteDoubleToArray(
          2, this_._internal_progress(), target);
    }
  }

  // int64 elapsedMs = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_elapsedms() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<3>(
              stream, this_._internal_elapsedms(), target);
    }
  }

  // int64 remainingMs = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_remainingms() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<4>(
              stream, this_._internal_remainingms(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.SyncProgressEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t SyncProgressEvent::ByteSizeLong(const MessageLite& base) {
  const SyncProgressEvent& this_ = static_cast<const SyncProgressEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t SyncProgressEvent::ByteSizeLong() const {
  const SyncProgressEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.SyncProgressEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // string userID = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_userid().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_userid());
      }
    }
    // double progress = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (::absl::bit_cast<::uint64_t>(this_._internal_progress()) != 0) {
        total_size += 9;
      }
    }
    // int64 elapsedMs = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_elapsedms() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_elapsedms());
      }
    }
    // int64 remainingMs = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_remainingms() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_remainingms());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void SyncProgressEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<SyncProgressEvent*>(&to_msg);
  auto& from = static_cast<const SyncProgressEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.SyncProgressEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_userid().empty()) {
        _this->_internal_set_userid(from._internal_userid());
      } else {
        if (_this->_impl_.userid_.IsDefault()) {
          _this->_internal_set_userid("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (::absl::bit_cast<::uint64_t>(from._internal_progress()) != 0) {
        _this->_impl_.progress_ = from._impl_.progress_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_elapsedms() != 0) {
        _this->_impl_.elapsedms_ = from._impl_.elapsedms_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_remainingms() != 0) {
        _this->_impl_.remainingms_ = from._impl_.remainingms_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void SyncProgressEvent::CopyFrom(const SyncProgressEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.SyncProgressEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void SyncProgressEvent::InternalSwap(SyncProgressEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.userid_, &other->_impl_.userid_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(SyncProgressEvent, _impl_.remainingms_)
      + sizeof(SyncProgressEvent::_impl_.remainingms_)
      - PROTOBUF_FIELD_OFFSET(SyncProgressEvent, _impl_.progress_)>(
          reinterpret_cast<char*>(&_impl_.progress_),
          reinterpret_cast<char*>(&other->_impl_.progress_));
}

::google::protobuf::Metadata SyncProgressEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

UserNotificationEvent::UserNotificationEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserNotificationEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.UserNotificationEvent)
}
PROTOBUF_NDEBUG_INLINE UserNotificationEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::grpc::UserNotificationEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        title_(arena, from.title_),
        subtitle_(arena, from.subtitle_),
        body_(arena, from.body_),
        userid_(arena, from.userid_) {}

UserNotificationEvent::UserNotificationEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UserNotificationEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UserNotificationEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UserNotificationEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:grpc.UserNotificationEvent)
}
PROTOBUF_NDEBUG_INLINE UserNotificationEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        title_(arena),
        subtitle_(arena),
        body_(arena),
        userid_(arena) {}

inline void UserNotificationEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
UserNotificationEvent::~UserNotificationEvent() {
  // @@protoc_insertion_point(destructor:grpc.UserNotificationEvent)
  SharedDtor(*this);
}
inline void UserNotificationEvent::SharedDtor(MessageLite& self) {
  UserNotificationEvent& this_ = static_cast<UserNotificationEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.title_.Destroy();
  this_._impl_.subtitle_.Destroy();
  this_._impl_.body_.Destroy();
  this_._impl_.userid_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UserNotificationEvent_class_data_ =
        UserNotificationEvent::InternalGenerateClassData_(UserNotificationEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserNotificationEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserNotificationEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UserNotificationEvent_class_data_.tc_table);
  return UserNotificationEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UserNotificationEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UserNotificationEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UserNotificationEvent_globals_));
  return UserNotificationEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UserNotificationEvent::ParseTableT_
    UserNotificationEvent::_table_ =
        UserNotificationEvent::InternalGenerateParseTable_(UserNotificationEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UserNotificationEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.UserNotificationEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.title_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.subtitle_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.body_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.userid_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UserNotificationEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UserNotificationEvent& this_ = static_cast<const UserNotificationEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UserNotificationEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UserNotificationEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.UserNotificationEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string title = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_title().empty()) {
      const ::std::string& _s = this_._internal_title();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.UserNotificationEvent.title");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string subtitle = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_subtitle().empty()) {
      const ::std::string& _s = this_._internal_subtitle();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.UserNotificationEvent.subtitle");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // string body = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_body().empty()) {
      const ::std::string& _s = this_._internal_body();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.UserNotificationEvent.body");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  // string userID = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_userid().empty()) {
      const ::std::string& _s = this_._internal_userid();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "grpc.UserNotificationEvent.userID");
      target = stream->WriteStringMaybeAliased(4, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.UserNotificationEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UserNotificationEvent::ByteSizeLong(const MessageLite& base) {
  const UserNotificationEvent& this_ = static_cast<const UserNotificationEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UserNotificationEvent::ByteSizeLong() const {
  const UserNotificationEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.UserNotificationEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // string title = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_title().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_title());
      }
    }
    // string subtitle = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_subtitle().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_subtitle());
      }
    }
    // string body = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_body().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_body());
      }
    }
    // string userID = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!this_._internal_userid().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_userid());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UserNotificationEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UserNotificationEvent*>(&to_msg);
  auto& from = static_cast<const UserNotificationEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.UserNotificationEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_title().empty()) {
        _this->_internal_set_title(from._internal_title());
      } else {
        if (_this->_impl_.title_.IsDefault()) {
          _this->_internal_set_title("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_subtitle().empty()) {
        _this->_internal_set_subtitle(from._internal_subtitle());
      } else {
        if (_this->_impl_.subtitle_.IsDefault()) {
          _this->_internal_set_subtitle("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_body().empty()) {
        _this->_internal_set_body(from._internal_body());
      } else {
        if (_this->_impl_.body_.IsDefault()) {
          _this->_internal_set_body("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!from._internal_userid().empty()) {
        _this->_internal_set_userid(from._internal_userid());
      } else {
        if (_this->_impl_.userid_.IsDefault()) {
          _this->_internal_set_userid("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UserNotificationEvent::CopyFrom(const UserNotificationEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.UserNotificationEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UserNotificationEvent::InternalSwap(UserNotificationEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.title_, &other->_impl_.title_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.subtitle_, &other->_impl_.subtitle_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.body_, &other->_impl_.body_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.userid_, &other->_impl_.userid_, arena);
}

::google::protobuf::Metadata UserNotificationEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

GenericErrorEvent::GenericErrorEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GenericErrorEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:grpc.GenericErrorEvent)
}
GenericErrorEvent::GenericErrorEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const GenericErrorEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GenericErrorEvent_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE GenericErrorEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void GenericErrorEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.code_ = {};
}
GenericErrorEvent::~GenericErrorEvent() {
  // @@protoc_insertion_point(destructor:grpc.GenericErrorEvent)
  SharedDtor(*this);
}
inline void GenericErrorEvent::SharedDtor(MessageLite& self) {
  GenericErrorEvent& this_ = static_cast<GenericErrorEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull GenericErrorEvent_class_data_ =
        GenericErrorEvent::InternalGenerateClassData_(GenericErrorEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GenericErrorEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GenericErrorEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(GenericErrorEvent_class_data_.tc_table);
  return GenericErrorEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GenericErrorEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GenericErrorEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&GenericErrorEvent_globals_));
  return GenericErrorEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const GenericErrorEvent::ParseTableT_
    GenericErrorEvent::_table_ =
        GenericErrorEvent::InternalGenerateParseTable_(GenericErrorEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void GenericErrorEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:grpc.GenericErrorEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.code_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL GenericErrorEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const GenericErrorEvent& this_ = static_cast<const GenericErrorEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL GenericErrorEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const GenericErrorEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:grpc.GenericErrorEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .grpc.ErrorCode code = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_code() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_code(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:grpc.GenericErrorEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t GenericErrorEvent::ByteSizeLong(const MessageLite& base) {
  const GenericErrorEvent& this_ = static_cast<const GenericErrorEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t GenericErrorEvent::ByteSizeLong() const {
  const GenericErrorEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:grpc.GenericErrorEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .grpc.ErrorCode code = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_code() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_code());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void GenericErrorEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<GenericErrorEvent*>(&to_msg);
  auto& from = static_cast<const GenericErrorEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:grpc.GenericErrorEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (from._internal_code() != 0) {
      _this->_impl_.code_ = from._impl_.code_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void GenericErrorEvent::CopyFrom(const GenericErrorEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:grpc.GenericErrorEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void GenericErrorEvent::InternalSwap(GenericErrorEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.code_, other->_impl_.code_);
}

::google::protobuf::Metadata GenericErrorEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace grpc
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_bridge_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
